/////////////////////////////////////////////////////////////////////////////
// Name:        PlotBandStructure.cpp
// Author:      XX
// Created:     XX/XX/XX
// Copyright:   
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUG__
    #pragma implementation "PlotBandStructure.h"
#endif

// For compilers that support precompilation
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// Include private headers
#include "OhmmsPETE/OhmmsMatrix.h"
#include "OhmmsPETE/TinyVector.h"
#include "PlotBandStructure.h"
#include "wx/file.h"
#include "wx/filename.h"
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <grace_np.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <string>
using namespace std;
#include "Utilities/SimpleParser.h"

// WDR: class implementations

//------------------------------------------------------------------------------
// PlotBandStructure
//------------------------------------------------------------------------------

// WDR: event table for PlotBandStructure

BEGIN_EVENT_TABLE(PlotBandStructure,wxFrame)
    EVT_MENU(ID_ABOUT, PlotBandStructure::OnAbout)
    EVT_MENU(ID_QUIT, PlotBandStructure::OnQuit)
    EVT_CLOSE(PlotBandStructure::OnCloseWindow)
    EVT_BUTTON( ID_FILEBROWER, PlotBandStructure::FileBrower )
    EVT_BUTTON( ID_PLOTBAND, PlotBandStructure::PlotIt )
END_EVENT_TABLE()

PlotBandStructure::PlotBandStructure( wxWindow *parent, wxWindowID id, const wxString &title,
    const wxPoint &position, const wxSize& size, long style ) :
    wxFrame( parent, id, title, position, size, style )
{
   
    PlotBandStructureDialog(this,false);
    CreateMyMenuBar();
    
    CreateMyToolBar();
    
    CreateStatusBar(1);
    SetStatusText( wxT("Welcome!") );
    
     // insert main window here
}

void PlotBandStructure::CreateMyMenuBar()
{
#ifdef __WXMAC__
    wxApp::s_macAboutMenuItemId = ID_ABOUT;
#endif

    SetMenuBar( MyMenuBarFunc() );
}

void PlotBandStructure::CreateMyToolBar()
{
    wxToolBar *tb = CreateToolBar( wxTB_HORIZONTAL|wxNO_BORDER /* |wxTB_FLAT */ );
    
    MyToolBarFunc( tb );
}

// WDR: handler implementations for PlotBandStructure

void PlotBandStructure::PlotIt( wxCommandEvent &event )
{

    wxCheckBox* dos = (wxCheckBox*) FindWindow(ID_PLOT_DOS);
    if(dos->GetValue()) {
        PlotDOS();
    } else {
        PlotBand();
    }
 
}
  
void PlotBandStructure::PlotDOS() {

   wxTextCtrl* temp = (wxTextCtrl*) FindWindow(ID_SIGMA);
   double sigma = atof(temp->GetValue().c_str());

   int ndos = 1000;
   double de = (Emax-Emin)/static_cast<double>(ndos+2);
   double deinv = 1.0/de;
   double halfde = 0.5*de;

   vector<double> DOS(ndos+4,0.0);

   double xmin = Emin-de;
    for(int i=0; i<e.nrows(); i++) {
      double fac = halfde*w[i];
      for(int j=0; j<e.ncols(); j++) {
	int ikm = static_cast<int>((e(i,j)-halfde-xmin)*deinv);
	int ikp = static_cast<int>((e(i,j)+halfde-xmin)*deinv);
	DOS[ikm] += fac;
	DOS[ikp] += fac;
      }
    }

    if(!GraceIsOpen()) {
      // Start xmgr with a buffer size of 2048 and open the pipe 
      if (GraceOpen(2048) == -1) {
	fprintf (stderr, "Can't run xmgr. \n");
	return;
      }
    }
    double ymax = *(max_element(DOS.begin(),DOS.end()));
    temp = (wxTextCtrl*) FindWindow(ID_MINVAL);
    double emin = atof(temp->GetValue().c_str());
    temp = (wxTextCtrl*) FindWindow(ID_MAXVAL);
    double emax = atof(temp->GetValue().c_str());
    GracePrintf ("title \"Band Structure %s\"", FileName.c_str());
    GracePrintf ("subtitle \"Energies are shifted by %f eV \"",FermiLevel);
    GracePrintf ("g0.s0 line linestyle 1");
    GracePrintf ("g0.s0 line color 1");
    GracePrintf ("g0.s0 line linewidth 2.0"); 
    GracePrintf ("page size 500, 600");
    GracePrintf ("view xmin 0.1");
    GracePrintf ("view xmax 0.95");
    GracePrintf ("view ymin 0.1");
    GracePrintf ("view ymax 0.85");
    GracePrintf ("world ymin 0");
    GracePrintf ("world ymax %f", ymax);
    GracePrintf ("world xmin %f", emin);
    GracePrintf ("world xmax %f", emax);
    GracePrintf ("xaxis tick major %f", (emax-emin)/10.);
    double x = xmin;
    for(int i=0; i<DOS.size(); i++) {
      GracePrintf ("g0.s0 point %f, %f",x,DOS[i]);
      x += de;
    }
    GraceClosePipe();

}
void PlotBandStructure::PlotBand() {
    wxTextCtrl* temp = (wxTextCtrl*) FindWindow(ID_INPUTFILE);
    wxString fname = temp->GetValue();

  /* Start xmgr with a buffer size of 2048 and open the pipe */
  if(!GraceIsOpen()) {
    /* Start xmgr with a buffer size of 2048 and open the pipe */
    if (GraceOpen(2048) == -1) {
      fprintf (stderr, "Can't run xmgr. \n");
      return;
    }
  }

  temp = (wxTextCtrl*) FindWindow(ID_MINVAL);
  double emin = atof(temp->GetValue().c_str());
  temp = (wxTextCtrl*) FindWindow(ID_MAXVAL);
  double emax = atof(temp->GetValue().c_str());

  /* Send some initialization commands to xmgr */
  GracePrintf ("page size 500, 600");
  GracePrintf ("view xmin 0.1");
  GracePrintf ("view xmax 0.95");
  GracePrintf ("view ymin 0.1");
  GracePrintf ("view ymax 0.85");
  GracePrintf ("world xmin 0");
  GracePrintf ("world xmax 1");
  GracePrintf ("world ymin %f", emin);
  GracePrintf ("world ymax %f", emax);
  GracePrintf ("yaxis tick major %f", (emax-emin)/10.);
  double x = 0;
  double dx = 1.0/static_cast<double>(e.nrows());

  for(int i=0; i<e.nrows(); i++) {
    GracePrintf ("g0.s%d line linestyle 1",i);
    GracePrintf ("g0.s%d line color 1",i);
    GracePrintf ("g0.s%d line linewidth 2.0",i);
    for(int ib=0; ib<e.ncols(); ib++) {
      GracePrintf ("g0.s%d point %f, %f",ib,x,e(i,ib));
    }  
    x += dx;
  }

  
  GracePrintf ("title size 1.5");
  GracePrintf ("title \"Band Structure %s\"", FileName.c_str());
  GracePrintf ("subtitle \"Energies are shifted by %f eV \"",FermiLevel);
  
  GracePrintf ("xaxis ticklabel off");
  GracePrintf ("yaxis label char size 1.2");
  GracePrintf ("yaxis label \"Energy (eV)\"");
  GracePrintf ("yaxis ticklabel char size 1.2");
    
  GraceClosePipe();
}

void PlotBandStructure::FileBrower( wxCommandEvent &event )
{
  wxFileDialog dialog(this,
              _T("Open a file"),
              _T("."),
              _T(""),
              _T("Data files (*.pot)|*.pot")
              );
  
  if (dialog.ShowModal() == wxID_OK)
    {
      wxString apath = dialog.GetPath();
      FileName = dialog.GetFilename();

      wxTextCtrl* temp = (wxTextCtrl*) FindWindow(ID_INPUTFILE);
      temp->SetValue(FileName);
    }

  //wxTextCtrl* temp = (wxTextCtrl*) FindWindow(ID_INPUTFILE);
  //wxString fname = temp->GetValue();
  ifstream fin(FileName.c_str());
  if(fin) {

    vector<string> slist;
    bool findit = true;
    FermiLevel = 0.0;
    TinyVector<double,3> pos;
    do {
      int nw = getXwords(slist,fin);
      if(nw == 0) continue;
      if(slist[0] == "fermi") {
	FermiLevel = atof(slist[1].c_str());
      } else  if(slist[0] == "eigvalues") {
	findit = false;
      } else if(slist[0] == "KPOINT") {
	int nk = atoi(slist[2].c_str());
	w.resize(nk);
	for(int ik=0; ik<nk; ik++) {
	  fin >> pos >> w[ik];
	}

      }
    } while(findit);

    int nk = atoi(slist[2].c_str());
    int nb = atoi(slist[4].c_str());

    e.resize(nk,nb);
    for(int ik=0; ik<nk; ik++) 
      for(int ib=0; ib<nb; ib++)
    fin >> e(ik,ib);
    e -= FermiLevel;

    int nbmax = nb-1;
    double emin = e(0,0);
    double emax = e(0,nbmax);
    for(int ik=1; ik<nk; ik++) {
      emin = min(e(ik,0),emin);
      emax = max(e(ik,nbmax),emax);
    }
    Emin = emin;
    Emax = emax;
    wxTextCtrl* temp = (wxTextCtrl*) FindWindow(ID_MINVAL);
    wxString a,b;
    a << emin;
    temp->SetValue(a);
    temp = (wxTextCtrl*) FindWindow(ID_MAXVAL);
    b << emax;
    temp->SetValue(b);
  }

}

void PlotBandStructure::OnAbout( wxCommandEvent &event )
{
    wxMessageDialog dialog( this, wxT("Welcome to SuperApp 1.0\n(C)opyright Joe Hacker"),
        wxT("About SuperApp"), wxOK|wxICON_INFORMATION );
    dialog.ShowModal();
}

void PlotBandStructure::OnQuit( wxCommandEvent &event )
{
     Close( TRUE );
}

void PlotBandStructure::OnCloseWindow( wxCloseEvent &event )
{
    // if ! saved changes -> return
    
    Destroy();
}

//------------------------------------------------------------------------------
// PlotBandStructureApp
//------------------------------------------------------------------------------

IMPLEMENT_APP(PlotBandStructureApp)

PlotBandStructureApp::PlotBandStructureApp()
{
}

bool PlotBandStructureApp::OnInit()
{
    PlotBandStructure *frame = new PlotBandStructure( NULL, -1, wxT("SuperApp"), wxPoint(20,20), 
    wxSize(400,300) );
    frame->Show( TRUE );
    
    return TRUE;
}

int PlotBandStructureApp::OnExit()
{
    return 0;
}

