// -*- C++ -*-

////////////////////////////////////////////////////////////////////////////
// Do experiment by removing add for each block.
////////////////////////////////////////////////////////////////////////////

#include <vector>
#include <iostream>
#include <fstream>
#include <iomanip>
using namespace std;
#include "Utilities/SpeciesCollection.h"
#include "Utilities/SimpleParser.h"
#include "ScaleFunctors/NRLScaleFunction.h"

int main(int argc, char** argv) {

  typedef double T;
  typedef NRLSplineScale<T> ScaleFunction_t;
  vector<vector<ScaleFunction_t>* > V;
  vector<vector<ScaleFunction_t>* > Vs;

  // make private references to Species
  SpeciesBase* mySpecies = SpeciesCollection::getSpecies();
  int icharge = SpeciesCollection::addAttrib("q");
  int ins = SpeciesCollection::addAttrib("ns");
  int irmax = SpeciesCollection::addAttrib("rmax");
  int imass   = SpeciesCollection::addAttrib("mass");

  int Lmax = 0;
  int ib = -1, ia, ja, L1, L2,vloc;
  int l1, l2, lt1, lt2, m;
  T lambda, rcut, screen;
  T h_sigma, h_rmin, s_sigma, s_rmin;
  T eps, onb, onc, ond; //onsite term
  T he,hf,hg;           //off-site functional
  T se,sf,sg;           //off-site functional

  int inbond[10][10]; // limit up to tenbonds
  for(int i=0; i<10; i++)
    for(int j=0; j<10; j++) inbond[i][j] = -1;

  ifstream fin(argv[1]);
  int nw=0;
  if(fin) {
    vector<string> slist,vlist;
    bool tofindend = true;
    do {
      nw = getwords(slist,fin);
    } while(nw == 0 || slist[0] != "<SK>"); 
    while(tofindend) {
      if(getwords(slist, fin) == 0) continue;
      if(slist[0] == "species") {
	int newid =SpeciesCollection::addSpecies(slist[1].c_str());
        //option can be checked, but assumes strict format 10/08/2001
	(*mySpecies)(imass,newid)=atof(slist[3].c_str());
	(*mySpecies)(icharge,newid)=atof(slist[5].c_str());
        int ns = 0;
        for(int il=7; il<slist.size(); il++) {
          int l = atoi(slist[il].c_str());
          Lmax = max(l,Lmax);
          ns += (2*l+1);
        }
	(*mySpecies)(ins,newid)=ns;
      } else if(slist[0] == "bond") {
	ia = SpeciesCollection::addSpecies(slist[1].c_str());
	ja = SpeciesCollection::addSpecies(slist[2].c_str());
        // assign unphysical L1 and L2
        L1 = -1; L2 = -1;
	ib++;
      } else if(slist[0] == "rcut") {
        rcut = atof(slist[1].c_str());
      } else if(slist[0] == "screenl") {
        screen = atof(slist[1].c_str());
      } else if(slist[0] == "lambda") {
        lambda = atof(slist[1].c_str());
      } else if(slist[0] == "h_rmin") {
        h_rmin = atof(slist[1].c_str());
      } else if(slist[0] == "h_sigma") {
        h_sigma = atof(slist[1].c_str());
      } else if(slist[0] == "s_rmin") {
        s_rmin = atof(slist[1].c_str());
      } else if(slist[0] == "s_sigma") {
        s_sigma = atof(slist[1].c_str());
      } else if(isdigit(slist[0][0])) { 
        l1 = atoi(slist[0].c_str());
        l2 = atoi(slist[1].c_str());
        m  = atoi(slist[2].c_str());
        lt1 = atoi(slist[3].c_str());
        lt2 = atoi(slist[4].c_str());
        if(m < 0) {
          eps = atof(slist[5].c_str());
          onb = atof(slist[6].c_str());
          onc = atof(slist[7].c_str());
          ond = atof(slist[8].c_str());

	  NRLOnsite<T> Rho;
	  Rho.set(onb,onc,ond);
	  double xxx=1.0;
	  double dx = 0.01,y,dy,y1;
	  while(xxx<8) {
	    dy = Rho(xxx,y);
	    y1 = Rho(xxx+0.0001);
	    cout << setw(15) << xxx << setw(15) << y 
		 << setw(15) << dy << setw(15) << (y1-y)*10000 << endl;
	    xxx+= dx;
	  }
	  cout << endl;
	} else {
          eps = 0.0;
          getwords(vlist,fin);
          he = atof(vlist[0].c_str());
          hf = atof(vlist[1].c_str());
          hg = atof(vlist[2].c_str());
          getwords(vlist,fin);
          se = atof(vlist[0].c_str());
          sf = atof(vlist[1].c_str());
          sg = atof(vlist[2].c_str());
	}

        if(l1 != L1 || l2 != L2) {// new L1-L2 pair is added
          vloc = V.size();
          V.push_back(new vector<ScaleFunction_t>);
          Vs.push_back(new vector<ScaleFunction_t>);

          int M = (l1<=l2)? l1 : l2;
          // create M+1 scaling functions for L1-L2 pair
	  V[vloc]->reserve(M+1);
	  for(int mm=0; mm<M+1; mm++) V[vloc]->push_back(ScaleFunction_t());

          // create M+1 scaling functions for L1-L2 pair
	  Vs[vloc]->reserve(M+1);
	  for(int mm=0; mm<M+1; mm++) Vs[vloc]->push_back(ScaleFunction_t());
        }

        L1 = l1; L2 = l2;
        if(m>=0) {
          V[vloc]->operator[](m).set(he,hf,hg,rcut,screen,h_rmin,h_sigma);
          Vs[vloc]->operator[](m).set(se,sf,sg,rcut,screen,s_rmin,s_sigma);
//  	  cout << l1 << " " << l2 << " " << m << " " << lt1 << " " << lt2 
//  	       << " 700 " << endl;
//            double xxx=1.0;
//            double dx = 0.01;
//            while(xxx<8) {
//              cout << setw(15) << xxx 
//  		 << setw(15) << V[vloc]->operator[](m)(xxx) 
//  		 << setw(15) << Vs[vloc]->operator[](m)(xxx) << endl;
//  	    xxx+= dx;
//  	  }          
	}
      } else if(slist[0] == "end" || slist[0] == "</SK>")  {
        tofindend = false; break;
      }
    }
  } else {
    cout << argv[1] << " does not exist. Stop execution.\n";
    abort();
  }


//    SmoothCutoff<T> smooth;
//    smooth.set(rcut,screen);
//    double xxx=1.0;
//    double dx = 0.01,y,dy,y1;
//    while(xxx<8) {
//      dy = smooth(xxx,y);
//      y1 = smooth(xxx+0.0001);
//      cout << setw(15) << xxx << setw(15) << y 
//  	 << setw(15) << dy << setw(15) << (y1-y)*10000 << endl;
//      xxx+= dx;
//    }

//    NRLDensity<T> Rho;
//    Rho.set(rcut,screen,lambda);
//    double xxx=1.0;
//    double dx = 0.01,y,dy,y1;
//    while(xxx<8) {
//      dy = Rho(xxx,y);
//      y1 = Rho(xxx+0.0001);
//      cout << setw(15) << xxx << setw(15) << y 
//  	 << setw(15) << dy << setw(15) << (y1-y)*10000 << endl;
//      xxx+= dx;
//    }


  return 0;
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: nrltb.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

