//////////////////////////////////////////////////////////////////
// (c) Copyright 2003-  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/**@example HowToUseParameter.cpp
 *This example shows how to simplify management of a set of parameters.
 *
 *Composite classes such as PropagatorBase accept a number of parameters.
 *ParameterSet manages a map with the name attribute and  provides interfaces to
 * - add OhmmsParameter<T> 
 * - assign the value given in the text node.
 *
 *This example shows how to add different data types, double, int, TinyVector<double,3>,
 * and Tensor<double,3> to a parameter set.
 *Any simple datatype with operator >> can be added to the ParameterSet.
 */
#include "ohmms-config.h"
#include <iostream>
#include <iomanip>
#include "OhmmsPETE/Tensor.h"
#include "OhmmsPETE/TinyVector.h"
#include "OhmmsData/ParameterSet.h"

class HowToUseParameter {
public:

  /** add the data memebrs to the parameter list */
  HowToUseParameter() { 
    m_param.add(scale,"scale","AA");
    m_param.add(size,"size","none");
    m_param.add(center,"center","posvec");
    m_param.add(lat,"lattice","tensor");
  }

  /** process the current node */
  bool put(xmlNodePtr cur) {
    xmlNodePtr parent = cur;
    std::cout << "This node is " << (const char*)cur->name << std::endl;
    m_param.put(cur);
    return true;
  }

  /** print out the content */
  void print(std::ostream& os) {
    os << "  scale " << scale << std::endl;
    os << "   size " << size << std::endl;
    os << " center " << center << std::endl;
    os << "lattice " << lat << std::endl;
  }

private:
  double scale;
  int size;
  TinyVector<double,3> center;
  Tensor<double,3> lat;
  ParameterSet m_param;
};


int main(int argc, char **argv) {

  xmlDocPtr m_doc = xmlParseFile("parameter.xml");
  if (m_doc == NULL) return false;
    
  // Check the document is of the right kind
  xmlNodePtr cur = xmlDocGetRootElement(m_doc);
  if (cur == NULL) {
    fprintf(stderr,"empty document\n");
    xmlFreeDoc(m_doc);
    return false;
  }
  
  HowToUseParameter showme;
  showme.put(cur);
  showme.print(std::cout);

  xmlFreeDoc(m_doc);
  return 0;
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 170 $   $Date: 2004-11-08 12:49:29 -0600 (Mon, 08 Nov 2004) $
 * $Id: HowToUseParameter.cpp 170 2004-11-08 18:49:29Z jnkim $ 
 ***************************************************************************/
