//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_DYNAMICSANALYSYSTOOLS_H
#define OHMMS_DYNAMICSANALYSYSTOOLS_H

namespace OHMMS {

  class PropagatorBase;

  enum { report_energies = 0, report_gofr};

  struct DynamicsAnalysis {

    int Stride;
    bool UsePlot;
    DynamicsAnalysis(int stride, bool useplot): 
      Stride(stride), UsePlot(useplot) { }

    virtual void put(std::istream& ) = 0;
    virtual bool Update( PropagatorBase* WorkingSystem, int iter) = 0;
  };

  void AddDynamicsAnalysis(std::map<int,DynamicsAnalysis*>& m, int ana,  
			   int stride, bool useplot);


  struct EnergiesVsTime: public DynamicsAnalysis {
    double oneovern;

    EnergiesVsTime(int stride, bool useplot): 
      DynamicsAnalysis(stride,useplot) { }

    void put(std::istream& );
    bool Update(PropagatorBase* WorkingSystem, int iter);
  };

  struct GofR: public DynamicsAnalysis {

    vector<double> gofrsum;

    GofR(int stride, bool useplot):  DynamicsAnalysis(stride,useplot) { }

    void put(std::istream& );
    bool Update( PropagatorBase* WorkingSystem, int iter);

  };

}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: DynamicsAnalysis.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
