//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "AppBase/ApplyConstraints.h"
#include "Utilities/OhmmsInfo.h"
/**@ingroup ohmmsxml
 * @ XML node with tag = ParticleLayout

   usage: 
   <freeze  
    target="name-of-particleset" 
    groups="group indices"
   />

   Action: Fix the positions of the particles that belong to the groups of the particle set.
 */
namespace OHMMS {

  FreezeParticles::FreezeParticles(ParticleSets& P,
				   const char* aname): 
    OhmmsElementBase(aname), PtclSetRef(P)  
  { }
  
  void FreezeParticles::reset() { }  
  
  bool FreezeParticles::put(xmlNodePtr cur) {

    ParticleSets::Particle_t* ptcl =  PtclSetRef.get();
    typedef ParticleSets::Particle_t::ParticleIndex_t ParticleIndex_t;

    if(ptcl) {
      const xmlChar* grs = xmlGetProp(cur,(const xmlChar*)"groups");
      if(grs && ptcl->hasAttrib("groupid")) {
	int fixg = atoi((const char*)grs);//only take one now
	LOGMSG("Freezing the particles in the group " << fixg)
	ParticleIndex_t& gid = *(ptcl->getIndexAttrib("groupid"));
	ParticleIndex_t& fixed = *(ptcl->getIndexAttrib("freeze"));
	for(int i=0; i<ptcl->getLocalNum(); i++) if(gid[i] == fixg) fixed[i] = 1;
      }
    }
    return true;
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 211 $   $Date: 2004-12-22 10:47:03 -0600 (Wed, 22 Dec 2004) $
 * $Id: ApplyConstraints.cpp 211 2004-12-22 16:47:03Z jnkim $ 
 ***************************************************************************/
