//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_GEOMETRYBUILDER_H
#define OHMMS_GEOMETRYBUILDER_H

#include "AppBase/TransformParticle.h"
#include "OhmmsData/ParameterSet.h"
namespace OHMMS {

  struct GeometryBuilder: public TransformParticleSet {

    GeometryBuilder(Particle_t& in, Particle_t& out ):TransformParticleSet(in,out){ }

    ~GeometryBuilder() { }
    bool put(xmlNodePtr cur);

  };

  /** Base class for a geometrical object
   *
   */
  struct GeometryComponent {

    typedef ParticleSet::SingleParticlePos_t SingleParticlePos_t;
    typedef ParticleSet::size_type size_type;
    bool exclude;
    SpeciesSet& SpeciesRef;
    ParameterSet m_param;
    std::vector<int> Swap;
    std::vector<double> SwapProb;
    GeometryComponent(SpeciesSet& species):
      exclude(false),SpeciesRef(species){}
    virtual ~GeometryComponent(){}

    /** swap species if the pos is in the region
     *@param sid the species index
     *@param pos the position in the lattice unit of the base ParticleSet
     *@return true, if the pos is in the region
     *
     *Upon exit, sid is the new species index. When the region is to be removed,
     *sid=-1.
     */
    virtual bool exchange(int& sid, const SingleParticlePos_t& pos)=0;
    bool canExchange(int i);
    bool put(xmlNodePtr cur);
  };

  /** Dummy geometry
   */
  struct DummyComponent: public GeometryComponent {
    DummyComponent(SpeciesSet& species, xmlNodePtr);
    bool exchange(int& sid, const SingleParticlePos_t& pos);
  };

  struct Slab: public GeometryComponent {
    int axis;
    double z_min;
    double z_max;
    Slab(SpeciesSet& species,xmlNodePtr cur);
    bool exchange(int& sid, const SingleParticlePos_t& pos);
  };

  struct Dome: public GeometryComponent {
    int axis;
    double z_min;
    double z_max;
    double height;
    double radius;
    double curvature, rcsq;
    SingleParticlePos_t center;
    Dome(SpeciesSet& species,xmlNodePtr cur);
    bool exchange(int& sid, const SingleParticlePos_t& pos);
  };

  struct Tube: public GeometryComponent {
    int axis, ind_x, ind_y;
    double r_min;
    double r_max;
    SingleParticlePos_t center;
    Tube(SpeciesSet& species,xmlNodePtr cur);
    bool exchange(int& sid, const SingleParticlePos_t& pos);
  };

  struct RegularPyramid: public GeometryComponent {
    int axis;
    double z_min;
    double dz_max;
    double height;
    double ratio;
    double base;
    double theta;
    SingleParticlePos_t center;
    RegularPyramid(SpeciesSet& species,xmlNodePtr cur);
    bool exchange(int& sid, const SingleParticlePos_t& pos);
  };

  struct Sphere:public GeometryComponent {
    double out_radius2;
    double in_radius2;
    SingleParticlePos_t center;
    Sphere(SpeciesSet& species,xmlNodePtr cur);
    bool exchange(int& sid, const SingleParticlePos_t& pos);
  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: GeometryBuilder.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
