//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_PARTICLESETS_H__
#define OHMMS_PARTICLESETS_H__

#include "OhmmsData/OhmmsElementBase.h"
#include "OhmmsData/RecordProperty.h"
#include "Particle/ParticleSet.h"

namespace OHMMS {

  /** \class ParticleSets
   *  \brief Similar to OhmmsXMLUI/MDParticle
   */
  class ParticleSets: public OhmmsElementBase {

  public:

    typedef ParticleSet                  Particle_t;
    typedef Particle_t::ParticleLayout_t ParticleLayout_t;

    /// constructors and destructors
    ParticleSets(const char* aname="particlesets");
    ~ParticleSets();

    bool get(ostream& os) const { return true;}
    bool put(istream& is) {return true;}
    bool put(xmlNodePtr cur);
    void reset();

    ///add a new particle set
    void addPtclSet(Particle_t* apt);

    ///returns the ActivePtcl
    Particle_t* get();

    ///returns a particleset with the name
    Particle_t* get(const string& aname);

    /** set the active particle to the input particle set
     *@param anew an external Particle_t* is assigned to ActivePtcl;
     */
    void setActivePtclSet(Particle_t* anew);

    /** create a record engine or engines for particle(s)
     *@param cur the current xml node
     *@param records record engines created
     *
     *For multiple particlesets, record can target "all" particle sets.
     */
    bool createRecordProperty(xmlNodePtr cur,
			      vector<RecordProperty*>& records);


    /** remove apt from the ParticleSet list that can be cleared
     */
    bool remove(Particle_t* apt);

    /** generate a particle name
     *@return a unique particle name
     */
    string setPtclSetName();
      
    /** clear all the internal particles except for the ActivePtcl
     */
    void clear();    

    /** clear all the internal particles including
     */
    void clearAll();    

  private:

    typedef list<Particle_t*>        PtclSetList_t;    
    typedef map<string,Particle_t*>  PtclSetMap_t;    

    int Counter;
    string RestartConfig;

    ///pointer to an active particle set
    Particle_t* ActivePtcl;

    ///container for particle sets that are created by ParticleSets and added by external objects
    PtclSetMap_t PtclCollection;

    ///container for particle sets that are created by ParticleSets
    PtclSetList_t NewPtclList;

    ///a global unitcell
    ParticleLayout_t Lattice;

    xmlNodePtr m_cur;

    bool transformParticle(xmlNodePtr cur);
  };
}

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 158 $   $Date: 2004-11-03 16:11:26 -0600 (Wed, 03 Nov 2004) $
 * $Id: ParticleSets.h 158 2004-11-03 22:11:26Z jnkim $ 
 ***************************************************************************/
