//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_TRANSFORMPPARTICLE_H
#define OHMMS_TRANSFORMPPARTICLE_H

#include "OhmmsData/OhmmsElementBase.h"
#include "Particle/ParticleSet.h"

namespace OHMMS {

  /*!\class abstract class to perform transformation on the particle sets
   */
  class TransformParticleSet {

  protected:

    typedef ParticleSet Particle_t;
    typedef Particle_t::size_type size_type;
    Particle_t& in_;
    Particle_t& out_;

  public:

    TransformParticleSet(Particle_t& pin, Particle_t& pout): in_(pin), out_(pout) { }

    ///read from DOM
    virtual bool put(xmlNodePtr cur)=0;
  };


  /** expand the particle set by duplicating a supercell
   */
  class ExpandParticle: public TransformParticleSet {

  public:

    ExpandParticle(Particle_t& pin, Particle_t& pout): TransformParticleSet(pin,pout) 
    {}

    bool put(xmlNodePtr cur);
  };

  /** move the particle position by a displacement 
   */
  class TranslateParticle: public TransformParticleSet {

  public:

    TranslateParticle(Particle_t& pin, Particle_t& pout): TransformParticleSet(pin,pout) 
    {}

    bool put(xmlNodePtr cur);

  };

  /** scale a supercell: the particle positions relative to the supercell unchanged
   */
  class ScaleParticleLayout: public TransformParticleSet {

  public:

    ScaleParticleLayout(Particle_t& pin, Particle_t& pout): 
      TransformParticleSet(pin,pout) 
    {}

    bool put(xmlNodePtr cur);

  };
}

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: TransformParticle.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
