//////////////////////////////////////////////////////////////////
//                        COPYRIGHT NOTICE
// (c) Copyright 1998-2002 by Jeongnim Kim
//   All rights reserved.
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Utilities/OhmmsInfo.h"
#include "Utilities/SimpleParser.h"
#include "CrystalSymmetry/KPointBase.h"
using namespace OHMMS;

void
createKPointMesh(KPointBase* kengine, istream& kfile) 
{

  typedef KPointBase::Particle_t          Particle_t;
  typedef KPointBase::Scalar_t            Scalar_t;
  typedef KPointBase::SingleParticlePos_t SingleParticlePos_t;

  if(kengine) {
    vector<string> slist;
    bool findkpt = true;
    do {
      int nw = getwords(slist,kfile);
      if(nw == 0) continue;
      if(slist[0] == "<KPOINT>") {
	findkpt = false;
      }
    } while(findkpt);
    getwords(slist,kfile);
    
    int nkpt =atoi(slist[0].c_str());
    
    bool getsymops = true;
    if(nkpt > 0) getsymops = false; // externally provided
    else {
      if(slist.size() > 1) { // MC grid but check if nosym option is given
	if(slist[1] == "nosym") 
	  getsymops = false;
	else if(slist[1] == "reduced") 
	  kengine->Reduced = true;
      }   
    }
    
    vector<SingleParticlePos_t> kptin;
    vector<Scalar_t>  wgtin;
    
    if(getsymops)  
      kengine->findSpaceGroup(*kengine->ref_);
    else {
      Particle_t ptcl_new;
      ptcl_new.Lattice = kengine->ref_->Lattice;
      ptcl_new.create(1);
      ptcl_new.R[0] = 0.0;
      ptcl_new.GroupID[0] = 0;
      kengine->findSpaceGroup(ptcl_new);
    }
    
    //if(nkpt <= 0) 
    //  kengine->findSpaceGroup(ptclin,cout);
    
    if(nkpt > 0) { // reading in nkpt points, do not get the space group 
      
      //\warning When kpoints and weights are given, assum that no need to extend k-points
      kengine->Reduced = false;
      
      kptin.resize(nkpt);
      wgtin.resize(nkpt);
      for(int i=0; i<nkpt; i++) {
	getwords(slist,kfile);
	kptin[i][0] = atof(slist[0].c_str());
	kptin[i][1] = atof(slist[1].c_str());
	kptin[i][2] = atof(slist[2].c_str());
	wgtin[i] = atof(slist[3].c_str());
      }
      kengine->buildKpoint(kptin,wgtin);
      
    } else if(nkpt == 0){ // creating MP special k-points
      
      int nk[3];
      SingleParticlePos_t origin;
      getwords(slist,kfile);
      nk[0] = atoi(slist[0].c_str());
      nk[1] = atoi(slist[1].c_str());
      nk[2] = atoi(slist[2].c_str());
      origin[0] = atoi(slist[3].c_str());
      origin[1] = atoi(slist[4].c_str());
      origin[2] = atoi(slist[5].c_str());
      kengine->buildKpoint(nk,origin);
      
    } else if(nkpt < 0) { // reading for band structure
      
      int nbranch = atoi(slist[1].c_str());
      
      vector<int> npts(nbranch);
      kptin.resize(nbranch);
      
      int nktot = 0;
      for(int i=0; i<nbranch; i++) {
	getwords(slist,kfile);
	kptin[i][0] = atof(slist[0].c_str());
	kptin[i][1] = atof(slist[1].c_str());
	kptin[i][2] = atof(slist[2].c_str());
	if(i) { // not for first k-point 
	  npts[i] = atoi(slist[3].c_str());
	  nktot += npts[i];
	}
      }
      
      kengine->K.reserve(nktot+1);//for efficiency
      kengine->W.reserve(nktot+1);//for efficiency
      
      Scalar_t wgts = 1.0/static_cast<double>(nktot+1); // not meaningful but...
      kengine->addKpt(kptin[0],wgts);// adding the first kpoint
      
      for(int i=1; i<nbranch; i++) {
	SingleParticlePos_t ki = kptin[i-1]; // starting adn end point
	SingleParticlePos_t kf = kptin[i];
	double dn = 1.0/static_cast<double>(npts[i]);
	SingleParticlePos_t dk =dn*(kf-ki);
	for(int ik=0; ik<npts[i]; ik++) {
	  ki += dk; 
	  kengine->addKpt(ki,wgts);// adding this kpoint
	}   
      }
      kengine->Reduced = false; // this kpoint set is not meant for total energy
      kengine->partition(1);
    }
  }
  //OhmmsInform kout(" ", "KPOINT.dat");//,false);
  //if(kout.open()) kengine->print(kout.getStream());
}

/*
KPointBase* 
createKPointMesh(Particle_t& ptclin, int *nk, SingleParticlePos_t& origin, 
		 bool getsymomp = true) {

  typedef KPointBase::SingleParticlePos_t SingleParticlePos_t;

  KPointBase *kengine= new KPointBase;

  if(getsymops)  
    kengine->findSpaceGroup(ptclin);
  else {
    Particle_t ptcl_new;
    ptcl_new.Lattice = ptclin.Lattice;
    ptcl_new.create(1);
    ptcl_new.R[0] = 0.0;
    ptcl_new.GroupID[0] = 0;
    kengine->findSpaceGroup(ptcl_new);
  }

  // creating MP special k-points
  kengine->buildKpoint(nk,origin);

  OhmmsInform kout(" ", "KPOINT.dat",false);
  if(kout.open()) kengine->print(kout.getStream());
  return kengine;
}
*/
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: KPointBuilder.cpp 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
