//////////////////////////////////////////////////////////////////
//                        COPYRIGHT NOTICE
// (c) Copyright 1998-2002 by Dallas Trinkle and Jeongnim Kim
//   All rights reserved.
//////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "CrystalSymmetry/FCCKPoint.h"

FCCKPoint::FCCKPoint(Lattice_t& lat):KPointBase(lat){

  fcc0 = lat.G(0,0);
  fcc1 = 1.5*fcc0;

  O.resize(47);
  // adding group operations excluding identity 
  O[ 0]=Tensor_t( -1.0,  -1.0,  -1.0,  0.0,   0.0,   1.0,  0.0,   1.0,   0.0);
  O[ 1]=Tensor_t(  0.0,   0.0,   1.0, -1.0,  -1.0,  -1.0,  1.0,   0.0,   0.0);
  O[ 2]=Tensor_t(  0.0,   1.0,   0.0,  1.0,   0.0,   0.0, -1.0,  -1.0,  -1.0);
  O[ 3]=Tensor_t(  0.0,   1.0,   0.0,  0.0,   0.0,   1.0,  1.0,   0.0,   0.0);
  O[ 4]=Tensor_t( -1.0,  -1.0,  -1.0,  1.0,   0.0,   0.0,  0.0,   0.0,   1.0);
  O[ 5]=Tensor_t(  1.0,   0.0,   0.0, -1.0,  -1.0,  -1.0,  0.0,   1.0,   0.0);
  O[ 6]=Tensor_t(  0.0,   0.0,   1.0,  0.0,   1.0,   0.0, -1.0,  -1.0,  -1.0);
  O[ 7]=Tensor_t(  0.0,   0.0,   1.0,  1.0,   0.0,   0.0,  0.0,   1.0,   0.0);
  O[ 8]=Tensor_t( -1.0,  -1.0,  -1.0,  0.0,   1.0,   0.0,  1.0,   0.0,   0.0);
  O[ 9]=Tensor_t(  0.0,   1.0,   0.0, -1.0,  -1.0,  -1.0,  0.0,   0.0,   1.0);
  O[10]=Tensor_t(  1.0,   0.0,   0.0,  0.0,   0.0,   1.0, -1.0,  -1.0,  -1.0);
  O[11]=Tensor_t(  0.0,  -1.0,   0.0, -1.0,   0.0,   0.0,  0.0,   0.0,  -1.0);
  O[12]=Tensor_t(  1.0,   1.0,   1.0,  0.0,   0.0,  -1.0, -1.0,   0.0,   0.0);
  O[13]=Tensor_t(  0.0,   0.0,  -1.0,  1.0,   1.0,   1.0,  0.0,  -1.0,   0.0);
  O[14]=Tensor_t( -1.0,   0.0,   0.0,  0.0,  -1.0,   0.0,  1.0,   1.0,   1.0);
  O[15]=Tensor_t( -1.0,   0.0,   0.0,  0.0,   0.0,  -1.0,  0.0,  -1.0,   0.0);
  O[16]=Tensor_t(  1.0,   1.0,   1.0,  0.0,  -1.0,   0.0,  0.0,   0.0,  -1.0);
  O[17]=Tensor_t(  0.0,  -1.0,   0.0,  1.0,   1.0,   1.0, -1.0,   0.0,   0.0);
  O[18]=Tensor_t(  0.0,   0.0,  -1.0, -1.0,   0.0,   0.0,  1.0,   1.0,   1.0);
  O[19]=Tensor_t(  0.0,   0.0,  -1.0,  0.0,  -1.0,   0.0, -1.0,   0.0,   0.0);
  O[20]=Tensor_t(  1.0,   1.0,   1.0, -1.0,   0.0,   0.0,  0.0,  -1.0,   0.0);
  O[21]=Tensor_t( -1.0,   0.0,   0.0,  1.0,   1.0,   1.0,  0.0,   0.0,  -1.0);
  O[22]=Tensor_t(  0.0,  -1.0,   0.0,  0.0,   0.0,  -1.0,  1.0,   1.0,   1.0);
  O[23]=Tensor_t( -1.0,   0.0,   0.0,  0.0,  -1.0,   0.0,  0.0,   0.0,  -1.0);
  O[24]=Tensor_t(  1.0,   1.0,   1.0,  0.0,   0.0,  -1.0,  0.0,  -1.0,   0.0);
  O[25]=Tensor_t(  0.0,   0.0,  -1.0,  1.0,   1.0,   1.0, -1.0,   0.0,   0.0);
  O[26]=Tensor_t(  0.0,  -1.0,   0.0, -1.0,   0.0,   0.0,  1.0,   1.0,   1.0);
  O[27]=Tensor_t(  0.0,  -1.0,   0.0,  0.0,   0.0,  -1.0, -1.0,   0.0,   0.0);
  O[28]=Tensor_t(  1.0,   1.0,   1.0, -1.0,   0.0,   0.0,  0.0,   0.0,  -1.0);
  O[29]=Tensor_t( -1.0,   0.0,   0.0,  1.0,   1.0,   1.0,  0.0,  -1.0,   0.0);
  O[30]=Tensor_t(  0.0,   0.0,  -1.0,  0.0,  -1.0,   0.0,  1.0,   1.0,   1.0);
  O[31]=Tensor_t(  0.0,   0.0,  -1.0, -1.0,   0.0,   0.0,  0.0,  -1.0,   0.0);
  O[32]=Tensor_t(  1.0,   1.0,   1.0,  0.0,  -1.0,   0.0, -1.0,   0.0,   0.0);
  O[33]=Tensor_t(  0.0,  -1.0,   0.0,  1.0,   1.0,   1.0,  0.0,   0.0,  -1.0);
  O[34]=Tensor_t( -1.0,   0.0,   0.0,  0.0,   0.0,  -1.0,  1.0,   1.0,   1.0);
  O[35]=Tensor_t(  0.0,   1.0,   0.0,  1.0,   0.0,   0.0,  0.0,   0.0,   1.0);
  O[36]=Tensor_t( -1.0,  -1.0,  -1.0,  0.0,   0.0,   1.0,  1.0,   0.0,   0.0);
  O[37]=Tensor_t(  0.0,   0.0,   1.0, -1.0,  -1.0,  -1.0,  0.0,   1.0,   0.0);
  O[38]=Tensor_t(  1.0,   0.0,   0.0,  0.0,   1.0,   0.0, -1.0,  -1.0,  -1.0);
  O[39]=Tensor_t(  1.0,   0.0,   0.0,  0.0,   0.0,   1.0,  0.0,   1.0,   0.0);
  O[40]=Tensor_t( -1.0,  -1.0,  -1.0,  0.0,   1.0,   0.0,  0.0,   0.0,   1.0);
  O[41]=Tensor_t(  0.0,   1.0,   0.0, -1.0,  -1.0,  -1.0,  1.0,   0.0,   0.0);
  O[42]=Tensor_t(  0.0,   0.0,   1.0,  1.0,   0.0,   0.0, -1.0,  -1.0,  -1.0);
  O[43]=Tensor_t(  0.0,   0.0,   1.0,  0.0,   1.0,   0.0,  1.0,   0.0,   0.0);
  O[44]=Tensor_t( -1.0,  -1.0,  -1.0,  1.0,   0.0,   0.0,  0.0,   1.0,   0.0);
  O[45]=Tensor_t(  1.0,   0.0,   0.0, -1.0,  -1.0,  -1.0,  0.0,   0.0,   1.0);
  O[46]=Tensor_t(  0.0,   1.0,   0.0,  0.0,   0.0,   1.0, -1.0,  -1.0,  -1.0);

}

FCCKPoint::~FCCKPoint(){

}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: FCCKPoint.cpp 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
