      subroutine group1 (iut,a1,a2,a3,nat,ty,ndim9,x,b1,b2,b3,ihg,ihc,
     &isy,li,nc,ib,v,f0,r)
c
c               written on september 10th - from the acmi complex
c (worlton and warren, comput.phys.commun. 8,71-84 (1974))
c (and 3,88-117 (1972))
c basic crystallographic information about a given crystal structure
c subroutines needed: pgl1,atftm1,rot1,rlv3
c input data:
c      iut ... number of the output unit for on-line printing
c               of various messages
c      a1,a2,a3 .. elementary translations of the lattice, in some
c               unit of length
c      nat .... number of atoms in the unit cell
c               all the dimensions are set for nat .le. 20
c      ty ..... integers distinguishing between the atoms of
c               different type. ty(i) is the type of the i-th atom
c               of the basis
c      x ...... cartesian coordinates of the nat atoms of the basis
c output data
c      b1,b2,b3 .. reciprocal lattice vectors, not multiplied by
c               any 2pi, in units reciprocal to those of a1,a2,a3
c      ihg .... point group of the primitive lattice, holohedral
c               group number:
c               ihg=1 stands for triclinic system
c               ihg=2 stands for monoclinic system
c               ihg=3 stands for orthorhombic system
c               ihg=4 stands for tetragonal system
c               ihg=5 stands for cubic system
c               ihg=6 stands for trigonal system
c               ihg=7 stands for hexagonal system
c               (holohedry is the point group of the lattice)
c      ihc .... code distinguishing between hexagonal and cubic
c               groups
c               ihc=0 stands for hexagonal groups
c               ihc=1 stands for cubic groups
c      isy .... code indicating whether the space group is
c               symmorphic or nonsymmorphic
c               isy=0 means nonsymmorphic group
c               isy=1 means symmorphic group
c               the group is considered symmorphic if for each
c               operation of the point group the sum of the 3
c               components of abs(v(n)) ( nonprimitive translation,
c               see below) is lt. 0.0005
c      li ..... code indicating whether the point group
c               of the crystal contains inversion or not
c               (operations 13 or 25 in respectively hexagonal
c               or cubic groups).
c               li=0 means: does not contain inversion
c               li.gt.0 means: there is inversion in the point
c                              group of the crystal
c      nc ..... total number of elements in the point group of the
c               crystal
c      ib ..... list of the rotations constituting the point group
c               of the crystal. the numbering is that defined in
c               worlton and warren, i.e. the one materialized in the
c               array r (see below)
c               only the first nc elements of the aray ib are
c               meaningful
c      v ...... nonprimitive translations (in the case of nonsymmor-
c               phic groups). v(i,n) is the i-th component
c               of the translation connected with the n-th element
c               of the point group (i.e. with the rotation
c               number ib(n) ).
c               attention: v(i) are not cartesian components,
c               they refer to the system a1,a2,a3.
c      f0 ..... the function defined in maradudin, ipatova by
c               eq. (3.2.12): atom transformation table.
c               the element f0(n,kapa) means that the n-th
c               operation of the space group (i.e. operation number
c               ib(n), together with an eventual nonprimitive
c               translation  v(n)) transfers the atom kapa into the
c               atom f0(n,kapa).
c      r ...... list of the 3 x 3 rotation matrices
c               (xyz representation of the o(h) or d(6)h groups)
c               all 48 or 24 matrices are listed.
c printed output
c      program prints the type of the lattice (ihg, in words),
c      lists the operations of the  point group of the
c      crystal, indicates whether the space group is symmorphic or
c      nonsymmorphic and whether the point group of the crystal
c      contains inversion.
c
      implicit none
      integer iut,ndim9,nat,ihc,ihg,isy,li,nc
      integer ty(ndim9),f0(48,ndim9),ib(48)
      double precision a1(3),a2(3),a3(3),x(3,ndim9),b1(3),b2(3),b3(3),
     &  v(3,48),r(49,3,3)
      double precision a(3,3), ai(3,3)
      integer i
c-----------------------------------------------------------------------
      if (nat .gt. ndim9) then
        write (iut,120) nat,ndim9
120     format ('nat =',i5,' is greater than ndim9 =',i5)
        return
        endif
c
      do 140 i = 1,3
        a(i,1) = a1(i)
        a(i,2) = a2(i)
        a(i,3) = a3(i)
        ai(1,i) = b1(i)
        ai(2,i) = b2(i)
        ai(3,i) = b3(i)
140     continue
c     a(i,j) is the i-th cartesian component of the j-th primitive
c     translation vector of the direct lattice
c     ty(i) is an integer distinguishing atoms of different type,i.e.,
c     different atomic species
c     x(j,i) is the j-th cartesian component of the position vector for
c     the i-th atom in the unit cell.
c
c     determine primitive lattice vectors for the reciprocal lattice.
c
      call pgl1 (a,ai,ihc,nc,ib,ihg,r)
c
      call atftm1 (iut,r,v,x,f0,ib,ty,ndim9,ihg,nat,nc,ai,li,isy)
      if (li .gt. 0) write (iut,*)
     &'the point group of the crystal contains the inversion'
      write (iut,*) 'subroutine group1 - end'
      return
      end
