      subroutine rot1 (ihc,r)
c                                     written on february 17th, 1976
c generate the CARTESIAN 3x3 x,y,z-transformation matrices 3x3 
c for hexagonal or cubic groups
c (cartesian same as fractional for cubic group, NOT for hexagonal)
c subroutines needed -- none
c this is identical with the subroutine rot of worlton-warren (in the ac
c -complex), only the way of transferring the data was changed
c input data
c      ihc...switch determining if we desire the hexagonal group (ihc=0)
c            or the cubic group (ihc=1)
c output data
c      r...the 3x3 matrices of the desired coordinate representation
c          their numbering corresponds to the symmetry elements as liste
c          in worlton-warren
c          for ihc=0 the first 24 matrices of the array r represent
c                    the full hexagonal group P6/mmm [191, D(6h)]
c          for ihc=1 the first 48 matrices of the array r represent
c                    the full cubic group Pm(3bar)m  [221, O(h)]
c
      implicit none
      integer ihc
      double precision r(49,3,3)
      integer i,j,n,k,nv
      double precision zero,one,half,f
c-----------------------------------------------------------------------
      zero=0.0d0
      one=1.0d0
      half=0.5d0

      do 100 j=1,3
      do 100 i=1,3
      do 100 n=1,49
        r(n,i,j)=zero
100   continue
      if (ihc .gt. 0) goto 160
c
c     define the generators for the rotation matrices--hexagonal group
c
c     f is sin(2*Pi/6); cos(2*Pi/6)=1/2
      f = 0.5d0*sqrt(3.0d0)
c     array 2 is rotation of 2*Pi/6 about z axis, cartesian coordinates
      r(2,1,1) = half
      r(2,1,2) = - f
      r(2,2,1) = f
      r(2,2,2) = half
c     array 7 is rotation of 2*Pi/2 about the [-1/2,sqrt(3)/2,0] axis
c     i.e. axis perpendicular to z axis, cartesian coordinates
c     (Note that Boisen+Gibbs, Mathematical Crystallography, 1990, p. 74,
c      suggest a set of cartesian coordinates which, for a hexagonal 
c      system, produces a*[100], a*[-1/2,sqrt(3)/2,0], c*[001]).
      r(7,1,1) = - half
      r(7,1,2) = - f
      r(7,2,1) = - f
      r(7,2,2) = half
      do 120 n      = 1,6
        r(n,3,3)    = one
        r(n+18,3,3) = one
        r(n+6,3,3)  = - one
        r(n+12,3,3) = - one
120     continue
c
c     generate the rest of the rotation matrices
c
      do 130 i = 1,2
        r(1,i,i) = one
        do 130 j = 1,2
          r(6,i,j) = r(2,j,i)
          do 130 k = 1,2
c           3 is 2 * 2 which is 2*Pi/3 rotation about z axis
            r(3,i,j)  = r(3,i,j) +  r(2,i,k)*r(2,k,j)
            r(8,i,j)  = r(8,i,j) +  r(2,i,k)*r(7,k,j)
            r(12,i,j) = r(12,i,j) + r(7,i,k)*r(2,k,j)
130         continue
      do 140 i = 1,2
        do 140 j = 1,2
          r(5,i,j) = r(3,j,i)
          do 140 k = 1,2
c           4 is 2 * 3 which is 2*Pi/2 rotation about z axis
            r(4,i,j)  = r(4,i,j)  + r(2,i,k)*r(3,k,j)
            r(9,i,j)  = r(9,i,j)  + r(2,i,k)*r(8,k,j)
            r(10,i,j) = r(10,i,j) + r(12,i,k)*r(3,k,j)
            r(11,i,j) = r(11,i,j) + r(12,i,k)*r(2,k,j)
140         continue
      do 150 n = 1,12
        nv = n + 12
        do 150 i = 1,2
          do 150 j = 1,2
c           the second 12 operations involve inversion
            r(nv,i,j) = - r(n,i,j)
150         continue
c
      return
c
c     define the generators for the rotation matrices--cubic group
c
160   r(9,1,3)  = one
      r(9,2,1)  = one
      r(9,3,2)  = one
      r(19,1,1) = one
      r(19,2,3) = - one
      r(19,3,2) = one
      do 170 i = 1,3
        r(1,i,i) = one
        do 170 j = 1,3
          r(20,i,j) = r(19,j,i)
          r(5,i,j)  = r(9,j,i)
          do 170 k  = 1,3
            r(2,i,j)  = r(2,i,j)  + r(19,i,k)*r(19,k,j)
            r(16,i,j) = r(16,i,j) + r(9,i,k)*r(19,k,j)
            r(23,i,j) = r(23,i,j) + r(19,i,k)*r(9,k,j)
170         continue
      do 180 i = 1,3
      do 180 j = 1,3
      do 180 k = 1,3
        r(6,i,j)  = r(6,i,j)  + r(2,i,k)*r(5,k,j)
        r(7,i,j)  = r(7,i,j)  + r(16,i,k)*r(23,k,j)
        r(8,i,j)  = r(8,i,j)  + r(5,i,k)*r(2,k,j)
        r(10,i,j) = r(10,i,j) + r(2,i,k)*r(9,k,j)
        r(11,i,j) = r(11,i,j) + r(9,i,k)*r(2,k,j)
        r(12,i,j) = r(12,i,j) + r(23,i,k)*r(16,k,j)
        r(14,i,j) = r(14,i,j) + r(16,i,k)*r(2,k,j)
        r(15,i,j) = r(15,i,j) + r(2,i,k)*r(16,k,j)
        r(22,i,j) = r(22,i,j) + r(23,i,k)*r(2,k,j)
        r(24,i,j) = r(24,i,j) + r(2,i,k)*r(23,k,j)
180     continue
      do 190 i=1,3
      do 190 j=1,3
      do 190 k=1,3
        r(3,i,j)  = r(3,i,j)  + r(5,i,k)*r(12,k,j)
        r(4,i,j)  = r(4,i,j)  + r(5,i,k)*r(10,k,j)
        r(13,i,j) = r(13,i,j) + r(23,i,k)*r(11,k,j)
        r(17,i,j) = r(17,i,j) + r(16,i,k)*r(12,k,j)
        r(18,i,j) = r(18,i,j) + r(16,i,k)*r(10,k,j)
        r(21,i,j) = r(21,i,j) + r(12,i,k)*r(15,k,j)
190     continue
      do 200 n = 1,24
        nv = n + 24
        do 200 i = 1,3
        do 200 j = 1,3
c         second 24 involve inversion
          r(nv,i,j) = - r(n,i,j)
200       continue
      return
      end
