      subroutine sort3(n,rsq,rvec)
c Sort floating point array rsq(n) into ascending numerical order 
c using Heapsort algorithm, while making corresponding rearrangement 
c of the integer array of vectors rvec(3,n).
c The main advantage of directly manipulating the additional array elements
c is the reduction of storage requirements.  The overhead of these 
c additional manipulations is pretty small.
c (Modified to help spchek.)
c Based on p. 231 of Numerical Recipes, 1986.
c Douglas C. Allan 
c Last revision 24 Mar 1995.  Last interface revision 24 Mar 1995.

      implicit none
      integer n
      integer rvec(3,n)
      double precision rsq(n)
      integer l,ir,i,j,rv1,rv2,rv3
      double precision rsqp

      if (n.eq.1) then
c      Accomodate case of array of length 1: already sorted!
       return
      else if (n.lt.1) then
c      Should not call with n<1
       write(06,1000) n
 1000  format(/,' sort3 has been called with array length n=',i12,/,
     $ ' having a value less than 1.  This is not allowed.',/)
       stop

      else

c      Conduct the usual sort

       l=n/2+1
       ir=n
10     continue
       if (l.gt.1) then
        l=l-1
        rsqp=rsq(l)
c       Whatever is done to rsq, do same to the components of rvec
        rv1=rvec(1,l)
        rv2=rvec(2,l)
        rv3=rvec(3,l)
       else
        rsqp=rsq(ir)
        rv1=rvec(1,ir)
        rv2=rvec(2,ir)
        rv3=rvec(3,ir)
        rsq(ir)=rsq(1)
        rvec(1,ir)=rvec(1,1)
        rvec(2,ir)=rvec(2,1)
        rvec(3,ir)=rvec(3,1)
        ir=ir-1
        if (ir.eq.1) then
         rsq(1)=rsqp
         rvec(1,1)=rv1
         rvec(2,1)=rv2
         rvec(3,1)=rv3
         return
        end if
       end if
       i=l
       j=l+l
20     if (j.le.ir) then
        if (j.lt.ir) then
         if (rsq(j).lt.rsq(j+1)) j=j+1
        end if
        if (rsqp.lt.rsq(j)) then
         rsq(i)=rsq(j)
         rvec(1,i)=rvec(1,j)
         rvec(2,i)=rvec(2,j)
         rvec(3,i)=rvec(3,j)
         i=j
         j=j+j
        else
         j=ir+1
        end if
        goto 20
       end if
       rsq(i)=rsqp
       rvec(1,i)=rv1
       rvec(2,i)=rv2
       rvec(3,i)=rv3
       goto 10
      end if

      end
