      subroutine speck(unit00,unit01,nk,gk0,b1,b2,b3,nsym,
     &               iinv,ck,mkpt,nkirr,symrec,ihc0,nbrav,symlat)

c A lot of this subroutine retains vestiges of primitive fortran coding, 
c i.e. many completely gratuitous goto's which could be removed in favor
c of cleaner coding.

c Input:
c  unit00=unit number for symmetry information
c  unit01=unit number for k point (ABINIT style) information
c  nk(3)=Monkhorst-Pack indices ("q")
c  gk0(3)=uniform shift of Monkhorst-Pack grid
c  b1(3),b2(3),b3(3)=reciprocal space primitive translations without 2*Pi
c  nsym=number of symmetries in the space group of the crystal
c  iinv= 0 for no inversion; 1 for inversion in the space group
c  mkpt=maximum number of allowed special k points (array dimension)
c  symrec(3,3,nsym)=(integer) symmetries of the crystal in the basis 
c   of (b1, b2, b3).
c  ihc0=0 for hexagonal groups, 1 for cubic groups
c  nbrav=number of lattice symmetries
c  symlat(3,3,nbrav)=symmetry matrices of lattice
c Output:
c  ck(4,mkpt)=special k points (in basis of b1,b2,b3) and weights
c  nkirr=number of special k points (in irreducible Brillouin zone)

      implicit none

c     ndim sets the maximum number of planes which may bound the 
c     1st Brillouin zone.
c     This number must lie in [6,14] in three dimensions (?).
c     Also, nmesh sets an upper limit on the number of k points in the
c     full Brillouin zone.
      integer ndim,nmesh
      parameter (ndim = 15, nmesh = 96000)
 
      integer unit00,unit01
      integer nsym,iinv,mkpt,nkirr,ihc0,nbrav
      integer nk(3),symrec(3,3,nsym),symlat(3,3,nbrav)
      double precision gk0(3),b1(3),b2(3),b3(3),ck(4,mkpt)

      integer imesh,iremov,nextra,nkitx,nkity,nkitz,count,isum,
     &  ir,ix,iy,iz,i,j,k,l,m,n,nplane,ntott,isym,iplace,jplace,igarbg
      integer include(nmesh),idir(3,ndim)
      double precision diff,dsum,dnk1,dnk2,dnk3,sum
      double precision symgc(3,3,48),rsdir(4,ndim),grt(3),gkt(3),gk(3)
      double precision kpt(3)
      
c H.J. Monkhorst and J.D. Pack, Phys. Rev. B13, 5188(1976) 
c comments by A. MacDonald, phys. rev. b18, 5897(1978).
c See J.D. Pack and H.J. Monkhorst, phys. rev. b16, 1748 (1977).
c n1, n2 and n3 are no. of meshes in the brillouin zone
c gk(i,j,k) = p1 * b1  + p2 * b2  + p3 * b3
c for cubic lattice 
c p1(i) = (2 * i - n1 - 1 ) / 2 * n1 .....e.t.c.
c for hexagonal lattice  
c pa(i) = pb(i) = (2 * i - nc - 1 ) / 2 * nc
c pc(i) = (i - 1 ) / nc

c     Set up symgc(3,3,.) to include all the crystal symmetries
c     and also time reversal symmetry if inversion is not present;
c     symmetries are expressed in basis of reciprocal lattice translations

      do isym=1,nsym
       do j=1,3
        do i=1,3
         symgc(i,j,isym)=symrec(i,j,isym)
        enddo
       enddo
      enddo

      ntott = nsym
 
c --- If inversion is not present in point group, include inversion
c     symmetry here based on time reversal symmetry
 
      if (iinv .le. 0) then
       ntott = nsym + nsym

c      ntott > 48 would be an error
       if (ntott .gt. 48) go to 302

       do isym=1,nsym
        do j=1,3
         do i=1,3
          symgc(i,j,isym+nsym)= - symrec(i,j,isym)
         enddo
        enddo
       enddo

      endif
 
c --- initialization
 
      do i  = 1, nmesh
       include(i) = 0
      enddo
      do 202 j = 1, ndim
      do 202 i = 1, 4
       rsdir(i,j) = 0.0d0
202   continue
 
c     Initialize the linked list
      do i = 1, 3
       gk(i) = 0.0d0
      enddo
      iplace = -2
      call listmg(unit00,gk,iplace)
      imesh = 0
 
c --- define the 1st brillouin zone
 
      call bzdef(unit00,b1,b2,b3,rsdir,ndim,nplane,idir)
 
c --- create Monkhorst-Pack mesh of k vectors
 
      if (ihc0 .eq. 1 ) then

c      cubic groups

       dnk1 = 1.0d0 / dble(2 * nk(1))
       nkitx = 2
       dnk2 = 1.0d0 / dble(2 * nk(2))
       nkity = 2
       dnk3 = 1.0d0 / dble(2 * nk(3))
       nkitz = 2

      else

c      hexagonal groups

       dnk1 = 1.0d0 / dble(nk(1))
       nkitx = 1
       dnk2 = 1.0d0 / dble(nk(2))
       nkity = 1
       dnk3 = 1.0d0 / dble(2 * nk(3))
       nkitz = 2

      endif

      count=0

c     Run loop over Monkhorst-Pack uniform grid of k points

      do 103 ix = nk(1) - 1, -nk(1) + 1, -nkitx
      do 103 iy = nk(2) - 1, -nk(2) + 1, -nkity
      do 103 iz = nk(3) - 1, -nk(3) + 1, -nkitz

       count=count+1
 
       kpt(1) = dble(ix) * dnk1 + gk0(1)
       kpt(2) = dble(iy) * dnk2 + gk0(2)
       kpt(3) = dble(iz) * dnk3 + gk0(3)
 
c ---  reduce the k-vector into the 1st bz if necessary
c      (note that bzred expects a vector in fractional coordinates)

       call bzred(kpt,b1,b2,b3,rsdir,ndim,nplane)

c      Apply all LATTICE symmetry operations and time reversal symmetry
c      This step is taken to create a grid with the full symmetry
c      of the LATTICE, not the possibly reduced symmetry of the crystal.

       do ir = 1, nbrav	
 
           do l = 1, 3
            gkt(l) = symlat(l,1,ir) * kpt(1) + 
     &               symlat(l,2,ir) * kpt(2) + 
     &               symlat(l,3,ir) * kpt(3)
           enddo

c        reduce rotated/reflected vector gkt to 1st bz

         call bzred(gkt,b1,b2,b3,rsdir,ndim,nplane)

c ---    check whether the new vector is in the list
 
         iplace = 0
         call listmg(unit00,gkt,iplace)
         if (iplace .gt. 0) imesh = iplace
         if (iplace .gt. nmesh) go to 305
 
       enddo
 
103   continue

      write(6,1110) count
 1110 format('number of k points in initial MP grid is ',3x,i10)
      write(6,1120) nbrav
 1120 format('number of symmetries of Bravais lattice nbrav is ',i5)
 
c --- in the mesh of k-vectors, now search for equivalent points
 
      iremov = 0
      do i = 1 , (imesh - 1)
       iplace = i
c      get the vector at location iplace in list
       call listmg(unit00,gkt,iplace)

c ---  now loop over all the rest of the mesh points
       do 280 j = (i + 1), imesh
        jplace = j
        call listmg(unit00,grt,jplace)

c       check (gkt-grt): is it integral (i.e. a recip lattice transl)?
        do 220 k = 1,3
         diff = gkt(k) - grt(k)
         if ( abs ( dble(nint(diff)) - diff ) .gt. 1.0d-6) then
          goto 280
         endif
220     continue
c       diff is integral: these represent the same k vector, so
c       remove grt from mesh and place in garbage list
        call remove (unit00,grt,jplace )
c       if grt actually removed, increment iremov
        if (jplace .gt. 0) iremov = iremov + 1
280    continue
      enddo

c     Now that unique points of full mesh are defined, construct
c     the irreducible Brillouin zone and tally up the weights
c     by considering the symmetries of the crystal.

      nkirr = 0

      do 106 i = 1, imesh

c      if mesh point was already included, skip to end of loop
       if(include(i) .eq. 1) go to 106

c      a new k-point is found
       include(i) = 1

c      Check whether number of k points in irreducible BZ, nkirr, exceeds
c      dimension mkpt (would be an error condition)
       if(nkirr .gt. mkpt) go to 302

c      get the k-point coordinates at mesh point i
       iplace = i
       call listmg(unit00,gkt,iplace) 
 
c      find out whether gkt is in the garbage list
       igarbg=0
       call garbag(unit00,gkt,igarbg)
 
c      if gkt is already in the garbage list then it isn't in the irr bz
       if (igarbg .gt. 0) go to 106
 
c      if not in garbage list, then add to irreducible bz
       nkirr = nkirr + 1
 
       do j = 1, 3
        ck(j,nkirr) = gkt(j)
       enddo
c      weighting factor
       ck(4,nkirr) = 1.0d0
 
c      find all the equivalent points
 
c      This involves applying all symmetries of the CRYSTAL, not merely
c      the lattice, and also applying time reversal if inversion is
c      not present

       do 108 ir = 1, ntott

        do 109 l = 1, 3
        grt(l) = 0.0d0
        do 109 m = 1, 3
        grt(l) = grt(l) + symgc(l,m,ir) * gkt(m)
109     continue

c       slap back into 1st bz again

        call bzred(grt,b1,b2,b3,rsdir,ndim,nplane)

c       find out whether grt is in the garbage list and, if so, skip
 
        call garbag(unit00,grt,igarbg)
 
        if (igarbg .gt. 0) go to 108
 
c       was grt already included in the irreducible bz ?
 
c       find vector grt in the list
        iplace = -1
        call listmg(unit00,grt,iplace)
  
        if (include(iplace) .gt. 0) go to 108
c       if was not already counted, count it up here (tally weight)
        ck(4,nkirr) = ck(4,nkirr) + 1.0d0
        include(iplace) = 1
108    continue
106   continue

c     Vectors of the irreducible Brillouin zone have now been defined
 
c     rotate special k-points such that they are positive (for convenience)
 
      do 118 i = 1, nkirr
       if(ck(1,i) .ge. 0.0d0 .and. ck(2,i) .ge. 0.0d0 .and.
     &    ck(3,i) .ge. 0.0d0) goto 118
       gkt(1) = ck(1,i)
       gkt(2) = ck(2,i)
       gkt(3) = ck(3,i)

c      Try all symmetries and time reversal
       do ir = 1, ntott
 
        do 120 l = 1, 3
         grt(l) = 0.0d0
         do 120 m = 1, 3
         grt(l) = grt(l) + symgc(l,m,ir) * gkt(m)
120     continue

c       Keep in 1st bz
        call bzred(grt,b1,b2,b3,rsdir,ndim,nplane)

c       If all components are positive, copy and jump out of this loop

        if (grt(1) .ge. 0.0d0 .and. grt(2) .ge. 0.0d0 .and. 
     &      grt(3) .ge. 0.0d0) then
         ck(1,i) = grt(1)
         ck(2,i) = grt(2)
         ck(3,i) = grt(3)
         go to 118
        endif
       enddo
118   continue     
 
c     normalize the weighting factors to be 1
c     Also tally up the weights to determine number of points in full zone.
 
      sum = 0.0d0
      do i = 1, nkirr
       sum = sum + ck(4,i)
      enddo
      dsum = 1.0d0 / sum
      isum=nint(sum)
 
      do n = 1, nkirr
       ck(4,n) = ck(4,n) * dsum
      enddo

      write(unit01,1100) nk,gk0,nkirr,isum
 1100 format(/,'# Monkhorst-Pack indices are ',3i5,/,
     &  '# with grid centered at k0=',3f12.8/,
     &  '# number of special points nkpt is ',i6,/,
     &  '# number of points in FULL Brillouin zone is ',i10)

      write(6,1150) isum
 1150 format('number of k points in FULL Brillouin zone is',i10)
 
      write(unit01,402)
402   format(/,'# nkpt, kpt(fractional), wtk in ABINIT format:')

      write(unit01,1000) nkirr
 1000 format(' nkpt    ',i6)
      write(unit01,1010)
 1010 format(' kpt')
      do n = 1, nkirr
       write(unit01,1030) (ck(i,n),i=1,3)
      enddo
 1030 format(3f22.15)
      write(unit01,1015)
 1015 format(' kptnrm     1')
      write(unit01,1020)
 1020 format(' wtk')
      nextra=mod(nkirr,3)
      do n=1,nkirr-nextra,3
       write(unit01,1030) (ck(4,n+i-1),i=1,3)
      enddo
      if (nextra.gt.0) then
       write(unit01,1030) (ck(4,nkirr-nextra+i),i=1,nextra)
      endif

      return
 
302   write(*,404) ntott, imesh, nkirr, mkpt
404   format(/'error in speck: failed in determining special k-points'/
     &      'total point group operators, total points in bz',2i6,
     &     /'total number of special k-points',i8, ' > dimensioned',
     &     ' mkpt',i8,
     &     /,'Raise mkpt in calling routine.'/)
      stop
c303   write(*,405) 
c405   format(/'could not locate k-point inside the 1st bz in speck')
c      stop
c304   write(*,406)
c406   format(/'rotated k-point was not found in the list in speck')
c      stop
305   write(*,407) iplace, nmesh
407   format(/,'iplace .gt. nmesh in speck: ',2i10,/,
     &  ' ==> too many k vectors for dimension nmesh in speck.',/)
      stop
      end
