      subroutine symatm(natom,xred,f0,nrt,is,tau,xredsym,tausym)
c Take fractional coordinates of atoms, in basis of real space
c primitive translations, and symmetrize them according to the input
c symmetry matrices.

c Doug Allan, Jan 1995.

c Input:
c  natom=number of atoms in cell
c  xred(3,natom)=reduced coordinates
c  f0(48,natom)=atom transformation table (integers)
c  nrt=number of symmetries in group
c  is(3,3,nrt)=integer rotation matrices in basis of primitive translations
c  tau(3,nrt)=nonsymmorphic translations
c Output:
c  xredsym(3,natom)=symmetrized fractional coordinates:
c     xredsym(j,n) = Sum(i) (is(j,k,i) * xred(k,n) ) / nrt
c  tausym(3,nrt)=nonprimitive translations forced to be simple rational
c     fractions
c xredsym may NOT be same array as xred; tausym MAY be same array as tau.

      implicit none
      integer natom,nrt
      integer f0(48,nrt),is(3,3,nrt)
      double precision xred(3,natom),tau(3,nrt),xredsym(3,natom),
     &  tausym(3,nrt)
      integer isym,mu,iatom,nu,itargt,ndiv
      double precision taut,sum,transl

c     First fix up the nonsymmorphic translations;
c     allowed fractions should be 1/6, 1/4, 1/3, and 1/2, EXCEPT
c     when unit cell is enlarged by some integer number of usual
c     lattice constants, in which case ANY additional integer should
c     be allowed in the denominator.  This makes the cleaning up process
c     more problematic.

c     This algorithm actually produces numbers to the nearest 1/48.

c THIS CHOICE, ndiv=48, could be inappropriate for certain weird
c supercells, this is not a robustly written algorithm.

      ndiv=48
      do isym=1,nrt
       do mu=1,3
        taut=tau(mu,isym)
        taut=dble(mod(nint(dble(ndiv)*taut)+ndiv,ndiv))/dble(ndiv)
        tausym(mu,isym)=taut
       enddo
      enddo

c     Next symmetrize each atomic position
      do iatom=1,natom
       do mu=1,3
        xredsym(mu,iatom)=0.d0
       enddo
      enddo

      do iatom=1,natom
       do isym=1,nrt
c       identify the target atom
        itargt=f0(isym,iatom)
c       write(6,'(''iatom,isym,itargt'',3i5,3f10.5)') iatom,isym,itargt,
c    &  (xred(mu,iatom),mu=1,3)

        do mu=1,3
         sum=0.d0
         do nu=1,3
c         real space rotations are transpose of is
          sum=sum+dble(is(nu,mu,isym))*xred(nu,iatom)
         enddo
         sum=sum+tausym(mu,isym)
c        Get rotated atom to agree with target atom after
c        removal of possible lattice translation
         transl=sum-xred(mu,itargt)
         sum=sum-dble(nint(transl))
c        write(6,'(''diff='',3f20.10)') sum-xred(mu,itargt),
c    &    sum,xred(mu,itargt)
         xredsym(mu,itargt)=xredsym(mu,itargt)+sum
        enddo
       enddo
      enddo

c     Average over rotated positions for final symmetrized coordinates
      do iatom=1,natom
       do mu=1,3
        xredsym(mu,iatom)=xredsym(mu,iatom)/dble(nrt)
       enddo
      enddo

      return
      end
