      subroutine symgen(a1,a2,a3,b1,b2,b3,nrt,rot,irot)
      implicit none
c
      integer nrt
      integer irot(3,3,48)
      double precision a1(3),a2(3),a3(3),b1(3),b2(3),b3(3),
     &  rot(3,3,48)
      integer ir,j,k,l
      double precision rrt(3,3,48),rr(3,3),rlatt(3,3),glatt(3,3)
c
c --- test symmetry operators for each case. run with symmetry
c --- of ibrav=i, indpg=1. then run ibrav=i,indpg=proper symmetry.
c --- these two cases should give identical results at gamma if
c --- symmetry is correct.
c --- rot(i,j,irot)*r(i) are the real-space rotation matices (note transpose)
c --- use rot(j,i,irot)*g(i) to operate on g-space vectors (?)
c --- rlatt(i,j) i-th cartesian component of j-th lattice vector

c In this code, a1, a2, and a3 are the real space (dimensional) 
c primitive translations in cartesian coordinates.  b1, b2, and b3
c are the reciprocal space primitive translations, given by
c (b1 b2 b3) = transpose (inverse ( a1 a2 a3) )
c where (a1 a2 a3) and (b1 b2 b3) are each 3x3 matrices.  Note NO 2*Pi.

c Applying rotation rot(j,i,irot)*g(i) to some g-space vector g(i) is
c the SAME as applying the transpose (=inverse) operation
c rot(i,j,irot)*r(i) to the real space vectors r(i) (in cartesian coords).

c     Make sure nrt does not exceed dimensioned 48
c     (optimized compilation with xlf on kptgen causes this to happen)
      if (nrt.gt.48) then
       write(6,1000) nrt
 1000  format(/,' Error in symgen: input nrt > 48, is ',i16,/)
       stop
      endif
 
c --- convert to lattice coordinates
 
c --- do the unitary transformation to the bases of b1,b2,b3
 
      do ir = 1, 3
       rlatt(ir,1) = a1(ir)
       rlatt(ir,2) = a2(ir)
       rlatt(ir,3) = a3(ir)
       glatt(ir,1) = b1(ir)
       glatt(ir,2) = b2(ir)
       glatt(ir,3) = b3(ir)
      enddo

c     Columns of rlatt are real space primitive translation vectors (a1 a2 a3)
c     Columns of glatt are reciprocal space     translation vectors (b1 b2 b3)
c     where (b1 b2 b3) = transpose(inverse(a1 a2 a3))

c     Rotation matrices expressed in the basis of real space primitive
c     translations are related to the cartesian matrices S expressing
c     rotations in g space by
c     symrel = R(inverse) * S(transpose) * R
c            = (b1 b2 b3)(transpose) * S(transpose) * (a1 a2 a3)
c     where symrel is in terms of columns of R and S(transpose) 
c     is the cartesian rotation matrix as applied in real space.
c     Rotation matrices expressed in the basis of reciprocal space
c     translations are given by
c     symrec = G(inverse) * S * G  where G = R (inverse transpose)
c            = (a1 a2 a3)(transpose) * S * (b1 b2 b3) 

      do 101 ir = 1, nrt
c      Compute transpose ( transpose(S) * R), S=rotation, R=(a1 a2 a3)
c       = R(transpose) * S
       do j = 1, 3
        do k = 1, 3
         rr(j,k) = 0.0d0
         do l = 1, 3
           rr(j,k) = rr(j,k) + rlatt(l,j) * rot(l,k,ir)
         enddo
        enddo
       enddo

c      Compute [R(transpose) * S] * G, G=(b1 b2 b3)
       do j = 1, 3
        do k = 1, 3
         rrt(j,k,ir) = 0.0d0
         do l = 1, 3
          rrt(j,k,ir) = rrt(j,k,ir) + rr(j,l) * glatt(l,k)
         enddo
        enddo
       enddo

101   continue

c     The rrt matrices are now  G(inverse) * rot * G, i.e.
c     are the rotations expressed in the basis of recip. translations
 
      do 108 ir = 1, nrt
      do 109 k = 1, 3
      do 109 j = 1, 3
c      should be all integers
       irot(j,k,ir) = nint(rrt(j,k,ir))
109   continue
c     write(6,2030) ((irot(j,k,ir),j=1,3),k=1,3),((rrt(j,k,ir),j=1,3),
c    & k=1,3)
c2030 format(9i8,/,9f8.5)
108   continue


c     TEST TEST
c     write(6,2000) a1,a2,a3,b1,b2,b3
c2000 format(' From symgen',/,'R=',9f8.3,/,'G=',9f8.3,/)
c     do ir=1,nrt
c      write(6,2010) ir,((rot(l,k,ir),l=1,3),k=1,3)
c      write(6,2020) ((irot(l,k,ir),l=1,3),k=1,3)
c     enddo
c2010 format(i5,9f8.3)
c2020 format(5x,i4,8i8)

      return
      end
