//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_NEB_PROPAGATOR_H
#define OHMMS_NEB_PROPAGATOR_H

#include "Propagators/PropagatorBase.h"
#include "OhmmsData/OhmmsElementBase.h"

namespace OHMMS {

  class NEBPotential;

  /*!\class NEBPropagator
   * \author Jeongnim Kim
   * \brief A propagator for NEB calculations
   *
   * \note If non-zero friction is used, using NEBPropagator results
   * in structural relaxation by steepest descent method. Because the
   * previous gradients as well as current gradients are used for update,
   * relaxation is very efficient. 
   */
  class NEBPropagator: public PropagatorBase { // using Verlet algorithm

  public:

    NEBPropagator();
    ~NEBPropagator() { }
 
    // implementing virutal functions of PropagatorBase
    void initialize(Particle_t* ptclin, Interaction_t* potin, bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(Interaction_t*);
    void applyBConds(bool wrapping);
    void copy(PropagatorBase* old);

    void setImages(vector<Particle_t*>& images);

  protected:

    size_type Rid;

    ///reference temperature
    Scalar_t T_ref;

    ///the time step
    Scalar_t Dt;

    ///the friction coefficient
    Scalar_t Friction;

    ///internal parameters for the EOM
    Scalar_t c0, c1, c2, oneover2dt;

    vector<Particle_t*> Images;
  };

}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 381 $   $Date: 2005-09-05 15:24:10 -0500 (Mon, 05 Sep 2005) $
 * $Id: NEBPropagator.h 381 2005-09-05 20:24:10Z jnkim $ 
 ***************************************************************************/
