//////////////////////////////////////////////////////////////////
// (c) Copyright 2004-  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_SLATERKOSTERTB_FORNANOAPP_H
#define OHMMS_SLATERKOSTERTB_FORNANOAPP_H

#include "Utilities/OhmmsInfo.h"
#include "SlaterKosterTB/SlaterKosterTB.h"
#include "OhmmsNumeric/ArpackSparseSolvers.h"
#include "OhmmsNumeric/ArpackSolverIO.h"

namespace OHMMS {

  ///Forward declaration
  class ArpackParam;

  /** Specialized SlaterKosterTB class using Arpack Solver
   */
  class ArpackSKTB: public SlaterKosterTB {

  public:

    ArpackSKTB();
    ~ArpackSKTB();

    /// implementing virtual functions for OhmmsElementBase
    bool get(ostream&) const;
    bool put(istream&);
    void reset();
    bool put(xmlNodePtr);

    Scalar_t el_ph_coupling(const vector<int>& psi_i, 
			    const PhononMode& phonon,
			    vector<int>& psi_f, Scalar_t del) { return 0.0;}

  protected:

    Scalar_t tbEnergy();
    Scalar_t tbForce(ParticlePos_t& f, ParticleScalar_t& e,
		       Tensor_t&, unsigned stressmode) { return 0.0;}
    void     tbDynamicMatrix(DynMatrix_t&) { }
    void     tbInit();
    void     getAtomicProperties(int i);

  private:

    bool NeedAtomicProperties;
    bool WriteForDx;
    ArpackSparseSolver *eig_solver;
    vector<ArpackParam*> Spectrum;
    hid_t hdfFile, hdfGroup;
  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 182 $   $Date: 2004-11-15 12:18:10 -0600 (Mon, 15 Nov 2004) $
 * $Id: ArpackSKTB.h 182 2004-11-15 18:18:10Z jnkim $
 ***************************************************************************/
