//////////////////////////////////////////////////////////////////
// (c) Copyright 2004-  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "NanoApp/NanoSim.h"
#include "NanoApp/ArpackSKTB.h"
#include "AppBase/ProjectData.h"
#include "AppBase/ParticleSets.h"
#include "Propagators/NoPropagator.h"
#include "ParticleIO/HDFParticleIO.h"

namespace OHMMS {

  NanoSim::NanoSim(OhmmsElementSet& nodelist, const char* aname): 
    OhmmsElementBase(aname), 
    NodeList(nodelist) {
  }

  bool NanoSim::get(std::ostream& os) const { return true;}
  bool NanoSim::put(std::istream& is) { return true;}

  bool NanoSim::put(xmlNodePtr cur) {

    //Create ArpackSKTB
    ArpackSKTB TBPot;
    RecordPropertyList records;

    cur = cur->children;
    xmlNodePtr tb_ptr = 0;
    while (cur != 0) {
      string cname((const char*)(cur->name));
      if(cname == "potential") {
	tb_ptr = cur;
      } else if(cname == "record") {
	xmlChar *t= xmlGetProp(cur, (const xmlChar *)"name");
	if(t) {
	  string pname((const char*)t);
	  if(pname == "particle" || pname == "Particle") {
	    ParticleSets* ptcl = (dynamic_cast<ParticleSets*>(NodeList.getNode(Key_particle)));
	    vector<RecordProperty*> alist;
	    bool success = ptcl->createRecordProperty(cur,alist);
	    if(success) {
	      for(int i=0;i<alist.size(); i++) {
		records.addRecord(alist[i]);
	      }
	    }
	  }
	}
      }
      cur = cur->next;
    }

    string fname("TBPARAM");
    if(tb_ptr) {
      cur = tb_ptr->children;
      while(cur != 0) {
	string cname((const char*)(cur->name));
	if(cname == "filename") {
	  putContent(fname,cur);
	}
	cur = cur->next;
      }
    }

    ProjectData *project = dynamic_cast<ProjectData*>(NodeList.getNode(Key_project));

    //assign the particleset to TBPot
    ParticleSets::Particle_t* workingptcl
      = (dynamic_cast<ParticleSets*>(NodeList.getNode(Key_particle)))->get();
    TBPot.set(workingptcl);
    HDFSaveParticle hdfout(*workingptcl);
    hdfout.reset(project->CurrentRoot());
    hdfout.report(1);
    hdfout.finalize();

    if(tb_ptr) {
    //initialize SK parameters etc
    TBPot.put(tb_ptr);
    vector<ShortRangePotential*> pot_s;
    bool success = TBPot.getSK().init(fname,pot_s);
    TBPot.setName(project->CurrentRoot());
    TBPot.initialize();

    //use NoPropagator
    NoPropagator mover;
    mover.initialize(workingptcl,false);
    mover.propagate(&TBPot);

    records.resetRecords(project->CurrentRoot());
    records.report(1);
    records.finalize();

    }
    return true;
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 183 $   $Date: 2004-11-15 15:00:18 -0600 (Mon, 15 Nov 2004) $
 * $Id: NanoSim.cpp 183 2004-11-15 21:00:18Z jnkim $
 ***************************************************************************/

