//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Utilities/OhmmsInfo.h"
#include "Message/Communicate.h"
#include "AppBase/OhmmsAppBase.h"
#include "NanoApp/NanoSim.h"
#include "NanoApp/Point2Grid.h"
#include "SparseMatrix/WorkSpace.h"

namespace OHMMS {

  class NanoApp: public OhmmsAppBase {

  public:

    ///constructor
    NanoApp(int argc, char** argv): OhmmsAppBase(argc,argv) {}

    ///destructor
    ~NanoApp() {}

    bool dump() {
      LOGMSG("Nothing to write")
      return true;
    }

    void initialize() {
      NodeList->addNode(new NanoSim(*NodeList));
      NodeList->addNode(new Point2Grid(*NodeList));
    }  
  };
}

/**\file main.cpp
 * The main routine that creates NanoApp to perform atomistic simulations.
 */
int main(int argc, char **argv) {

  WorkSpace::create(1024000);

  OHMMS::Controller->initialize(argc,argv);

  OHMMS::NanoApp app(argc,argv);
  app.initialize();

  xmlKeepBlanksDefault(0);
  xmlDocPtr doc;
  if(argc == 2) {
    if(!app.parse(argv[1])) {
      ERRORMSG("Error parsing file " << argv[1])
    }
  } else if(argc > 2) {
    int iargc=0;
    while(iargc<argc) {
      if(!strcmp(argv[iargc],"-f")) {
	if(!app.parse(argv[++iargc])) {
          ERRORMSG("Error parsing file " << argv[iargc])
	}
      }
      iargc++;
    }
  } else {
    ERRORMSG("No input file is provided.")
    ERRORMSG("Usage: ohmms.x input-xml")
  }

  /* Clean up everything else before quitting. */
  xmlCleanupParser();
  OHMMS::Controller->finalize();
  return(0);
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 255 $   $Date: 2005-01-14 11:41:16 -0600 (Fri, 14 Jan 2005) $
 * $Id: nano.cpp 255 2005-01-14 17:41:16Z jnkim $ 
 ***************************************************************************/
