//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Utilities/OhmmsInfo.h"
#include "Message/Communicate.h"
#include "OhmmsApp/OhmmsApp.h"
//#include "OhmmsData/OhmmsElementBase.h"
//#include "OhmmsApp/Configuration.h"
//#include "OhmmsApp/ProjectData.h"
//#include "OhmmsApp/IonicPotential.h"
//#include "OhmmsApp/ParticleSets.h"
//#include "ParticleIO/RecordParticle.h"
#include "SKTB.h"
using namespace OHMMS;

struct QDApp {

  QDApp(int argc, char**argv){
    initialize(argc,argv);
  }

  ~QDApp();

  int initialize(int argc, char **argv);
  void apply();

  vector<RecordProperty*> Properties;
  SKTB TBPot;
};


int main(int argc, char** argv) {

  OhmmsData::Add(Key_project, new ProjectData);
  OhmmsData::Add(Key_ptclin, new ParticleSets);

  QDApp apps(argc,argv);
  apps.apply();

}

void QDApp::apply() {

  Particle_t* workingptcl
    = (dynamic_cast<ParticleSets*>(OhmmsData::Members[Key_ptclin]))->get();

  TBPot.set(workingptcl);

  for(int i=0; i<Properties.size(); i++) 
    Properties[i]->report(0);

  TBPot.evalEnergy();
}

QDApp::~QDApp() {
  for(int i=0; i<Properties.size(); i++)  delete Properties[i];
}

int QDApp::initialize(int argc, char **argv) {

  xmlDocPtr doc;
  xmlNsPtr ns;
  xmlNodePtr cur;

  // build an XML tree from a the file;
  doc = xmlParseFile(argv[1]);
  if (doc == NULL) return(NULL);
  
  // Check the document is of the right kind
  cur = xmlDocGetRootElement(doc);
  if (cur == NULL) {
    fprintf(stderr,"empty document\n");
    xmlFreeDoc(doc);
    return(NULL);
  }

  ns = NULL;
  if (xmlStrcmp(cur->name, (const xmlChar *) "Main")) {
    fprintf(stderr,"document of the wrong type, root node != Main");
    xmlFreeDoc(doc);
    return(NULL);
  }

  // First level we expect just Jobs
  cur = cur->xmlChildrenNode;
  while ( cur && xmlIsBlankNode ( cur ) )
    {
      cur = cur -> next;
    }
  if ( cur == 0 )
    return false;
  if ((xmlStrcmp(cur->name, (const xmlChar *) "simulation"))) {
    fprintf(stderr,"document of the wrong type, was '%s', simulation expected",
	    cur->name);
    fprintf(stderr,"xmlDocDump follows\n");
    xmlDocDump ( stderr, doc );
    fprintf(stderr,"xmlDocDump finished\n");
    xmlFreeDoc(doc);
    return(NULL);
  }

  cur = cur->xmlChildrenNode;
  while (cur != NULL) {
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"project"))) {
      OhmmsData::Members[Key_project]->put(doc,ns,cur);
    }
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"particlesets"))) {
      OhmmsData::Members[Key_ptclin]->put(doc,ns,cur);
    }

    if ((!xmlStrcmp(cur->name, (const xmlChar *)"record"))) {
      string pname = (const char*)(xmlGetProp(cur, (const xmlChar *) "name"));
      if(pname == "particle") {
	Particle_t* thisptcl 
	  = (dynamic_cast<ParticleSets*>(OhmmsData::Members[Key_ptclin]))->get();
	Properties.push_back(new RecordParticle(*thisptcl));
	Properties.back()->put(doc,ns,cur);
      }
    }
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"potential"))) {
      TBPot.put(doc,ns,cur);
      TBPot.initialize();
    }
    cur=cur->next;
  }

  ProjectData* project 
    = dynamic_cast<ProjectData*>(OhmmsData::Members[Key_project]);
  project->reset();

  for(int i=0; i<Properties.size(); i++) 
    Properties[i]->reset(project->CurrentRoot());

  TBPot.setName(project->CurrentRoot());
  Particle_t* workingptcl
    = (dynamic_cast<ParticleSets*>(OhmmsData::Members[Key_ptclin]))->get();
  workingptcl->put(doc,ns,cur);
  return 0;
}


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: quantumdot.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

