//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifdef HAVE_CONFIG_H
#include "ohmms-config.h"
#endif
#include "Utilities/OhmmsInfo.h"
#include "Message/Communicate.h"
#include "OhmmsApp/OhmmsApp.h"
#include "AppBase/ProjectData.h"
#include "AppBase/ParticleLayoutManager.h"
#include "OhmmsApp/PropagatorBuilder.h"
#include "OhmmsApp/AppConfig.h"
#include "OhmmsApp/IonicPotential.h"
#include "OhmmsApp/MDRun.h"
#include "OhmmsApp/NEBRun.h"
//#include "OhmmsApp/Test.h"
#if defined(ENABLE_DIMER)
#include "OhmmsApp/DimerRun.h"
#endif
#if defined(ENABLE_FEMC)
#include "FreeEnergyMC/FreeEnergyMC.h"
#endif
#if defined(ENABLE_ELPH)
#include "OhmmsApp/ElectronPhononCoupling.h"
#endif
#include "OhmmsApp/ParticleSetBuild.h"
#include "OhmmsApp/MinimaHopping.h"
//#include "OhmmsApp/ScanUnitCell.h"

namespace OHMMS {

  /*! \brief Constructor 
   * Register OhmmsElementBase objects corresponding to the major tags of input xml files.
   * OhmmsElementSet contains default objects \ref ohmmsdefaults and the constructor
   * adds OhmmsElementBase subclasses that depend on the default objects.
   *
   */
  OhmmsApp::OhmmsApp(int argc, char** argv): OhmmsAppBase(argc,argv) {
  }
    
  void OhmmsApp::initialize() {

    NodeList->addNode(new PropagatorBuilder);
    NodeList->addNode(new IonicPotential);
    NodeList->addNode(new ParticleLayoutManager(*NodeList));

    NodeList->addNode(new MDRun(*NodeList));
    NodeList->addNode(new NEBRun(*NodeList));
#if defined(ENABLE_DIMER)
    NodeList->addNode(new DimerRun(*NodeList));
#endif
#if defined(ENABLE_FEMC)
    NodeList->addNode(new FreeEnergyMC(*NodeList));
#endif
#if defined(ENABLE_ELPH)
    NodeList->addNode(new ElectronPhononCoupling(*NodeList));
#endif
    NodeList->addNode(new ParticleSetBuild(*NodeList));
    //NodeList->addNode(new Test(*NodeList));
    NodeList->addNode(new MinimaHopping(*NodeList));
    //NodeList->addNode(new ScanUnitCell(*NodeList));

    //call virtual function for initialization
    //initialize();
  }

  OhmmsApp::~OhmmsApp() {
  }

  bool OhmmsApp::dump() {
     
    if(Controller->master()) {

      ProjectData* project = dynamic_cast<ProjectData*>(NodeList->getNode(Key_project));

      (*NodeList)[Key_particle]->reset();
      (*NodeList)[Key_ensemble]->reset();
      (*NodeList)[Key_potential]->reset();

      for(int k=Key_random; k<NodeList->size(); k++)
        (*NodeList)[k]->reset();

      string newxml(project->CurrentRoot());
      newxml.append(".cont.xml");
      xmlSaveFormatFile(newxml.c_str(),m_doc,1);
    }
    return true;
  }
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 317 $   $Date: 2005-03-01 17:13:41 -0600 (Tue, 01 Mar 2005) $
 * $Id: OhmmsApp.cpp 317 2005-03-01 23:13:41Z jnkim $ 
 ***************************************************************************/
