//////////////////////////////////////////////////////////////////
// (c) Copyright 2004  by Jeongnim Kim and Charles Cornwell
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//
// Original c code was developed by C. Cornwell at ARL.
// Modified for OHMMS and other related products.
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "SWCNTBuilder.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <math.h>

SWCNTBuilder::SWCNTBuilder() {
  m_bond = 1.460412;
}

void 
SWCNTBuilder::build(int n1,int n2,int n_uc)
{
  int 	i,j,k,i1,j1,k1,f,g,type,at_ct=0;
  int	syssize,atring,rings,sys;
  int	p1,p2,N,loc=0;
  double  x,y,z,sum,a,b,a1,a2,xij,yij,zij,rij,M,T;
  double  vx[2],vy[2],vz[2],abR,Rc,cir,am,ct;
  double  th,alpha,th1,th2,h,h1,x1,y1,z1,uc_trans;
  const double pi = 3.141592654;
  
  vec suv[3],stv[3],tmp,tmp1,L,L1,L2,e[3],trans;
  vec R,min,max,dist,uc[2],uc_cross,uc_rot[2];
  vec R1,R2,H,d,r,cyl_r,R1xR2; /* *motif */
  std::vector<vec> motif;

  e[0].x = 1.0; 
  e[0].y = 0.0;
  e[0].z = 0.0;
  trans.x = 0.0;
  trans.y = 0.0;
  trans.z = 0.0;
  trans.d = 0.0;
  

  double hexa = m_bond*sqrt(3.0)*0.5;  
  R1.x = hexa*2.0;//R1.x = 1.264753892 * 2.0;
  R1.y = 0.0;
  R1.z = 0.0;
  
  R2.x = hexa;//R2.x = 1.264753892;
  R2.y = 0.0;
  R2.z = (3.0/2.0) * m_bond;//1.460412;
  
  R.x = n1 * R1.x + n2 * R2.x;
  R.y = n1 * R1.y + n2 * R2.y;;
  R.z = n1 * R1.z + n2 * R2.z;;
  R.d = sqrt(R.x * R.x + R.y * R.y + R.z * R.z);
  Rc = R.d/(2.0*pi);
  
  
  // finds p1, p2 and N
  p1p2(&R1,&R2,&p1,&p2,n1,n2,&N);
  
  H.x = p1 * R1.x + p2 * R2.x;
  H.y = p1 * R1.y + p2 * R2.y;
  H.z = p1 * R1.z + p2 * R2.z;
  H.d = sqrt(H.x * H.x + H.y * H.y + H.z * H.z);
  
  //motif = (vec *)  malloc(sizeof(vec) * 2 * N);
  motif.resize(2*N);
  
  d.x = (R1.x+R2.x)/3.0;
  d.y = (R1.y+R2.y)/3.0;
  d.z = (R1.z+R2.z)/3.0;
  d.d = sqrt(d.x * d.x + d.y * d.y + d.z * d.z);
  
  // positions the atoms in the unit cell
  uc[0].x = d.x;
  uc[0].y = d.y;
  uc[0].z = d.z;
  uc[0].d = sqrt(uc[0].x*uc[0].x + uc[0].y*uc[0].y + uc[0].z*uc[0].z);
  uc[1].x = 2.0 * d.x;
  uc[1].y = 2.0 * d.y;
  uc[1].z = 2.0 * d.z;
  uc[1].d = sqrt(uc[1].x*uc[0].x + uc[1].y*uc[1].y + uc[1].z*uc[1].z);
  
  // finds the cross product of H and R
  cross(&H,&R,&r);
  
  // calculates h and alpha for translation and rotation of motif
  h = r.d/R.d;
  alpha = 2.0* pi * dot(&H,&R) /(R.d*R.d);
  
  // calculates the length of the unit cell
  L.d = LENGTH(n1,n2,&R);
  
  // calculates the number of times the translation and 
  // rotation must be applied to the motif to create a unit cell
  M = sqrt(3.0)*R.d/h;
  M = L.d/h;
  
  // defines a translation vector
  trans.x = 0.0;
  trans.y = h;
  trans.z = 0.0;
  trans.d = h;
  
  for(i = 0; i < 2; i++)
    {
      uc_rot[i].x = uc[i].x;
      uc_rot[i].y = uc[i].y;
      uc_rot[i].z = uc[i].z;
      uc_rot[i].x -= d.x;
      uc_rot[i].y -= d.y;
      uc_rot[i].z -= d.z;
      uc_rot[i].x += Rc;
      uc_rot[i].y += 0.0;
      uc_rot[i].z += 0.0;
    }
  th = 2.0* pi * (dot(&d,&R))/(R.d*R.d);
  cross(&d,&R,&uc_cross);
  uc_trans = uc_cross.d/R.d;
  rot(&uc_rot[0],&uc_rot[1],th,1);
  uc_rot[1].y = uc_trans;
  xij = uc_rot[1].x -uc_rot[0].x;
  yij = uc_rot[1].y -uc_rot[0].y;
  zij = uc_rot[1].z -uc_rot[0].z;
  rij = sqrt(xij*xij + yij*yij + zij*zij);       
  for(i = 0; i < N; i++) {
    th1 = i * (2.0*pi)/N;
    for(j = 0; j < 2; j++) {
      loc = i * 2 + j;
      rot(&uc_rot[j],&motif[loc],th1,1); 
    }
  }
  
  at_ct = 0; 
  
  std::ofstream fout("swcnt.xyz");
  fout <<int((M+.5))*n_uc*N*2 << std::endl;
  fout << "# z = " << std::setprecision(10) << L.d*n_uc << " Rc = " << Rc << std::endl;
  //fprintf(fw,"%d\n\n",(int)(M+.5)*n_uc*N*2);
  for(k = 0; k < (int) (M+.5) * n_uc; k++) {
    th1 = k * alpha;
    h1 = k * h;
    for(i = 0; i < N; i++) {
      for(j = 0; j < 1; j++) {
	loc = i * 2 + j;
	rot(&motif[loc],&tmp,th1,1);
	tmp.y += h1;
	rot(&motif[loc+1],&tmp1,th1,1);
	tmp1.y += h1;
	xij = tmp1.x - tmp.x;
	yij = tmp1.y - tmp.y;
	zij = tmp1.z - tmp.z;
	rij = sqrt(xij*xij + yij*yij + zij*zij);
	x = sqrt(tmp.x * tmp.x + tmp.z * tmp.z);
	y = sqrt(tmp1.x * tmp1.x + tmp1.z * tmp1.z);
	
	//fprintf(fw,"C %f\t%f\t%f\n",tmp.x,tmp.y,tmp.z);
	//fprintf(fw,"C %f\t%f\t%f\n",tmp1.x,tmp1.y,tmp1.z);
	fout << "C " << std::setw(20) << tmp.x 
	     << std::setw(20) << tmp.z 
	     << std::setw(20) << tmp.y << std::endl;
	fout << "C " << std::setw(20) <<tmp1.x 
	     << std::setw(20) << tmp1.z 
	     << std::setw(20) << tmp1.y << std::endl;
	at_ct += 2;
      }
    }
  }
}

int 
SWCNTBuilder::p1p2(vec *R1, vec *R2, int *p1, int *p2, int n1, int n2, int *N)
{
  double amin=100000.0,NN=10000.0;
  int    i,j,p1min,p2min,Nmin=10000,LOOP=100;
  vec    a,b,c,d;

  *N = LCD(n1,n2);

  b.x = n1 * R1->x + n2 * R2->x;
  b.y = n1 * R1->y + n2 * R2->y;;
  b.z = n1 * R1->z + n2 * R2->z;;
  b.d = sqrt(b.x * b.x + b.y * b.y + b.z * b.z);

  for(i = 1; i < n1; i++)
    {
      if((*N+i * n2)%n1 == 0) {
	*p1 = i;
	*p2 = (*N+i*n2)/n1;
	a.x = *p1 * R1->x + *p2 * R2->x;
	a.y = *p1 * R1->y + *p2 * R2->y;
	a.z = *p1 * R1->z + *p2 * R2->z;
	a.d = sqrt(a.x * a.x + a.y * a.y + a.z * a.z);
	
	if(a.d < amin) {
	  amin = a.d;
	  p1min = *p1;
	  p2min = *p2;
	  Nmin = *N;
	}
      }
    }
  return 1;
}

double SWCNTBuilder::dot(vec *a, vec *b)
{
  return (a->x * b->x + a->y * b->y + a->z * b->z);
}

int SWCNTBuilder::cross(vec *a, vec *b, vec *c)
{
  c->x = (a->y*b->z - a->z*b->y); 
  c->y = (a->z*b->x - a->x*b->z);
  c->z = (a->x*b->y - a->y*b->x);
  c->d = sqrt(c->x * c->x + c->y * c->y + c->z * c->z);
  return 1;
}

double SWCNTBuilder::cos_th(vec *a, vec *b)
{
  double	dot;
  dot = (a->x * b->x + a->y * b->y + a->z * b->z);
  dot /= (a->d * b->d);
  return dot;
}

double SWCNTBuilder::theta(vec *a, vec *b)
{
  double	dot;
  dot = (a->x * b->x + a->y * b->y + a->z * b->z);
  dot /= (a->d * b->d);
  dot = acos(dot);
  return dot;
}

int SWCNTBuilder::rot(vec *a, vec *b, double th2, int q)
{
  switch(q)
    {
    case 0:
      {
        b->x = a->x;
        b->y = a->y*cos(th2) - a->z * sin(th2);
        b->z = a->y*sin(th2) + a->z* cos(th2);
	break;
      }
    case 1:
      {
        b->x = a->x*cos(th2) - a->z * sin(th2);
        b->y = a->y;
        b->z = a->x*sin(th2) + a->z* cos(th2);
	break;
      }
    case 2:
      {
        b->x = a->x*cos(th2) - a->y * sin(th2);
        b->y = a->x*sin(th2) + a->y* cos(th2);
        b->z = a->z;
	break;
      }
    }
  b->d = sqrt(b->x * b->x + b->y * b->y + b->z * b->z);
  return 1;
}

int SWCNTBuilder::unit(vec *a)
{
  a->x /= a->d;
  a->y /= a->d;
  a->z /= a->d;
  a->d = sqrt(a->x * a->x + a->y * a->y + a->z * a->z);
}

int SWCNTBuilder::LCD(int a, int b)
{
  int i,max=0,L=0;
  
  if(a > b) 
    { max = a;}
  else { max = b;}

  for(i = 1; i < max+1; i++) {
    if(a%i == 0 && b%i == 0) {
      if( i > L) {
      L = i;
    }
    }
  }

  return L;
}

double SWCNTBuilder::LENGTH(int a, int b, vec *r)
{
  int i,max=0,L=0,N=0;
  double length,M,T;

  L = LCD((2*a+b),(2*b+a));

   length = sqrt(3.0)*r->d/L;

  return length;
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SWCNTBuilder.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
