//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Message/Communicate.h"
#include "Particle/ParticleSet.h"
#include "ParticleIO/XMLParticleIO.h"
#include "Potentials/PotentialCollection.h"
//#include "Potentials/SWPotential.h"
#include "Potentials/MEAMPotential.h"
//#include "Potentials/KeatingPotential.h"
//#include "Potentials/CoulombPotential.h"
#include "Propagators/NoPropagator.h"
#include "OhmmsApp/Test.h"
#include "AppBase/ProjectData.h"
#include "AppBase/ParticleSets.h"

using namespace OHMMS;

  Test::Test(OhmmsElementSet& nodelist, const char* aname): 
    OhmmsElementBase(aname),NodeList(nodelist)
  {  }

  bool Test::get(ostream& os) const {
    return true;
  }

  bool Test::put(istream& is) { 
    return true;
  }

  void Test::reset() {
  }

#if defined(HAVE_LIBXML2)

  bool Test::put(xmlNodePtr cur){

    cur = cur->xmlChildrenNode;

    map<string,int> nodes;
    nodes["particlesets"]   = Key_particle;
    nodes["particlelayout"] = Key_playout;
    nodes["ensemble"]       = Key_ensemble;
    nodes["potential"]      = Key_potential;
    double rmax = 2.8;

    while (cur != NULL) {
      ///The first character of the tag is capitalized.
      string cname((const char*)(cur->name));
      if(cname == "Rcut") {
	putContent(rmax,cur);
      }
      map<string,int>::iterator it = nodes.find(cname);
      if(it != nodes.end()) {
	int inode = (*it).second;
	NodeList.getNode(inode)->put(cur);
      }
      cur = cur->next;
    }

    //================================================
    typedef ParticleSets::Particle_t Particle_t;

    Particle_t* sys
      = (dynamic_cast<ParticleSets*>(NodeList.getNode(Key_particle)))->get();
    
    char fname[128],nnfile[128];
    sprintf(fname,"bulk.p%0d",Controller->mycontext());    
    sprintf(nnfile,"nnlist.p%0d",Controller->mycontext());    
    ofstream nnout(nnfile);
    //sys->getNNEngine()->getPairlist(*sys,3.0,0);

    PotentialCollection pot;
    //pot.addPotential(new CoulombPotential,"coulomb");
    pot.addShortRangePotential(new MEAMPotential,"meam",0);
    //KeatingPotential swwpot;
    
    NoPropagator nothing;
    nothing.initialize(sys,&pot,false);

    nothing.propagate(&pot);

    double e = nothing.getProperty(PropagatorBase::POT_ENERGY);

    //cout << "Energy per particle " << e/static_cast<double>(sys->getLocalNum()) << endl;
    cout << "Energy per particle " << e/static_cast<double>(sys->getTotalNum()) << endl;
//     Particle_t::ParticlePos_t& f = *(sys->getVectorAttrib(ParticleTags::force_tag));
//     Particle_t::ParticleScalar_t& e = *(sys->getScalarAttrib(ParticleTags::energy_tag));

//     Particle_t::Tensor_t stress;
//     pot.evalForce(f,e,stress,0);
//     nnout << 0 << " " << f[0] << endl;

//     Particle_t::ParticlePos_t R(sys->R);
//     R.setUnit(sys->R.getUnit());
//     double dx = 0.001;        
//     Particle_t::SingleParticlePos_t r0 = R[0];
//     f = 0.0;      
//     e = 0.0;      
//     if(!Controller->mycontext()) R[0][2] = r0[2]-dx;
//     sys->updateR(R);
//     pot.evalForce(f,e,stress,0);
//     nnout << -dx << " " << f[0] << endl;

//     f = 0.0;      
//     e = 0.0;      
//     if(!Controller->mycontext()) R[0][2] = r0[2]+dx;
//     sys->updateR(R);
//     pot.evalForce(f,e,stress,0);
//     nnout << dx << " " << f[0] << endl;

    XMLSaveParticle ptclsave(*sys);
    ptclsave.reset(fname);
    ptclsave.report(0);

    return true;
  }
#else
  bool Test::put(xmlNodePtr cur){
    return true;
  }
#endif
//}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 317 $   $Date: 2005-03-01 17:13:41 -0600 (Tue, 01 Mar 2005) $
 * $Id: Test.cpp 317 2005-03-01 23:13:41Z jnkim $ 
 ***************************************************************************/

