//////////////////////////////////////////////////////////////////
// (c) Copyright 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_ARPACKREALSPARSESOLVERS_H
#define OHMMS_ARPACKREALSPARSESOLVERS_H

#ifndef HAVE_CONFIG_H
#include "ohmms-config.h"
#endif
#include "OhmmsNumeric/SparseEigenSolver.h"
#if defined(USE_PARPACK)
#include "EigSolver/ParpackSymSolver.h"
#else
#if defined(ENABLE_OPENMP)
#include "EigSolver/ArpackSymSolverOMP.h"
#else
#include "EigSolver/ArpackSymSolver.h"
#endif
#endif

namespace OHMMS {

#if defined(USE_PARPACK) 
  typedef ParpackSymSolver<OHMMS_PRECISION,RealMatrix_t>  ArpackSolver_t;
#else
#if defined(ENABLE_OPENMP)
  typedef ArpackSymSolverOMP<OHMMS_PRECISION,RealMatrix_t>  ArpackSolver_t;
#else
  typedef ArpackSymSolver<OHMMS_PRECISION,RealMatrix_t> ArpackSolver_t;
#endif
#endif

  /**class to solve eigen problems of an input sparse matrix using ArpackSymSolver
   */
  struct ArpackSparseSolver: public SymEigenSolver, public ArpackSolver_t
  {

    typedef OHMMS_PRECISION real_type;

    ArpackSparseSolver(EigenSpaceDimension& space);
    ~ArpackSparseSolver();

    void getEigvectors(SparseMatrix_t& hmat, RealMatrix_t& E);
    void getEigvalues(SparseMatrix_t& hmat, RealMatrix_t& E);
    void getEigvectors(SparseMatrix_t& hmat, SparseMatrix_t& smat, RealMatrix_t& E);
    void getEigvalues(SparseMatrix_t& hmat, SparseMatrix_t& smat,RealMatrix_t& E);

    bool put(xmlNodePtr);

    ///reset the cummulative offset
    void resetOffset() { accumShift = 0.0;}

    bool ArpackCheck;
    real_type accumShift;

  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 180 $   $Date: 2004-11-12 13:34:26 -0600 (Fri, 12 Nov 2004) $
 * $Id: ArpackSparseSolvers.h 180 2004-11-12 19:34:26Z jnkim $
 ***************************************************************************/
