//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_EIGENSOLVER_H
#define OHMMS_EIGENSOLVER_H

#ifndef OHMMSNUMERIC_BLAS_H
#include "OhmmsNumeric/OhmmsBlasf.h"
#endif

#ifndef OHMMS_NUMERIC_MATRIX_H
#include "OhmmsNumeric/Matrix.h"
#endif

#if defined(USE_NAG) /* using NAG library */
#include "EigSolver/NagSymSolver.h"

/*! define eigensolvers chosen at compiler times
 * SymSolver_t : a symmetric positive-definite eigen solver
 * HerSolver_t : a hermition eigen solver
 */

namespace OHMMS {

  typedef NagSymSolver<Scalar_t, RealMatrix_t,RealMatrix_t> SymSolver_t;
  typedef NagSymSolver<complex<Scalar_t>, ComplexMatrix_t,ComplexMatrix_t> HerSolver_t;

}
#elif defined(USE_ESSL) /* using IBM ESSL */

#include "EigSolver/EsslSymSolver.h"

namespace OHMMS {

//  typedef EsslSymSolver<Scalar_t, PackedSymMatrix_t,RealMatrix_t> SymSolver_t;
//  typedef EsslSymSolver<complex<Scalar_t>, PackedHerMatrix_t,ComplexMatrix_t> HerSolver_t;
   typedef EsslSymSolver<Scalar_t, RealMatrix_t,RealMatrix_t> SymSolver_t;
   typedef EsslSymSolver<complex<Scalar_t>, ComplexMatrix_t,ComplexMatrix_t> HerSolver_t;

}
#else /* using Lapack */

#include "EigSolver/LapackSymSolverPacked.h"

namespace OHMMS {

  typedef LapackPackedSymSolver<Scalar_t,PackedSymMatrix_t,RealMatrix_t> 
  SymSolver_t;
  typedef 
  LapackPackedSymSolver<complex<Scalar_t>, PackedHerMatrix_t,ComplexMatrix_t> 
  HerSolver_t;
}
#endif
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: EigSolvers.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
