//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2003 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/***************************************************************************
 *
 ***************************************************************************/
#ifndef OHMMS_SPARSESOLVER_H
#define OHMMS_SPARSESOLVER_H
#ifndef OHMMSNUMERIC_BLAS_H
#include "OhmmsNumeric/OhmmsBlasf.h"
#endif

#ifndef OHMMS_NUMERIC_MATRIX_H
#include "OhmmsNumeric/Matrix.h"
#endif
//#include "SlaterKosterTB/SKTypes.h"
#include "SparseMatrix/SparseMatrix2MatrixCopy.h"
#include "SparseMatrix/Matrix2SparseMatrixProduct.h"
#include "EigSolver/LapackSymSolverPacked.h"

namespace OHMMS {

  struct RealEigenSolver {

    EigenSpaceDimension States;
    RealMatrix_t Psi;

    RealEigenSolver(int n, int m): States(n,m) { }
    virtual void getEigvalues(const SparseMatrix_t& hmat, RealMatrix_t& E) = 0;
    virtual void getEigvalues(const SparseMatrix_t& hmat, const SparseMatrix_t& smat, 
			      RealMatrix_t& E) = 0;
    virtual void getEigvectors(const SparseMatrix_t& hmat, RealMatrix_t& E) = 0;
    virtual void getEigvectors(const SparseMatrix_t& hmat, const SparseMatrix_t& smat, 
			       RealMatrix_t& E) = 0;

    virtual void product(int i, const vector<Scalar_t>& w, SparseMatrix_t& rho) {
      OHMMS::product(Psi,Psi,w,rho); // only need Rho
      //thisSolver->product(0,w,RhoS);
    }

  };
  
  struct LapackPackedSparseSolver: public RealEigenSolver {

    typedef LapackPackedSymSolver<Scalar_t,PackedSymMatrix_t,RealMatrix_t> 
    SymSolver_t;
    SymSolver_t *thisSolver;

    LapackPackedSparseSolver(int n, int m): RealEigenSolver(n,m), thisSolver(NULL) 
    {  }


    void getEigvectors(const SparseMatrix_t& hmat, RealMatrix_t& E) {
      if(!thisSolver) {
        thisSolver = new SymSolver_t(States,false);
      }
      thisSolver->getEigvectors(hmat,Psi);
      for(int is=0; is<E.cols(); is++)  E[0][is] = thisSolver->eigval(is);
    }

    void getEigvalues(const SparseMatrix_t& hmat, RealMatrix_t& E) {
      if(!thisSolver) {
        thisSolver = new SymSolver_t(States,false);
      }
      thisSolver->getEigvalues(hmat);
      for(int is=0; is<E.cols(); is++)  E[0][is] = thisSolver->eigval(is);
    }

    void getEigvectors(const SparseMatrix_t& hmat, const SparseMatrix_t& smat, 
		       RealMatrix_t& E) {
    }


    void getEigvalues(const SparseMatrix_t& hmat, const SparseMatrix_t& smat, 
		      RealMatrix_t& E) {

    }

  };

}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: LapackPackedSparseSolver.h 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
