//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2003 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/***************************************************************************
 *
 ***************************************************************************/
#include "OhmmsNumeric/LapackSymSparseSolvers.h"
#include "SparseMatrix/SparseMatrix2MatrixCopy.h"
using namespace OHMMS;

LapackSymA::~LapackSymA() {
  if(thisSolver) delete thisSolver;
}

void 
LapackSymA::getEigvectors(SparseMatrix_t& hmat, RealMatrix_t& E) {

  if(!thisSolver) {
    thisSolver = new SymSolver_t(States,false);
    H.resize(States.N,States.N);
  }
  if(!Psi.size()) Psi.resize(States.M, States.N);
  Sparse2DenseMatrix::copy(hmat,H); 
  thisSolver->Jobz = 'V';
  thisSolver->solve(H.data(), Psi.data()); 
  for(int is=0; is<E.cols(); is++)  E[0][is] = thisSolver->eigval(is);
}

void 
LapackSymA::getEigvalues(SparseMatrix_t& hmat, RealMatrix_t& E) {
  if(!thisSolver) {
    thisSolver = new SymSolver_t(States,false);
    H.resize(States.N,States.N);
  }
  thisSolver->Jobz = 'N';
  Sparse2DenseMatrix::copy(hmat,H); 
  thisSolver->solve(H.data(), H.data());
  for(int is=0; is<E.cols(); is++)  E[0][is] = thisSolver->eigval(is);
}

void 
LapackSymA::getEigvectors(SparseMatrix_t& hmat, 
			  SparseMatrix_t& smat, 
			  RealMatrix_t& E) {
  if(!thisSolver) {
    thisSolver = new SymSolver_t(States,false);
    H.resize(States.N,States.N);
    S.resize(States.N,States.N);
  }
  if(!Psi.size()) Psi.resize(States.M, States.N);
  thisSolver->Jobz = 'V';
  Sparse2DenseMatrix::copy(hmat,H); 
  Sparse2DenseMatrix::copy(smat,S); 
  thisSolver->solve(H.data(), S.data(), Psi.data());
  for(int is=0; is<E.cols(); is++)  E[0][is] = thisSolver->eigval(is);
}

void 
LapackSymA::getEigvalues(SparseMatrix_t& hmat, 
			 SparseMatrix_t& smat, 
			 RealMatrix_t& E) {
  if(!thisSolver) {
    thisSolver = new SymSolver_t(States,false);
    H.resize(States.N,States.N);
    S.resize(States.N,States.N);
  }
  thisSolver->Jobz = 'N';
  Sparse2DenseMatrix::copy(hmat,H); 
  Sparse2DenseMatrix::copy(smat,S); 
  thisSolver->solve(H.data(), S.data(), NULL);
  for(int is=0; is<E.cols(); is++)  E[0][is] = thisSolver->eigval(is);
}


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: LapackSymSparseSolvers.cpp 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
