//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_NUMERIC_MATRIX_H
#define OHMMS_NUMERIC_MATRIX_H

#ifndef HAVE_CONFIG_H
#include "ohmms-config.h"
#endif

#ifdef OHMMS_UBLAS
#include <boost/numeric/ublas/config.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/triangular.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/numeric/ublas/traits.hpp>

namespace OHMMS {

  typedef boost::numeric::ublas::matrix<OHMMS_PRECISION>           RealMatrix_t;
  typedef boost::numeric::ublas::matrix<complex<OHMMS_PRECISION> > ComplexMatrix_t;

  typedef boost::numeric::ublas::upper                             upper;
  typedef boost::numeric::ublas::triangular_matrix<OHMMS_PRECISION,upper>   
  PackedSymMatrix_t;
  typedef boost::numeric::ublas::triangular_matrix<complex<OHMMS_PRECISION>,upper> 
  PackedHerMatrix_t;
}
#elif defined(OHMMS_MTL)
#include <mtl/matrix.h>
#include <mtl/mtl.h>
#include <mtl/utils.h>


namespace OHMMS {
  template <class T> struct type_traits { };

  template <>
  struct type_traits<double> {
    typedef double value_type;
    typedef double real_type;
  };

  template <>
  struct type_traits<std::complex<double> > {
    typedef std::complex<double> value_type;
    typedef double real_type;
  };

  using namespace mtl;	 

  typedef mtl::matrix<OHMMS_PRECISION>::type            RealMatrix_t;
  typedef mtl::matrix<complex<OHMMS_PRECISION> >::type  ComplexMatrix_t;

  typedef mtl::matrix<OHMMS_PRECISION, 
                      mtl::triangle<mtl::upper>, 
		      mtl::packed<>, 
                      mtl::row_major>::type PackedSymMatrix_t;
  typedef mtl::matrix<complex<OHMMS_PRECISION>, 
                      mtl::triangle<mtl::upper>, 
		      mtl::packed<>, 
                      mtl::row_major>::type PackedHerMatrix_t;
}
#elif defined(OHMMS_BLITZ)
#include <blitz/array.h>
namespace OHMMS {

  typedef blitz::Array<OHMMS_PRECISION,2>              RealMatrix_t;
  typedef blitz::Array<complex<OHMMS_PRECISION>,2 >    ComplexMatrix_t;
  typedef blitz::Array<OHMMS_PRECISION,1>              PackedSymMatrix_t;
  typedef blitz::Array<complex<OHMMS_PRECISION>,1>     PackedHerMatrix_t;

}
#else

#include "OhmmsPETE/OhmmsMatrix.h"
#include "OhmmsPETE/OhmmsSymmetricMatrix.h"
#include <complex>
namespace OHMMS {

  template <class T> struct type_traits { };

  template <>
  struct type_traits<double> {
    typedef double value_type;
    typedef double real_type;
  };

  template <>
  struct type_traits<std::complex<double> > {
    typedef std::complex<double> value_type;
    typedef double real_type;
  };

  typedef Matrix<OHMMS_PRECISION>                RealMatrix_t;
  typedef Matrix<std::complex<OHMMS_PRECISION> > ComplexMatrix_t;

  typedef SymmetricMatrix<OHMMS_PRECISION>           PackedSymMatrix_t;
  typedef SymmetricMatrix<complex<OHMMS_PRECISION> > PackedHerMatrix_t;
}
#endif
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: Matrix.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

