//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//
//////////////////////////////////////////////////////////////////
// -*- c++ -*-
#ifndef OHMMS_BLOCK_BLAS_THREADSAFE
#define OHMMS_BLOCK_BLAS_THREADSAFE

// threadsafe blas equivalents. When using Linux/Intel-mkl and KAPPRO,
// blas can be threaded and there isn't an easy way to tell a compiler
// to use a single-node blas due to the name conflicts

//! Xaxpy
template<class T>
inline 
void ohmms_axpy_mp(int nb, T scal,const T* restrict aptr, T* restrict bptr) {

  for(int ib=0; ib<nb; ib++, aptr++, bptr++) (*bptr) += scal*(*aptr);
  
}

//! Xdot
template<class T>
inline 
T ohmms_dot_mp(int nb, const T* restrict aptr, const T* restrict bptr) {
  T res = 0.0; 
  for(int ib=0; ib<nb; ib++, aptr++, bptr++) res += (*bptr)*(*aptr);
  return res;  
}

//! Xcopy
template<class T>
inline 
void ohmms_copy_mp(int nb, const T* restrict aptr, T* restrict bptr) {
  for(int ib=0; ib<nb; ib++, aptr++, bptr++) (*bptr) = (*aptr);
}

//! Xscal
template<class T>
inline
void ohmms_scal_mp(int nb, T x, T* restrict bptr) {
  for(int ib=0; ib<nb; ib++,bptr++) (*bptr) *= x;
}

//! Xassign ???
template<class T>
inline
void ohmms_assign_mp(int nb, T x, T* restrict bptr) {
  for(int ib=0; ib<nb; ib++,bptr++) (*bptr) = x;
}

//! extra for block matrix
template<class T>
inline
void ohmms_adddiag_mp(int nb, T x, T* restrict bptr) {
  for(int ib=0; ib<nb; ib++) bptr[ib+ib*nb] += x;
}


#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: OhmmsBlasOMP.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
