//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2003 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/***************************************************************************
 *
 ***************************************************************************/
#ifndef OHMMS_SPARSESOLVER_H
#define OHMMS_SPARSESOLVER_H
#include "OhmmsData/OhmmsElementBase.h"
#ifndef OHMMS_NUMERIC_MATRIX_H
#include "OhmmsNumeric/Matrix.h"
#endif
#ifndef OHMMS_SPARSEMATRIX_H
#include "SparseMatrix/SparseMatrix.h"
#endif
#ifndef OHMMS_EIGENSPACEDIMENSION_H
#include "EigSolver/EigenSpaceDimension.h"
#endif
#include "SparseMatrix/Matrix2SparseMatrixProduct.h"

namespace OHMMS {

  struct SymEigenSolver {

    typedef type_traits<RealMatrix_t::Type_t>::real_type real_type;

    EigenSpaceDimension& States;
    RealMatrix_t Psi;

    SymEigenSolver(EigenSpaceDimension& space): States(space) { }
    virtual ~SymEigenSolver() { }

    virtual void 
    getEigvalues(SparseMatrix_t& hmat, RealMatrix_t& E) = 0;

    virtual void 
    getEigvectors(SparseMatrix_t& hmat, RealMatrix_t& E) = 0;
    
    virtual void 
    getEigvalues(SparseMatrix_t& hmat, SparseMatrix_t& smat, RealMatrix_t& E) = 0;
    virtual void 
    getEigvectors(SparseMatrix_t& hmat, SparseMatrix_t& smat, RealMatrix_t& E) = 0;

    virtual void product(const vector<real_type>& w, SparseMatrix_t& rho) {
      Dense2SparseMatrix::product(Psi,Psi,w,rho); // only need Rho
    }

  };


  struct HerEigenSolver {

    typedef type_traits<ComplexMatrix_t::Type_t>::value_type value_type;
    typedef type_traits<ComplexMatrix_t::Type_t>::real_type  real_type;
    EigenSpaceDimension& States;
    vector<ComplexMatrix_t*> Psi;

    HerEigenSolver(EigenSpaceDimension& space): States(space) { }
    virtual ~HerEigenSolver() { }

    inline const ComplexMatrix_t& psi(int i) const { return *Psi[i];}

    virtual void 
    getEigvalues(const SparseMatrix_t& hmat, const ComplexMatrix_t& phi, 
		 RealMatrix_t& E, int first, int last) = 0;
    virtual void 
    getEigvalues(const SparseMatrix_t& hmat, const SparseMatrix_t& smat,
		 const ComplexMatrix_t& phi, RealMatrix_t& E, int first, int last) = 0;
    virtual void 
    getEigvectors(const SparseMatrix_t& hmat, const ComplexMatrix_t& phi,
		  RealMatrix_t& E, int first, int last) = 0;
    virtual void 
    getEigvectors(const SparseMatrix_t& hmat, const SparseMatrix_t& smat, 
		  const ComplexMatrix_t& phi,
		  RealMatrix_t& E, int first, int last) = 0;

    virtual void 
    product(int i, int j, const vector<real_type>& w, 
	    SparseMatrix_t& rhor, SparseMatrix_t& rhoi) {
      Dense2SparseMatrix::product(*Psi[i],*Psi[j],w,rhor,rhoi); // only need Rho
    }
  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseEigenSolver.h 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
