//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifdef HAVE_CONFIG_H
#include "ohmms-config.h"
#endif
#include "Utilities/OhmmsInfo.h"
#include "Message/Communicate.h"
#include "OhmmsElementSet.h"
#include "ProjectData.h"
#include "OhmmsQt.h"

#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qsimplerichtext.h>
#include <qsplitter.h>
#include <qlistview.h>

#include "filesave.xpm"
#include "fileopen.xpm"
#include "fileprint.xpm"

namespace OHMMS {

  /*! \brief Constructor 
   * Register OhmmsElementBase objects corresponding to the major tags of input xml files.
   * OhmmsElementSet contains default objects \ref ohmmsdefaults and the constructor
   * adds OhmmsElementBase subclasses that depend on the default objects.
   *
   */
  OhmmsQt::OhmmsQt() 
    : QMainWindow( 0, "ohmms application main window", WDestructiveClose | WGroupLeader ),
      m_doc(NULL), m_context(NULL) 
  {
    QPixmap openIcon, saveIcon, printIcon;

    QToolBar * fileTools = new QToolBar( this, "file operations" );
    fileTools->setLabel( "File Operations" );

    openIcon = QPixmap( fileopen );
    QToolButton * fileOpen
	= new QToolButton( openIcon, "Open File", QString::null,
			   this, SLOT(choose()), fileTools, "open file" );

    saveIcon = QPixmap( filesave );
    QToolButton * fileSave
	= new QToolButton( saveIcon, "Save File", QString::null,
			   this, SLOT(save()), fileTools, "save file" );


    (void)QWhatsThis::whatsThisButton( fileTools );

    const char * fileOpenText = "<p><img source=\"fileopen\"> "
	         "Click this button to open a <em>new file</em>.<br>"
                 "You can also select the <b>Open</b> command "
                 "from the <b>File</b> menu.</p>";

    QWhatsThis::add( fileOpen, fileOpenText );

    QMimeSourceFactory::defaultFactory()->setPixmap( "fileopen", openIcon );

    const char * fileSaveText = "<p>Click this button to save the file you "
                 "are editing. You will be prompted for a file name.\n"
                 "You can also select the <b>Save</b> command "
                 "from the <b>File</b> menu.</p>";

    QWhatsThis::add( fileSave, fileSaveText );

    const char * filePrintText = "Click this button to print the file you "
                 "are editing.\n"
		 "You can also select the Print command "
		 "from the File menu.";

    QPopupMenu * file = new QPopupMenu( this );
    menuBar()->insertItem( "&File", file );


    int id;
    id = file->insertItem( openIcon, "&Open...",
			   this, SLOT(choose()), CTRL+Key_O );
    file->setWhatsThis( id, fileOpenText );

    id = file->insertItem( saveIcon, "&Save",
			   this, SLOT(save()), CTRL+Key_S );
    file->setWhatsThis( id, fileSaveText );

    id = file->insertItem( "Save &As...", this, SLOT(saveAs()) );
    file->setWhatsThis( id, fileSaveText );

    file->insertSeparator();

    file->insertItem( "&Close", this, SLOT(close()), CTRL+Key_W );

    file->insertItem( "&Quit", qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );

    menuBar()->insertSeparator();

    QPopupMenu * help = new QPopupMenu( this );
    menuBar()->insertItem( "&Help", help );

    help->insertItem( "&About", this, SLOT(about()), Key_F1 );
    help->insertSeparator();
    help->insertItem( "What's &This", this, SLOT(whatsThis()), SHIFT+Key_F1 );

    QSplitter *d_view = new QSplitter( Qt::Vertical, this);
    setCentralWidget(d_view);

    NodeList = new OhmmsElementSet;
    nodeView = new QListView(d_view,"whatever");
    nodeView->addColumn("Node Name");
    nodeView->addColumn("Value");
    nodeView->addColumn("Unit");
    nodeView->setSorting(-1);
    nodeView->setRootIsDecorated(TRUE);

    e = new QTextEdit( d_view, "editor" );
    e->setFocus();
    statusBar()->message( "Ready", 2000 );

    resize( 450, 600 );
  }

  OhmmsQt::~OhmmsQt() {
     delete NodeList;
  }

  void OhmmsQt::closeEvent( QCloseEvent* ce )
  {
    if ( !e->isModified() ) {
      ce->accept();
      return;
    }
    
    switch( QMessageBox::information( this, "Ohmms Session",
				      "Do you want to save the changes"
				      " to the document?",
				      "Yes", "No", "Cancel",
				      0, 1 ) ) {
    case 0:
      save();
      ce->accept();
      break;
    case 1:
      ce->accept();
      break;
    case 2:
    default: // just for sanity
      ce->ignore();
      break;
    }
  }

  void OhmmsQt::choose()
  {
    QString fn = QFileDialog::getOpenFileName( QString::null, QString::null,
					       this);
    if ( !fn.isEmpty() )
      load( fn );
    else
      statusBar()->message( "Loading aborted", 2000 );
  }

  void OhmmsQt::load( const QString &fileName )
  {
    parse(fileName.ascii());    
    setCaption( fileName );
    statusBar()->message( "Loaded document " + fileName, 2000 );
  }

  /*!\fn bool OhmmsQt::parse(const char* fname) {
   *\param fname an input xml file to be parsed.
   *\return true if successful.
   *\page parsexml
   * The main routine to process an xml file.
   * Valid input files should contain
   * <Main>
   * <Jobs>
   *    ................
   * </Jobs>
   * </Main>
   *
   * Each <Jobs/> is a self-contained application to perform a simulation.
   * The XML elements registered by OhmmsElementSet and OhmmsQt 
   * by OhmmsElementSet::addNode function are recognized and other unknown
   * elements are ignored.
   */
  bool OhmmsQt::parse(const char* fname) {
    
    d_filename = fname; //QString("%1.%2").arg(fname).arg(1);

    LOGMSG("Going to parse " << fname)
    xmlNsPtr ns;
    xmlNodePtr cur;
    
    // build an XML tree from a the file;
    m_doc = xmlParseFile(fname);
    if (m_doc == NULL) return false;
    
    // Check the document is of the right kind
    cur = xmlDocGetRootElement(m_doc);
    if (cur == NULL) {
      fprintf(stderr,"empty document\n");
      xmlFreeDoc(m_doc);
      return false;
    }

    ns = NULL;

    if (xmlStrcmp(cur->name, (const xmlChar *) "Main")) {
      fprintf(stderr,"document of the wrong type, root node != Main");
      xmlFreeDoc(m_doc);
      return false;
    }
    
    // First level we expect just Jobs
    cur = cur->xmlChildrenNode;
    while ( cur && xmlIsBlankNode ( cur ) )
      {
	cur = cur -> next;
      }
    if ( cur == 0 )
      return false;
    
    
    ///using XPath instead of recursive search
    xmlXPathObjectPtr result;
    m_context = xmlXPathNewContext(m_doc);

    result = xmlXPathEvalExpression((const xmlChar*)"//Jobs",m_context);
    
    if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {
      
      xmlNodeSetPtr jobs = result->nodesetval;
      
      if(jobs->nodeNr > 1) {
	LOGMSG("Launching a multiple jobs.")
        if(jobs->nodeNr < Controller->ncontexts()) {
	  WARNMSG("MPI processors < the number of Jobs. Perform sequential simulations")
	  for(int ijob=0; ijob<jobs->nodeNr; ijob++) {
	    NodeList->put(jobs->nodeTab[ijob]);
	  }
	} else {
	  cur = jobs->nodeTab[Controller->mycontext()];
	  xmlDocPtr doc1 = xmlParseFile((const char*)(xmlGetProp(cur, (const xmlChar *) "file")));
	  if (doc1 == NULL) return false;
	  xmlXPathContextPtr context1;
	  xmlXPathObjectPtr result1;
	  context1 = xmlXPathNewContext(doc1);
	  result1 = xmlXPathEvalExpression((const xmlChar*)"//Jobs",context1);
	  if(!xmlXPathNodeSetIsEmpty(result1->nodesetval)) {
	    NodeList->put(result1->nodesetval->nodeTab[0]);
	  }
	  xmlXPathFreeObject(result1);
	}
      } else {

	QListViewItem *a=new QListViewItem(nodeView,"Jobs");
	NodeList->buildTree(a,jobs->nodeTab[0]);
	//NodeList->buildTree(NULL,jobs->nodeTab[0]);
	//NodeList->put(jobs->nodeTab[0]);
      }
    } else {
      fprintf(stderr,"document of the wrong type, was '%s', Jobs expected",
	      cur->name);
      fprintf(stderr,"xmlDocDump follows\n");
      xmlDocDump ( stderr, m_doc );
      fprintf(stderr,"xmlDocDump finished\n");
      xmlFreeDoc(m_doc);
      return false;
    }
    
    xmlXPathFreeObject(result);

    nodeView->show();

    //savedump();
    //LOGMSG("Completed the jobs. Bye!")
    return true;
  }

  void OhmmsQt::save() {
     
    if(Controller->master()) {
//       ProjectData* project =  dynamic_cast<ProjectData*>(NodeList->getNode(Key_project));
//       (*NodeList)[Key_particle]->reset();
//       string newxml(project->CurrentRoot());
//       newxml.append(".cont.xml");
//       xmlSaveFormatFile(newxml.c_str(),m_doc,1);
      xmlSaveFormatFile(d_filename.ascii(),m_doc,1);

    }
  }

  void OhmmsQt::saveAs()
  {
    QString fn = QFileDialog::getSaveFileName( QString::null, QString::null,
					       this );
    if ( !fn.isEmpty() ) {
      d_filename = fn;
      save();
    } else {
      statusBar()->message( "Saving aborted", 2000 );
    }
  }

  void OhmmsQt::about()
  {
    QMessageBox::about( this, "Ohmms Session",
			"This example demonstrates simple use of "
			"QMainWindow,\nQMenuBar and QToolBar.");
  }
  

}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: OhmmsQt.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
