//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "ParticleLayoutManager.h"
#include "Particle/NNEngine.h"

/**@ingroup ohmmsxml
 * @ XML node with tag = ParticleLayout

   usage: 
   <ParticleLayout
    method="grid|N2|metis|optimize"
    scale="double"
    mpi="on|off"
    omp="on|off"/>

   Default: <ParticleLayout method="grid" scale="1" mpi="off" omp="off"/>

   Action: intialize/reset the nearest neighbor engine and the method
   The action is valid only after <ParticleSets/> is processed.
 */
namespace OHMMS {

  ParticleLayoutManager::ParticleLayoutManager(ParticleSets& P,
					       const char* aname): 
    OhmmsElementBase(aname), PtclSetRef(P)  
  { }

  void ParticleLayoutManager::reset() { }  

  bool ParticleLayoutManager::put(xmlNodePtr cur) {

    Particle_t* ptcl =  PtclSetRef.get();
    if(ptcl) {
      // initialize the NNEngine without specifying the cutoff radius
      //ptcl->getPairlist(-1);
      //return ptcl->getNNEngine()->put(cur);
      NNEngine* eng = NNEngineBuilder::create(cur);
      ptcl->setNNEngine(eng);
      return true;
    } else {
      ERRORMSG("No ParticleSet exists")
      return false;
    }
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: ParticleLayoutManager.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
