//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_PARTICLESETS_H__
#define OHMMS_PARTICLESETS_H__

#include "OhmmsData/OhmmsElementBase.h"
#include "OhmmsData/RecordProperty.h"
#include "Particle/OhmmsParticle.h"

namespace OHMMS {

  /** \class ParticleSets
   *  \brief Similar to OhmmsXMLUI/MDParticle
   */
  class ParticleSets: public OhmmsElementBase {

  public:

    typedef vector<Particle_t*> ParticleSet_t;    

    /// constructors and destructors
    ParticleSets(const char* aname="particlesets");
    
    bool get(ostream& os) const { return true;}
    bool put(istream& is) {return true;}
    bool put(xmlNodePtr cur);
    void reset();

    ///add a new particle set
    void addParticleSet(Particle_t* apt);

    ///returns PtclCollection
    Particle_t* get();

    /*!\fn  void set(Particle_t* anew)
     * \param anew an external Particle_t* is assigned to PtclCollection
     */
    void set(Particle_t* anew) { 
      if(PtclCollection) delete PtclCollection;
      PtclCollection = anew;
    }


    RecordProperty* createRecordProperty(xmlNodePtr cur);

  private:

    ///set of input particle sets
    ParticleSet_t PtclSet;

    ///main particle set
    Particle_t* PtclCollection;

    /// a reference unitcell
    ParticleLayout_t Lattice;

    xmlNodePtr m_cur;

    string RestartConfig;

  };
}

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: ParticleSets.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
