//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_TRANSFORMPPARTICLE_H
#define OHMMS_TRANSFORMPPARTICLE_H

#include "OhmmsData/OhmmsElementBase.h"
#include "Particle/OhmmsParticle.h"

namespace OHMMS {

  /*!\class TransformParticleSet
   *\brief abstract class to perform transformation on the particle sets
   */
  class TransformParticleSet {

  protected:

    Particle_t& in_;
    Particle_t& out_;

  public:

    TransformParticleSet(Particle_t& pin, Particle_t& pout): in_(pin), out_(pout) { }

    ///read from DOM
    virtual bool put(xmlNodePtr cur)=0;
  };


  /*! \class ExpandParticle
   */
  class ExpandParticle: public TransformParticleSet {

  public:

    ExpandParticle(Particle_t& pin, Particle_t& pout): TransformParticleSet(pin,pout) 
    {}

    bool put(xmlNodePtr cur);
  };

  /*! \class TranslateParticle
   */
  class TranslateParticle: public TransformParticleSet {

  public:

    TranslateParticle(Particle_t& pin, Particle_t& pout): TransformParticleSet(pin,pout) 
    {}

    bool put(xmlNodePtr cur);

  };

  /*! \class ShapeParticle
   */
  class ShapeParticle: public TransformParticleSet {

  public:

    ShapeParticle(Particle_t& pin, Particle_t& pout): TransformParticleSet(pin,pout) 
    {}

    bool put(xmlNodePtr cur);

  };

}

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: TransformParticle.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
