//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
#include <math.h>

#include <iostream>
#include <complex>
#include <vector>
using namespace std;

void 
laguer(vector<complex<double> >& a,int m,
       complex<double>& x,double eps,bool polish)
{
        const double EPSS = 6.e-8;
        const int MAXIT = 100;
	int j,iter;
	double err,dxold,cdx,abx;
        typedef complex<double> dcomplex;
	dcomplex sq,h,gp,gm,g2,g,b,d,dx,f,x1;

        double m_d = static_cast<double>(m);
        double m_1_d = static_cast<double>(m-1);
	dxold=abs(x);
	for (iter=1;iter<=MAXIT;iter++) {
		b=a[m];
		err=abs(b);
		d=f=dcomplex(0.0,0.0);
		abx=abs(x);
		for (j=m-1;j>=0;j--) {
			f= x*f + d;    // Cadd(Cmul(*x,f),d)
			d= x*d + b;    // Cadd(Cmul(*x,d),b)
			b= x*b + a[j]; // Cadd(Cmul(*x,b),a[j])
			err=abs(b) + abx*err; //Cabs(b)+abx*err
		}
		err *= EPSS;
		if (abs(b) <= err) return;
		g=d/b;   // Cdiv(d,b)
		g2=g*g;  // Cmul(g,g)
		h= g2 - 2.0*(f/b); //Csub(g2,RCmul(2.0,Cdiv(f,b)));
		sq=sqrt(m_1_d*(m_d*h - g2));
    //     Csqrt(RCmul((float) (m-1),Csub(RCmul((float) m,h),g2)));
		gp=g + sq;                 // Cadd(g,sq);
		gm=g - sq;                 // Csub(g,sq);
		if (abs(gp) < abs(gm)) gp=gm;
		dx=dcomplex(m_d,0.0e0)/gp; // Cdiv(Complex((float) m,0.0),gp);
		x1=x - dx;                 // Csub(*x,dx);
		if (x.real() == x1.real() && x.imag() == x1.imag()) return;
	        x=x1;
		cdx=abs(dx);
		dxold=cdx;
		if (!polish)
		  if (cdx <= eps*abs(x)) return;
	}
	
}

void 
zroots(vector<complex<double> >& a,int m,
       vector<complex<double> >& roots,bool polish)
{
        const double EPS = 1.0e-6;
	int jj,j,i;
        typedef complex<double> dcomplex;
	dcomplex x,b,c;
        vector<dcomplex> ad(2*m);
//	void laguer();

	for (j=0;j<=m;j++) ad[j]=a[j];
	for (j=m;j>=1;j--) {
		x=dcomplex(0.0,0.0);
		laguer(ad,j,x,EPS,0);
		if (fabs(x.imag()) <= (2.0*EPS*abs(x.real()))) 
                   x = dcomplex(x.real(), 0.0e0);
		roots[j]=x;
		b=ad[j];
		for (jj=j-1;jj>=0;jj--) {
			c=ad[jj];
			ad[jj]=b;
			b=x*b + c; // Cadd(Cmul(x,b),c);
		}
	}
	if (polish)
		for (j=1;j<=m;j++)
			laguer(a,m,roots[j],EPS,1);
	for (j=2;j<=m;j++) {
		x=roots[j];
		for (i=j-1;i>=1;i--) {
			if (roots[i].real() <= x.real()) break;
			roots[i+1]=roots[i];
		}
		roots[i+1]=x;
	}
}

double getOptima(const vector<double>& e, bool getMinimum) {
  const double small = 1.0e-6;
  int m = e.size() - 1;  // order of polynomial [0....m]
  
  typedef complex<double> dcomplex;
  vector<dcomplex> a(m+1);
  vector<dcomplex> roots(m+1);

  a[0] = dcomplex(0.0e0, 0.0e0);
  for(int i=1; i <= m; i++) {
    a[i] = dcomplex(static_cast<double>(i)*e[i], 0.0e0);
  }

  double amaxinv = 1.0e0/a[m].real();


  for(int i=1; i<= m; i++) {
    a[i] *= amaxinv;
  }
  

  zroots(a, m, roots, true);

  cout << endl;
  if(getMinimum) {

    double xmin = 1.0e10;  
    double z0 = 0.0e0; 
    for(int i=0; i <= m; i++) {
      if(fabs(roots[i].imag()) < small) {
        double x = 0.0e0;
        double z = roots[i].real();

// will be substituted by more efficient routines
//        for(int j=1; j<= m; j++) x +=pow(z,j)*e[j];
        double zp = 1;
        for(int j=1; j<= m; j++) { zp *= z; x += zp*e[j];} 
        if(x < xmin) { xmin = x; z0 = z;}
     }
    }

    return z0;
  } else {

    double xmax = -1.e10;
    double z0 = 0.0e0;

    for(int i=0; i <= m; i++) {
      if(abs(roots[i].imag()) < small) {
        double x = 0.0e0;
        double z = roots[i].real();

// will be substituted by more efficient routines
//       for(int j=1; j<= m; j++) x += pow(z,j)*e[j];
        double zp = 1;
        for(int j=1; j<= m; j++) { zp *= z; x += zp*e[j];} 
        if(x > xmax) { xmax = x; z0 = z;}
      }
    }
    return z0;
  }
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: laguer.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
