//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Message/Communicate.h"
#include "Message/OpenMP.h"
#include "Particle/NNEngine.h"
#include "Particle/NNSimpleEngine.h"
#include "Particle/NNRegGridEngine.h"
#include "Particle/NNSpatialOMPEngine.h"
#ifdef HAVE_MPI
#include "Particle/NNSpatialEngine.h"
#endif
#include "Utilities/OhmmsInfo.h"
namespace OHMMS {

  int NNEngineBuilder::Counter=0;

  NNEngine* 
  NNEngineBuilder::create(xmlNodePtr cur) {

    ///default engine = NNRegGridEngine: safe
    int nneng = NNRegGridEngine::ThisMethod;
    NNEngine* eng = 0;

    string what("grid");
    if(cur) {
      const xmlChar* am = xmlGetProp(cur,(const xmlChar *)"method");
      if(am) what = (const char*)am;
    }

    //std::map<std::string,int> nnmap;
    //nnmap["grid"] = 0;
    //nnmap["N2"] = 1;
    //nnmap["metis"] = 2;
    //nnmap["distributed"] = 3;
    //nnmap["ompgrid"] = 4;
    //std::map<std::string,int>::iterator it(nnmap.find(what));
    //if(it != nnmap.end()) { nneng = (*it).second; }

    int nthreads = omp_get_max_threads();
    int ncontexts = Controller->ncontexts();
    bool enable_omp=(nthreads>1);
    bool enable_mpi=(ncontexts>1);

    const xmlChar* aptr=xmlGetProp(cur,(const xmlChar*)"omp");
    if(aptr) {
      string vname((const char*)aptr);
      if(vname == "no" || vname == "off" || vname == "false") {
        enable_omp=false;
      }
    }
    aptr=xmlGetProp(cur,(const xmlChar*)"mpi");
    if(aptr) {
      string vname((const char*)aptr);
      if(vname == "no" || vname == "off" || vname == "false") {
        enable_mpi=false;
      }
    }

    if(enable_omp || enable_mpi) {
      if(enable_omp) { //when OpenMP is enabled, only NNSpatialOMPEngine can be used
        eng = new NNSpatialOMPEngine();
      } else {
#if defined(HAVE_MPI)
        if(enable_mpi) {
          eng = new NNSpatialEngine();
        } 
#else
        ERRORMSG("THIS SHOULD NOT BE REACHED.")
        eng = new NNRegGridEngine();
#endif
      }
    } else {//serial runs
      if(what == "N2") {
        eng = new NNSimpleEngine();
      } else {
        eng = new NNRegGridEngine();
      }
    } 

    char aname[16];
    sprintf(aname,"nnegine%02d",Counter);
    eng->setName(aname);
    Counter++;

    eng->Distributed[NNEngine::enable_omp]=enable_omp;
    eng->Distributed[NNEngine::enable_mpi]=enable_mpi;
    if(cur)  eng->put(cur);
    return eng;
  }

  NNEngine* 
  NNEngineBuilder::create(int& argc, char**& argv) {

    ///default engine = NNRegGridEngine: safe
    int nneng = 0;
    NNEngine* eng = NULL;
    TinyVector<int,OHMMS_DIM> Grid[3];
    Grid[0] = TinyVector<int,OHMMS_DIM>(1);
    Grid[1] = TinyVector<int,OHMMS_DIM>(1);
    Grid[2] = TinyVector<int,OHMMS_DIM>(1);
    bool ModifyGrid[]={false,false,false};
    int retargc = 1;
    char **retargv = new char *[argc];
    retargv[0] = argv[0];
    int i=1;
    while(i<argc) {
      string aword(argv[i]);
      if(aword == "--method") {
	string a(argv[++i]);
	if(a == "N2") 
	  nneng = 1;
	else if(a == "metis") 
	  nneng = 2;
	else if(a == "distributed") 
	  nneng = 3;
      } else if(aword == "--mpi") {
	for(int idim=0; idim<OHMMS_DIM; idim++) 
	  Grid[0][idim] = atoi(argv[++i]);
        ModifyGrid[0] = true;
      } else if(aword == "--omp") {
	for(int idim=0; idim<OHMMS_DIM; idim++) 
	  Grid[1][idim] = atoi(argv[++i]);
        ModifyGrid[1] = true;
      } else if(aword == "--grid") {
	for(int idim=0; idim<OHMMS_DIM; idim++) 
	  Grid[2][idim] = atoi(argv[++i]);
        ModifyGrid[2] = true;
      } else {
	retargv[retargc++]  = argv[i];
      }
      ++i;
    }
  
    argc = retargc;
    for (i = 0; i < retargc; ++i) {argv[i] = retargv[i];}
    delete [] retargv;
    
    //Fix to N2
    switch(nneng) {
    case(0):
      LOGMSG("\tCreating NNRegGridEngine for a nearest-neighbor engine")
	eng= new NNRegGridEngine();
      break;
    case(1):
      LOGMSG("\tCreating NNSimpleEngine for a nearest-neighbor engine")
	eng = new NNSimpleEngine();
      break;
    case(2):
      LOGMSG("\tCreating NNMetisEngine for a nearest-neighbor engine")
	eng= new NNRegGridEngine();
	//eng = new NNMetisEngine();
      break;
#ifdef HAVE_MPI
    case(3):
      LOGMSG("\tCreating NNSpatialEngine for a nearest-neighbor engine")
      eng = new NNSpatialEngine();
      break;
#endif
    }
    for(int i=0; i<3; i++) eng->Grid[i] = Grid[i];
    for(int i=0; i<3; i++) eng->ModifyGrid[i] = ModifyGrid[i];
    return eng;
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 361 $   $Date: 2005-07-13 13:31:31 -0500 (Wed, 13 Jul 2005) $
 * $Id: NNEngineBuilder.cpp 361 2005-07-13 18:31:31Z jnkim $ 
 ***************************************************************************/

