//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Message/Communicate.h"
#include "Message/OpenMP.h"
#include "Utilities/OhmmsInfo.h"
#include "Particle/NNSimpleEngine.h"
#include "ParticleBase/ParticleUtility.h"
#include "Particle/PLayoutFunctions.h"
using namespace OHMMS;
#include <bitset>
#include <set>
#include <math.h>

NNSimpleEngine::NNSimpleEngine()
{ 
  setID(ThisMethod);
  UseLayout = false;
  LOGMSG("Creating NNSimpleEngine")
}

NNSimpleEngine::~NNSimpleEngine()
{ 
}

/** initialization of this NNEngine
 *\param p the particle whose nearest-neighbor list is to be evaluated.
 *\param rmax the cutoff radius of the interaction. No safe zone is needed.
 *\return false This engine does not use grid-partition.
 */
bool
NNSimpleEngine::initialize(Particle_t& p, bool needvisitingforce) {

  DEBUGMSG("NNSimpleEngine::initialize")
  Rmax = p.Lattice.InteractionRadius;
  RmaxSq = Rmax*Rmax;

  //UseVisitingForce=needvisitingforce;
  MaxPairsPerPtcl=p.getLocalNum();
  makeFairPartition(p);
  allocateAttributes(p,UseVisitingForce);

  LOGMSG("NNSimpleEngine::UseVisitingForce = false")
  UseVisitingForce=false;

  return false;
}

NNSimpleEngine::Index_t
NNSimpleEngine::updateGridData(const ParticlePos_t& pos, ParticlePos_t& pu,
    ParticleLayout_t& lattice){
  PLayoutFunc::applyBC(lattice,pos,pu);
  return pos.size();
}

void 
NNSimpleEngine::evalPairsPerThread(const ParticleLayout_t& lattice, 
    const ParticlePos_t& pos, const ParticleIndex_t& grID,
    PtclPairList_t& myppl) {

  myppl.reset(); // reset the pair lists

  Index_t LocalNum(myppl.getLocalNum());
  Index_t iat(myppl.First);
  for(Index_t iL=0; iL<LocalNum; iL++,iat++) {
    myppl.I[iL] =iat;
    for(Index_t jat=0; jat < LocalNum; jat++) {
      if(iat == jat) continue;
      /**calculate the minimum-image distance
       *NNSimpleEngine is valid only when the cutoff radius is
       *less than the half of the unit cell size.
       *
       *ParticleBCond<T,D>::apply(const TinyVector<T,D>& dr) 
       *should work fine as far as a compiler can support inlining
       *and constructor/destructor optimization.
       *Unfortunately, the majority of the compilers cannot do it right.
       *Instead, do all the calculations.
       */
#ifdef DISABLE_PETE
      SingleParticlePos_t dr(pos[jat]-pos[iat]);
      if(dr[0]<-0.5)      dr[0]+=1.0;
      else if(dr[0]>=0.5) dr[0]-=1.0;
      if(dr[1]<-0.5)      dr[1]+=1.0;
      else if(dr[1]>=0.5) dr[1]-=1.0;
      if(dr[2]<-0.5)      dr[2]+=1.0;
      else if(dr[2]>=0.5) dr[2]-=1.0;
#else
      SingleParticlePos_t 
        dr(lattice.toCart(lattice.BConds.apply(pos[jat]-pos[iat])));
#endif
      //evaluate the cartesian distance:not that dr is in the unit-cell unit
      Scalar_t rsq = dot(dr,dr);
      if(rsq< RmaxSq && rsq > RminSq) { 
        myppl.addPair(iL,jat,jat,grID[jat],sqrt(rsq),dr);
      }
    } // jat
  }//iL

  myppl.checkPairs(UseVisitingForce);
}
 

bool NNSimpleEngine::putExtra(xmlNodePtr cur){
  return true;
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 355 $   $Date: 2005-07-08 12:56:01 -0500 (Fri, 08 Jul 2005) $
 * $Id: NNSimpleEngine.cpp 355 2005-07-08 17:56:01Z jnkim $ 
 *******************************************************************/
