//////////////////////////////////////////////////////////////////
// (c) Copyright 2004- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Utilities/OhmmsInfo.h"
#include "Particle/ParticleSet.h"
#include "Particle/FastParticleOperators.h"
#include "Message/OpenMP.h"
namespace OHMMS {

  void ParticleSet::convert(const ParticlePos_t& pin, ParticlePos_t& pout){

    if(pin.getUnit() == pout.getUnit())   {
      pout = pin;
      return;
    }
    if(pin.getUnit() == PosUnit::LatticeUnit) { //convert to CartesianUnit
      ConvertPosUnit<ParticlePos_t,Tensor_t,OHMMS_ORTHO>::apply(pin,Lattice.R,pout,0,pin.size());
    } else { //convert to LatticeUnit
      ConvertPosUnit<ParticlePos_t,Tensor_t,OHMMS_ORTHO>::apply(pin,Lattice.G,pout,0,pin.size());
    }
  }

  void ParticleSet::convert2Unit(const ParticlePos_t& pin, ParticlePos_t& pout){
    pout.setUnit(PosUnit::LatticeUnit);
    if(pin.getUnit() == PosUnit::LatticeUnit) 
      pout = pin;
    else 
      ConvertPosUnit<ParticlePos_t,Tensor_t,OHMMS_ORTHO>::apply(pin,Lattice.G,pout,0,pin.size());
  }

  void ParticleSet::convert2Cart(const ParticlePos_t& pin, ParticlePos_t& pout) {
    pout.setUnit(PosUnit::CartesianUnit);
    if(pin.getUnit() == PosUnit::CartesianUnit) 
      pout = pin;
    else 
      ConvertPosUnit<ParticlePos_t,Tensor_t,OHMMS_ORTHO>::apply(pin,Lattice.R,pout,0,pin.size());
  }

  void ParticleSet::convert2Unit(ParticlePos_t& pinout) {
    if(pinout.getUnit() == PosUnit::LatticeUnit) 
      return;
    else {
      pinout.setUnit(PosUnit::LatticeUnit);
      ConvertPosUnit<ParticlePos_t,Tensor_t,OHMMS_ORTHO>::apply(pinout,Lattice.G,0,pinout.size());
    }
  }

  void ParticleSet::convert2Cart(ParticlePos_t& pinout) {
    if(pinout.getUnit() == PosUnit::CartesianUnit) 
      return;
    else {
      pinout.setUnit(PosUnit::CartesianUnit);
      ConvertPosUnit<ParticlePos_t,Tensor_t,OHMMS_ORTHO>::apply(pinout,Lattice.R,0,pinout.size());
    }
  }

  void ParticleSet::applyBC(const ParticlePos_t& pin, ParticlePos_t& pout) {
    applyBC(pin,pout,0,pin.size());
  }

  void ParticleSet::applyBC(const ParticlePos_t& pin, ParticlePos_t& pout, size_type first, size_type last) {
    const bool orthogonal = ParticleLayout_t::IsOrthogonal;
    size_type mode = pin.getUnit()*2+pout.getUnit();
    switch(mode) {
    case(0):
      ApplyBConds<ParticlePos_t,Tensor_t,orthogonal>::Cart2Cart(pin,Lattice.G,Lattice.R,pout,first,last);
      break;
    case(1):
      ApplyBConds<ParticlePos_t,Tensor_t,orthogonal>::Cart2Unit(pin,Lattice.G,pout,first,last);
      break;
    case(2):
      ApplyBConds<ParticlePos_t,Tensor_t,orthogonal>::Unit2Cart(pin,Lattice.R,pout,first,last);
      break;
    case(3):
      ApplyBConds<ParticlePos_t,Tensor_t,orthogonal>::Unit2Unit(pin,pout,first,last);
      break;
    }
  }

  void ParticleSet::applyBC(ParticlePos_t& pos) {
    const bool orthogonal = ParticleLayout_t::IsOrthogonal;
    if(pos.getUnit()==PosUnit::LatticeUnit) {
      ApplyBConds<ParticlePos_t,Tensor_t,orthogonal>::Unit2Unit(pos,0,LocalNum);
    } else {
      ApplyBConds<ParticlePos_t,Tensor_t,orthogonal>::Cart2Cart(pos,Lattice.G,Lattice.R,0,LocalNum);
    }
  }

  void ParticleSet::getMinimumDisplacement(ParticlePos_t& displ) {
    const bool orthogonal = ParticleLayout_t::IsOrthogonal;
    if(displ.getUnit()==PosUnit::LatticeUnit) {
      GetMinimumDifference<ParticlePos_t,Tensor_t,orthogonal>::Unit2Unit(displ,0,LocalNum);
    } else {
      GetMinimumDifference<ParticlePos_t,Tensor_t,orthogonal>::Cart2Cart(displ,Lattice.G,Lattice.R,0,LocalNum);
    }
  }

}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 362 $   $Date: 2005-07-13 13:32:27 -0500 (Wed, 13 Jul 2005) $
 * $Id: ParticleSet.2.cpp 362 2005-07-13 18:32:27Z jnkim $ 
 ***************************************************************************/

