// -*- C++ -*-
/***************************************************************************
 *
 * The POOMA Framework
 * 
 * This program was prepared by the Regents of the University of
 * California at Los Alamos National Laboratory (the University) under
 * Contract No.  W-7405-ENG-36 with the U.S. Department of Energy (DOE).
 * The University has certain rights in the program pursuant to the
 * contract and the program should not be copied or distributed outside
 * your organization.  All rights in the program are reserved by the DOE
 * and the University.  Neither the U.S.  Government nor the University
 * makes any warranty, express or implied, or assumes any liability or
 * responsibility for the use of this software
 *
 * Visit http://www.acl.lanl.gov/POOMA for more details
 *
 ***************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// MDParticle: public Var
//   contains a SimpleParticle that are updated by propagator
//
///////////////////////////////////////////////////////////////////////////////

#ifndef MDEVENTS_H
#define MDEVENTS_H

#ifndef USER_INTERFACE_H
#include "UserInterface/UserInterface.h"
#endif
#ifndef CASNAMESPACE_H
#include "CasDefs.h"
#endif

class SimpleParticle;
class PotentialBase;

class MDEvents: public Var {

public:
  enum MDEventFlags {DISPLACEMENT, ENERGY, Y2, Y4};

  typedef CAS_DEFS::Scalar_t            Scalar_t;
  typedef CAS_DEFS::SingleParticlePos_t SingleParticlePos_t;
  typedef CAS_DEFS::ParticlePos_t       ParticlePos_t;
  typedef CAS_DEFS::ParticleScalar_t    ParticleScalar_t;

  ParticlePos_t    RefPos, DeltaR;
  ParticleScalar_t RefEne, Ene;
  vector<ParticleAttribBase* > RefConfig;

  MDEvents(): Var("events") { setup();}
  MDEvents(const char* name): Var(name) { setup();}

  ~MDEvents(){ PtclRef = NULL; PotRef = NULL;}

  int set(int argc, char **argv);
  void print ( ostream &os );

  void initialize(SimpleParticle* p, PotentialBase* pb, const string&, bool addbc = true);

  bool detectEvents(ParticlePos_t&, Scalar_t& drmaxsq);

  void setRefConfig(ParticlePos_t&, int); 
  void setRefConfig(ParticlePos_t&, ParticlePos_t&, int); 
  void saveRefConfig(Scalar_t);
  void saveRefConfig(int );

protected:
  int Counter;
  int LocalNum;
  SingleParticlePos_t RefCM;
  double Ecut, Rcut, RcutSq;  // cut-off radius to determine an event
  MDEventFlags d_Event; 
  bool AdjustCM, AddBC;
  string RefPosFile;
  SimpleParticle* PtclRef;
  PotentialBase*  PotRef;

  ParticlePos_t  AccumBC;

private:
  void setup();
};

// equivalent to  "set events ..."
class SetMDEvent: public Cmd {

public:
  SetMDEvent():Cmd("events"){ }
  SetMDEvent(const char* name):Cmd(name){ }
  char *help_msg();
  int action(int argc, char **argv);
};


#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * POOMA_VERSION_ID: $Id: MDEvents.h 151 2004-11-02 23:51:14Z jnkim $ 
***************************************************************************/

