//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <bitset>
#include <set>
using namespace std;
#include <math.h>

#include "Message/Communicate.h"
#include "Message/OpenMP.h"
#include "Particle/N2Method.h"
using namespace OHMMS;
#include "ParticleBase/ParticleUtility.h"

N2Method::N2Method():NNMethodBase(ThisMethod) { }

N2Method::~N2Method() { }

int 
N2Method::init(Particle_t& p, Scalar_t rmax) {

  RmaxSq = rmax*rmax;

  convert2Unit(p.Lattice,p.R);

  simplePartition(p);

  return p.getLocalNum();
}

void 
N2Method::partition(Particle_t& p) {
}

void 
N2Method::get(const Particle_t& p, PtclPairList_t& myppl, int myID) {

  int LocalNum = p.getLocalNum();
  int cur_nn = myppl.getTotNadj();

  myppl.reset(); // reset the pair lists

  SingleParticlePos_t dr;
  Scalar_t rsq;
  int maxnn = 0;
  int first = p.Lattice.dGrid[1]->PtclDist[myID];
  int last = p.Lattice.dGrid[1]->PtclDist[myID+1];
  int iL = 0;
  for(int ig= first; ig<last; ig++, iL++) {
    int iG = p.ID[ig];
    myppl.I[iL] =iG;
    int nn = 0;
    for(int jG=0; jG < LocalNum; jG++) 
      {
	if(iG == jG) continue;
	dr = p.Lattice.BConds.apply(p.R[jG]-p.R[iG]);
	//dr = p.R[jG]-p.R[iG];
	rsq = p.Lattice.Dot(dr,dr);
	if(rsq< RmaxSq && rsq > RminSq) 
	  {
	    nn++;
	    rsq = sqrt(rsq);
	    Scalar_t rinv = 1.0e0/rsq;
	    myppl.add(iL,p.ID[jG],rsq,rinv*p.Lattice.toCart(dr));
	  }
      } // all the atoms
    maxnn = (maxnn>nn)? maxnn:nn;
  }
  myppl.add(iL, -1, rsq, dr);
  myppl.setMaxNadj(maxnn);
}
 
#if defined(ENABLE_LIBXML2)
bool N2Method::putExtra(xmlNodePtr cur){
  return true;
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: N2Method.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

