//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Particle/NNImageMethod.h"
using namespace OHMMS;
#include "ParticleBase/DuplicateSuperCell.h"
#include "ParticleBase/ParticleUtility.h"

NNImageMethod::NNImageMethod() { }
NNImageMethod::~NNImageMethod() { }

int 
NNImageMethod::init(Particle_t& p, Scalar_t rmax) {

  RmaxSq = rmax*rmax;

  convert2Unit(p.Lattice,p.R);

  // call a linked cell calculation
  DuplicateSuperCell<ParticleLayout_t>::apply(p.Lattice, nextcell, rmax);

  return p.getLocalNum()*nextcell.size();

}

void 
NNImageMethod::get(const Particle_t& p, PtclPairList_t& myppl, int myID) {

  int LocalNum = p.getLocalNum();
  int cur_nn = myppl.getTotNadj();
  myppl.reset(); // reset the pair lists

  SingleParticlePos_t dr;
  Scalar_t rsq;
  int maxnn = 0;

  for(int iat= 0;iat < LocalNum; iat++) 
    {
      int nn = 0; // reset nn
      myppl.I[iat] = iat;
      for(int ib=0; ib<nextcell.size(); ib++) {

	for(int jat=0; jat < LocalNum; jat++) {
	  dr = p.Lattice.toCart(p.R[jat]-p.R[iat]+nextcell[ib]);
	  rsq = dot(dr,dr);
	  if(rsq< RmaxSq && rsq > RminSq) {// new neighbor
	    rsq = sqrt(rsq);
	    Scalar_t rinv = 1.0e0/rsq;
	    myppl.add(iat,jat,rsq,rinv*dr); 
	    nn++;
	  }
	} // all the atoms
      } // boxes
      maxnn = (maxnn>nn)? maxnn:nn;
    }

  myppl.add(LocalNum, -1, rsq, dr); // finalize it
  myppl.setMaxNadj(maxnn);
}
 
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: NNImageMethod.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

