//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <bitset>
#include "Particle/NNMethodBuilder.h"
#include "Particle/NNImageMethod.h"
#include "Particle/NNRegGridMethod.h"
#include "Utilities/OhmmsInfo.h"

NNMethodBase*
NNMethodBuilder::create(Particle_t& p, Scalar_t rmax, int& nnmax) {

  int ng[OHMMS_DIM];
  std::bitset<OHMMS_DIM> unitcell, bigcell;

  for(int idim=0; idim<OHMMS_DIM; idim++) {
    ng[idim] = p.Lattice.dGrid[2]->NP[idim];  
    if(ng[idim] == 1 && p.Lattice.BoxBConds[idim]) unitcell.set(idim);
    if(ng[idim] > 2)  bigcell.set(idim);
  } 

  NNMethodBase* nnmethod;

//   nnmethod = new NNRegGridMethod();
//   nnmax = nnmethod->init(p,rmax);
//   cout << "Mximum nearest neighbors " << nnmax << endl;
//   return nnmethod;

  /////////////////////////////////////////////
  //\todo mixed configuration

  // all directions
  if(bigcell.any()) {
    DEBUGMSG("\tCreating NNRegGridMethod for a nearest-neighbor engine")
    nnmethod = new NNRegGridMethod();
    nnmax = nnmethod->init(p,rmax);
    return nnmethod;
  }

  // any direction requires a image cell
  if(unitcell.any()) { 
    DEBUGMSG("\tCreating NNImageMethod for a nearest-neighbor engine")
    nnmethod = new NNImageMethod();
    nnmax = nnmethod->init(p,rmax);
    return nnmethod;
  }

//   // all directions
//   if(bigcell.any()) {
//     LOGMSG("\tCreating NNRegGridMethod for a nearest-neighbor engine")
//     nnmethod = new NNRegGridMethod();
//     nnmax = nnmethod->init(p,rmax);
//     return nnmethod;
//   }

  DEBUGMSG("\tCreating N2Method for a nearest-neighbor engine")
  nnmethod = new N2Method();
  nnmax = nnmethod->init(p,rmax);
  return nnmethod;
  //NNMethodBase* nnmethod = new N2Method();
}

/*
void
NNMethodBase::init(const ParticleLayout_t& lat, Scalar_t rmax) {


  RmaxSq = rmax*rmax;
  Scalar_t rmax2 = sqrt(2.0)*rmax;

  // number of the lowest grids
  int ngtot = lat.dGrid[2]->getTotalNum();
  int ng[OHMMS_DIM], curgrid[OHMMS_DIM];
  Scalar_t dr[OHMMS_DIM];

  for(int idim=0; idim<OHMMS_DIM; idim++) {
    dr[idim] = lat.dGrid[2]->Delta[idim];
    ng[idim] = lat.dGrid[2]->NP[idim];
  }


  int nmax[OHMMS_DIM]; 
  bool ncopy[OHMMS_DIM];

  //check boundary conditions and grid to determine 
  //if cells need to be duplicated or not
  for(int idim=0; idim<OHMMS_DIM; idim++) {
    nmax[idim] = 1; //  left/right cell to be examined
    ncopy[idim] = false;
    if(ng[idim] == 1) { // one grid in this direction, possible to duplicate
      if(lat.BoxBConds[idim]) {
	SingleParticlePos_t u0;
	u0[idim] = dr[idim];
	Scalar_t drc = lat.Dot(u0,u0);    
	Scalar_t d = drc*0.5;
	while(d < rmax2) { d+= drc; nmax[idim]++;}
	ncopy[idim] = true;
      } else {
	nmax[idim] = 0;
      } 
    }
  }

  // itself is not included
  for(int ig=0; ig<ngtot; ig++) {

    next.push_back(new vector<int>());
    image.push_back(new vector<int>());
    displ.push_back(new vector<SingleParticlePos_t>());

    
    // get the center coordinate of this grid
//     SingleParticlePos_t center(lat.dGrid[2]->Node[ig].Ri[0]+dr[0]*0.5,
// 			       lat.dGrid[2]->Node[ig].Ri[1]+dr[1]*0.5,
// 			       lat.dGrid[2]->Node[ig].Ri[2]+dr[2]*0.5);

    SingleParticlePos_t center(lat.dGrid[2]->Node[ig].Ri[0],
			       lat.dGrid[2]->Node[ig].Ri[1],
			       lat.dGrid[2]->Node[ig].Ri[2]);

    // get the index coordinate of this grid
    lat.dGrid[2]->getcoord(center,curgrid); // get the current point
    SingleParticlePos_t nc; // displacement vector under boundary conditions

    for(int ix = -nmax[0]; ix <= nmax[0]; ix++) {

      int ic = ix + curgrid[0]; // grid index
      if(ncopy[0]) {
        ic = curgrid[0]; // always itself
        nc[0] = static_cast<Scalar_t>(ix); // disp-component
      } else {
	nc[0] = 0.0;
	if(lat.BoxBConds[0]) {
          if(ic < 0)  { ic += ng[0]; nc[0] = 1.0;}
          if(ic>= ng[0]) {ic -= ng[0]; nc[0] =-1.0;}
	}
      }
      if(ic < 0 || ic >= ng[0]) continue;

      for(int iy = -nmax[1]; iy <= nmax[1]; iy++) {
	int jc = iy + curgrid[1];

	if(ncopy[1]) {
	  jc = curgrid[1]; // always itself
	  nc[1] = static_cast<Scalar_t>(iy);;
	} else {
	  nc[1] = 0.0;
	  if(lat.BoxBConds[1]) {
	    if(jc < 0)  {jc += ng[1]; nc[1] = 1.0;}
	    if(jc >= ng[1]) {jc -= ng[1]; nc[1] = -1.0;}
	  }
	}

	if(jc < 0 || jc >= ng[1]) continue;


	for(int iz = -nmax[2]; iz <= nmax[2]; iz++) {

	  // skip oneself
	  if(ix == 0 && iy == 0 && iz == 0) {
	     continue;
	  }

	  int kc = iz + curgrid[2];
	  if(ncopy[2]) {
	    kc = curgrid[2]; // always itself
	    nc[2] = static_cast<Scalar_t>(iz);;
	  } else {
	    nc[2] = 0.0;
	    if(lat.BoxBConds[2]) {
	      if(kc < 0) {kc += ng[2]; nc[2] = 1.0;}
	      if(kc >= ng[2]) {kc -= ng[2]; nc[2] = 1.0;}
	    }
	  }
	  if(kc < 0 || kc >= ng[2]) continue;

	  image[ig]->push_back(lat.dGrid[2]->loc(ic,jc,kc));
	  displ[ig]->push_back(nc);
	}
      }
    } // search all the related cells
    cout << "grid = " << center << endl;
    for(int ii=0; ii<image[ig]->size(); ii++) {
      cout << image[ig]->operator[](ii)
	   << displ[ig]->operator[](ii) << endl;
    }
  }

}
*/
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: NNMethodBuilder.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

