//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Particle/NNMetisMethod.h"
#include "Particle/NNRegGridMethod.h"
#include "ParticleBase/ParticleUtility.h"
#include "Particle/MetisGraphPartition.h"
using namespace OHMMS;

NNMetisMethod::NNMetisMethod(): partTool(NULL), NNMethodBase(ThisMethod) {
  LOGMSG("NNMetisMethod::NNMetisMethod is created.")
}
NNMetisMethod::~NNMetisMethod() { 
  if(partTool) delete partTool;
}

void 
NNMetisMethod::updateR(Particle_t& p, const ParticlePos_t& pos) {
  convert(p.Lattice,pos,p.R);
//   if(pos.InUnit) {
//     for(int iG=0; iG<p.getLocalNum(); iG++){
//       p.R[iG] = p.Lattice.BConds.wrap(pos[iG]);
//     }
//   } else {
//     for(int iG=0; iG<p.getLocalNum(); iG++){
//       p.R[iG] = p.Lattice.BConds.wrap(p.Lattice.toUnit(pos[iG]));
//     }
//   }
}

void 
NNMetisMethod::partition(Particle_t& p) {

  if(Distributed[enable_mpi] && p.NeedPartition) {

    p.partition(false);
    const ParticleLayout_t::PtclGrid_t& bg = *(p.Lattice.dGrid[2]);
    int ng = bg.getTotalNum();

    //weight propotional to the number of particles per grid
    vector<int> w(ng);
    for(int ig=0; ig<ng; ig++) {
      w[ig] = bg.PtclDist[ig+1]-bg.PtclDist[ig];
    }
      
    partTool->partition(NextCell.M, NextCell.ID, w);

    //reset the map(logical,physical)
    int iL = 0;
    for(int ipart=0; ipart<partTool->NumPart; ipart++) {
      for(int k=0;k<partTool->gSet[ipart].size(); k++) {
	int ic = partTool->gSet[ipart][k];
	for(int ig=bg.PtclDist[ic];ig<bg.PtclDist[ic+1]; ig++) {
	  p.ID[iL++] = bg.I[ig];
	}
      }
    }
  }
}
  
void 
NNMetisMethod::get(const Particle_t& p, PtclPairList_t& myppl, int myID) {

  myppl.reset(); // reset the pair lists

  const ParticleLayout_t::PtclGrid_t& bg = *(p.Lattice.dGrid[2]);

  SingleParticlePos_t dr;
  Scalar_t rsq;    
  const ParticleLayout_t& lat = p.Lattice;

  int maxnn = 0;
  int iL = 0;
  //for(int ipart=0; ipart<partTool->NumPart; ipart++) {

    for(int k = 0; k<partTool->gSet[myID].size(); k++) {

      ///get the index of base grid
      int ic = partTool->gSet[myID][k];
      int ni = partTool->firstIndex[ic];
      int nf = partTool->lastIndex[ic];
      iL = ni;
      for(int ig=ni; ig<nf; ig++, iL++) {
	int iG = p.ID[ig];

	myppl.I[iL] = iG; // save the physical index
	SingleParticlePos_t r0 = p.R[iG];
	int nn = 0;

	///check the particles in the current grid
	for(int jg=ni; jg<nf;jg++) {
	  if(ig == jg) continue; //do not interact with itself
	  int jG = p.ID[jg];
	  dr = p.R[jG]-r0;
	  rsq = dot(dr,dr);
	  if(rsq< RmaxSq && rsq > RminSq) {
	    rsq = sqrt(rsq);
	    Scalar_t rinv = 1.0e0/rsq;
	    myppl.add(iL,jg,rsq,rinv*dr); //  logical index jg is used
	    nn++;
	  }
	} // ni <= jg < nf

	  ////////////////////////////////
	  // neighboring cells
	  ////////////////////////////////
	for(int nc=NextCell.first(ic); nc< NextCell.last(ic); nc++) {
	  int jc = NextCell.id(nc);
	  
	  SingleParticlePos_t del = NextCell.bc(nc)-r0;
	  for(int jg=partTool->firstIndex[jc]; jg<partTool->lastIndex[jc]; jg++) {
	    int jG = p.ID[jg];
	    dr = p.R[jG] + del;
	    rsq = dot(dr,dr);
	    if(rsq< RmaxSq && rsq  > RminSq){
	      rsq = sqrt(rsq);
	      Scalar_t rinv = 1.0e0/rsq;
	      myppl.add(iL,jg,rsq,rinv*dr); //  physical index jg is used
	      nn++;
	    }
	  }// particles in a connected box
	}///connected boxes
	maxnn = (maxnn>nn)? maxnn:nn;
      }///ni <= ig < nf
    }
    //}
  myppl.add(iL,-1,rsq,dr);
  myppl.setMaxNadj(maxnn);
}

int 
NNMetisMethod::init(Particle_t& p, Scalar_t rmax) {
    
  // grid parititon
  RmaxSq = rmax*rmax;
  p.partition(false);

  int nnmax = 0;
  const ParticleLayout_t::PtclGrid_t& bg = *(p.Lattice.dGrid[2]);

  int maxnat = 0;
  int maxnc = NextCell.connect(p.Lattice,rmax,2);

  int ng = bg.getTotalNum();
  ///weight propotional to the number of particles per grid
  vector<int> w(ng);
  for(int ig=0; ig<ng; ig++) {
    w[ig] = bg.PtclDist[ig+1]-bg.PtclDist[ig];
    maxnat = max(maxnat,w[ig]);
  }

  partTool->partition(NextCell.M, NextCell.ID, w);

  int iL = 0;
  for(int ipart=0; ipart<partTool->NumPart; ipart++) {
    for(int k=0;k<partTool->gSet[ipart].size(); k++) {
      int ic = partTool->gSet[ipart][k];
      for(int ig=bg.PtclDist[ic];ig<bg.PtclDist[ic+1]; ig++) {
	p.ID[iL++] = bg.I[ig];
      }
    }
  }

  if(partTool->Reorder) {
    p.NeedPartition =true;
    p.partition(true);
  }

  convert2Cart(p.Lattice,p.R);

//   int ogrid[] = {1,1,1};
//   if(Distributed[enable_omp]) ogrid[0] = omp_get_max_threads();
//   p.Lattice.dGrid[1]->set(ogrid);
//   p.Lattice.dGrid[1]->distribute(p.getLocalNum());
  if(Distributed[enable_omp]) {
    int ogrid[] = {1,1,1};
    ogrid[0] = partTool->NumPart;
    p.Lattice.resetGrid(1,ogrid);
    p.Lattice.dGrid[1]->PtclDist = partTool->M;
    p.Lattice.dGrid[0]->distribute(p.getLocalNum());
  } else {
    p.Lattice.dGrid[1]->distribute(p.getLocalNum());
    p.Lattice.dGrid[0]->distribute(p.getLocalNum());
  }

  LOGMSG("\tNumber of maximum linked cells, particles per cell " << maxnc << " " << maxnat)
  return (maxnc+1)*(maxnat+1);
}

#if defined(ENABLE_LIBXML2)
bool NNMetisMethod::putExtra(xmlNodePtr cur){

  if(!partTool) {
    partTool = new MetisGraphPartition;
  }

  if(xmlHasProp(cur, (const xmlChar *) "reorder")) {
    if(!xmlStrcmp(xmlGetProp(cur, (const xmlChar *) "reorder"),(const xmlChar*)"true")){
      partTool->Reorder = true;
    }
  }

  if(Distributed[enable_omp]) {
    partTool->NumPart = omp_get_max_threads();
  }

//   cur = cur->xmlChildrenNode;
//   while(cur != NULL) {
//     ///start random number generator
//    if ((!xmlStrcmp(cur->name, (const xmlChar *)"partition"))) {
//      putXMLNodeContent<int>::put(partTool->NumPart,doc,cur);
//    } 
//    cur = cur->next;
//   }
  return true;
}
#endif
//}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: NNMetisMethod.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

