//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Particle/NNRegGridMethod.h"
using namespace OHMMS;
#include "ParticleBase/ParticleUtility.h"

NNRegGridMethod::NNRegGridMethod() {
  PartitionMode = -1;
}
NNRegGridMethod::~NNRegGridMethod() { }

void 
NNRegGridMethod::partition(Particle_t& p) {

  p.partition(false);

}

void 
NNRegGridMethod::get(const Particle_t& p, PtclPairList_t& myppl, int myID) {

  myppl.reset(); // reset the pair lists

  const ParticleLayout_t::PtclGrid_t& bg = *(p.Lattice.dGrid[2]);
  SingleParticlePos_t dr;
  Scalar_t rsq;    

  const ParticleLayout_t& lat = p.Lattice;
  int maxnn = 0;
  int iL = 0;

  for(int ic=bg.NodeDist[myID]; ic<bg.NodeDist[myID+1]; ic++) {

    int ni=bg.PtclDist[ic];
    int nf=bg.PtclDist[ic+1];
    for(int ig=ni; ig<nf; ig++, iL++) {

      int iG = p.ID[ig];// physical index
      myppl.I[iL] = iG; // save the physical index

      int nn = 0;
      ////////////////////////////////
      // in-cell 
      ////////////////////////////////
      //for(int jG=ni; jG<nf; jG++) {
      /////////////////////////////////
      for(int jg=ni; jg<nf; jg++) {

        int jG = p.ID[jg]; // physical index
      	if(iG == jG) continue; //do not interact with itself

	//////////////////////
	//dr = lat.toCart(p.R[jG]-p.R[iG]);
	//////////////////////

#ifdef INLINE_TINYVECTOR
	dr[0] = p.R[jG][0]-p.R[iG][0];
	dr[1] = p.R[jG][1]-p.R[iG][1];
	dr[2] = p.R[jG][2]-p.R[iG][2];
	rsq = dr[0]*dr[0] + dr[1]*dr[1] + dr[2]*dr[2];
#else
	dr = p.R[jG]-p.R[iG];
	rsq = dot(dr,dr);
#endif
	if(rsq< RmaxSq && rsq > RminSq) {
	  rsq = sqrt(rsq);
	  Scalar_t rinv = 1.0e0/rsq;

	  ///////////////////////////////////////
	  // adding a new neighrbor
	  //myppl.add(iL,jG,rsq,rinv*dr); // physical index jG
	  ///////////////////////////////////////
	  ///////////////////////////////////////
	  // IMPORTANT
	  ///////////////////////////////////////
#ifdef INLINE_TINYVECTOR
	  dr[0]*=rinv;	  dr[1]*=rinv;  dr[2]*=rinv;
	  myppl.add(iL,jg,rsq,dr); //  logical index jg is used
#else
	  myppl.add(iL,jg,rsq,rinv*dr); //  logical index jg is used
#endif
	  nn++;
	}          
      }//

      ////////////////////////////////
      // neighboring cells
      ////////////////////////////////
      for(int nc=NextCell.first(ic); nc<NextCell.last(ic);nc++) {

      //for(int nc=0; nc<NodeConn[ic]->size(); nc++) {

	//commented out: see note below
	//int jc = NodeConn[ic]->operator[](nc);
	int jc = NextCell.id(nc);

	// corrections due to the boundary conditions
	//SingleParticlePos_t del = (*Delta[ic])[nc];
	SingleParticlePos_t del = NextCell.bc(nc);
	for(int jg=bg.PtclDist[jc]; jg<bg.PtclDist[jc+1]; jg++) {

	  int jG = p.ID[jg];

#ifdef INLINE_TINYVECTOR
	  dr[0] = p.R[jG][0]-p.R[iG][0]+del[0];
	  dr[1] = p.R[jG][1]-p.R[iG][1]+del[1];
	  dr[2] = p.R[jG][2]-p.R[iG][2]+del[2];
	  rsq = dr[0]*dr[0] + dr[1]*dr[1] + dr[2]*dr[2];
#else
	  dr = p.R[jG]-p.R[iG]+del;
	  rsq = dot(dr,dr);
#endif
	  if(rsq< RmaxSq && rsq > RminSq) {
	    rsq = sqrt(rsq);
	    Scalar_t rinv = 1.0e0/rsq;

	    ///////////////////////////////////////
	    // adding a new neighrbor
	    //myppl.add(iL,jG,rsq,rinv*dr); // new neighbor
	    ///////////////////////////////////////
	    ///////////////////////////////////////
	    // IMPORTANT
	    ///////////////////////////////////////

#ifdef INLINE_TINYVECTOR
	    dr[0]*=rinv; dr[1]*=rinv;  dr[2]*=rinv;
	    myppl.add(iL,jg,rsq,dr); //  logical index jg is used
#else
	    myppl.add(iL,jg,rsq,rinv*dr); //  logical index jg is used
#endif
	    nn++;
	  }
	}
      }
      
      maxnn = (maxnn>nn)? maxnn:nn;
    }//atoms in a cell
  }//go thru all the cells
  myppl.add(iL,-1,rsq,dr);
  myppl.setMaxNadj(maxnn);
}

int 
NNRegGridMethod::init(Particle_t& p, Scalar_t rmax) {

  // grid parititon
  RmaxSq = rmax*rmax;

  const ParticleLayout_t::PtclGrid_t& basegrid = *(p.Lattice.dGrid[2]);
  int maxnat = 0;
  for(int ig=0; ig<basegrid.getTotalNum(); ig++) {
    maxnat = max(maxnat,basegrid.PtclDist[ig+1]-basegrid.PtclDist[ig]);
  }

  int maxnc = 26;
  SingleParticlePos_t dx(basegrid.Delta[0],
			 basegrid.Delta[1],
			 basegrid.Delta[2]),d;
  Scalar_t x,y,z;    

  NextCell.M.resize(basegrid.getTotalNum()+1);
  NextCell.ID.reserve(maxnc*basegrid.getTotalNum());
  NextCell.BC.reserve(maxnc*basegrid.getTotalNum());

  //for each grid, search for connected grid
  NextCell.M[0] = 0;
  for(int ig=0; ig<basegrid.getTotalNum(); ig++) {
    SingleParticlePos_t org(basegrid.Node[ig].Ri[0]+0.5*dx[0], 
			    basegrid.Node[ig].Ri[1]+0.5*dx[1], 
			    basegrid.Node[ig].Ri[2]+0.5*dx[2]);
    int nconnected = 0;
    for(int ix=-1; ix<=1; ix++) {
      x = org[0]+double(ix)*dx[0];
      d[0] = x;
      if(p.Lattice.BoxBConds[0]) {
	if(x<0)       x += 1;
	else if(x>=1) x -= 1;
      }
      if(x<0 || x>=1) continue;
      d[0] -= x;
      for(int jx=-1; jx<=1; jx++) {
	y = org[1]+double(jx)*dx[1];
	d[1] = y;
	if(p.Lattice.BoxBConds[1]) {
	  if(y<0)       y += 1;
	  else if(y>=1) y -= 1;
	}
	if(y<0 || y>=1) continue;
	d[1] -= y;
	for(int kx=-1; kx<=1; kx++) {
	  if(ix == 0 && jx == 0 && kx == 0) continue;
	  z = org[2]+double(kx)*dx[2];
	  d[2] = z;
	  if(p.Lattice.BoxBConds[2]) {
	    if(z<0)       z += 1;
	    else if(z>=1) z -= 1;
	  }
	  if(z<0 || z>=1) continue;
	  d[2] -= z;
	  int iloc = basegrid.loc(x,y,z);
	  NextCell.ID.push_back(iloc);
	  NextCell.BC.push_back(p.Lattice.toCart(d));
	  nconnected++;
	}
      }
    }
    NextCell.M[ig+1] = NextCell.M[ig]+nconnected;
  }

  convert2Cart(p.Lattice,p.R);
  LOGMSG("\tNumber of maximum linked cells " << maxnc)
  return (maxnc)*(maxnat+1)/2;

}

/////////////////////////////////////////////////////////
// Jeongnim Kim 10/24/2002
// init(Particle_t& p, Scalar_t rmax) is substitued by above.
// This routine does not include image cells.
// Maybe more efficient but cannot handle cases like (1 1 N)
/////////////////////////////////////////////////////////
/*
int 
NNRegGridMethod::init(Particle_t& p, Scalar_t rmax) {

  // grid parititon
  RmaxSq = rmax*rmax;

  const ParticleLayout_t::PtclGrid_t& basegrid = *(p.Lattice.dGrid[2]);
  int maxnat = 0;
  for(int ig=0; ig<basegrid.getTotalNum(); ig++) {
    maxnat = max(maxnat,basegrid.PtclDist[ig+1]-basegrid.PtclDist[ig]);
  }


  const ParticleLayout_t::PtclGrid_t& basegrid = *(p.Lattice.dGrid[2]);
  SingleParticlePos_t dx(basegrid.Delta[0],
			 basegrid.Delta[1],
			 basegrid.Delta[2]);

  double x,y,z;
  int maxnat = 0;
  int maxnc = 0;
  for(int ig=0; ig<basegrid.getTotalNum(); ig++) {

    maxnat = max(maxnat,basegrid.PtclDist[ig+1]-basegrid.PtclDist[ig]);
    NodeConn.push_back(new vector<int>);
    Delta.push_back(new vector<SingleParticlePos_t>);

    //set<int> ghosts;
    SingleParticlePos_t org(basegrid.Node[ig].Ri[0]+0.5*dx[0],
			    basegrid.Node[ig].Ri[1]+0.5*dx[1],
			    basegrid.Node[ig].Ri[2]+0.5*dx[2]);
    SingleParticlePos_t d;

    for(int ic=-1; ic<=1; ic++) {
      //\note round-off error can lead to an interesting case
      //the center of the next cell is referenced (factor 1/2).
      x = org[0] + static_cast<double>(ic)*dx[0];
      d[0] = x;
      if(p.Lattice.BoxBConds[0]) {
	if(x<0)       x += 1;
	else if(x>=1) x -= 1;
      }
      if(x<0 || x >= 1) continue;
      d[0] -= x;

      for(int jc=-1; jc<=1; jc++) {
	y = org[1] + static_cast<double>(jc)*dx[1];
	d[1] = y;
	if(p.Lattice.BoxBConds[1]) {
	  if(y<0)       y += 1; 
	  else if(y>=1) y -= 1;
	}

	if(y<0 || y >= 1 ) continue;
        d[1] -= y;

	for(int kc=-1; kc<=1; kc++) {
	  z = org[2] + static_cast<double>(kc)*dx[2];
          d[2] = z;
	  if(ic == 0 && jc == 0 && kc == 0) continue;
	  if(p.Lattice.BoxBConds[2]) {
	    if(z<0)       z += 1;
	    else if(z>=1) z -= 1; 
	  }
	  if(z<0 || z >= 1)  continue;
          d[2] -= z;
 	  int newloc = basegrid.loc(x,y,z);
	  NodeConn[ig]->push_back(newloc);
	  Delta[ig]->push_back(p.Lattice.toCart(d));
//           if(newloc == ig) continue;exclude itself
//           ghosts.insert(newloc);
	}
      }
    }

//     NodeConn.push_back(new vector<int>);
//     NodeConn[ig]->insert(NodeConn[ig]->begin(), ghosts.begin(), ghosts.end());
    if(NodeConn[ig]->size() > maxnc) {
      maxnc = NodeConn[ig]->size();
    }
//     cout << ig << " has neighboring cell " << endl;
//     for(int ii=0; ii<NodeConn[ig]->size(); ii++) {
//       cout << NodeConn[ig]->operator[](ii) << " "
// 	   << Delta[ig]->operator[](ii) << endl;
//     }
  }//basegrid

  if(p.R.InUnit) {
    ParticlePos_t ptmp(p.getLocalNum());
    ptmp = p.R;
    ptmp.InUnit = true;
    p.R.InUnit = false;
    convert(p.Lattice, ptmp, p.R);
  }

  return maxnc*maxnat/2;

}
*/
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: NNRegGridMethodNP.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

