//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Particle/UniformGridConnection.h"
using namespace OHMMS;

//!< return the maximum number of connected cells
int 
UniformGridConnection::connectGrid(ParticleLayout_t& Lattice, 
                                   Scalar_t rmax, int glevel) {

    Displacement_t u0(Lattice.Delta[0],0.0,0.0);
    Displacement_t u1(0.0,Lattice.Delta[1],0.0);
    Displacement_t u2(0.0,0.0,Lattice.Delta[2]);
    Scalar_t RmaxSq = rmax*rmax;

    ///calculate the extend of linked cells
    int nx = static_cast<int>(sqrt(RmaxSq/Lattice.Dot(u0,u0)))+1;
    int ny = static_cast<int>(sqrt(RmaxSq/Lattice.Dot(u1,u1)))+1;
    int nz = static_cast<int>(sqrt(RmaxSq/Lattice.Dot(u2,u2)))+1;

    //M.resize(basegrid.getTotalNum()+1);
    M.resize(Lattice.getTotalNum()+1);
    int ntot = Lattice.getTotalNum()*(2*nx+1)*(2*ny+1)*(2*nz+1);
    if(ID.capacity() < ntot) ID.reserve(ntot);
    if(BC.capacity() < ntot) BC.reserve(ntot);

    M[0] = 0;
    int maxnc = 0;
    Displacement_t dx(Lattice.Delta[0], Lattice.Delta[1], Lattice.Delta[2]);

    ///for each grid, search for connected grids
    for(int ig=0; ig<Lattice.getTotalNum(); ig++) {
      Displacement_t org = Lattice.center(ig), d;
      Scalar_t x,y,z;
      int nconnected = 0;
      for(int ix=-nx; ix<=nx; ix++) {
	d[0] = x = org[0]+double(ix)*dx[0];
	if(Lattice.BoxBConds[0]) {
	  x = fmod(d[0],1.0); if(x<0) x += 1.0;
	}
	if(x<0 || x>=1) continue;
	d[0] -= x;
	for(int jx=-ny; jx<=ny; jx++) {
	  d[1] = y = org[1]+double(jx)*dx[1];
	  if(Lattice.BoxBConds[1]) {
	    y = fmod(d[1],1.0); if(y<0) y += 1.0;
	  }
	  if(y<0 || y>=1) continue;
	  d[1] -= y;
	  for(int kx=-nz; kx<=nz; kx++) {

	    d[2] = z = org[2]+double(kx)*dx[2];
	    if(Lattice.BoxBConds[2]) {
	      z = fmod(d[2],1.0); if(z<0) z += 1.0;
	    }
	    if(z<0 || z>=1) continue;
	    d[2] -= z;

	    int iloc = Lattice.loc(x,y,z);

	    if(iloc ==ig && ix == 0 && jx == 0 && kx == 0) continue;
	    ID.push_back(iloc);
	    BC.push_back(Lattice.toCart(d));
	    nconnected++;
	  }
	}
      }
      M[ig+1] = M[ig]+nconnected;
      maxnc = max(maxnc,nconnected);
    }

    //for each grid, search for connected grid
    //for(int ig=0; ig<Lattice.getTotalNum(); ig++) {
    //  cout << ig << " has " << M[ig+1]-M[ig] << endl;
    //  for(int j=M[ig]; j<M[ig+1]; j++) {
    //	cout << ID[j] << " " << BC[j] << endl;
    //  }
    //}
    return maxnc; // return the maxmimum number of connected cells
  }

// int UniformGridConnection::connectGrid(ParticleLayout_t& Lattice, Scalar_t rmax, int glevel) {

//   ParticleLayout_t::PtclGrid_t&  basegrid = *(Lattice.dGrid[glevel]);

//     Displacement_t u0(basegrid.Delta[0],0.0,0.0);
//     Displacement_t u1(0.0,basegrid.Delta[1],0.0);
//     Displacement_t u2(0.0,0.0,basegrid.Delta[2]);
//     Scalar_t RmaxSq = rmax*rmax;

//     ///calculate the extend of linked cells
//     int nx = static_cast<int>(sqrt(RmaxSq/Lattice.Dot(u0,u0)))+1;
//     int ny = static_cast<int>(sqrt(RmaxSq/Lattice.Dot(u1,u1)))+1;
//     int nz = static_cast<int>(sqrt(RmaxSq/Lattice.Dot(u2,u2)))+1;

//     M.resize(basegrid.getTotalNum()+1);
//     int ntot = basegrid.getTotalNum()*(2*nx+1)*(2*ny+1)*(2*nz+1);
//     if(ID.capacity() < ntot) ID.reserve(ntot);
//     if(BC.capacity() < ntot) BC.reserve(ntot);

//     M[0] = 0;
//     int maxnc = 0;
//     Displacement_t dx(basegrid.Delta[0], basegrid.Delta[1], basegrid.Delta[2]);

//     ///for each grid, search for connected grids
//     for(int ig=0; ig<basegrid.getTotalNum(); ig++) {
//       Displacement_t org = basegrid.center(ig), d;
//       Scalar_t x,y,z;
//       int nconnected = 0;
//       for(int ix=-nx; ix<=nx; ix++) {
// 	d[0] = x = org[0]+double(ix)*dx[0];
// 	if(Lattice.BoxBConds[0]) {
// 	  x = fmod(d[0],1.0); if(x<0) x += 1.0;
// 	}
// 	if(x<0 || x>=1) continue;
// 	d[0] -= x;
// 	for(int jx=-ny; jx<=ny; jx++) {
// 	  d[1] = y = org[1]+double(jx)*dx[1];
// 	  if(Lattice.BoxBConds[1]) {
// 	    y = fmod(d[1],1.0); if(y<0) y += 1.0;
// 	  }
// 	  if(y<0 || y>=1) continue;
// 	  d[1] -= y;
// 	  for(int kx=-nz; kx<=nz; kx++) {

// 	    d[2] = z = org[2]+double(kx)*dx[2];
// 	    if(Lattice.BoxBConds[2]) {
// 	      z = fmod(d[2],1.0); if(z<0) z += 1.0;
// 	    }
// 	    if(z<0 || z>=1) continue;
// 	    d[2] -= z;

// 	    int iloc = basegrid.loc(x,y,z);

// 	    if(iloc ==ig && ix == 0 && jx == 0 && kx == 0) continue;
// 	    ID.push_back(iloc);
// 	    BC.push_back(Lattice.toCart(d));
// 	    nconnected++;
// 	  }
// 	}
//       }
//       M[ig+1] = M[ig]+nconnected;
//       maxnc = max(maxnc,nconnected);
//     }

//     //for each grid, search for connected grid
//     //for(int ig=0; ig<basegrid.getTotalNum(); ig++) {
//     //  cout << ig << " has " << M[ig+1]-M[ig] << endl;
//     //  for(int j=M[ig]; j<M[ig+1]; j++) {
//     //	cout << ID[j] << " " << BC[j] << endl;
//     //  }
//     //}
//     return maxnc; // return the maxmimum number of connected cells
//   }


  void UniformGridConnection::print(ostream& os) {
    for(int ig=0; ig<M.size()-1; ig++) {
      cout << ig << " has neighboring cell "  << M[ig+1]-M[ig]<< endl;
      for(int ii=M[ig]; ii<M[ig+1]; ii++) {
	cout << ID[ii] << " " << BC[ii] << endl;
      }
    }
  }
  
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: UniformGridConnection.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
