//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_PARTICLE_INPUTOUTPUT_HDF_UTILITY_H
#define OHMMS_PARTICLE_INPUTOUTPUT_HDF_UTILITY_H

#include "OhmmsData/OhmmsElementBase.h"
#include "OhmmsData/RecordProperty.h"
#include "Particle/ParticleSet.h"
#include "OhmmsData/HDFAttribIO.h"

namespace OHMMS {

  struct HDFParticleIO: public ParticleTags {

    typedef ParticleSet Particle_t;

    HDFParticleIO(Particle_t& aptcl):ref_(aptcl) { }

    bool read(hid_t pid);
    bool write(hid_t pid);
        
    Particle_t& ref_;

  };

  class HDFParticleParser: public ParticleTags {

  public:

    typedef ParticleSet Particle_t;

    HDFParticleParser(Particle_t& aptcl):ref_(aptcl) { }

    ///reading from a file
    bool put(const char*, xmlNodePtr cur);

    bool put(xmlNodePtr cur);

  private:
    Particle_t& ref_;

    bool read_v02_format(hid_t h, xmlNodePtr cur);
    bool read_v03_format(hid_t pgroup);
  };

  class HDFSaveParticle:  
    public ParticleTags,
    public RecordProperty {

  public:

    typedef ParticleSet Particle_t;

    HDFSaveParticle(Particle_t& pin): ref_(pin){ }

    ~HDFSaveParticle();

    void reset(const char* fileroot);

    void report(int iter);

    void finalize() { }

    bool put(xmlNodePtr cur);

  private:

    Particle_t& ref_;
    string FileRoot;

  };
}

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 299 $   $Date: 2005-02-16 09:37:33 -0600 (Wed, 16 Feb 2005) $
 * $Id: HDFParticleIO.h 299 2005-02-16 15:37:33Z jnkim $
 ***************************************************************************/
