//////////////////////////////////////////////////////////////////
// (c) Copyright 2005- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/**@file MCFRecordParticle.cpp
 * @brief Implementation of threaded Record engines.
 */
#include "Particle/ParticleSet.h"
#include "ParticleIO/MCFRecordParticle.h"
#include "ParticleIO/MCFRecordParticleAttrib.h"
#include "Message/OpenMP.h"
using namespace OHMMS;

MCFRecordParticle::MCFRecordParticle(const Particle_t& a): ref_(a) {
}

MCFRecordParticle::~MCFRecordParticle(){ 
  for(int i=0; i<Records.size(); i++) {
    vector<RecordProperty*>::iterator it(Records[i]->begin());
    vector<RecordProperty*>::iterator it_end(Records[i]->end());
    while(it != it_end) {
      delete *it;
      ++it;
    }
    delete Records[i];
  }
}

void MCFRecordParticle::reset(const char* fileroot) {
#pragma omp parallel
  {
    int ip = omp_get_thread_num();
    vector<RecordProperty*>::iterator it(Records[ip]->begin());
    vector<RecordProperty*>::iterator it_end(Records[ip]->end());
    while(it != it_end) {
      char fname[128];
      sprintf(fname,"%s.t%03d",fileroot,ip);
      (*it)->reset(fname);
      ++it;
    }
  }
}

void MCFRecordParticle::finalize() {
#pragma omp parallel
  {
    int ip = omp_get_thread_num();
    vector<RecordProperty*>::iterator it(Records[ip]->begin());
    vector<RecordProperty*>::iterator it_end(Records[ip]->end());
    while(it != it_end) {
      (*it)->finalize();
      ++it;
    }
  }
}

bool MCFRecordParticle::put(xmlNodePtr cur) {

  if(Records.empty()) {
    Records.resize(omp_get_max_threads(),0);
  }

#pragma omp parallel
  {
    int ip = omp_get_thread_num();
    int np = omp_get_max_threads();
    vector<int> locind(np+1);
    FairDivide(ref_.getLocalNum(),np,locind);
    int first = locind[ip];
    int last = locind[ip+1];

    if(Records[ip] == 0) Records[ip] = new vector<RecordProperty*>;

    xmlNodePtr cur1=cur->children;
    while(cur1 != NULL) {
      string cname((const char*)cur1->name);
      if(cname == ParticleTags::attrib_tag) {
        const xmlChar* nptr = xmlGetProp(cur1,(const xmlChar*)"name");
        if(nptr) {
          string aname((const char*)nptr);
          RecordProperty* arecord = 0;
          if(aname == ParticleTags::position_tag) {
            arecord = new MCFRecordPosAttrib(ref_.R, first, last);
          } else if(aname == ParticleTags::energy_tag) {
            arecord = new MCFRecordScalarAttrib(ref_.E, first, last);
          }
          if(arecord) {
            LOGMSG("\tCreating mcf record engine for " << aname)
            arecord->put(cur1);
            Records[ip]->push_back(arecord);
          }
        }
      }
      cur1 = cur1->next;
    }
  }

  return true;
}

void MCFRecordParticle::report(int iter) {
#pragma omp parallel 
  {
    int ip = omp_get_thread_num();
    vector<RecordProperty*>::iterator it(Records[ip]->begin());
    vector<RecordProperty*>::iterator it_end(Records[ip]->end());
    while(it != it_end) {
      (*it)->report(iter);
      ++it;
    }
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 299 $   $Date: 2005-02-16 09:37:33 -0600 (Wed, 16 Feb 2005) $
 * $Id: MCFRecordParticle.cpp 299 2005-02-16 15:37:33Z jnkim $ 
 ***************************************************************************/
