////////////////////////////////////////////////////////////////
// (c) Copyright 2005- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
//
/**@file MCFRecordParticle.h
 * @brief Declaration of record engine for OpenMP parallelization.
 */
#ifndef OHMMS_MCF_RECORD_PARTICLE_H
#define OHMMS_MCF_RECORD_PARTICLE_H

#include "OhmmsData/RecordProperty.h"

namespace OHMMS {

  /** Record a number of Particle Attributes of a ParticleSet.
   *
   * Each thread (OpenMP node) writes into different files.
   * The number of files is the product of the number of threads and the number
   * of attributes. The hardware limit should be consider when using a large
   * number of threads.
   * Although this class is used with MCFRecordAttrib, the use can be extended to
   * other RecordProperty.
   */
  class MCFRecordParticle: public RecordProperty
  {

    public:

      typedef ParticleSet Particle_t;
      MCFRecordParticle(const Particle_t& a);
      ~MCFRecordParticle();

      void reset(const char* fileroot);

      void report(int iter);

      void finalize();

      bool put(xmlNodePtr cur);

      void compress();

    private:
      ///reference to the ParticleSet
      const Particle_t& ref_;

      /** Record engines.  
       *
       * Each thread owns vector<RecordProperty*> for parallel output.
       */
      vector<vector<RecordProperty*>* > Records;
  };

}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 299 $   $Date: 2005-02-16 09:37:33 -0600 (Wed, 16 Feb 2005) $
 * $Id: MCFRecordParticle.h 299 2005-02-16 15:37:33Z jnkim $ 
 ***************************************************************************/
