//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/**@file RecordParticle.h
 * @brief Declare the builder class for particle-related recorders.
 */
#ifndef OHMMS_RECORDPARTICLELIST_H_
#define OHMMS_RECORDPARTICLELIST_H_

#include "Particle/ParticleSet.h"
#include "OhmmsData/RecordProperty.h"

namespace OHMMS {


  /** Record engine for a ParticleSet
   *
   * A number of particle attributes is written at every stride step.
   */
  class RecordParticle: 
    public ParticleTags,
    public RecordProperty {

  public:

    typedef ParticleSet Particle_t;

    RecordParticle(Particle_t& pin): ref_(pin) { }

    ~RecordParticle();

    void reset(const char* fileroot);

    void report(int iter);

    void finalize();

    bool put(xmlNodePtr cur);

    void setAnimationRecord(int freq);

  private:
    ///The reference to the ParticleSet whose attribute(s) will be recorded.
    Particle_t& ref_;

    ///The list of recorded attributes.
    vector<RecordProperty*> Recorders;
  };

  /** Save engine for a ParticleSet
   *
   * The current status of ParticleSet is saved. The content is overwritten
   * at every stride step. Only the attributes in ParticleSet::RestartList
   * are saved. 
   */
  class SaveParticle:
    public ParticleTags,
    public RecordProperty {
  public:

    typedef ParticleSet Particle_t;

    SaveParticle(Particle_t& pin): ref_(pin), Dump(NULL), DumpType(0)
    { }

    ~SaveParticle();

    void reset(const char* fileroot);

    void report(int iter);

    void finalize() { }

    bool put(xmlNodePtr cur);

  private:
    int DumpType;
    Particle_t& ref_;
    RecordProperty *Dump;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 299 $   $Date: 2005-02-16 09:37:33 -0600 (Wed, 16 Feb 2005) $
 * $Id: RecordParticle.h 299 2005-02-16 15:37:33Z jnkim $ 
 ***************************************************************************/
