//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "ParticleIO/RecordParticleAttrib.h"

#define PTCLATTRIB_HEAD(o,a,b,c,d) \
	{o << "# attrib : name=" << a << " datatype=" << b << " size=" << c \
          << " format=" << d << endl;}

namespace OHMMS {

  ////////////////////////////////////////////////////////////////////
  //functions of RecordIndexAttrib
  ////////////////////////////////////////////////////////////////////
  void RecordIndexAttrib::reset(const char* fileroot) {
    FileName = fileroot;
    FileName.append(".");
    FileName.append(ref_.objName());
    ofstream fout(FileName.c_str());
    PTCLATTRIB_HEAD(fout,ref_.objName(),ParticleTags::indextype_tag,ref_.size(),"format -xny")
  }

  void RecordIndexAttrib::report(int iter) {
    ofstream fout(FileName.c_str(),ios::app);
    fout << iter << " ";
    copy(ref_.begin(), ref_.end(), ostream_iterator<value_type>(fout," "));
    fout << endl;
  }

  bool RecordIndexAttrib::put(xmlNodePtr cur) {
    return true;
  }

  ////////////////////////////////////////////////////////////////////
  //functions of RecordScalarAttrib
  ////////////////////////////////////////////////////////////////////
  void RecordScalarAttrib::reset(const char* fileroot) {
    FileName = fileroot;
    FileName.append(".");
    FileName.append(ref_.objName());
    ofstream fout(FileName.c_str());
    PTCLATTRIB_HEAD(fout,ref_.objName(),ParticleTags::scalartype_tag,ref_.size(),"format -xny")
  }

  void RecordScalarAttrib::report(int iter) {
    ofstream fout(FileName.c_str(),ios::app);
    fout.precision(5);
    fout << iter << " ";
    copy(ref_.begin(), ref_.end(), ostream_iterator<value_type>(fout," "));
    fout << endl;
  }

  bool RecordScalarAttrib::put(xmlNodePtr cur) {
    return true;
  }

  ////////////////////////////////////////////////////////////////////
  //functions of RecordVectorAttrib
  ////////////////////////////////////////////////////////////////////
  void RecordVectorAttrib::reset(const char* fileroot) {
    FileName = fileroot;
    FileName.append(".");
    FileName.append(ref_.objName());
    ofstream fout(FileName.c_str());
    PTCLATTRIB_HEAD(fout,ref_.objName(),ParticleTags::postype_tag,ref_.size(),"xyz")
  }
  void RecordVectorAttrib::report(int iter) {
    ofstream fout(FileName.c_str(),ios::app);
    fout.precision(5);
    fout << "# iteration " << iter << endl;
    copy(ref_.begin(), ref_.end(), ostream_iterator<value_type>(fout,"\n"));
  }

  bool RecordVectorAttrib::put(xmlNodePtr cur) {
      return true;
  }


  ////////////////////////////////////////////////////////////////////
  //functions of RecordVectorAttrib
  ////////////////////////////////////////////////////////////////////
  void RecordTrajectory::reset(const char* fileroot) {
    if(_file) {
      *_file << "</particlesets>" << endl;
      delete _file;
    }
    string FileName(fileroot);
    FileName.append(".cxyz");
    _file = new ofstream(FileName.c_str());
    _file->precision(5);
    *_file << "<?xml version=\"1.0\"?>" << endl;
    *_file << "<particlesets type=\"animation\">" << endl;
  }

  void RecordTrajectory::finalize() {
    *_file << "</particlesets>" << endl;
  }

  void RecordTrajectory::report(int iter) {
    *_file << "<particleset name=\"frame"  << iter << "\" size=\"" 
	   << ref_.size() << "\">" << endl;
    lattice_out.get(*_file);
    *_file << "<attrib name=\"" << ref_.getName() 
           << "\" " <<ParticleTags::datatype_tag << "=\"" << ParticleTags::postype_tag
           << "\" " <<ParticleTags::condition_tag << "=\"" << ref_.getUnit() <<"\">" << endl;
    copy(ref_.begin(), ref_.end(), ostream_iterator<value_type>(*_file,"\n"));
    *_file << "</attrib>" << endl;
    *_file << "</particleset>" << endl;
  }

  bool RecordTrajectory::put(xmlNodePtr cur) {
      return true;
  }

  ////////////////////////////////////////////////////////////////////
  //functions of RecordComboAttrib
  ////////////////////////////////////////////////////////////////////
  void RecordComboAttrib::reset(const char* fileroot) {
    FileName = fileroot;
    FileName.append(".xyz");
    ofstream fout(FileName.c_str());
  }
  void RecordComboAttrib::report(int iter) {
    ofstream fout(FileName.c_str(),ios::app); 
    fout.setf(ios::scientific, ios::floatfield);
    fout.precision(8);
    fout << ID.size() << endl;
    fout << "# iteration " << iter << " " 
	 << Lattice.a(0) << Lattice.a(1) << Lattice.a(2) << endl;
    if(R.getUnit() == PosUnit::LatticeUnit) {
      for(int i=0; i<ID.size(); i++)
	fout << Species[ID[i]] << " " << Lattice.toCart(R[i]) << endl;
    } else {
      for(int i=0; i<ID.size(); i++)
	fout << Species[ID[i]] << " " << R[i] << endl;
    }
  }

  bool RecordComboAttrib::put(xmlNodePtr cur) {
    return true;
  }


  ////////////////////////////////////////////////////////////////////
  //functions of RecordAddAttrib
  ////////////////////////////////////////////////////////////////////
  void RecordAddAttrib::reset(const char* fileroot) {
    FileName = fileroot;
    FileName.append(".trajectory");
    ofstream fout(FileName.c_str());
    PTCLATTRIB_HEAD(fout,R1.objName(),ParticleTags::postype_tag,R1.size(),"xyz")
  }
  void RecordAddAttrib::report(int iter) {
    ofstream fout(FileName.c_str(),ios::app);
    fout << "# iteration " << iter << endl;
    if(R1.getUnit() == PosUnit::LatticeUnit) {
      for(int i=0; i<R1.size(); i++)
	fout << Lattice.toCart(R1[i]) + R2[i] << endl;
    } else {
      for(int i=0; i<R1.size(); i++)
	fout << R1[i]+R2[i] << endl;
    }
  }

  bool RecordAddAttrib::put(xmlNodePtr cur) {
    return true;
  }

}


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 255 $   $Date: 2005-01-14 11:41:16 -0600 (Fri, 14 Jan 2005) $
 * $Id: RecordParticleAttrib.cpp 255 2005-01-14 17:41:16Z jnkim $ 
 ***************************************************************************/
