//////////////////////////////////////////////////////////////////
//                        COPYRIGHT NOTICE
//   (c) Copyright 1998-2002 David A. Richie and Jeongnim Kim.
//   All rights reserved.
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   David A. Richie
//   Department of Physics, Ohio State University
//   174 W 18th Avenue
//   Columbus, OH 43210
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//
// \class HDFParticleIO
//
// Records HDF 5 library, directly using C not c++ hdf5 implementation
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_HDFPARTICLEIO_H
#define OHMMS_HDRPARTICLEIO_H

#include <bitset>

#include "Particle/OhmmsParticle.h"
using namespace OHMMS;

#include "ParticleIO/ParticleIOBase.h"
#include "Utilities/SpeciesCollection.h"
#include "Utilities/SimpleParser.h"


/*! \class HDFParticleIO
 *  \author Jeongnim Kim
 *  \brief A dervied class from ParticleIOBase
 *  
 *  Record in HDF format.
 */

class  HDFParticleIO: public ParticleIOBase<Scalar_t,OHMMS_DIM> {

public:

  typedef  Scalar_t T;
  typedef  Particle_t::ParticlePos_t       ParticlePos_t;
  typedef  Particle_t::ParticleScalar_t    ParticleScalar_t;
  typedef  Particle_t::ParticleIndex_t     ParticleIndex_t;
  typedef  Particle_t::SingleParticlePos_t SingleParticlePos_t;
 
  HDFParticleIO(ThisParticle_t& p, int i, const char* name = NULL);

  ~HDFParticleIO() {  }

  void read(istream&);
  void write();
  void record(int);// record a configuration (time-stepping)
  void finalize();
  bool put(const char* );

private:
  void setup();

  vector<ParticlePos_t*>    pos_list;
  vector<ParticleScalar_t*> val_list;
  vector<ParticleIndex_t*>  ind_list;
  vector<string> pos_file;
  vector<string> val_file;
  vector<string> ind_file;
  char metafile[128], datafile[128];
  bool rec_lattice;
  std::bitset<8> HDFMode;
};

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: HDFParticleIO.old.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
