//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#if defined(__GNUC__)
#include "Utilities/OhmmsBitset.h"
#else
#include <bitset>
using std::bitset;
#endif

#include "Particle/OhmmsParticle.h"
using namespace OHMMS;
#include "Particle/MetisParticleIO.h"
#include "ParticleBase/ParticleInputTemplate.h"
#include "Particle/PtclGeometryBase.h"
#include "Particle/NNEngine.h"

MetisParticleIO::~MetisParticleIO() {

}

void MetisParticleIO::read() {

  if(ioMode == INPUT) {

    DEBUGMSG("Performing MetisParticleIO::read()");
    ifstream fpos(FileRoot.c_str());
    if(!fpos) {
      ERRORMSG("Cannot find an input file " << FileRoot)
      return;
    }

    vector<string> slist;
    int uc_grid[D];
    makeLattice<ThisParticleLayout_t>::apply(MyPtcl.Lattice, uc_grid, fpos);

    fpos.seekg(0,ios::beg);
    int natom = 0;
    int numpart = 1; // number of partition
    bool posInUnit=false, reshaped=false;
    do {
      int argc = getwords(slist,fpos);
      if(argc == 0) continue;
      if(slist[0] == "number_of_ions") {
        for(int i=1; i<slist.size(); i++)  natom += atoi(slist[i].c_str());
      } else if(slist[0] == "<shape>") {
        options.insert(options.end(),slist[0]);
	do {
	  getwords(vlist,fpos);
	  options.insert(options.end(),vlist.begin(), vlist.end());
	} while(vlist[0] != "</shape>");
	//ParticleBuilder::set(fpos);
        reshaped = true;
      } else if(slist[0] == "positions") {
        if(slist[1][0] == 'u' || slist[1][0] == 'U')  
	  posInUnit = true; 
        else
	  posInUnit = false; 
        break;
      }
    } while(!fpos.eof());

    SpeciesBase* species = SpeciesCollection::getSpecies();

    ThisParticle_t bPtcl;
    bPtcl.Lattice = MyPtcl.Lattice;
    MyPtcl.Lattice.set(bPtcl.Lattice,uc_grid);

    int nctot = uc_grid[0]*uc_grid[1]*uc_grid[2];
    vector<SingleParticlePos_t> ptmp;//temporary position vector
    vector<int>                 itmp;//temporary ionid vector

    if(reshaped || nctot > 1) {
      //if the input configuration is used for a new conf..
      if(natom) {
	bPtcl.create(natom);
	for(int i=0; i<natom; i++) {
	  int argc =  getwords(slist, fpos);
	  if(isdigit(slist[0][0])) 
	    bPtcl.IonID[i] = atoi(slist[0].c_str())-1;
	  else 
	    bPtcl.IonID[i] = species->getSpeciesID(slist[0]);
	  getValue<SingleParticlePos_t>::get(bPtcl.R[i], slist.begin()+1);
	}
      }

      if(!posInUnit) {//first convert to unit
       	for(int i=0; i<natom; i++)
          bPtcl.R[i] = bPtcl.Lattice.toUnit(bPtcl.R[i]);
      }	

      ptmp.reserve(nctot*natom);//reserve a vector
      itmp.reserve(nctot*natom);//reserve a vector

      PtclGeometryBase* sysgeom = createPtclGeometry(options,bPtcl.Lattice);

      SingleParticlePos_t pos,del;
      int ionid;
      for(int ic=0; ic<uc_grid[0]; ic++) {
        del[0] = static_cast<Scalar_t>(ic);
	for(int jc=0; jc<uc_grid[1]; jc++) {
          del[1] = static_cast<Scalar_t>(jc);
	  for(int kc=0; kc<uc_grid[2]; kc++) {
            del[2] = static_cast<Scalar_t>(kc);
	    for(int nb=0; nb<bPtcl.getLocalNum(); nb++) {
              pos = bPtcl.Lattice.toCart(bPtcl.R[nb]+del);
              ionid = bPtcl.IonID[nb];
              if(sysgeom->keepit(pos,ionid)) {
                ptmp.push_back(pos);
                itmp.push_back(ionid);
	      }
	    }
	  }
	}
      }

      delete sysgeom;

      MyPtcl.create(ptmp.size());
      for(int i=0; i<ptmp.size(); i++) MyPtcl.R[i] = ptmp[i];
      for(int i=0; i<ptmp.size(); i++) MyPtcl.IonID[i] = itmp[i];
      for(int i=0; i<ptmp.size(); i++) MyPtcl.ID[i] = i;
    } else {

      if(natom) {
	MyPtcl.create(natom);
	for(int i=0; i<natom; i++) {
	  int argc =  getwords(slist, fpos);
	  if(isdigit(slist[0][0])) 
	    MyPtcl.IonID[i] = atoi(slist[0].c_str())-1;
	  else 
	    MyPtcl.IonID[i] = species->getSpeciesID(slist[0]);
	  getValue<SingleParticlePos_t>::get(MyPtcl.R[i], slist.begin()+1);
          MyPtcl.ID[i] = i;
	}
      }
      ptmp.resize(natom);      
      itmp.resize(natom);      
      for(int i=0; i<natom; i++) ptmp[i] = MyPtcl.R[i];
      for(int i=0; i<natom; i++) itmp[i] = MyPtcl.IonID[i];
    }
    
    NearestNeighborBase *nn_engine = NNEngine::getNNEngine(MyPtcl,3);
    nn_engine->getPairlist(MyPtcl, MyPtcl.R, numpart);

    if(numpart>1) {
      int nloc=0;
      for(int ipart=0; ipart<numpart; ipart++) {
	for(int i=0; i<MyPtcl.getLocalNum(); i++) {
	  if(MyPtcl.OmpID[i] != ipart) continue;
	  MyPtcl.R[nloc] = ptmp[i];
	  MyPtcl.IonID[nloc] = itmp[i];
	  MyPtcl.ID[nloc] = ipart;
	  nloc++;
	}
      }
    }


  } else {
    ERRORMSG("read for OUPUT/PROGRESS mode is not allowed.\n")
  }
}

void MetisParticleIO::record(int iter) {

}

void MetisParticleIO::write() {

}

void MetisParticleIO::setup() {

}


void MetisParticleIO::finalize() {

}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: MetisParticleIO.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

