//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_PARTICLE_INPUT_FACTORY_H
#define OHMMS_PARTICLE_INPUT_FACTORY_H

/////////////////////////////////////////////
// \author Jeongnim Kim
// \class ParticleInputFactory
// \brief Declaration of static functions to intialize a particle set from a file
// or an istream
//
// Instead of using ParticleIOFactory, two functions are introduced.
// The functions instantiate an ParticleIOBase to read data and
// properly initialize the particle sets.
// Currently available formats are:
//  hdf5 : using hdf5 library, always *.meta and *.data pair, see hdf*.meta
//  tbmd : using tbmd format, the particle data can be included in a file
//         can use *.meta to include an external file
//  xyz  : using xyz format, always *.meta and *.data pair
/////////////////////////////////////////////

#include <iostream>
using namespace std;

#include "Ohmms.h"
#include "Particle/OhmmsParticle.h"
using namespace OHMMS;

struct ParticleInputFactory {

  //initialize a particle set using options
  // options typically contain a file name which containt 
  // lattice and particle data
  static void createParticle(Particle_t&, int argc, char **agrv);
  static void createParticle(Particle_t&, vector<string>& arglist);

  //initialize a particle set from an istream
  static void createParticle(Particle_t&, istream& );

  // process lattice information
  static void read_lattice(istream& fpos, ParticleLayout_t& lat, int *uc_grid);

};

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: ParticleInputFactory.h 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
