//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_PARTICLEINPUTTEMPLATE_H
#define OHMMS_PARTICLEINPUTTEMPLATE_H
#include "Utilities/SimpleParser.h"

template<class T>
struct getValue{};

template<class T, unsigned D>
struct getValue<TinyVector<T,D> >{

};

template<class T>
struct getValue<TinyVector<T,3> > {
  typedef vector<string>::iterator iter_t;
  inline static void get(TinyVector<T,3>& r, iter_t it) {
    r[0] = atof((*it).c_str());
    r[1] = atof((*(++it)).c_str());
    r[2] = atof((*(++it)).c_str());
  }
};

/////////////////////////////////////////////////////////////
// Generic functor to expand a set of unit cell by l x m x n
/////////////////////////////////////////////////////////////

template<class T> struct expandLattice { };

/////////////////////////////////////////////////////////////
// specialized for CrystalLattice<T,D>
/////////////////////////////////////////////////////////////

template<class T, unsigned D>
struct expandLattice< CrystalLattice<T,D> > { };

/////////////////////////////////////////////////////////////
// specialized for CrystalLattice<T,3>
/////////////////////////////////////////////////////////////
template<class T>
struct expandLattice< CrystalLattice<T,3> > {

  static void apply(ParticleBase<CrystalLattice<T,3> >& p, int *ng) {

    typedef typename CrystalLattice<T,3>::SingleParticlePos_t SingleParticlePos_t;
    int ngtot = ng[0]*ng[1]*ng[2];

    if(ngtot > 1) {
      bool inUnit = p.R.InUnit;
      CrystalLattice<T,3> lat0 = p.Lattice;
      p.Lattice.set(lat0, ng); // expand the supercell

      int nloc = p.getLocalNum();
      int nglo = p.getTotalNum();
      int ntot = nloc*(ngtot-1);
      int nat = nloc;

      p.create(ntot);
      for(int ig=0; ig<ng[2]; ig++) {
        for(int jg=0; jg<ng[1]; jg++) {
          for(int kg=0; kg<ng[0]; kg++) {

	    // the original particle set already exists
            if(ig == 0 && jg == 0 && kg == 0) continue;

            SingleParticlePos_t  shift(static_cast<T>(kg),
	                               static_cast<T>(jg),
	                               static_cast<T>(ig));
            for(int i=0; i<nat; i++) {
              p.ID[nloc] = nglo++;
              p.IonID[nloc] = p.IonID[i];
              if(inUnit) {
                p.R[nloc] = p.Lattice.toUnit(lat0.toCart(p.R[i]+ shift));
	      } else {
                p.R[nloc] = p.R[i] + lat0.toCart(shift);
	      }
              nloc++;
            } // number of atoms
          } // x-direction
        } // y-direction
      } // z-direction

      // special care for the original position vectors
      // cartesian vectors do not need any update
      if(inUnit) { // convert positions in the new lattice
        for(int i=0; i<nat; i++) p.R[i] = p.Lattice.toUnit(lat0.toCart(p.R[i]));
      }
      p.update();
    } // ngtot > 1
  }

};

/////////////////////////////////////////////////////////////
// Generic functor to make CrystalLattice from an input
/////////////////////////////////////////////////////////////
template<class T>
struct makeLattice {
};

/////////////////////////////////////////////////////////////
// Specialized for CrystalLattice<T,D>: do nothing
/////////////////////////////////////////////////////////////
template<class T, unsigned D>
struct makeLattice< CrystalLattice<T,D> > {
};

/////////////////////////////////////////////////////////////
// Specialized for CrystalLattice<T,2>
/////////////////////////////////////////////////////////////
template<class T>
struct makeLattice< CrystalLattice<T,2> > {

  static void apply(CrystalLattice<T,2>& lat, int *ng, iostream& fpos) {

    ng[0] = 1; ng[1] = 1;
    T a0tmp=1.0;
    T vtmp[] = {1.0,0.0,0.0, 1.0};
    do {
      getwords(slist,fpos);
    } while(slist[0] != "<lattice>" && !(fpos.eof()));

    if(fpos.eof()) {
      ERRORMSG("No Lattice Parameters are set. Making a huge supercell.")
      vtmp[0] = 1e10; vtmp[3] = 1e10; 
      lat.set(a0tmp, vtmp);
      return;
    }

    bool tofindend = true;
    vector<string> slist;
    do {
      int nw = getwords(slist,fpos);
      if(nw == 0) continue;
      if(slist[0] == "a0") {
        a0tmp = atof(slist[1].c_str());
      }else if(slist[0] == "a_1") {
        vtmp[0] = atof(slist[1].c_str());
        vtmp[1] = atof(slist[2].c_str());
      } else if(slist[0] == "a_2") {
        vtmp[2] = atof(slist[1].c_str());
        vtmp[3] = atof(slist[2].c_str());
      } else if(slist[0] == "unit_cell_grid") {
        ng[0] = atoi(slist[1].c_str());
        ng[1] = atoi(slist[2].c_str());
      } else if(slist[0] == "grid") {
        lat.Grid[0] = atoi(slist[1].c_str());
        lat.Grid[1] = atoi(slist[2].c_str());
      } else if(slist[0] == "bconds") { // boundary conditions
        if(slist[1][0] == 'n' || slist[1][0] == 'N') {
	  lat.BConds[0] = ParticleNoBCond;
          lat.BoxBConds[0] = false;
        } 
        if(slist[2][0] == 'n' || slist[2][0] == 'N'){
	  lat.BConds[1] = ParticleNoBCond;
          lat.BoxBConds[1] = false;
        }
      } else if(slist[0] == "</lattice>") {
        break;
      }
    } while(!fpos.eof()); //while(tofindend);

    lat.set(a0tmp, vtmp);
  }
};

/////////////////////////////////////////////////////////////
// Specialized for CrystalLattice<T,3>
/////////////////////////////////////////////////////////////
template<class T>
struct makeLattice< CrystalLattice<T,3> > {

  static void apply(CrystalLattice<T,3>& lat, int *ng, istream& fpos) {

    ng[0] = 1;    ng[1] = 1;    ng[2] = 1;
    vector<string> slist;
    T a0tmp=1.0;
    T vtmp[] = {1.0e0, 0.0e0, 0.0e0, 
                0.0e0, 1.0e0, 0.0e0, 
                0.0e0, 0.0e0, 1.0e0};
    do {
      getwords(slist,fpos);
    } while(slist[0] != "<lattice>" && !(fpos.eof()));

    if(fpos.eof()) {
      ERRORMSG("No Lattice Parameters are set. Making a huge supercell.")
      vtmp[0] = 1e10; vtmp[4] = 1e10;  vtmp[8] = 1e10;
      lat.set(a0tmp, vtmp);
      return;
    }

    do 
    {
      int nw = getwords(slist,fpos);
      if(nw == 0) continue;
      if(slist[0] == "a0") {
        a0tmp = atof(slist[1].c_str());
      }else if(slist[0] == "a_1") {
        vtmp[0] = atof(slist[1].c_str());
        vtmp[1] = atof(slist[2].c_str());
        vtmp[2] = atof(slist[3].c_str());
      } else if(slist[0] == "a_2") {
        vtmp[3] = atof(slist[1].c_str());
        vtmp[4] = atof(slist[2].c_str());
        vtmp[5] = atof(slist[3].c_str());
      } else if(slist[0] == "a_3") {
        vtmp[6] = atof(slist[1].c_str());
        vtmp[7] = atof(slist[2].c_str());
        vtmp[8] = atof(slist[3].c_str());
      } else if(slist[0] == "unit_cell_grid") {
        ng[0] = atoi(slist[1].c_str());
        ng[1] = atoi(slist[2].c_str());
        ng[2] = atoi(slist[3].c_str());
      } else if(slist[0] == "grid") {
        lat.Grid[0] = atoi(slist[1].c_str());
        lat.Grid[1] = atoi(slist[2].c_str());
        lat.Grid[2] = atoi(slist[3].c_str());
      } else if(slist[0] == "bconds") { // boundary conditions
        if(slist[1][0] == 'n' || slist[1][0] == 'N') {
	  lat.BConds[0] = ParticleNoBCond;
          lat.BoxBConds[0] = false;
        }
        if(slist[2][0] == 'n' || slist[2][0] == 'N') {
	  lat.BConds[1] = ParticleNoBCond;
          lat.BoxBConds[1] = false;
        }
        if(slist[3][0] == 'n' || slist[3][0] == 'N') {
	  lat.BConds[2] = ParticleNoBCond;
          lat.BoxBConds[2] = false;
        }
      } else if(slist[0] == "</lattice>") {
        break;
      }
    } while(!fpos.eof()); //while(tofindend);
    lat.set(a0tmp, vtmp);
  }
};



#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: ParticleInputTemplate.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
