//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_PARTICLE_GEOMETRY_BASE_H
#define OHMMS_PARTICLE_GEOMETRY_BASE_H

#include "Particle/OhmmsParticle.h"
using namespace OHMMS;

  /*! \class PtclGeometryBase
   *  \author Jeongnim Kim
   *  \brief An abstract class to define a shape of a particle system
   */
  struct PtclGeometryBase {

    typedef Particle_t::ParticlePos_t       ParticlePos_t;
    typedef Particle_t::ParticleIndex_t     ParticleIndex_t;
    typedef Particle_t::SingleParticlePos_t SingleParticlePos_t;

    PtclGeometryBase() { }
    ~PtclGeometryBase() { }

    virtual void set(const vector<string>&) {}
    virtual void set(const ParticleLayout_t&) { }

    virtual bool keepit(SingleParticlePos_t& p, int& id) {
      return true;
    }
  };

  ////////////////////////////////////////////////////////////////
  //PtclGeometryBase is usally temporary object. Create and delete.
  ////////////////////////////////////////////////////////////////
  extern PtclGeometryBase* 
  createPtclGeometry(const vector<string>&, const ParticleLayout_t&);

  extern bool
  changePtclNumbers(const vector<string>&);

///////////////////////////////////////////////////////////////
//SingletonPattern may work but not necessary
//////////////////////////////////////////////////////////////////
//  class ParticleBuilder {
//   public:
//    static void create(const Particle_t&, Particle_t&, const int*);  
//    static void set(istream& );
//    static PtclGeometryBase* getPtclGeometry();
//    static PtclGeometryBase* getPtclGeometry(const vector<int>&);
//   private:
//    ParticleBuilder() { }  
//    ~ParticleBuilder() { }  
//    static PtclGeometryBase* Shape;
//  };

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: PtclGeometryBase.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
