//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include <stdio.h>
#include "Particle/ParticleIOFactory.h"
#include "Utilities/SpeciesCollection.h"
#include "Utilities/SimpleParser.h"
#include "Particle/QuantumDotIO.h"
#include "ParticleBase/ParticleInputTemplate.h"

void QuantumDotIO::read() {

  ifstream fpos(FileRoot.c_str());
  if(!fpos) {
    ERRORMSG("Cannot find an input file " << FileRoot)
    return;
  }

  int grid[3];  
  makeLattice<ThisParticleLayout_t>::apply(MyPtcl.Lattice, grid, fpos);

  SpeciesBase* species = SpeciesCollection::getSpecies();
  MyPtcl.R.InUnit = false; 

  vector<SingleParticlePos_t> baseatom, selectedatoms;
  vector<int> ionid,selectedids;

  fpos.seekg(0,ios::beg);
  vector<string> slist;
  int  natom = 0;
  double rcut = 0;
  SingleParticlePos_t 
  origin(static_cast<double>(grid[0])*0.5, 
         static_cast<double>(grid[1])*0.5,
         static_cast<double>(grid[2])*0.5);

  do 
    {
      int argc = getwords(slist, fpos);
      if(argc == 0) continue;
      if(slist[0] == "radius") {
        rcut = atof(slist[1].c_str());
      } else if(slist[0] == "center") {
        origin[0] = atof(slist[1].c_str());
        origin[1] = atof(slist[2].c_str());
        origin[2] = atof(slist[3].c_str());
      } else if(slist[0] == "number_of_ions") {
        for(int i=1; i<slist.size(); i++)  natom += atoi(slist[i].c_str());
      } else if(slist[0] == "positions") {
        break;
      }
    } while(!fpos.eof());

  if(natom) {
    baseatom.resize(natom);
    ionid.resize(natom);
    for(int i=0; i<natom; i++) {
      getwords(slist, fpos);
      ionid[i] = atoi(slist[0].c_str())-1;
      getValue<SingleParticlePos_t>::get(baseatom[i],slist.begin()+1);
    }
  } else {
    ERRORMSG("No particles are given. No action is taken.")
    return;
  }

  cout << "Center of QD = " << origin << endl;
  cout << "Radius of QD = " << rcut << endl;

  selectedatoms.resize(natom*grid[0]*grid[1]*grid[2]);
  selectedids.resize(natom*grid[0]*grid[1]*grid[2]);

  rcut = rcut*rcut; // radius of a quantum dots

  int ntot=0;  

  SingleParticlePos_t x,del;
  for(int ic=0; ic<grid[0]; ic++) {
    for(int jc=0; jc<grid[1]; jc++) {
      for(int kc=0; kc<grid[2]; kc++) {
        del = SingleParticlePos_t(double(ic), double(jc), double(kc))-origin;
	for(int nb=0; nb<natom; nb++) {
          x = baseatom[nb]+del;
          if(dot(x,x) < rcut) {

            selectedatoms[ntot] = x + origin;
            selectedids[ntot] = ionid[nb];
            ntot++;
	  }                        
	}
      }
    }
  }
  cout << "Number of atoms in a QD = " << ntot << endl;

  // so far, in the unit of input lattice
  ThisParticleLayout_t Lattice0 = MyPtcl.Lattice;  
  MyPtcl.Lattice.set(Lattice0,grid);

  MyPtcl.create(ntot);
  ofstream fout("lattice.xyz");
  MyPtcl.R.InUnit = false; // using cartesian

  //MyPtcl.addAttribute(SCALARSTR,"radius");
  //ParticleScalar_t& rad = *(MyPtcl.getScalarAttrib("radius"));

  fout << ntot << endl << endl; 
  for(int i=0; i<ntot; i++) {
    MyPtcl.ID[i] = i;
    MyPtcl.IonID[i] = selectedids[i];
    MyPtcl.R[i] = Lattice0.toCart(selectedatoms[i]);
    //SingleParticle dr = MyPtcl.R[i]-origin; // R in cartesian (convert)
    //rad[i] = sqrt(dot(dr,dr));
    fout << species->Name[selectedids[i]] << MyPtcl.R[i] << endl;
  }

}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: QuantumDotIO.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

