//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#if defined(__GNUC__)
#include "Utilities/OhmmsBitset.h"
#else
#include <bitset>
using std::bitset;
#endif
#include "Message/OpenMP.h"
#include "Particle/OhmmsParticle.h"
using namespace OHMMS;
#include "Utilities/OhmmsInfo.h"
#include "Particle/RegGridParticleIO.h"
#include "ParticleBase/ParticleInputTemplate.h"
#include "ParticleBase/TbmdParticleIO.h"
#include "Particle/PtclGeometryBase.h"


RegGridParticleIO::~RegGridParticleIO() {

}
//to be used for grid-based calculations
void RegGridParticleIO::read(istream& fpos, int *uc_grid) {

  if(ioMode == INPUT) {

    DEBUGMSG("Performing RegGridParticleIO::read()");

    int  natom = 0;
    vector<string> slist;
    do {
      int argc = getwords(slist,fpos);
      if(argc == 0) continue;
      if(slist[0] == "number_of_ions") {
        for(int i=1; i<slist.size(); i++)  natom += atoi(slist[i].c_str());
      } else if(slist[0] == "positions") {
        if(slist[1][0] == 'u' || slist[1][0] == 'U')  
	  MyPtcl.R.InUnit = true; 
        else
	  MyPtcl.R.InUnit = false; 
        break;
      }
    } while(!fpos.eof());

    int  NumThreads = omp_get_max_threads();

    int grid[D];

    if(natom>0) {

      grid[0] = MyPtcl.Lattice.Grid[0];
      grid[1] = MyPtcl.Lattice.Grid[1];
      grid[2] = MyPtcl.Lattice.Grid[2];

      MyPtcl.Lattice.Grid[0] = uc_grid[0];  
      MyPtcl.Lattice.Grid[1] = uc_grid[1];  
      MyPtcl.Lattice.Grid[2] = uc_grid[2];  

      if(InputFlag.any()) {

        if(InputFlag[UNITCELLGRID]) {
	  LOGMSG ("\tBuilding OMP Particle: expanding or shaping.")
	  buildOMPDistPtcl(fpos,natom,NumThreads);
	} else {
	  LOGMSG("\tBuilding OMP Particle from an existing configuatation.") 
          rebuildOMPDistPtcl(fpos,natom,NumThreads);
	}
      } else {
	LOGMSG("\tBuilding Particle: single thread mode.");
        buildBasicPtcl(fpos,natom,NumThreads);
      }

      ////////////////////////////////////////////////////////
      // all return R in cartesian unit
      ////////////////////////////////////////////////////////
      MyPtcl.R.InUnit = false;

      //reset the grid to commensurate with ompgrid
      MyPtcl.Lattice.Grid[0] = grid[0];  
      MyPtcl.Lattice.Grid[1] = grid[1];  
      MyPtcl.Lattice.Grid[2] = grid[2];  
    }


  } else {
    ERRORMSG("read for OUPUT/PROGRESS mode is not allowed.\n")
  }
}

void 
RegGridParticleIO::buildOMPDistPtcl(istream& fpos, int natom, 
				      int NumThreads) {

  vector<string> slist;
  int uc_grid[3];

  uc_grid[0] = MyPtcl.Lattice.Grid[0];
  uc_grid[1] = MyPtcl.Lattice.Grid[1];
  uc_grid[2] = MyPtcl.Lattice.Grid[2];
  
  //a new configuration has to be build based on lattice or geometry
  SpeciesBase* species = SpeciesCollection::getSpecies();
  ThisParticle_t bPtcl;
  bPtcl.Lattice = MyPtcl.Lattice;
  MyPtcl.Lattice.set(bPtcl.Lattice,uc_grid);

  bool posInUnit = MyPtcl.R.InUnit;
  bPtcl.create(natom);
  SingleParticlePos_t pin;
  for(int i=0; i<natom; i++) {
    int argc =  getwords(slist, fpos);
    if(isdigit(slist[0][0])) 
      bPtcl.IonID[i] = atoi(slist[0].c_str())-1;
    else 
      bPtcl.IonID[i] = species->getSpeciesID(slist[0]);

    //getValue<SingleParticlePos_t>::get(bPtcl.R[i], slist.begin()+1);
    //////////////////////////////////////////////////////////
    //\note Input particle position is restricted in a unit cell under
    // boundary condition
    // bPtcl.R -> temporary position to expand or shaping
    //            in unit-cell unit for efficient set up
    //////////////////////////////////////////////////////////

    getValue<SingleParticlePos_t>::get(pin, slist.begin()+1);
    if(posInUnit) {
      bPtcl.R[i] = bPtcl.Lattice.BConds.wrap(pin);
    } else {
      bPtcl.R[i] = bPtcl.Lattice.BConds.wrap(bPtcl.Lattice.toUnit(pin));
    }
    bPtcl.ID[i] = i;//assign ID according to the local index
  }


  vector<vector<SingleParticlePos_t>* > ptmp;
  vector<vector<int>* >                 itmp;

  int nx=(uc_grid[0]/MyPtcl.Lattice.dGrid[1]->NP[0]);
  if(nx*MyPtcl.Lattice.dGrid[1]->NP[0] != uc_grid[0]) {
    nx++;
  }
  int ny=(uc_grid[1]/MyPtcl.Lattice.dGrid[1]->NP[1]);
  if(ny*MyPtcl.Lattice.dGrid[1]->NP[1] != uc_grid[1]) {
    ny++;
  }
  int nz=(uc_grid[2]/MyPtcl.Lattice.dGrid[1]->NP[2]);
  if(nz*MyPtcl.Lattice.dGrid[1]->NP[2] != uc_grid[2]) {
    nz++;
  }

  int nest = nx*ny*nz*natom;

  for(int inode=0; inode <NumThreads; inode++) {
    ptmp.push_back(new vector<SingleParticlePos_t>(nest));
    itmp.push_back(new vector<int>(nest));
  }

  vector<int> nat(MyPtcl.Lattice.dGrid[2]->getTotalNum(),0);

  {

    // not necessary to OpenMP
    typedef ParticleLayout_t::PtclGrid_t PtclGrid_t;
    typedef Particle_t::SingleParticlePos_t SingleParticlePos_t;
    const PtclGrid_t& ompgrid = *MyPtcl.Lattice.dGrid[1];//omp level
    const PtclGrid_t& basegrid = *MyPtcl.Lattice.dGrid[2];//grid level
    PtclGeometryBase* sysgeom = createPtclGeometry(options,bPtcl.Lattice);

    //force nodeid = 0 when mppgrid has only one mesh element
    int nodeid = 0;// get MPI node
    SingleParticlePos_t dx(1.0/static_cast<Scalar_t>(uc_grid[0]),
			   1.0/static_cast<Scalar_t>(uc_grid[1]),
			   1.0/static_cast<Scalar_t>(uc_grid[2]));

    SingleParticlePos_t del,pos,upos;
    int newid, myID;
    for(int ic=0; ic<uc_grid[0]; ic++) {
      del[0] = static_cast<Scalar_t>(ic);
      for(int jc=0; jc<uc_grid[1]; jc++) {
	del[1] = static_cast<Scalar_t>(jc);
	for(int kc=0; kc<uc_grid[2]; kc++) {
	  del[2] = static_cast<Scalar_t>(kc);

	  for(int nb=0; nb<bPtcl.getLocalNum(); nb++) {

	    pos = bPtcl.Lattice.toCart(bPtcl.R[nb]+del);
            upos = MyPtcl.Lattice.toUnit(pos);

	    //if(mpigrid.loc(upos) != myID) continue;
            //if(ompgrid.loc(upos) != myID) continue;
            myID = ompgrid.loc(upos);
	    newid = bPtcl.IonID[nb];
	    if(sysgeom->keepit(pos,newid)) {
              int iloc = nat[myID];
	      ptmp[myID]->operator[](iloc) = pos;       
	      itmp[myID]->operator[](iloc) = newid;
	      nat[myID]++;
	    }
	  }
	}
      }
    }
    if(sysgeom) delete sysgeom;// clean up
  }

  int nattot = 0;
  MyPtcl.Lattice.dGrid[1]->PtclDist.resize(NumThreads+1);
  MyPtcl.Lattice.dGrid[1]->PtclDist[0] = 0;
  for(int inode=0; inode<NumThreads; inode++) {
    nattot += nat[inode];
    MyPtcl.Lattice.dGrid[1]->PtclDist[inode+1] = nattot;
  }

  MyPtcl.create(nattot);

#pragma omp parallel
  {
    int myID = omp_get_thread_num();
    typedef ParticleLayout_t::PtclGrid_t PtclGrid_t;
    const PtclGrid_t& bgrid = *MyPtcl.Lattice.dGrid[2];//base level

    //starting particle
    int iat=MyPtcl.Lattice.dGrid[1]->PtclDist[myID];

    //number of particle on this thread
    int nloc =MyPtcl.Lattice.dGrid[1]->PtclDist[myID+1] -iat;

    SingleParticlePos_t upos;

    for(int ig=MyPtcl.Lattice.dGrid[2]->NodeDist[myID];
	ig<MyPtcl.Lattice.dGrid[2]->NodeDist[myID+1]; ig++) {

      int n_ig = 0;//number of particle on this node

      for(int iloc=0; iloc<nloc; iloc++) {

        upos = MyPtcl.Lattice.toUnit((*ptmp[myID])[iloc]);
        if(bgrid.loc(upos) != ig) continue;

	MyPtcl.R[iat] = ptmp[myID]->operator[](iloc);        
	MyPtcl.IonID[iat] = itmp[myID]->operator[](iloc);        

#ifdef USE_OPENMP
	MyPtcl.OmpID[iat] = myID;
#endif
	MyPtcl.ID[iat] = iat;
	iat++;
	n_ig++;
      }
      nat[ig] = n_ig;
    }
  }

  int ngtot = MyPtcl.Lattice.dGrid[2]->getTotalNum();
  MyPtcl.Lattice.dGrid[2]->PtclDist.resize(ngtot+1);
  MyPtcl.Lattice.dGrid[2]->PtclDist[0] = 0;
  nattot  = 0;
  for(int inode=0; inode<ngtot; inode++) {
    nattot += nat[inode];
    MyPtcl.Lattice.dGrid[2]->PtclDist[inode+1] = nattot;
  }


  for(int inode=0; inode<NumThreads; inode++)  delete ptmp[inode];
  for(int inode=0; inode<NumThreads; inode++)  delete itmp[inode];

}

void 
RegGridParticleIO::rebuildOMPDistPtcl(istream& fpos, int natom, int NumThreads) {

  ///////////////////////////////////////////////////////
  // Not creating any new OMP map
  ///////////////////////////////////////////////////////
  MyPtcl.create(natom);
  vector<string> slist,vlist;
  SpeciesBase* species = SpeciesCollection::getSpecies();
  bool posInUnit = MyPtcl.R.InUnit;
  SingleParticlePos_t pin;

  for(int i=0; i<natom; i++) {
    int argc =  getwords(slist, fpos);
    if(isdigit(slist[0][0])) 
      MyPtcl.IonID[i] = atoi(slist[0].c_str())-1;
    else 
      MyPtcl.IonID[i] = species->getSpeciesID(slist[0]);

    //getValue<SingleParticlePos_t>::get(MyPtcl.R[i], slist.begin()+1);
    getValue<SingleParticlePos_t>::get(pin, slist.begin()+1);

    // At run time, the particle positions are limited to a unit cell
    if(posInUnit) {
      MyPtcl.R[i] = MyPtcl.Lattice.toCart(MyPtcl.Lattice.BConds.wrap(pin));
    } else {
      MyPtcl.R[i] 
	= MyPtcl.Lattice.toCart(MyPtcl.Lattice.BConds.wrap(MyPtcl.Lattice.toUnit(pin)));
    }
  }

  MyPtcl.R.InUnit = false; // unit is converted in cartesin

  bool hasomp = false;
  vector<int> nloc(NumThreads,0);

  while(!fpos.eof()) { // reading extra attributes
    int argc = getwords(slist,fpos);
    if(argc == 0) continue;
    if(slist[0] == POSSTR) // reading a position type
      { 
	MyPtcl.addAttribute(POSSTR, slist[1].c_str());
	ParticlePos_t& pref = *(MyPtcl.getVectorAttrib(slist[1].c_str()));
	for(int i=0; i<natom; i++) 
	  {
	    argc = getwords(vlist,fpos);
	    getValue<SingleParticlePos_t>::get(pref[i], vlist.begin());
	  }
      }
    else if(slist[0] == SCALARSTR) 
     {
	MyPtcl.addAttribute(SCALARSTR, slist[1].c_str());

	ParticleScalar_t& sref = *(MyPtcl.getScalarAttrib(slist[1].c_str()));
	for(int i=0; i<natom; i++) 
	  {
	    argc = getwords(vlist,fpos);
	    sref[i] = atof(vlist[0].c_str());
	  } 
      }
#ifdef USE_OPENMP
    else if(slist[1] == OMPNODEID_TAG) {
      for(int i=0; i<natom; i++) 
	{
	  argc = getwords(vlist,fpos);
	  MyPtcl.OmpID[i] = atoi(vlist[0].c_str());
	  nloc[MyPtcl.OmpID[i]]++;
	}
      hasomp = true;
    }
#endif
    else if(slist[0] == "end") break;
    else if(slist[0] == "end-mdframe") break;
  } 

  ParticleLayout_t::PtclGrid_t& ompgrid = *(MyPtcl.Lattice.dGrid[1]);
  ompgrid.PtclDist.resize(NumThreads+1,0);
  ompgrid.PtclDist[0] = 0;

  if(hasomp) {
    //\todo when the number of processors has changed
    // use old omp distribution
    for(int myID = 0; myID < NumThreads; myID++) {
      ompgrid.PtclDist[myID+1] = ompgrid.PtclDist[myID] + nloc[myID];
    }

  } else {
    int bat=natom/NumThreads;
    int residue = natom%NumThreads;
    for(int myID=0; myID < NumThreads; myID++) {
      if(myID<residue){
        ompgrid.PtclDist[myID] = myID*(bat+1);
      }
      else {
        ompgrid.PtclDist[myID] = myID*(bat+1) + (myID-residue)*bat;
      }
    }
    ompgrid.PtclDist[NumThreads] = natom;


#ifdef USE_OPENMP
    //#pragma omp parallel for
    for(int myID=0; myID<NumThreads; myID++) {
      for(int iat=ompgrid.PtclDist[myID]; iat<ompgrid.PtclDist[myID+1]; iat++)
        MyPtcl.OmpID[iat] = myID;
    }
#endif
  }

//  #ifdef USE_OPENMP
//    else {
//      if(!hasomp) {
//        ParticleIndex_t ionid(natom),id(natom);
//        ParticlePos_t   R(natom);
//        ionid = MyPtcl.IonID;
//        id = MyPtcl.ID;
//        R = MyPtcl.R;

//        for(int i=0; i<natom; i++) {
//  	int nodeid = ompgrid.loc(R[i]);
//  	MyPtcl.OmpID[i] = nodeid;
//  	nloc[nodeid]++;
//        }

//        for(int inode=0; inode < NumThreads; inode++) {
//  	ompgrid.PtclDist[inode+1] = ompgrid.PtclDist[inode] + nloc[inode];
//        }

//  #pragma omp parallel
//        {
//  	int myID = omp_get_thread_num();
//  	int n0 = ompgrid.PtclDist[myID],ng;

//  	for(int i=0; i<natom; i++) {
//  	  if(MyPtcl.OmpID[i] != myID) continue;
//  	  ng = nloc[myID] + n0;
//  	  MyPtcl.ID[ng] = id[i];
//  	  MyPtcl.R[ng] = R[i];
//  	  MyPtcl.IonID[ng] = ionid[i];
//  	}
//        }
//      }
//    }
//  #endif
}

void 
RegGridParticleIO::buildBasicPtcl(istream& fpos, int natom, int NumThreads) {

  SpeciesBase* species = SpeciesCollection::getSpecies();
  int loc = MyPtcl.getLocalNum();

  MyPtcl.create(natom);
  vector<string> slist;
  bool posInUnit = MyPtcl.R.InUnit;
  SingleParticlePos_t pin;
  for(int i=0; i<natom; i++) {
    int argc =  getwords(slist, fpos);
    if(isdigit(slist[0][0])) 
      MyPtcl.IonID[loc] = atoi(slist[0].c_str())-1;
    else 
    MyPtcl.IonID[loc] = species->getSpeciesID(slist[0]);

    //getValue<SingleParticlePos_t>::get(MyPtcl.R[loc], slist.begin()+1);
    getValue<SingleParticlePos_t>::get(pin, slist.begin()+1);

    // At run time, the particle positions are limited to a unit cell
    if(posInUnit) {
      MyPtcl.R[i] = MyPtcl.Lattice.BConds.wrap(pin);
    } else {
      MyPtcl.R[i] 
	= MyPtcl.Lattice.toCart(MyPtcl.Lattice.BConds.wrap(MyPtcl.Lattice.toUnit(pin)));
    }

    loc++;
  }

  int uc_grid[3];
  uc_grid[0] = MyPtcl.Lattice.Grid[0];
  uc_grid[1] = MyPtcl.Lattice.Grid[1];
  uc_grid[2] = MyPtcl.Lattice.Grid[2];

  PtclGeometryBase* sysgeom = createPtclGeometry(options,MyPtcl.Lattice);

  expandLattice<ThisParticleLayout_t>::apply(MyPtcl,uc_grid);

  SingleParticlePos_t pos;
  for(int i=0; i<MyPtcl.getLocalNum(); i++) {

    if(MyPtcl.R.InUnit) 
      pos = MyPtcl.Lattice.toCart(MyPtcl.R[i]);
    else                
      pos = MyPtcl.R[i];

    sysgeom->keepit(pos,MyPtcl.IonID[i]);
    MyPtcl.R[i] = pos;
    MyPtcl.ID[i] =i;
#ifdef USE_OPENMP
    MyPtcl.OmpID[i] = 0;
#endif
  }

  MyPtcl.update(Particle_t::LOCupdate); //local grid update

  while(!fpos.eof()) { // reading extra attributes
    vector<string> vlist;
    int argc = getwords(slist,fpos);
    if(argc == 0) continue;
    if(slist[0] == POSSTR) // reading a position type
      { 
	MyPtcl.addAttribute(POSSTR, slist[1].c_str());
	ParticlePos_t& pref = *(MyPtcl.getVectorAttrib(slist[1].c_str()));
	for(int i=0; i<natom; i++) 
	  {
	    argc = getwords(vlist,fpos);
	    getValue<SingleParticlePos_t>
	      ::get(pref[MyPtcl.ID[i]], vlist.begin());
	  }
      }
    else if(slist[0] == SCALARSTR) 
      {
	MyPtcl.addAttribute(SCALARSTR, slist[1].c_str());

	ParticleScalar_t& sref = *(MyPtcl.getScalarAttrib(slist[1].c_str()));
	for(int i=0; i<natom; i++) 
	  {
	    argc = getwords(vlist,fpos);
	    sref[MyPtcl.ID[i]] = atof(vlist[0].c_str());
	  } 
      }
    else if(slist[0] == "end") break;
    else if(slist[0] == "end-mdframe") break;
  } 

  MyPtcl.update(Particle_t::IDupdate);// update ID
  delete sysgeom;
}


void RegGridParticleIO::record(int iter) {

}

void RegGridParticleIO::write() {

}

void RegGridParticleIO::setup() {

}


void RegGridParticleIO::finalize() {

}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: RegGridParticleIO.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

