//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_SPHERICAL_GEOMETRY_H
#define OHMMS_SPHERICAL_GEOMETRY_H

/*! \class SphericalQD a inherited class from PtclGeometryBase
 *  \author Jeongnim Kim
 *  \brief 
 <shape>
 sphere
 a0 value # scaling function
 radius value
 center value value value 
 exchange A B
 </shape>

 \note unit for center is an input lattice before expanding
 
 exchange makes a embbeded sphere. Species A will be substituted by B
 in the outer layer. 
 IMPORTANT: GaAs/InAs where As in the inner and outer layers are identical,
 add "exchange As As". I decided to force this to reduce additional
 boolean evalulation.

 e.g. to create a spherical qd of radius 2*a AA
 <shape>
 sphere
 a0 5.43
 radius 2 
 center 2 2 2
 </shape>
 */

struct SphericalQD:public PtclGeometryBase {

  SphericalQD():CenterInUnit(true) { }
  ~SphericalQD() { }

  SingleParticlePos_t Center;
  double Radius2;
  bool CenterInUnit;
  map<int,int> exchange;

  void set(const vector<string>& argv) {

    SpeciesBase* species = SpeciesCollection::getSpecies();
    vector<string>::const_iterator it = argv.begin();
    double a0 = 1;
    map<int,int> extmp;
    while(it != argv.end()) {
      if((*it) == "a0") {
	a0 = atof((*(++it)).c_str());
      } else if((*it) == "radius") {
	Radius2 = atof((*(++it)).c_str());
      } else if((*it) == "center") {
	Center[0] = atof((*(++it)).c_str());
	Center[1] = atof((*(++it)).c_str());
	Center[2] = atof((*(++it)).c_str());
      } else if((*it) == "exchange") {
        int a = species->getSpeciesID(*(++it));
        int b = species->getSpeciesID(*(++it));
        exchange[a] = b;
      }
      it++;
    }
    Radius2 *= a0;
  }

  void set(const ParticleLayout_t& lattice) { 
    Center = lattice.toCart(Center);
    Radius2  = Radius2*Radius2;
  }

  bool keepit(SingleParticlePos_t& p, int& id) {
    SingleParticlePos_t pp = p - Center;
    double rsq = dot(pp,pp);
    if(exchange.empty()) {
      if(rsq < Radius2) return true;
      else              return false;
    } else {
      if(rsq > Radius2) id = exchange[id];
      return true;
    }
  }

};


#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SphericalQD.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
