//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <string>
#include <vector>
#include <utility>
#include <iostream>
#include <fstream>
using namespace std;
#include "Utilities/OhmmsInfo.h"
#include "ParticleIO/ParticleIOUtility.h"
#include "ParticleIO/TBMDParticleIO.h"

using namespace OHMMS;
TBMDSaveParticle::~TBMDSaveParticle() { 
}

void TBMDSaveParticle::reset(const char* fileroot){
  FileRoot = fileroot;
  FileRoot.append(".tbmd");
}

void TBMDSaveParticle::report(int iter) {

  ofstream ftbmd(FileRoot.c_str()); // always overwrite
  ftbmd << "<unitcell>" << endl;
  ref_.Lattice.print(ftbmd);
  ftbmd << "</unitcell>" << endl;
  ftbmd <<"<ptclset oname=\"" << ref_.getName() << "\">" << endl;
  ftbmd << "\t<format> tbmd </format> " << endl;
  ftbmd << "\t<name> none  </name>" <<  endl;
  ftbmd <<"<TBMD>" << endl;
  ftbmd << "number_of_ions = " << ref_.getLocalNum()<< endl;
  ftbmd << "positions = " ;
  if(ref_.R.InUnit) ftbmd << " unit\n";
  else                ftbmd << " cartesian\n";
  for(int i=0; i<ref_.getLocalNum(); i++)
    ftbmd << ref_.IonID[i]+1 << " " << ref_.R[i] << endl;
  
  for(int i=3; i<ref_.getNumAttrib(); i++) {
    OhmmsObject& ooref=ref_.getAttrib(i);
    ftbmd << ooref.typeName() << " " << ooref.objName() << " ";
    ooref.get(ftbmd);
    ftbmd << endl;
    if(!strcmp(ooref.typeName(), POSSTR)) {
      const ParticlePos_t& rtmp =*(ref_.getVectorAttrib(i));
      for(int iat=0; iat<ref_.getLocalNum(); iat++) {
	ftbmd << rtmp[iat] << endl;
      }
    } else if(!strcmp(ooref.typeName(), SCALARSTR)) {
      const ParticleScalar_t& stmp =*(ref_.getScalarAttrib(i));
      for(int iat=0; iat<ref_.getLocalNum(); iat++) {
	ftbmd << stmp[iat] << " ";
	if(iat%5 == 4) ftbmd << endl;
      }
      if(ref_.getLocalNum()%5 != 0) ftbmd<< endl;
    } else if(!strcmp(ooref.typeName(), INDEXSTR)){
      const ParticleIndex_t& itmp =*(ref_.getIndexAttrib(i));
      for(int iat=0; iat<ref_.getLocalNum(); iat++) {
	ftbmd << itmp[iat] << " ";
	if(iat%40 == 39) ftbmd << endl;
      }
      if(ref_.getLocalNum()%40 != 0) ftbmd<< endl;
    }
  }
  ftbmd <<"</TBMD>" << endl;
  ftbmd << "</ptclset>" << endl;
  ftbmd.close();
}

bool TBMDSaveParticle::put(xmlNodePtr cur) {
  
  return true;
}


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: TBMDParticleIO.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
