//////////////////////////////////////////////////////////////////
// (c) Copyright 2004- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/*! \class CoulombPotential
 * \author Jeongnim Kim
 * \brief Derived class from PotentialBase.  Implements \f$q_i q_j/r_{ij}\f$.
 */
#ifndef OHMMS_COULOMBPOTENTIAL_H
#define OHMMS_COULOMBPOTENTIAL_H

#include "Potentials/PotentialBase.h"
#include "OhmmsUnits.h"
#include "Potentials/EwaldSum.h"

namespace OHMMS {

  class CoulombPotential:public PotentialBase {

  public:

    typedef EwaldSum<PtclOnLatticeTraits> EwaldEngineType;

    CoulombPotential();
    ~CoulombPotential();

    void setParticleSet(Particle_t* ptclset);
    Scalar_t evalEnergy();
    Scalar_t evalForce(ParticlePos_t& f, unsigned stressmode = 0);
    bool evalDynamicMatrix(DynMatrix_t&);

    bool get(ostream&) const;
    bool put(istream&);
    bool put(xmlNodePtr cur);


  protected:

    bool NotInitialized;

    EwaldEngineType* ewald_engine;
    ParticlePos_t Rcart;
    ParticleScalar_t* Charge;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 308 $   $Date: 2005-02-21 09:21:44 -0600 (Mon, 21 Feb 2005) $
 * $Id: CoulombPotential.h 308 2005-02-21 15:21:44Z jnkim $ 
 ***************************************************************************/

