//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Richard Hennig and Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Potentials/EAMPotential.h"
#include "Message/OpenMP.h"
namespace OHMMS {
  template<class T>
  struct SaveRho{
    unsigned id;
    T drho;
    TinyVector<T,3> displ,dcos;
    inline SaveRho() { } // do nothing
    inline ~SaveRho() { }
  };
    
  //template<class T>
  //struct SaveF{
  //  unsigned id;
  //  T r, rinv, f, df;
  //  TinyVector<T,3> dr;
  //  SaveF() { }
  //  ~SaveF() { }
  //};
  //
  //template<class T>
  //struct SaveG{
  //  T lcos, g, dg;
  //  SaveG() {}
  //  ~SaveG() {}
  //};


  EAMPotential::Scalar_t 
  EAMPotential::evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale) const {
    
    typedef PtclPairList_t::PairData_t    PairData_t;
    const ParticleIndex_t& grID(myPtcl->GroupID);
    // storing three-body function
    int nnloc(0);
    Scalar_t esum(0.0);
    for(int iL = 0; iL<nnlist.getLocalNum(); iL++) {
      int iat(nnlist.I[iL]);
      int num_nn(nnlist.nadj(iL));
      int n3(0);
      int n2(0);
      Scalar_t elocal(0.0);
      Scalar_t val;
      int itype(grID[iat]);
      int bond(itype*EAMSet->nspecies);
      Scalar_t emaX(0.e0);
      PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
      while(nit != nit_end) {
        const PairData_t& cur(*nit);
        Scalar_t r(cur.sep);
	if(r < EAMSet->clsphircut) {
	  int jtype(cur.Group);
	  Scalar_t deriv(EAMSet->phi[bond+jtype].splint(r, val));
	  elocal += 0.5*val; 
          // save data associated with rho(r), if \f$r < r_{cut}^{\rho}\f$
          if(r < EAMSet->clsrhorcut) {
            deriv = EAMSet->rho[jtype].splint(r, val, 0);
            emaX+= val;
          }
	}
        ++nit;
      } // while(nit != nit_end)

      Scalar_t dux = EAMSet->f[itype].splint(emaX, val);
      elocal += val; 
      esum += elocal;
    }
    return esum;
  }

  EAMPotential::Scalar_t 
  EAMPotential::evalForceNN(PtclPairList_t& nnlist, 
			    ParticlePos_t& fv, 
			    unsigned stressmode) const {

    typedef PtclPairList_t::PairData_t    PairData_t;
    ParticleScalar_t& q = myPtcl->E;
    const ParticleIndex_t& grID(myPtcl->GroupID);

    //temporary data for multi-body functions
    vector<SaveRho<Scalar_t> > srho(nnlist.getMaxNadj());
    Scalar_t esumloc(0.0);
    Tensor_t stressloc;
    for(int iL = 0; iL<nnlist.getLocalNum(); iL++) {
      int iat(nnlist.I[iL]);
      int itype(grID[iat]);
      int bond(itype*EAMSet->nspecies);
      int n2(0);
      Scalar_t elocal(0.0);
      Scalar_t val;
      SingleParticlePos_t fon(0.0);
      Scalar_t emaX (0.e0);
      PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
      while(nit != nit_end) {
        const PairData_t& cur(*nit);
	Scalar_t sep(cur.sep);
	if(sep < EAMSet->clsphircut) {
	  int jtype(cur.Group);
	  Scalar_t deriv(0.5*(EAMSet->phi[bond+jtype].splint(sep, val)));
	  elocal += 0.5*val;     
          Scalar_t rinv(1.0/sep);
	  SingleParticlePos_t dcos(rinv*cur.displ);
	  SingleParticlePos_t df(deriv*dcos); 
	  fon += 2.0*df;
	  if(stressmode) stressloc -= outerProduct(df,cur.displ);

	  // save data associated with rho(r), if \f$r < r_{cut}^{\rho}\f$ 
	  if(sep < EAMSet->clsrhorcut) { 
	    deriv = EAMSet->rho[jtype].splint(sep, val, 0);
	    srho[n2].id = cur.Where;
	    srho[n2].drho = deriv;
	    srho[n2].displ = cur.displ;    
	    srho[n2].dcos = dcos;    
            emaX+=val;
	    ++n2;
	  }
	}
        ++nit;
      } // while(nit != nit_end)

      //Scalar_t dux;               // du/dx, used for a force prefactor
      Scalar_t dux(EAMSet->f[itype].splint(emaX, val));
      elocal +=val; 
      
      for(int nj=0; nj < n2; nj++) { // forces from drho
	SingleParticlePos_t df(dux * srho[nj].drho * srho[nj].dcos);
	fon += df;
        fv[srho[nj].id]-=df;
        if(stressmode) stressloc -= outerProduct(df, srho[nj].displ);
      }

      fv[iL] += fon; 
      q[iat] = elocal;
      esumloc += elocal;
    }

    //Assignment to PtclPairList
    nnlist.Stress = stressloc;
    return esumloc;
  }

}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: EAMPotential.1.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

