//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Richard Hennig and Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/*!\class AlloyEAMPotential
 * \author Richard Hennig
 * \author Jeongnim Kim
 * \brief Derived class from PotentialBase.  Implements AlloyEAM potential for single
 *  species.
 * \note  
 * Implemented by Richard Hennig 6/2001
 *
 * Based on MAlloyEAMPotential originally written by Thomas Lenosky, LLNL, 12/8/99
 *
 * Modified to confirm new interfaces of PotentialBase Jeongnim Kim 9/27/2001
 ************************************************************************/
#ifndef OHMMS_EAMPOTENTIAL_H
#define OHMMS_EAMPOTENTIAL_H

#include "Potentials/ShortRangePotential.h"
#include "Potentials/EAMParam.h"

class SpeciesSet;

namespace OHMMS {

  class EAMPotential: public ShortRangePotential{

  public:

    EAMPotential();
    ~EAMPotential();

    /// implementing virtual functions for OhmmsElementBase
    bool get(ostream&) const;
    bool put(istream&);
    void reset();
    bool put(xmlNodePtr cur);

    void resetParticleSet(Particle_t* ptclset) {
      myPtcl=ptclset;
    }

    Scalar_t evalForceNN(PtclPairList_t& nnlist, 
			 ParticlePos_t& f0, 
			 unsigned stressmode) const;

    Scalar_t evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale=1.0) const;

  protected:

    EAMParam<Scalar_t>* EAMSet;
    SpeciesSet* mySpecies;
    bool parseXML(const string& fname);
  };

}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 363 $   $Date: 2005-07-13 13:33:18 -0500 (Wed, 13 Jul 2005) $
 * $Id: EAMPotential.h 363 2005-07-13 18:33:18Z jnkim $ 
 ***************************************************************************/
