//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002, 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_GENERIC_ENVDEP_SHORTRANGEPOTENTIAL_H
#define OHMMS_GENERIC_ENVDEP_SHORTRANGEPOTENTIAL_H

#include "Potentials/ShortRangePotential.h"

namespace OHMMS {

  /** Environment-dependent potential
   *
   * \f[E_{i} = \sum_k f_k (\sum_{j\nei} \phi(r_{ij}))^k\f]
   */
  template<class PPT>
  struct EnvDepPotential: public ShortRangePotential {

    typedef PtclPairList_t::PairData_t PairData_t;
    PPT* PairPot;
    vector<Scalar_t> fc;

    EnvDepPotential(PPT* pp):PairPot(pp){}
    void setFc(const vector<Scalar_t>& f) {
      fc=f;
    }

    ~EnvDepPotential() {
      if(PairPot) delete PairPot;
    }

    /// implementing virtual functions for OhmmsElementBase
    bool get(ostream&) const { return true;}
    bool put(istream&) { return true;}
    bool put(xmlNodePtr cur) { return true;}
    void reset() {}

    void setParticleSet(Particle_t* ptclset) { 
      myPtcl=ptclset;
      PairPot->setParticleSet(ptclset);
    }

    void resetParticleSet(Particle_t* ptclset) { 
      myPtcl=ptclset;
      PairPot->resetParticleSet(ptclset);
    }    

    Scalar_t 
    evalForceNN(PtclPairList_t& nnlist, ParticlePos_t& f0, unsigned stressmode) const {
      ParticleScalar_t& q(myPtcl->E);
      Scalar_t rep, sumloc = 0.0e0,psum = 0.0e0;
      Scalar_t drep;
      Tensor_t stressloc;
      ParticleScalar_t dphi(nnlist.getLocalNum());
      const typename PPT::RadFunc_t& phi(*(PairPot->Phi[0]));
      for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
        int iat(nnlist.I[iL]);
        PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
        Scalar_t xxx=0.0;
        while(nit != nit_end) {
          const PairData_t& cur(*nit);
          if(cur.sep<Rmax){
            xxx+=phi(cur.sep);
          }
          ++nit;
        }
        rep=fc[0]+xxx*(fc[1]+xxx*(fc[2]+xxx*(fc[3]+xxx*fc[4])));
        dphi[iL]=fc[1]+xxx*(2.0*fc[2]+xxx*(3.0*fc[3]+4.0*xxx*fc[4]));
        q[iat] += rep;
        sumloc+=rep;
      }

      for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
        int iat(nnlist.I[iL]);
        PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
        SingleParticlePos_t fon;
        while(nit != nit_end) {
          const PairData_t& cur(*nit);
          if(cur.sep<Rmax){
            drep=phi(cur.sep,rep)*(dphi[iL]+dphi[cur.Next])/cur.sep;
            SingleParticlePos_t df(drep*cur.displ);
            fon+=df;
            f0[cur.Where]-=df;
            stressloc -= outerProduct(df,cur.displ);
          }
          ++nit;
        }
      }


      nnlist.Stress+=stressloc;
      return sumloc;
    }

    Scalar_t 
    evalEnergyNN(const PtclPairList_t& nnlist,Scalar_t rscale=1.0) const {
      Scalar_t esumloc = 0.0e0;
      const typename PPT::RadFunc_t& phi(*(PairPot->Phi[0]));
      for(int iL=0; iL<nnlist.getLocalNum();iL++) {
        Scalar_t xxx=0.0;
        PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
        while(nit != nit_end) {
          Scalar_t sep((*nit).sep);
          if(sep<Rmax) {
            xxx += phi(sep);
          }
          ++nit;
        }
        esumloc +=fc[0]+xxx*(fc[1]+xxx*(fc[2]+xxx*(fc[3]+xxx*fc[4])));
      }
      return esumloc;
    }
  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 397 $   $Date: 2006-01-28 13:07:56 -0600 (Sat, 28 Jan 2006) $
 * $Id: EnvDepPotential.h 397 2006-01-28 19:07:56Z jnkim $ 
 ***************************************************************************/
