//////////////////////////////////////////////////////////////////
// (c) Copyright 2004- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/**@file FastPotential.h
 * @brief Declare FastPotential class.
 */
#ifndef OHMMS_FASTPOTENTIAL_COLLECTION_H
#define OHMMS_FASTPOTENTIAL_COLLECTION_H
#include "Potentials/PotentialBase.h"
namespace OHMMS {
  
  class ShortRangePotential;

  /** Specialized potential for a short-range interaction
   *
   * Template parameter SRP stands for ShortRangePotential.
   */
  class FastPotential: public PotentialBase {

    public:

      ///default constructor
      FastPotential(xmlNodePtr p_ptr=0, xmlNodePtr pl_ptr=0): srp_impl(0) {
        PotXmlNode=p_ptr;
        LayoutXmlNode=pl_ptr;
      }

      ///destructor
      ~FastPotential();

      //@{Implement virtual functions of OhmmsElementBase
      bool get(ostream&) const;
      bool put(istream&);
      bool put(xmlNodePtr cur);
      void reset();
      //@}

      //@{Implement virtual functions of PotentialBase
      void setParticleSet(Particle_t* ptclset);
      void resetParticleSet(Particle_t* ptclset);
      Scalar_t evalEnergy();
      Scalar_t evalForce(ParticlePos_t& f, unsigned fmode = 0);
      bool evalDynamicMatrix(DynMatrix_t&){return false;}
      /** assign xmlNodePtr to Particle Layout object
       * @param pl_ptr an xml node
       */
      void setLayout(xmlNodePtr pl_ptr){LayoutXmlNode=pl_ptr;}

      /** Set the current position to be pos
       *@param pos position array
       */
      void setCurrentPos(ParticlePos_t& pos);

      /** Set the current position to be pos
       *@param pos position array
       *@param wrap bool, if true, call myPtcl->applyBConds()
       *@param update bool, if true (default), update the cell information.
       */
      void setCurrentPos(ParticlePos_t& pos, bool wrap, bool updatecell=true);

      /** Set the current position to be pos and set the displacement vector dr
       *@param pos position array
       *@param dr  displacement array
       */
      void setCurrentPos(const ParticlePos_t& pos, const ParticlePos_t& dr);
      //@}      

      void setSRP(ShortRangePotential *srp) {
        srp_impl=srp;
        CollectVisitingForces = srp_impl->CollectVisitingForces;
      }
    private:

      ShortRangePotential* srp_impl;

      Scalar_t evalForcePairOnly(ParticlePos_t& f, unsigned fmode = 0);
      Scalar_t evalForceMultiple(ParticlePos_t& f, unsigned fmode = 0);
    };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 383 $   $Date: 2005-09-05 15:30:56 -0500 (Mon, 05 Sep 2005) $
 * $Id: FastPotential.h 383 2005-09-05 20:30:56Z jnkim $ 
 ***************************************************************************/
