//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim and Thomas Lenosky
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
#ifndef OHMMS_KEATINGPARAM_H
#define OHMMS_KEATINGPARAM_H
#include <vector>
#include <string>
#include "Utilities/SpeciesSet.h"

struct KeatingParam
{ 

  typedef double      value_t;
  KeatingParam(SpeciesSet& species);

  bool isReady() {
    if(nSpecies) return true;
    else         return false;
  }                

  inline int doublet(int i, int j) const { return j+i*nSpecies;}
  inline int triplet(int i, int j, int k) const { 
    return k+nSpecies*(j+i*nSpecies);
  }

  inline value_t f2(int ib, int r2) const {  
    // expect r*r
    // NOTE: alpha = (3/8 * alpha0/r0^2)/2 
    // twobody: alpha*(r^2 - r0^2)^2
    return alpha[ib]*(r2-r0sq[ib])*(r2-r0sq[ib]);
  }

  inline value_t df2(int ib, value_t r2) const {
    // expect r*r
    // alpha*4*(r^2-r0^2)[r*\hat r] to be multiplied
    return 4.0e0*alpha[ib]*(r2-r0sq[ib]); 
  }

  inline value_t df2(int ib, value_t r, value_t& y) const {
    // expect r*r
    // alpha*4*(r^2-r0^2)[r*\hat r] to be multiplied
    value_t r2 = r*r;
    value_t tmp = alpha[ib]*(r2-r0sq[ib]);
    y = tmp*(r2-r0sq[ib]);
    return 4.0e0*r*tmp;
  }

  inline value_t d2f2(int ib, value_t r, value_t& y_over_r) const {
    // expect r*r
    // alpha*4*(r^2-r0^2)[r*\hat r] to be multiplied
    value_t r2 = r*r;
    value_t tmp = 4*alpha[ib]*(r2-r0sq[ib]);
    y_over_r = tmp;
    return tmp + 8.0e0*alpha[ib]*r2;
  }

  inline value_t f3(unsigned id, value_t r123) const
  { 
    // NOTE: beta = [3/8 * beta0/(r01*r02)]
    return  beta[id]*(r123-theta[id])*(r123-theta[id]);
  }

  inline value_t df3(unsigned id, value_t r123) const
  {  

    return 2.0e0*beta[id]*(r123-theta[id]);
  
  }  

  inline value_t rmax() const {
    return Rmax;
  }

  void print(std::ostream& o) const;
  void readKeatingParam(const char *fname);
      
  SpeciesSet& mySpecies;
  const value_t three_over_eight;
  const value_t tetra;
  const value_t econv;
  int nSpecies;
  value_t Rmax;
  std::vector<value_t>  r0,r0sq;
  std::vector<value_t>  alpha0,alpha;
  std::vector<value_t>  theta0, theta,beta0,beta;
  void initializeParam(int n);
};

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * POOMA_VERSION_ID: $Id: KeatingParam.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

