//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Message/OpenMP.h"
#include "Potentials/KeatingPotential.h"
#include "Potentials/KeatingParam.h"
#include "Utilities/SimpleParser.h"
#include "Utilities/OhmmsInfo.h"
#include "Potentials/StressAnalysis.h"
using namespace OHMMS;

KeatingPotential::KeatingPotential(): 
  VSet(NULL){ 
  CollectVisitingForces = true;
  myName = "KEATING"; 
  RecordPropertyList::registerRecord("vff",new StressAnalysis<KeatingPotential>(*this));
}

KeatingPotential::~KeatingPotential() {
  DEBUGMSG("Calling KeatingPotential::~KeatingPotential()");
}


bool KeatingPotential::get(ostream& os) const { 
  return true;
}
bool KeatingPotential::put(istream&) {return true;}

void KeatingPotential::reset() {

}
bool KeatingPotential::put(xmlNodePtr cur){

  string PotFile("KeatingParam");

#if defined(HAVE_LIBXML2)
  if(cur) {
    const xmlChar* s=xmlGetProp(cur,(const xmlChar*)"src");
    if(s == NULL) { //try with file
      s=xmlGetProp(cur,(const xmlChar*)"file");
    }
    if(s) PotFile = (const char*)s;
    cur = cur->children;
    while(cur != NULL) {
      string cname((const char*)cur->name);
      if(cname == "filename") {
        putContent(PotFile,cur);
      }
      cur = cur->next;
    }
  }
#endif
  if(myPtcl && !VSet) {
    VSet = new KeatingParam(myPtcl->getSpeciesSet());
    VSet->readKeatingParam(PotFile.c_str());
  }
  Rmax = VSet->rmax();
  LOGMSG("\tMaximum cutoff radius " << Rmax)
  return true;
}

//  Scalar_t 
//  KeatingPotential::evalForce(ParticlePos_t& f0, ParticleScalar_t& q,
//                         Tensor_t& stress, unsigned stressmode) {

//    getPairList(); // using a pair-list

//    Scalar_t esum = 0.0;
//    Scalar_t p00 = 0.0e0, p11=0.0e0, p22 = 0.e0, p01 = 0.0, p02=0.0, p12 = 0.0;

//  #if defined(SGI) && defined(ENABLE_OPENMP)
//  #pragma omp parallel reduction(+:esum,p00,p11,p22,p01,p02,p12)
//   {
//      int myID = omp_get_thread_num();
//  #else
//  #pragma omp parallel for reduction(+:esum,p00,p11,p22,p01,p02,p12)
//    for(int myID=0; myID<nnListEngine->size(); myID++)  {
//  #endif
//      const PtclPairList_t& nnlist = nnListEngine->getPairAttrib(myID);
   
//      map<int,SingleParticlePos_t> fv;   //visiting forces on non-local particles 

//      int nnloc = 0;

//      Scalar_t  esumloc=0.0;
//      Tensor_t  stressloc;
//      SingleParticlePos_t dr_ij, dr_ik, df, dfj, dfk;

//      int first = nnListEngine->first_id(myID);
//      int last  = nnListEngine->last_id(myID);

//      for(int iL = 0; iL < nnlist.getLocalNum(); iL++) {

//        int iG = nnlist.I[iL];
//        int num_nn = nnlist.nadj(iL);
//        int itype = myPtcl->GroupID[iG];
//        Scalar_t elocal = 0.0;

//        int nnloc_save = nnloc; 
//        for(int nni = 0; nni< num_nn; nni++, nnloc++) {

//          Scalar_t r = nnlist.R[nnloc]; 

//          if(r < Rmax) {
//            int jg = nnlist.iadj(iL,nni);
//            int jG = myPtcl->ID[jg];

//            int jtype = myPtcl->GroupID[jG];
//            int bond_ij = VSet->doublet(itype,jtype);

//  	  // dr = cartensian
//            dr_ij = r*nnlist.dR[nnloc];

//            Scalar_t rsq = r*r;
//            df = VSet->df2(bond_ij,rsq)*dr_ij; // r*directional cosine
//            elocal += VSet->f2(bond_ij,rsq);

//            if(stressmode) {stressloc -= outerProduct(df, dr_ij);}

//  	  f0[iG] += 2.0*df;
//            int nnloc2 = nnloc_save+nni+1;
//            for(int nnk=nni+1; nnk<num_nn; nnk++, nnloc2++) { // j<k

//              int kg = nnlist.iadj(iL,nnk);
//  	    int kG = myPtcl->ID[kg];

//              if(nnlist.R[nnloc2] < Rmax) {
//                int ktype = myPtcl->GroupID[kG];
//                int bond_ik = VSet->doublet(itype, ktype);

//  	      // j-i-k triplet index
//                int i_jik = VSet->triplet(jtype,itype,ktype);             

//                dr_ik = nnlist.R[nnloc2]*nnlist.dR[nnloc2]; // r_jk*\hat r_jk
//                Scalar_t lcos = dot(dr_ij,dr_ik);
//                elocal += VSet->f3(i_jik,lcos);
//                Scalar_t df3 = VSet->df3(i_jik,lcos); // beta*2*(r_ij*r_ik-beta)

//                dfj = df3*dr_ik;
//                dfk = df3*dr_ij;

//  	      if(stressmode) {
//  		stressloc -= outerProduct(dfj, dr_ij);
//  		stressloc -= outerProduct(dfk, dr_ik);
//  	      }
//  #ifdef ENABLE_OPENMP
//  	      // Forgot to add - (unary) operator, substract later for fv
//  	      map<int,SingleParticlePos_t>::iterator it;
//  	      int jG = myPtcl->ID[jg];
//  	      if(jg < first || jg >= last) {//logically belongs to other node
//  		it = fv.find(jG); 
//  		if(it == fv.end()) 
//  		  fv[jG] = dfj;
//  		else  
//  		  (*it).second += dfj;
//  	      } else { //logically belongs to this node
//  		f0[jG] -= dfj;
//  	      }

//  	      int kG = myPtcl->ID[kg];
//  	      if(kg < first || kg >= last) {// logically belongs to other node
//  		it = fv.find(kG); 
//  		if(it == fv.end()) 
//  		  fv[kG] = dfk;
//  		else
//  		  (*it).second += dfk;
//  	      } else { // jG logically belongs to this node
//  		f0[kG] -= dfk;
//  	      }

//  #else
//  	      f0[jG] -= dfj;
//  	      f0[kG] -= dfk;
//  #endif
//  	      f0[iG] += (dfj+dfk);
//  	    } // r_ik < rmax
//  	  } // k-indices
//  	} // r<rmax
//        } // j-indices

//        q[iG] = elocal;
//        esumloc += elocal;
//      }

//      esum += esumloc;
//      if(stressmode) {
//        p00 += stressloc(0,0);
//        p11 += stressloc(1,1);
//        p22 += stressloc(2,2);
//        p01 += stressloc(0,1);
//        p02 += stressloc(0,2);
//        p12 += stressloc(1,2);
//      }

//  #pragma omp critical
//      {    
//  #ifdef ENABLE_OPENMP
//        map<int,SingleParticlePos_t>::iterator it = fv.begin();
//        while(it != fv.end()) {
//  	f0[(*it).first] -= (*it).second;
//  	it++;
//        }
//  #endif
//      }
//    }//end-of-omp

//    //myclock.stop();
//    //ERRORMSG("FORCE = " << myclock.cpu_time());
   
//    if(stressmode)
//      stress += Tensor_t(p00,p01,p02,-p01,p11,p12,-p02,-p12,p22);
//    return esum;
//  }
// bool
// KeatingPotential::evalDynamicMatrix(DynMatrix_t& dymat) {

//   if(ompnode.size() < myPtcl->getLocalNum()) {
//     getPairList(); // using a pair-list
//     // ompnode is recorded to simplify the calculation
//     ompnode.resize(myPtcl->getLocalNum());
//     int ig0=0;
//     for(int myID=0; myID<nnListEngine->size(); myID++)  {
//       const PtclPairList_t& nnlist = nnListEngine->getPairAttrib(myID);
//       for(int iL = 0; iL < nnlist.getLocalNum(); iL++, ig0++) 
// 	ompnode[ig0] = myID;
//     }
//   } else {
//     nnListEngine->update(*myPtcl,-1);
//   }

// #if defined(ENABLE_PFOR)
// #pragma omp parallel for
//   for(int myID=0; myID<nnListEngine->size(); myID++)  {
// #else
// #pragma omp parallel
//  {
//     int myID = omp_get_thread_num();
// #endif
//     const PtclPairList_t& nnlist = nnListEngine->getPairAttrib(myID);
//     int first = nnListEngine->first_id(myID);
//     int last  = nnListEngine->last_id(myID);
//     int nnloc = 0;
//     int nnmax = 2*nnlist.getMaxNadj();
//     vector<SingleParticlePos_t> dcos(nnmax);
//     vector<Scalar_t> dist(nnmax);
//     vector<int> J(nnmax);

//     for(int iL = 0; iL < nnlist.getLocalNum(); iL++) {

//       int iG = nnlist.I[iL];
//       int num_nn = nnlist.nadj(iL);
//       int itype = myPtcl->GroupID[iG];
//       int i0 = iG*OHMMS_DIM; int i1 = i0+1; int i2 = i0+2;
//       for(int nnj = 0; nnj< num_nn; nnj++, nnloc++) {
// 	dcos[nnj] = nnlist.dR[nnloc];
// 	dist[nnj]  = nnlist.R[nnloc];
//         J[nnj]  = nnlist.iadj(iL,nnj);
//       }

//       for(int nnj=0; nnj<num_nn; nnj++) {

// 	int jg = J[nnj];        // logical index
// 	int jG = myPtcl->ID[jg];// physical index
// 	int j0 = jG*OHMMS_DIM; int j1=j0+1; int j2=j0+2;

// 	// calculate two body terms
// 	int jtype = myPtcl->GroupID[jG];
// 	int bond_ij = VSet->doublet(itype,jtype);
// 	Scalar_t dy_over_r; // 1/r df(r)/dr 
// 	// d^2f(r)/dr^2
// 	Scalar_t d2y = 4*VSet->d2f2(bond_ij, dist[nnj], dy_over_r);
// 	dy_over_r *= 4.0;
// 	d2y -= dy_over_r;
// 	Tensor_t dxy = d2y*outerProduct(dcos[nnj],dcos[nnj]);
// 	dymat(i0,j0) -= dxy(0,0) + dy_over_r;
// 	dymat(i0,j1) -= dxy(0,1);
// 	dymat(i0,j2) -= dxy(0,2);
// 	dymat(i1,j0) -= dxy(1,0);
// 	dymat(i1,j1) -= dxy(1,1) + dy_over_r;
// 	dymat(i1,j2) -= dxy(1,2);
// 	dymat(i2,j0) -= dxy(2,0);
// 	dymat(i2,j1) -= dxy(2,1);
// 	dymat(i2,j2) -= dxy(2,2) + dy_over_r;

// // 	// triplet j-i-k relations
// // 	for(int nnk=nnj+1; nnk<num_nn; nnk++) { // j<k
// // 	  int kG = myPtcl->ID[J[nnk]];
// // 	  int ktype = myPtcl->GroupID[jG];
// // 	  // j-i-k triplet index
// // 	  int i_jik = VSet->triplet(jtype,itype,ktype);             

// // 	  Scalar_t lcos = dist[nnj]*dist[nnk]*dot(dcos[nnj],dcos[nnk]);
// // 	  Scalar_t df3 = VSet->df3(i_ijk,lcos); // beta*2*(r_ij*r_ik-beta)

// // 	  Tensor_t dfj_i = df3*dist[nnk]*outerProduct(dcos[nnj],dcos[nnj]);
// // 	  Tensor_t dfk_i = df3*dist[nnj]*outerProduct(dcos[nnk],dcos[nnk]);
// // 	  Tensor_t dfj_k = df3*outerProduct(dcos[nnk],dcos[nnj]);

// // 	}
	

// // 	/////////////////////////////////////////
// // 	// update i-j-k triplet interaction
// // 	/////////////////////////////////////////
// // 	int jnode = ompnode[jg];
// // 	int jL = jg - nnListEngine->first_id(jnode); // find the local index
// // 	const PtclPairList_t& otherlist = nnListEngine->getPairAttrib(jnode);
// // 	int nnloc_j  = nnlist.loc(jL,0);// starting location of the jL's neighbors

// // 	for(int nnk = 0; nnk< otherlist.nadj(jL); nnk++, nnloc_j++) {
// // 	  int kg = nnlist.iadj(jL,nnk);
// // 	  int kG = myPtcl->ID[kg];
// // 	  if(kG == iG) continue; // i-j-i connection does not exist
// // 	  int ktype = myPtcl->GroupID[kG];
// // 	  // j-i-k triplet index
// // 	  int i_ijk = VSet->triplet(itype,jtype,ktype);             
// // 	}
//       }
//     }

//   }//end-of-omp


// //   // experimental version to calculate dynamic matrix with finite difference method
// //   // utilizing the nearest neighbor relations
// //   ParticlePos_t f0(myPtcl->getLocalNum());
// //   ParticlePos_t q(myPtcl->getLocalNum());
// //   Tensor_t stress;

// //   evalForce(f0,q,stress,0); // first calculate the forces

// //   // ompnode is recorded to simplify the calculation
// //   ParticlePos_t ompnode(myPtcl->getLocalNum());
// //   int ig0=0;
// //   for(int myID=0; myID<nnListEngine->size(); myID++)  {
// //     const PtclPairList_t& nnlist = nnListEngine->getPairAttrib(myID);
// //     for(int iL = 0; iL < nnlist.getLocalNum(); iL++, ig0++) 
// //       ompnode[ig0] = myID;
// //   }

// // #if defined(SGI) && defined(ENABLE_OPENMP)
// // #pragma omp parallel
// //  {
// //     int myID = omp_get_thread_num();
// // #else
// // #pragma omp parallel for
// //   for(int myID=0; myID<nnListEngine->size(); myID++)  {
// // #endif
// //     const PtclPairList_t& nnlist = nnListEngine->getPairAttrib(myID);
// //     Scalar_t delta = 0.001; // AA
// //     Scalar_t invdelta = -1.0/delta, factor;
// //     bool tocart = false;

// //     if(myPtcl->R.InUnit) tocart = true;//need conversion to cartesian coordinates

// //     vector<SingleParticlePos_t> disp(OHMMS_DIM); // 3D Cartesian displacement vectors
// //     disp[0] = SingleParticlePos_t(delta,0,0);  // +x
// //     disp[1] = SingleParticlePos_t(-delta,0,0); // -x
// //     disp[2] = SingleParticlePos_t(0,delta,0);  // +y
// //     disp[3] = SingleParticlePos_t(0,-delta,0); // -y
// //     disp[4] = SingleParticlePos_t(0,0,delta);  // +z
// //     disp[5] = SingleParticlePos_t(0,0,-delta); // -z

// //     // temporary storages for neighbor data   
// //     vector<SingleParticlePos_t> R0(2*nnlist.getMaxNadj()); // shifted positions
// //     vector<SingleParticlePos_t> dr(2*nnlist*getMaxNadj()); // R[j] - (R[i] + disp)
// //     vector<SingleParticlePos_t> dcos(2*nnlist*getMaxNadj()); // directional cosines
// //     vector<Scalar_t>            dist(2*nnlist*getMaxNadj());// distance
// //     vector<int>                 J(2*nnlist.getMaxNadj());   // physical index
// //     ParticlePos_t fv(2*nnlist.getMaxNadj());  // visiting forces

// //     vector<int> iv3(nnlist.getMaxNadj()*nnlist.getMaxNadj()); // next-next id
// //     vector<SingleParticlePos_t> 
// //       fv3(nnlist.getMaxNadj()*nnlist.getMaxNadj()); // force on next-next particle

// //     int first = nnListEngine->first_id(myID);
// //     int last  = nnListEngine->last_id(myID);
// //     int nnloc = 0;
// //     for(int iL = 0; iL < nnlist.getLocalNum(); iL++) {

// //       int iG = nnlist.I[iL];
// //       int num_nn = nnlist.nadj(iL);
// //       int itype = myPtcl->GroupID[iG];
// //       int nnloc0 = nnloc;
// //       SingleParticlePos_t r0 = myPtcl->R[iG];
// //       if(tocart) 
// // 	r0 = myPtcl->Lattice.toCart(r0); // need cartesin coordinate (see below)

// //       // copy the position of the neighbor particles
// //       for(int nnj = 0; nnj< num_nn; nnj++, nnloc++) {
// // 	int jg = nnlist.iadj(iL,nnj);
// //         int jG = myPtcl->ID[jg];

// // 	J[nnj] = jG; // physical index

// // 	// determine the position of the neighbor atom
// // 	// note that r0 is in cartesian and nnlist.R and nnlist.dR already
// // 	// contain corrections due to the boundary conditions
// // 	// displacement to calculate force constants cannot move particles too far.
// // 	R0[nnj] = nnlist.R[nnloc]*nnlist.dR[nnloc]+ r0;
// //       }
          
// //       // for each atom, apply a finite displacement
// //       for(int jdisp=0; jdisp<2*OHMMS_DIM; jdisp++) {

// // 	// first, calculate new distance and directional cosines of i-j pairs
// // 	// note that R0 is shifted by R[iG]
// // 	for(int nnj=0; nnj<num_nn; nnj++) {
// // 	  dr[nnj] = R0[nnj]-disp[jdisp];          //displacement vector
// // 	  dist[nnj] = sqrt(dot(dr[nnj],dr[nnj])); //new distance
// // 	  Scalar_t rinv = 1.0/dist[nnj];
// // 	  dcos[nnj] = rinv*dr[nnj];               //new consine vector
// // 	}

// // 	// get forces on iG and neighbor and next neighbor particles
// // 	SingleParticlePos_t fplus(0,0,0); // force on iG
// // 	fv = 0.0;                         // forces on the neighbor particle
// // 	int nnjik = 0;                    // number of next-next particles

// // 	for(int nnj=0; nnj<num_nn; nnj++) {

// // 	  // calculate two body terms
// // 	  int jtype = myPtcl->GroupID[J[nnj]];
// // 	  int bond_ij = VSet->doublet(itype,jtype);

// // 	  Scalar_t rsq = r[nnj]*r[nnj];
// // 	  SingleParticlePos_t df = VSet->df2(bond_ij,rsq)*dr[nnj];

// // 	  fplus   += 2.0*df; // force on the iG
// // 	  fv[nnj] -= 2.0*df; // force on the neighbor due to a displacement on iG

// // 	  // triplet j-i-k relations
// // 	  for(int nnk=nnj+1; nnk<num_nn; nnk++) { // j<k
// // 	    int ktype = myPtcl->GroupID[J[nnk]];
// // 	    // j-i-k triplet index
// // 	    int i_jik = VSet->triplet(jtype,itype,ktype);             
// // 	    Scalar_t lcos = dot(dr[nnj],dr[nnk]);
// // 	    Scalar_t df3 = VSet->df3(i_jik,lcos); // beta*2*(r_ij*r_ik-beta)

// // 	    SingleParticlePos_t dfj = df3*dist[nnk]*dcos[nnj];
// // 	    SingleParticlePos_t dfk = df3*dist[nnj]*dcos[nnk];
// // 	    fplus   += (dfj+dfk);
// // 	    fv[nnj] -= dfj;
// // 	    fv[nnk] -= dfk;
// // 	  }


// // 	  /////////////////////////////////////////
// // 	  // update i-j-k triplet interaction
// // 	  /////////////////////////////////////////
// // 	  int jnode = ompnode[jg];
// // 	  int jtype = myPtcl->GroupID[jG];
// // 	  int jL = jg - nnListEngine->first_id(jnode); // find the local index
// // 	  const PtclPairList_t& otherlist = nnListEngine->getPairAttrib(jnode);
// // 	  int nnloc_j  = nnlist.loc(jL,0);// starting location of the jL's neighbors

// // 	  for(int nnk = 0; nnk< otherlist.nadj(jL); nnk++, nnloc_j++) {
// // 	    int kg = nnlist.iadj(jL,nnk);
// // 	    int kG = myPtcl->ID[kg];
// // 	    if(kG == iG) continue; // i-j-i connection does not exist
// // 	    int ktype = myPtcl->GroupID[kG];
// // 	    int bond_ik = VSet->doublet(jtype, ktype);
	    
// // 	    // j-i-k triplet index
// // 	    int i_ijk = VSet->triplet(itype,jtype,ktype);             

// // 	    //note the negative applied applied to dr[nnj]
// // 	    Scalar_t lcos = -nnlist.R[nnloc_j]*dot(dr[nnj],nnlist.dR[nnloc_j]); 
// // 	    Scalar_t df3 = VSet->df3(i_ijk,lcos); // beta*2*(r_ij*r_ik-beta)

// // 	    // negative sign applied to dr[nnj]: dr = R[j] - R[i]
// // 	    SingleParticlePos_t dfi = -df3*nnlist.R[nnloc_j]*dcos[nnj]; 
// // 	    SingleParticlePos_t dfk =  df3*dist[nnj]*nnlist.dR[nnloc_j];

// // 	    fv[nnj] += (dfi+dfk); // i-j-k
// // 	    fplus   -= dfi;       // force on iL

// // 	    // save forces on the k atom in a vector, nnjik <  nnmax*(nnmax-1)
// // 	    // the number of terms is not big 12 for the typical case.
// // 	    // unlike force calculation where visiting forces may require
// // 	    // global operations, the force constant calculations do not.
// // 	    iv3[nnjik] = kG;	
// // 	    fv3[nnjik] = -dfk;
// // 	    nnjik++;
// // 	  }
// // 	} // loop over the neighbor particles and accumulate forces

// // 	if(jdisp%2 == 0) 
// // 	  factor = invdelta; // even + displacement
// // 	else             
// // 	  factor = -invdelta; // odd - displacement

// // 	int idir=jdisp%3; // direction
// // 	int ii = iG*OHMMS_DIM; 
// // 	int ii0 = ii+idir;

// // 	SingleParticlePos_t df = fplus-f0[iG];
// // 	fcmat(ii0,ii++) += factor*df[0];
// // 	fcmat(ii0,ii++) += factor*df[1];
// // 	fcmat(ii0,ii++) += factor*df[2];

// // 	// add the visiting force contribution: i-j and j-i-k
// // 	for(int nnj=0; nnj<num_nn; nnj++) {
// // 	  int jG = J[nnj];
// // 	  int jj = jG*OHMMS_DIM;
// // 	  df = fv[nnj] - f0[jG];
// //           fcmat(ii0,jj++) += factor*df[0];
// //           fcmat(ii0,jj++) += factor*df[1];
// //           fcmat(ii0,jj++) += factor*df[2];
// // 	}

// // 	// add i-j-k components on k
// // 	for(int k=0; k<nnjik; k++) {
// // 	  int kG = iv3[k];
// // 	  int kk = kG*OHMMS_DIM;
// // 	  df = fv3[k] - f0[kG];
// //           fcmat(ii0,kk++) += factor*df[0];
// //           fcmat(ii0,kk++) += factor*df[1];
// //           fcmat(ii0,kk++) += factor*df[2];
// // 	}
// //       }
// //     }
// //   }//end-of-omp

//   return true;

// }


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 308 $   $Date: 2005-02-21 09:21:44 -0600 (Mon, 21 Feb 2005) $
 * $Id: KeatingPotential.cpp 308 2005-02-21 15:21:44Z jnkim $ 
 ***************************************************************************/
