//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/**@file KeatingPotential.h
 * @author Jeongnim Kim
 * @brief Declaration of Keating Potential.
 */
#ifndef OHMMS_KEATINGPOTENTIAL_H
#define OHMMS_KEATINGPOTENTIAL_H

#include <vector>
using namespace std;

#include "Potentials/ShortRangePotential.h"
class KeatingParam;

namespace OHMMS {

  /** Implements Keating Valence-Force-Field model, excluding long-range interactions
   */
  class KeatingPotential:public ShortRangePotential {

  public:

    KeatingPotential();
    ~KeatingPotential();

    /// implementing virtual functions for OhmmsElementBase
    bool get(ostream&) const;
    bool put(istream&);
    void reset();
    bool put(xmlNodePtr cur);

    void resetParticleSet(Particle_t* ptclset) {
      myPtcl=ptclset;
    }

    Scalar_t evalForceNN(PtclPairList_t& nnlist, 
			 ParticlePos_t& f0, 
			 unsigned stressmode) const;

    Scalar_t evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale=1.0) const;

    void analize(const PtclPairList_t& nnlist,size_type first, size_type last); //ParticleTensor_t& astress);

  protected:

    KeatingParam* VSet;  
    ParticleIndex_t ompnode;
  };


  /**class to record the results of KeatingPotential calculations
  struct KeatingPotentialOutput: public RecordProperty {
    
    typedef KeatingPotential::ParticleTensor_t ParticleTensor_t;

    int OutputFormat;
    
    KeatingPotential& ref_;
    
    KeatingPotentialOutput(KeatingPotential& kp);
    
    void reset(const char* fileroot);
    
    void report(int iter);
    
    void finalize();
    
    bool put(xmlNodePtr cur);

  };
  */
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 363 $   $Date: 2005-07-13 13:33:18 -0500 (Wed, 13 Jul 2005) $
 * $Id: KeatingPotential.h 363 2005-07-13 18:33:18Z jnkim $ 
 ***************************************************************************/

