//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
/////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Message/OpenMP.h"
#include <vector>
using namespace std;
#include "Potentials/LJPotential.h"
#include "Utilities/SpeciesCollection.h"
#include "Utilities/SimpleParser.h"
using namespace OHMMS;

LJPotential::LJPotential() :LJSet(0)
{
  CollectVisitingForces = false;
  myName = "LJ";
}
  
// Scalar_t 
// LJPotential::e(const ParticlePos_t& R, int i, int myID) {

//   // use pre-calculated neighbor lists
//   ////////////////////////////////////////
//   // only works if the logical and physical orders are identical
//   ////////////////////////////////////////
//   int iL = i - nnListEngine->first_id(myID);
//   const PtclPairList_t& nnlist = nnListEngine->getPairAttrib(myID);

//   Scalar_t esum = 0;
//   SingleParticlePos_t dr;
//   Scalar_t r;
//   int num_nn = nnlist.nadj(iL);

//   for(int nni=0; nni < num_nn; ++nni) {
//     int j = nnlist.iadj(iL,nni);  //atomic index of jL-th nn of i
//     if(R.InUnit) 
//       dr = myPtcl->Lattice.BConds.apply(R[j]-R[i]);
//     else
//       dr = myPtcl->Lattice.BConds.apply(myPtcl->Lattice.toUnit(R[j]-R[i]));

//     r = sqrt(myPtcl->Lattice.Dot(dr,dr));
//     if(r < Rmax) {
//       esum += LJSet->f(r);
//     }
//   }  // for jL < nn

//   return esum;
// }

LJPotential::Scalar_t 
LJPotential::evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale) const {
  typedef PtclPairList_t::PairData_t    PairData_t;
  int nnloc(0);
  Scalar_t  esum(0.0e0);
  for(int iL = 0; iL<nnlist.getLocalNum(); iL++) {
    int num_nn(nnlist.nadj(iL));
    for(int nni = 0; nni< num_nn; nni++, nnloc++) {
      const PairData_t& cur(nnlist[nnloc]);
      if(cur.sep<Rmax) esum += LJSet->f(cur.sep);
    }
  }
  return esum;
}


LJPotential::Scalar_t 
LJPotential::evalForceNN(PtclPairList_t& nnlist, 
			 ParticlePos_t& f0, 
			 unsigned stressmode) const {
  //int iG = nnListEngine->myAtoms[myID][0];
  int nnloc(0);
  Scalar_t  esum(0.0), phi;
  for(int iL = 0; iL<nnlist.getLocalNum(); iL++) {
    int iat(nnlist.I[iL]);
    int num_nn(nnlist.nadj(iL));
    Scalar_t elocal(0.0); // local energy to be accumulated
    SingleParticlePos_t fon;
    for(int nni = 0; nni< num_nn; nni++, nnloc++) {
      const PairData_t& cur(nnlist[nnloc]);
      Scalar_t r(cur.sep);
      if(r < Rmax) {
	Scalar_t dphi(LJSet->df(r,phi)/r);
	elocal += phi;
	SingleParticlePos_t df = dphi*cur.displ;
	fon += 2*df; // openmp optimal, does not matter anyway
	if(stressmode) 
	  stressloc -= outerProduct(df, cur.displ);
      }
    }
    f0[iL] += fon;
    q[iat] = elocal;
    esum += elocal;
  }

  nnlist.Stress = stressloc;
  return esum;
}

// bool LJPotential::evalDynamicMatrix(DynMatrix_t& dymat) {

//   getPairList(); // using a pair-list
// #if defined(ENABLE_PFOR)
// #pragma omp parallel for 
//   for(int myID=0; myID<nnListEngine->size(); myID++)  {
// #else
// #pragma omp parallel 
//  {
//     int myID = omp_get_thread_num();
// #endif
//     const PtclPairList_t& nnlist = nnListEngine->getPairAttrib(myID);

//     int nnloc = 0;
//     Scalar_t  phi, dphi_over_r;
//     for(int iL = 0; iL < nnlist.getLocalNum(); iL++) {

//       int iG = nnlist.I[iL]; // get the index, physical index
//       int i0 = iG*OHMMS_DIM;
//       int i1 = i0+1;
//       int i2 = i0+2;
//       int num_nn = nnlist.nadj(iL);

//       for(int nni = 0; nni< num_nn; nni++, nnloc++) {

//         Scalar_t r = nnlist.R[nnloc];

//         if(r < Rmax) {
// 	  int jG = myPtcl->ID[nnlist.iadj(iL,nni)];
//           Scalar_t d2phi = 4.0*LJSet->d2f(r,phi,dphi_over_r);
// 	  dphi_over_r *= 4.0;
// 	  d2phi -= dphi_over_r;
//           SingleParticlePos_t dr = nnlist.dR[nnloc];
// 	  int j0 = jG*OHMMS_DIM;
// 	  int j1 = j0+1;
// 	  int j2 = j0+2;
// 	  dymat(i0,j0) -= (d2phi*dr[0]*dr[0] + dphi_over_r);
// 	  dymat(i0,j1) -= (d2phi*dr[0]*dr[1]);
// 	  dymat(i0,j2) -= (d2phi*dr[0]*dr[2]);
// 	  dymat(i1,j0) -= (d2phi*dr[1]*dr[0]);
// 	  dymat(i1,j1) -= (d2phi*dr[1]*dr[1] + dphi_over_r);
// 	  dymat(i1,j2) -= (d2phi*dr[1]*dr[2]);
// 	  dymat(i2,j0) -= (d2phi*dr[2]*dr[0]);
// 	  dymat(i2,j1) -= (d2phi*dr[2]*dr[1]);
// 	  dymat(i2,j2) -= (d2phi*dr[2]*dr[2] + dphi_over_r);
// 	}
//       }
//     }
//   }//end-of-omp

//   return true;
// }

bool LJPotential::put(xmlNodePtr cur) {
  if(!LJSet)  {
    LJSet = new LJScale<Scalar_t>;
    SpeciesBase& mySpecies = myPtcl->getSpeciesSet();
   int newid, irmax, imass;
   newid = mySpecies.addSpecies("Ar");
   irmax = mySpecies.addAttrib("rmax");
   imass = mySpecies.addAttrib("mass");
   mySpecies(imass,newid)= 39.948;

    Rmax = LJSet->rmax();
    if(mySpecies(irmax,newid) < Rmax*0.5) {
      mySpecies(irmax,newid) = Rmax*0.5;
    }
  }
  return true;
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 308 $   $Date: 2005-02-21 09:21:44 -0600 (Mon, 21 Feb 2005) $
 * $Id: LJPotential.cpp 308 2005-02-21 15:21:44Z jnkim $ 
 ***************************************************************************/
