//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim and Thomas Lenosky
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_MEAMPOTENTIAL_PARAM_H
#define OHMMS_MEAMPOTENTIAL_PARAM_H
#include <math.h>
#include <vector>
#include "Potentials/CubicSplineType.h"
#include "Utilities/SimpleParser.h"

namespace OHMMS {

  /*! \class MEAMParam
   *  \brief Container for the parameters defined by MEAM potential by T. Lenosky.
   */
  struct MEAMParam {
    typedef double value_type;
    int clsusephi, clsusetri, clsmrho, clsmhtb, clsmepstb, clsmrho2;

    //@{Ranges of various splines in potential
    value_type clsgnecut,   R2sq;  
    value_type clsrhonecut, Rrhosq;
    value_type clstricut1,  R3sq;
    //@}

    std::vector<value_type> eps;
    SplineType phi,f,g;
    std::vector<SplineType*> u, rho, h;
    std::vector<SplineType*> xf, xg, xu, xrho;
    std::vector<SplineType*> xh, xl, xq;

    MEAMParam(const char *fname = "coeff.cls");

    void readMEAMParam(const char *fname);

    value_type rcutmax() { 
      value_type rmax = R2sq;
      rmax = (Rrhosq > rmax)? Rrhosq : rmax;
      rmax = (R3sq > rmax)? R3sq : rmax;
      return sqrt(rmax);
    }

  };          
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 308 $   $Date: 2005-02-21 09:21:44 -0600 (Mon, 21 Feb 2005) $
 * $Id: MEAMParam.h 308 2005-02-21 15:21:44Z jnkim $ 
 ***************************************************************************/

