//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim and Thomas Lenosky
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Potentials/MEAMPotential.h"
#include "Potentials/MEAMParam.h"
#include "Message/OpenMP.h"
using namespace OHMMS;

//TESTING IF MEAM_STRUCT CAN HELP CACHE MANAGEMENT 
//#define USE_MEAM_STRUCT
MEAMPotential::Scalar_t 
MEAMPotential::evalForceNN(PtclPairList_t& nnlist, 
    ParticlePos_t& fv, unsigned stressmode) const {

  typedef PtclPairList_t::PairData_t    PairData_t;
  typedef MEAM2Body<Scalar_t,OHMMS_DIM> TwoBodyType;
  typedef MEAM3Body<Scalar_t>           ThreeBodyType;

  ParticleScalar_t& q = myPtcl->E;

  size_type max_est(nnlist.getMaxNadj());
  vector<TwoBodyType> SF(max_est);
  vector<TwoBodyType> SRho(max_est);
  vector<ThreeBodyType> SG(max_est*max_est);
  
  Scalar_t esumloc = 0.0e0;
  Tensor_t stressloc = 0.0;
  size_type nnloc = 0;
  for(size_type iL=0; iL<nnlist.getLocalNum(); iL++) {
    size_type iat(nnlist.I[iL]),n3(0),n2(0);
    Scalar_t elocal = 0.0;
    Scalar_t emaX = 0.0;
    SingleParticlePos_t fon(0.0);
    Scalar_t val, deriv;
    PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
    while(nit != nit_end) {
      const PairData_t& cur(*nit);
      Scalar_t r(cur.sep);
      if(r < MEAMSet[0]->clsgnecut) {// going thru all the nn atoms within a cutoff of phi
        Scalar_t rinv(1.0/r);
        SingleParticlePos_t dcos(rinv*cur.displ);
        deriv = 0.5*MEAMSet[0]->phi.splint(r, val); // phi(r)
        //The factor 0.5 is due to the double counting
        elocal += 0.5*val; 
        SingleParticlePos_t df(deriv*dcos);
        fon += 2*df; 
        if(stressmode) stressloc -= outerProduct(df, cur.displ);
        int jloc(cur.Where);
        if(r < MEAMSet[0]->clsrhonecut) { // save data associated with rho(r) 
          SRho[n2].Loc=jloc;
          SRho[n2].df = MEAMSet[0]->xrho[0]->splint(r, SRho[n2].f);
          SRho[n2].r=r;
          SRho[n2].dcos=dcos;
          emaX += SRho[n2].f;
          n2++;
        }

        if(r < MEAMSet[0]->clstricut1) { // save data associated with f(r)
          //deriv = MEAMSet[0]->xf[0]->splint(r, val);
          SF[n3].Loc=jloc;
          SF[n3].df= MEAMSet[0]->xf[0]->splint(r, SF[n3].f);
          SF[n3].r=r;
          SF[n3].rinv=rinv;
          SF[n3].dcos=dcos;
          n3++;
        }
      }
      ++nit;
    } //while(nit != nit_end)

    int n3seq = 0;
    // save data associated with g(costheta)
    for(int nj=0; nj < n3-1; nj++) {
      for(int nk=nj+1; nk < n3; nk++) {
        Scalar_t lcos = dot(SF[nj].dcos, SF[nk].dcos); // costheta
        SG[n3seq].lcos = lcos;  // costheta
        SG[n3seq].dg = MEAMSet[0]->xg[0]->splint(lcos, SG[n3seq].g);// g(costheta)
        emaX += SF[nj].f*SF[nk].f*SG[n3seq].g;    // f(rij)*f(rik)*g(costheta)
        n3seq++;
      }
    }    // Done with emaX, calculate the EMA term

    // du/dx, used for a force prefactor
    Scalar_t dux(MEAMSet[0]->xu[0]->splint(emaX, val));
    elocal +=val; 

    for(int nj=0; nj < n2; nj++) { // forces from drho
      SingleParticlePos_t df(dux * SRho[nj].df*SRho[nj].dcos);
      fon += df; fv[SRho[nj].Loc] -= df;
      if(stressmode) stressloc -= SRho[nj].r*outerProduct(df, SRho[nj].dcos);
    }//nj
    //No need for specialization for serial version
    //for(int nj=0; nj < n2; nj++) { // forces from drho
    //  SingleParticlePos_t df = dux * SRho[nj].df*SRho[nj].dcos;
    //  fv[iat] += df; fv[SRho[nj].Loc] -= df;
    //  if(stressmode) stressloc -= SRho[nj].r*outerProduct(df, SRho[nj].dcos);
    //}//nj

    n3seq = 0;  
    for(int nj=0; nj<n3-1; nj++) {
      const TwoBodyType& srho = SRho[nj];
      const TwoBodyType& sf_j = SF[nj];
      int jat = sf_j.Loc;
      SingleParticlePos_t df(dux * srho.df*srho.dcos);
      SingleParticlePos_t drj(sf_j.dcos);
      for(int nk=nj+1; nk<n3; nk++) {
        const TwoBodyType& sf_k = SF[nk];
        const ThreeBodyType& sg = SG[n3seq];
        SingleParticlePos_t drk(sf_k.dcos);

        Scalar_t dV3j(sg.g*sf_j.df * sf_k.f);
        Scalar_t dV3k(sg.g*sf_j.f  * sf_k.df);
        Scalar_t V3(sf_j.f * sf_k.f * sg.dg);

        Scalar_t vlj(V3*sf_j.rinv);
        Scalar_t vlk(V3*sf_k.rinv);
        Scalar_t vv3j(dV3j-vlj*sg.lcos);
        Scalar_t vv3k(dV3k-vlk*sg.lcos);

        SingleParticlePos_t dfj(vv3j*drj + vlj*drk);
        SingleParticlePos_t dfk(vv3k*drk + vlk*drj);
        
	//SingleParticlePos_t dfj = dV3j*drj + vlj*(drk- sg.lcos*drj);
	//SingleParticlePos_t dfk = dV3k*drk + vlk*(drj- sg.lcos*drk);

	fon      += dux*(dfj+dfk);
	fv[jat]     -=dux*dfj;
	fv[sf_k.Loc]-=dux*dfk;
	if(stressmode) {
	  stressloc -= sf_j.r*dux * outerProduct(dfj, drj);
	  stressloc -= sf_k.r*dux * outerProduct(dfk, drk);
	}
        n3seq++; // advance the sequence number of three body terms 
      } // nk
    } // nj

    //Remove the atomic energy calculation
    fv[iL] += fon;
    q[iat] = elocal;
    esumloc += elocal;
  }

  nnlist.Stress = stressloc;
  nnlist.Energy = esumloc;
  return esumloc;
 }

/*** legacy
MEAMPotential::Scalar_t 
MEAMPotential::evalForceNN(PtclPairList_t& nnlist, 
			   ParticlePos_t& f0, 
			   ParticleScalar_t& q, 
			   unsigned stressmode) const {

  int myid=nnlist.myID;

#ifdef ENABLE_OPENMP
  PtclPairList_t::IndexList_t& fvind = nnlist.IDv;
  PtclPairList_t::VectorList_t& fv = nnlist.Fv;
#endif

  int max_est = nnlist.getMaxNadj();

  vector<int> sf_id(max_est);
  vector<Scalar_t> sf_f(max_est);
  vector<Scalar_t> sf_df(max_est);
  vector<Scalar_t> sf_r(max_est);
  vector<Scalar_t> sf_rinv(max_est);
  vector<SingleParticlePos_t> sf_dr(max_est);
  vector<int> srho_id(max_est);
  vector<Scalar_t> srho_r(max_est);
  vector<Scalar_t> srho_rho(max_est);
  vector<Scalar_t> srho_drho(max_est);
  vector<SingleParticlePos_t> srho_dr(max_est);
  vector<SingleParticlePos_t> f2(max_est),f3(max_est);
  
  int max_est_2 = max_est*max_est;
  vector<Scalar_t> sg_lcos(max_est_2);
  vector<Scalar_t> sg_g(max_est_2);
  vector<Scalar_t> sg_dg(max_est_2);

  Scalar_t esumloc = 0.0e0, elocal;
  Tensor_t stressloc = 0.0;
  Scalar_t val, deriv, r;
  int nnloc = 0;
  
  for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
    int iat = nnlist.I[iL];
    int num_nn = nnlist.nadj(iL);
    int n3 = 0;
    int n2 = 0;
    elocal = 0.0;

    for(int nni = 0; nni< num_nn; nni++, nnloc++) {
      int jat = nnlist.iadj(iL,nni);
      r = nnlist.R[nnloc];
      f2[nni]=0.0;	
      f3[nni]=0.0;	

      // going thru all the nn atoms within a cutoff of phi
      if(r < MEAMSet[0]->clsgnecut) {

        deriv = MEAMSet[0]->phi.splint(r, val); // phi(r)
        SingleParticlePos_t dr = nnlist.dR[nnloc];
        elocal += val*0.5e0;                     // double counting
        
        SingleParticlePos_t df = 0.5e0*deriv*dr; // double counting
        
#ifdef ENABLE_OPENMP
        fv[iL] += 2*df;  int jloc = fvind[nnloc];
#else
        f0[iat] += 2*df; int jloc = jat;
#endif
        if(stressmode) stressloc -= r*outerProduct(df, dr);
        
        if(r < MEAMSet[0]->clsrhonecut) { // save data associated with rho(r) 
          deriv = MEAMSet[0]->xrho[0]->splint(r, val);
          srho_id[n2] = jloc;   // location to which the visiting force goes
          srho_r[n2] = r;       // distance
          srho_rho[n2] = val;   // rho(r)
          srho_drho[n2] = deriv;// d rho(r)/ dr
          srho_dr[n2] = dr;     // (r[j]-r[i])/r
          n2++;
        }

        if(r < MEAMSet[0]->clstricut1) { // save data associated with f(r)
          deriv = MEAMSet[0]->xf[0]->splint(r, val);
          sf_id[n3] = jloc;    // location to which the visiting force goes
          sf_r[n3] = r;        // r[j]-r[i]
          sf_rinv[n3]  = 1/r;  // 1/r
          sf_f[n3]  = val;     // f(r)
          sf_df[n3] = deriv;   // d f(r)/dr
          sf_dr[n3] = dr;      // (r[j]-r[i])/r
          n3++;
        }
      }
    } //for(int nni=0; nni < nadj; ++nni) 

      Scalar_t emaX = 0.0;
      // calculating x for U(x) for the embedded atom term
      for(int nj=0; nj < n2; nj++) {
        emaX += srho_rho[nj];
      }

      int n3seq = 0;
      // save data associated with g(costheta)
      for(int nj=0; nj < n3-1; nj++) {
        for(int nk=nj+1; nk < n3; nk++) {
          Scalar_t lcos = dot(sf_dr[nj], sf_dr[nk]); // costheta
	  deriv = MEAMSet[0]->xg[0]->splint(lcos, val);// g(costheta)
	  emaX += sf_f[nj]*sf_f[nk]*val;    // f(rij)*f(rik)*g(costheta)
	  // ID's are not saved, since sf contains necessary information
	  sg_lcos[n3seq] = lcos;  // costheta
	  sg_g[n3seq] = val;      // g(costheta)
	  sg_dg[n3seq] = deriv;   // dg(costheta)/dcostheta
	  n3seq++;
	}
      }    // Done with emaX, calculate the EMA term

      // du/dx, used for a force prefactor
      Scalar_t dux = MEAMSet[0]->xu[0]->splint(emaX, val);
      elocal +=val; 

      for(int nj=0; nj < n2; nj++) { // forces from drho
	int jat = srho_id[nj];
        SingleParticlePos_t df = dux * srho_drho[nj] * srho_dr[nj];
#ifdef ENABLE_OPENMP
        fv[iL] += df; fv[jat] -= df;
#else
        f0[iat] += df; f0[jat] -= df;
#endif
	if(stressmode)
	  stressloc -= srho_r[nj]*outerProduct(df, srho_dr[nj]);
      }

      n3seq = 0;  
      for(int nj=0; nj<n3-1; nj++) {
	int jat = sf_id[nj]; 
        SingleParticlePos_t drj = sf_dr[nj];
	for(int nk=nj+1; nk<n3; nk++) {
	  int kat = sf_id[nk];

	  SingleParticlePos_t drk = sf_dr[nk];

	  Scalar_t dV3j = sg_g[n3seq] * sf_df[nj] * sf_f[nk];
	  Scalar_t dV3k = sg_g[n3seq] * sf_f[nj]  * sf_df[nk];

	  Scalar_t V3 = sf_f[nj] * sf_f[nk] * sg_dg[n3seq]; 
	  Scalar_t vlj = V3*sf_rinv[nj];
	  Scalar_t vlk = V3*sf_rinv[nk];

	  SingleParticlePos_t dfj = dV3j*drj + vlj*(drk- sg_lcos[n3seq]*drj);
	  SingleParticlePos_t dfk = dV3k*drk + vlk*(drj- sg_lcos[n3seq]*drk);

#ifdef ENABLE_OPENMP
	  fv[iL] += dux*(dfj+dfk); fv[jat]-=dux*dfj; fv[kat]-=dux*dfk;
#else
	  f0[iat] += dux*(dfj+dfk); f0[jat]-=dux*dfj; f0[kat]-=dux*dfk;
#endif
	  if(stressmode) {
	    stressloc -= sf_r[nj]*dux * outerProduct(dfj, drj);
	    stressloc -= sf_r[nk]*dux * outerProduct(dfk, drk);
	  }
	  n3seq++; // advance the sequence number of three body terms
	} // nk
      } // nj
      q[iat] = elocal;
      esumloc += elocal;
    }

    nnlist.Stress = stressloc;
    return esumloc;
 }
*/
// Scalar_t 
// MEAMPotential::evalForce(ParticlePos_t& f0, ParticleScalar_t& q,
//                          Tensor_t& stress, unsigned stressmode) {

//   DEBUGMSG("Entering AlloyEAMPotential::evalForce.\n")
//   getPairList(); // using a pair-list

//   Scalar_t esum = 0.0;

//   if(nnListEngine->size() >1) {
// #pragma omp parallel reduction(+:esum)
//     {
//       int myID = omp_get_thread_num();
//       ParticlePos_t ft(f0.size());
//       Tensor_t stressloc;
//       Scalar_t esumloc= evalForceNN(nnListEngine->getPairAttrib(myID), myID,
// 				    ft,q,stressloc,stressmode);
// #pragma omp critical
//       {    
// 	esum += esumloc;
// 	stress += stressloc;
// 	f0 += ft;
//       }
//     }
//   } else 
//     esum = evalForceNN(nnListEngine->getPairAttrib(0),0, f0,q,stress,stressmode);

//   if(nnListEngine->distributed()) {
//     gsum(f0,0);
//     gsum(q,0);
//     gsum(esum,0);
//   }
  
//   return esum;
// }


//   Scalar_t 
//     MEAMPotential::evalForceNN_MAP(const PtclPairList_t& nnlist, int myID,
//  			      ParticlePos_t& f0, ParticleScalar_t& q,
//  			      Tensor_t& stressloc, unsigned stressmode) {

//     DEBUGMSG("Entering MEAMPotential::evalForce.\n")

//      // storing three-body function
//      int max_est = 2*nnlist.getTotNadj()/nnlist.getLocalNum();
//      vector<int> sf_id(max_est);
//      vector<Scalar_t> sf_f(max_est);
//      vector<Scalar_t> sf_df(max_est);
//      vector<Scalar_t> sf_r(max_est);
//      vector<Scalar_t> sf_rinv(max_est);
//      vector<SingleParticlePos_t> sf_dr(max_est);
//      vector<int> srho_id(max_est);
//      vector<Scalar_t> srho_r(max_est);
//      vector<Scalar_t> srho_rho(max_est);
//      vector<Scalar_t> srho_drho(max_est);
//      vector<SingleParticlePos_t> srho_dr(max_est);

//      max_est = nnlist.getTotNadj()/nnlist.getLocalNum();
//      max_est = max_est*max_est*2;
//      vector<Scalar_t> sg_lcos(max_est);
//      vector<Scalar_t> sg_g(max_est);
//      vector<Scalar_t> sg_dg(max_est);

//      map<int,SingleParticlePos_t> fv;   //visiting forces on non-local particles 

//      Scalar_t esumloc = 0.0e0, elocal;
//      Scalar_t val, deriv, r;

//      int nnloc = 0;
//      int first = nnListEngine->first_id(myID);
//      int last  = nnListEngine->last_id(myID);

//      for(int iL = 0; iL < nnlist.getLocalNum(); iL++) {

//        int iG = nnlist.I[iL]; // get the index, physical index

//        int num_nn = nnlist.nadj(iL);
//        int n3 = 0;
//        int n2 = 0;
//        elocal = 0.0;

//        for(int nni = 0; nni< num_nn; nni++, nnloc++) {
//          int jg = nnlist.iadj(iL,nni);
//          r = nnlist.R[nnloc];

//          // going thru all the nn atoms within a cutoff of phi
//          if(r < MEAMSet[0]->clsgnecut) {
//  	  deriv = MEAMSet[0]->phi.splint(r, val); // phi(r)
//  	  SingleParticlePos_t dr = nnlist.dR[nnloc];
//  	  elocal += val*0.5e0;                     // double counting

//  	  SingleParticlePos_t df = 0.5e0*deriv*dr; // double counting

//            f0[iG] += 2*df; 
//  	  //f0[jG] -= df;
//            if(stressmode) {
//              stressloc -= r*outerProduct(df, dr);
//            }

//            if(r < MEAMSet[0]->clsrhonecut) { // save data associated with rho(r) 
//  	    deriv = MEAMSet[0]->xrho[0]->splint(r, val);
//  	    srho_id[n2] = jg;     // ID
//  	    srho_r[n2] = r;       // distance
//  	    srho_rho[n2] = val;   // rho(r)
//  	    srho_drho[n2] = deriv;// d rho(r)/ dr
//  	    srho_dr[n2] = dr;     // (r[j]-r[i])/r
//  	    n2++;
//  	  }

//            if(r < MEAMSet[0]->clstricut1) { // save data associated with f(r)
//  	    deriv = MEAMSet[0]->xf[0]->splint(r, val);
//  	    sf_id[n3] = jg;      // logical ID
//  	    sf_r[n3] = r;        // r[j]-r[i]
//  	    sf_rinv[n3]  = 1/r;  // 1/r
//  	    sf_f[n3]  = val;     // f(r)
//  	    sf_df[n3] = deriv;   // d f(r)/dr
//  	    sf_dr[n3] = dr;      // (r[j]-r[i])/r
//  	    n3++;
//  	  }
//  	}
//        } //for(int nni=0; nni < nadj; ++nni) 

//        Scalar_t emaX = 0.0;
//        // calculating x for U(x) for the embedded atom term
//        for(int nj=0; nj < n2; nj++) {
//          emaX += srho_rho[nj];
//        }

//        int n3seq = 0;
//        // save data associated with g(costheta)
//        for(int nj=0; nj < n3-1; nj++) {
//          for(int nk=nj+1; nk < n3; nk++) {
//            Scalar_t lcos = dot(sf_dr[nj], sf_dr[nk]); // costheta
//  	  deriv = MEAMSet[0]->xg[0]->splint(lcos, val);// g(costheta)
//  	  emaX += sf_f[nj]*sf_f[nk]*val;    // f(rij)*f(rik)*g(costheta)
//          // ID's are not saved, since sf contains necessary information
//  	  sg_lcos[n3seq] = lcos;  // costheta
//  	  sg_g[n3seq] = val;      // g(costheta)
//  	  sg_dg[n3seq] = deriv;   // dg(costheta)/dcostheta
//  	  n3seq++;
//  	}
//        }    // Done with emaX, calculate the EMA term

//        // du/dx, used for a force prefactor
//        Scalar_t dux = MEAMSet[0]->xu[0]->splint(emaX, val);
//        elocal +=val; 

//        for(int nj=0; nj < n2; nj++) { // forces from drho
//          int jg = srho_id[nj];
//  	int jG = myPtcl->ID[jg];

//          SingleParticlePos_t df = dux * srho_drho[nj] * srho_dr[nj];
//          f0[iG] += df;

//  #ifdef ENABLE_OPENMP
//          // Forgot to add - (unary) operator, substract later for fv
//  	map<int,SingleParticlePos_t>::iterator it;
//  	if(jg < first || jg >= last) {//jg logically belongs to other node
//  	  it = fv.find(jG); 
//  	  if(it == fv.end()) 
//  	    fv[jG] = df;
//  	  else  
//  	    (*it).second += df;
//  	} else { // jG logically belongs to this node
//  	  f0[jG] -= df;
//  	}
//  #else
//  	f0[jG] -= df;
//  #endif

//  	if(stressmode) { 
//  	  stressloc -= srho_r[nj]*outerProduct(df, srho_dr[nj]);
//  	}
//        }

//        n3seq = 0;  
//        for(int nj=0; nj<n3-1; nj++) {
//  	int jg = sf_id[nj]; // logical index
//  	int jG = myPtcl->ID[jg];//physical index

//          SingleParticlePos_t drj = sf_dr[nj];

//  	for(int nk=nj+1; nk<n3; nk++) {
//  	  int kg = sf_id[nk];
//  	  int kG = myPtcl->ID[kg];

//  	  SingleParticlePos_t drk = sf_dr[nk];

//  	  Scalar_t dV3j = sg_g[n3seq] * sf_df[nj] * sf_f[nk];
//  	  Scalar_t dV3k = sg_g[n3seq] * sf_f[nj]  * sf_df[nk];

//  	  Scalar_t V3 = sf_f[nj] * sf_f[nk] * sg_dg[n3seq]; 
//  	  Scalar_t vlj = V3*sf_rinv[nj];
//  	  Scalar_t vlk = V3*sf_rinv[nk];

//  	  SingleParticlePos_t dfj = dV3j*drj + vlj*(drk- sg_lcos[n3seq]*drj);
//  	  SingleParticlePos_t dfk = dV3k*drk + vlk*(drj- sg_lcos[n3seq]*drk);
//  	  f0[iG] += dux*(dfj+dfk);

//  #ifdef ENABLE_OPENMP
//          // Forgot to add - (unary) operator, substract later for fv
//  	  map<int,SingleParticlePos_t>::iterator it;
//  	  if(jg < first || jg >= last) {//jg logically belongs to other node
//  	    it = fv.find(jG); 
//  	    if(it == fv.end()) 
//  	      fv[jG] = dux*dfj;
//  	    else 
//                (*it).second += dux*dfj;
//  	  } else
//  	    f0[jG] -= dux*dfj;

//  	  if(jg < first || jg >= last) {//jg logically belongs to other node
//  	    it = fv.find(kG); 
//  	    if(it == fv.end()) 
//  	      fv[kG] = dux*dfk;
//  	    else 
//                (*it).second += dux*dfk;
//  	  } else
//  	    f0[kG] -= dux*dfk;
//  #else
//  	  f0[jG] -= dux*dfj;
//  	  f0[kG] -= dux*dfk;
//  #endif
//  	  if(stressmode) {
//  	    stressloc -= sf_r[nj]*dux * outerProduct(dfj, drj);
//  	    stressloc -= sf_r[nk]*dux * outerProduct(dfk, drk);
//  	  }
//  	  n3seq++; // advance the sequence number of three body terms
//  	} // nk
//        } // nj
//        q[iG] = elocal;
//        esumloc += elocal;
//      }

//  #pragma omp critical
//      {    
//  #ifdef ENABLE_OPENMP
//        map<int,SingleParticlePos_t>::iterator it = fv.begin();
//        while(it != fv.end()) {
//  	f0[(*it).first] -= (*it).second;
//  	it++;
//        }
//  #endif
//      }
//      return esumloc;
//   }
//  bool
//  MEAMPotential::evalDynamicMatrix(DynMatrix_t&) 
//  {

//    return false;

//  }
//  Scalar_t 
//  MEAMPotential::e(const ParticlePos_t& R, int i, int myID) 
//  {

//    int iL = i - nnListEngine->first_id(myID);

//    const PtclPairList_t& nnlist = nnListEngine->getPairAttrib(myID);

//    Scalar_t elocal = 0;
//    SingleParticlePos_t dr;
//    Scalar_t r, rinv;
//    Scalar_t emaX = 0.0;

//    int num_nn = nnlist.nadj(iL);
//    vector<Scalar_t> sf_f(num_nn);
//    vector<SingleParticlePos_t> sf_dr(num_nn);

//    int n3 = 0;
//    for(int nni=0; nni < num_nn; ++nni) {
//      int j = nnlist.iadj(iL,nni);  //atomic index of jL-th nn of i
//      if(R.InUnit) 
//        dr = myPtcl->Lattice.BConds.apply(R[j]-R[i]);
//      else
//        dr = myPtcl->Lattice.BConds.apply(myPtcl->Lattice.toUnit(R[j]-R[i]));

//      r = sqrt(myPtcl->Lattice.Dot(dr,dr));

//      // check all the nearest atoms within a cutoff of phi
//      if(r < MEAMSet[0]->clsgnecut) 
//        {
//  	elocal += 0.5e0*MEAMSet[0]->phi.splint(r); //phi(r),double counting
//  	if(r < MEAMSet[0]->clsrhonecut) 
//  	  emaX += MEAMSet[0]->xrho[0]->splint(r); 
//  	if(r < MEAMSet[0]->clstricut1) 
//  	  {
//  	    rinv = 1/r;
//  	    sf_f[n3]  = MEAMSet[0]->xf[0]->splint(r); 
//  	    sf_dr[n3] = myPtcl->Lattice.toCart(dr)*rinv;
//  	    n3++;
//  	  }
//        }
//      } // for(jL=0;jL<nn; ++jL)

//      // calculating x for U(x) for the embedded atom term
//      // NOTE that terms from rho have been already added to emaX
//      for(int nj=0; nj < n3-1; nj++) 
//      {
//        for(int nk=nj+1; nk < n3; nk++) 
//  	{
//            Scalar_t lcos = dot(sf_dr[nj], sf_dr[nk]);
//            emaX += (sf_f[nj]*sf_f[nk]*MEAMSet[0]->xg[0]->splint(lcos));
//  	}
//      }
//      elocal += MEAMSet[0]->xu[0]->splint(emaX); // adding u(x) to elocal
//      return elocal; 
//  }

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: MEAMPotential.1.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

