/////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2004- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/** @file PotentialCollection.cpp
 * @brief Declare manager member functions.
 *
 * - constructor
 * - destructor
 * - get/put/initialize
 * - addPotential/addShortRangePotential/addRecord
 */
#include "Potentials/PotentialCollection.h"
#include "Potentials/ShortRangePotential.h"
#include "Particle/NNEngine.h"
#include "Utilities/OhmmsInfo.h"
namespace OHMMS {

  ///default constructor
  PotentialCollection::PotentialCollection():
    Ready(false),
    UseLatticeUnit(true),
    UseGridData(false),
    UseNNList(false),
    LastAddedTerm(-1),
    LayoutXmlNode(0)
  { }
  
  PotentialCollection::~PotentialCollection() {
    DEBUGMSG("Calling PotentialCollection::~ShortrangePotential()")
    for(int s=0; s<H_short.size(); s++) delete H_short[s];
    for(int l=0; l<H_long.size(); l++) delete H_long[l];
  }

  bool PotentialCollection::get(ostream& os) const {
    return true;
  }

  bool PotentialCollection::put(istream& os) {
    return false;
  }

  bool PotentialCollection::put(xmlNodePtr cur) {
    //if(LastAddedTerm == 0) {
    //  H_long.back()->put(cur);
    //} else if(LastAddedTerm == 1) {
    //  H_short.back()->put(cur);
    //}
    return true;
  }
  
  void PotentialCollection::initialize() {

    //Assign the ParticleSet to each potential term
    for(int s=0; s<H_short.size(); s++) {
      H_short[s]->setParticleSet(myPtcl);
    }      

    for(int s=0; s<H_long.size(); s++) {
      H_long[s]->setParticleSet(myPtcl);
    }      

    //Process input parameters
    std::map<string,CheckType>::iterator it(H_long_list.begin());
    std::map<string,CheckType>::iterator it_end(H_long_list.end());
    while(it != it_end) {
      if((*it).second.second) {
        H_long[(*it).second.first]->put((*it).second.second);
        (*it).second.second=0;
        ++it;
      }
    }
    it = H_short_list.begin();
    it_end = H_short_list.end();
    while(it != it_end) {
      if((*it).second.second) {
        H_short[(*it).second.first]->put((*it).second.second);
        (*it).second.second=0;
        ++it;
      }
    }
  }

  void 
  PotentialCollection::addPotential(PotentialBase* aterm, 
      const string& aname, xmlNodePtr cur) {
    std::map<string,CheckType>::iterator it = H_long_list.find(aname);
    if(it == H_long_list.end()) {
      H_long_list[aname] = CheckType(H_long.size(),cur);
      H_long.push_back(aterm);
      LastAddedTerm = 0;
    } else {
      WARNMSG("Interaction " << aname << " exists. Ignore the addtion.")
    }

  }

  void 
  PotentialCollection::addShortRangePotential(ShortRangePotential* aterm, 
                                              const string& aname,
                                              xmlNodePtr cur) {
    std::map<string,CheckType>::iterator it = H_short_list.find(aname);
    if(it == H_short_list.end()) {
      H_short_list[aname] = CheckType(H_short.size(),cur);
      H_short.push_back(aterm);
      CollectVisitingForces |= aterm->CollectVisitingForces;
      LastAddedTerm = 1;
    } else {
      WARNMSG("Short-range interaction " << aname << " exists. Ignore the addtion.")
    }
  }

  bool PotentialCollection::remove(const string& aname){
    std::map<string,CheckType>::iterator it(H_short_list.find(aname));
    std::map<string,CheckType>::iterator it_end(H_short_list.find(aname));
    while(it != it_end) {
      size_type n=(*it).second.first;
      delete H_short[n];
      for(size_type i=n+1; i<H_short.size(); i++) H_short[i-1]=H_short[i];
      H_short.pop_back();
      std::map<string,CheckType>::iterator jt=it++;
      while(it != H_short_list.end()) {
	(*it).second.first -= 1;it++;
      }
      H_short_list.erase(jt);
      return true;
    }

    it = H_long_list.find(aname);
    if(it != H_long_list.end()) {
      size_type n=(*it).second.first;
      delete H_long[n];
      for(size_type i=n+1; i<H_long.size(); i++) H_long[i-1]=H_long[i];
      H_long.pop_back();
      std::map<string,CheckType>::iterator jt=it++;
      while(it != H_long_list.end()) {
	(*it).second.first -= 1;
	it++;
      }
      H_long_list.erase(jt);
      return true;
    }

    return false;
  }      

  bool 
  PotentialCollection::addRecord(RecordPropertyList& reclist, xmlNodePtr cur) {
    bool has_record=false;
    for(int l=0; l<H_long.size(); l++) {
      RecordProperty* rec = H_long[l]->getRecorder();
      if(rec) {
	reclist.addRecord(rec);
	has_record =true;
      }
    }
    return has_record;
  }
 
  void 
  PotentialCollection::setLayout(xmlNodePtr pl_ptr) {
    LayoutXmlNode = pl_ptr;
  }

  /** Assign a ParticleSet* to the PotentialBase::myPtcl
   *
   * setParticleSet of the components is called.
   */
  void 
  PotentialCollection::reset() {
  }

  void 
  PotentialCollection::setParticleSet(Particle_t* ptclset) {

    if(myPtcl == ptclset) return;

    myPtcl=ptclset;
    initialize();

    LOGMSG("\tPotentialCollection::setParticleSet initialize the layout")
    Rmax = -1.0;
    for(int s=0; s<H_short.size(); s++) {
      Rmax = std::max(H_short[s]->Rmax,Rmax);
    }      
    GridRadius = -1.0;
    for(int s=0; s<H_long.size(); s++) {
      GridRadius = std::max(H_long[s]->getRmax(),GridRadius);
    }      
    if(GridRadius>0) Rmax = GridRadius;
    if(Rmax>0){
      nnListEngine = myPtcl->getNNEngine();
      if(nnListEngine == 0) {
	nnListEngine = NNEngineBuilder::create(LayoutXmlNode);
      } else {
        ERRORMSG("\tUsing an existing layout ")
      }
      UseNNList = true;        
      UseGridData = nnListEngine->UseLayout;
      Rmax *= nnListEngine->Rscale;
      LOGMSG("\tCut-off radius for nnlist = " << Rmax)
      //if(GridRadius<0) GridRadius = nnListEngine->Rscale;
      GridRadius = Rmax;
      myPtcl->setNNEngine(nnListEngine);
    }

    myPtcl->updateLayout(Rmax,GridRadius);
    //Counter = 0;
    //myPtcl->setCounter(Counter);

    if(UseNNList) {
      if(!Ready) {
	nnListEngine->initialize(*myPtcl,CollectVisitingForces);
        //Calculate the lis
	Ready = true;
      }
    }
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: PotentialCollection.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

