//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//
// Tue Jun 20 15:58:36 CDT 2000 Local pressure calculation is added.
// Fri Dec 28 10:20:23 CST 2001 OpenMP parallelization
// Tue Nov 30 15:04:41 CST 2004 Multiple-species and put function to read the parameters
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Potentials/SWPotential.h"
#include "Utilities/OhmmsInfo.h"
#include "ParticleBase/ParticleMessage.h"
#include "OhmmsData/ParameterSet.h"
#include "Potentials/SWParam.h"
#include "Potentials/StressAnalysis.h"
#include <set>

namespace OHMMS {

  SWPotential::Scalar_t
  SWPotential::evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale) const{

    typedef PtclPairList_t::PairData_t    PairData_t;
    size_type nnloc = 0;
    Scalar_t  esum = 0.0e0;

    // storing three-body function
    vector<Scalar_t> g(nnlist.getMaxNadj());
    vector<SingleParticlePos_t> dcos(nnlist.getMaxNadj());
    for(int iL = 0; iL<nnlist.getLocalNum();iL++) {
      size_type iat = nnlist.I[iL];
      size_type ibond = myPtcl->GroupID[iat]*NumSpecies;
      size_type n2=0;
      PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
      while(nit != nit_end) {
        const PairData_t& cur(*nit);
	const SWParam<Scalar_t>& sw(*(SWSet[ibond+cur.Group]));
	if(sw.inside(cur.sep)) {
          Scalar_t rinv(1.0/cur.sep);
          esum+=sw.evaluate(cur.sep,rinv,g[n2]);
          dcos[n2]=rinv*cur.displ;
          ++n2;
        }
        ++nit;
      }// while(nit!=nit_end)

      for(size_type nj=0; nj<n2-1; nj++) {
        for(size_type nk=nj+1; nk<n2; nk++) {
          Scalar_t x(dot(dcos[nj], dcos[nk]) - Theta0);
          //esum+= (Lambda * g[nj] * g[nk]* x * x);
          esum+= g[nj]*g[nk]* x * x;
        }
      }
    }
    return esum;
  }


  SWPotential::Scalar_t 
  SWPotential::evalForceNN(PtclPairList_t& nnlist, 
			   ParticlePos_t& fv, 
			   unsigned stressmode) const {

    typedef PtclPairList_t::PairData_t    PairData_t;
    ParticleScalar_t& q = myPtcl->E;
    vector<SWPair<Scalar_t> > v2(nnlist.getMaxNadj());
    Scalar_t  esumloc = 0.0;
    Tensor_t  stressloc;
    size_type nnloc(0);
    SingleParticlePos_t df;
    for(size_type iL=0; iL<nnlist.getLocalNum(); iL++) {
      size_type iat(nnlist.I[iL]);
      size_type n2(0);
      size_type ibond(myPtcl->GroupID[iat]*NumSpecies);
      Scalar_t elocal(0.0);
      SingleParticlePos_t fon(0.0);
      PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
      while(nit != nit_end) {
        const PairData_t& cur(*nit);
	const SWParam<Scalar_t>& sw(*(SWSet[ibond+ cur.Group]));
	if(sw.inside(cur.sep)) {
	  elocal += sw.evaluate(cur.Where,cur.sep,cur.displ,v2[n2],df);
          fon += 2.0*df;
	  if(stressmode) stressloc -= outerProduct(df,cur.displ);
	  n2++;
	}
        ++nit;
      } // while(nit!=nit_end)

      for(size_type nj=0; nj<n2-1; nj++) {

	const SWPair<Scalar_t>& v3j=v2[nj];
	SingleParticlePos_t drj(v3j.dcos);
	Scalar_t rij(v3j.sep);

	for(size_type nk=nj+1; nk<n2; nk++) {
	  const SWPair<Scalar_t>& v3k=v2[nk];
	  SingleParticlePos_t drk(v3k.dcos);
	  Scalar_t lcos(dot(drj, drk));
	  Scalar_t x(lcos - Theta0);
	  Scalar_t xx(x*x);

	  Scalar_t V3(v3j.g * v3k.g);
	  Scalar_t dV3j(xx * v3j.dg * v3k.g);
	  Scalar_t dV3k(xx * v3j.g  * v3k.dg);

	  Scalar_t dVl(V3*2.0e0*x); 
	  Scalar_t vlj(dVl*v3j.rinv);
	  Scalar_t vlk(dVl*v3k.rinv);
	
	  SingleParticlePos_t dfj(dV3j*drj + vlj*(drk-lcos*drj));
	  SingleParticlePos_t dfk(dV3k*drk + vlk*(drj-lcos*drk));
	
	  elocal+= (V3*xx);
	
	  fon += (dfj+dfk);
          fv[v3j.J] -=dfj;
          fv[v3k.J] -=dfk;
	  if(stressmode) {
	    stressloc -= rij*outerProduct(dfj, drj);
	    stressloc -= v3k.sep*outerProduct(dfk, drk);
	  }
	}//k
      }//j

      fv[iL] += fon;
      q[iat] = elocal;
      esumloc += elocal;
    }
    nnlist.Stress = stressloc;
    return esumloc;
  }

  void
  SWPotential::analize(const PtclPairList_t& nnlist, size_type first, size_type last) {

    typedef PtclPairList_t::PairData_t    PairData_t;
    ParticleScalar_t& exx = *(myPtcl->getScalarAttrib("exx"));
    ParticleScalar_t& eyy = *(myPtcl->getScalarAttrib("eyy"));
    ParticleScalar_t& ezz = *(myPtcl->getScalarAttrib("ezz"));
    ParticleScalar_t& exy = *(myPtcl->getScalarAttrib("exy"));
    ParticleScalar_t& eyz = *(myPtcl->getScalarAttrib("eyz"));
    ParticleScalar_t& ezx = *(myPtcl->getScalarAttrib("ezx"));

    //the maximum number of neighbors per atom
    size_type maxnn = nnlist.getMaxNadj();
    vector<SWPair<Scalar_t> > v2(maxnn);

    Scalar_t  elocal(0.0);
    SingleParticlePos_t df;

    //for(int iat=first, iL = 0; iat<last; iat++, iL++) {
    for(size_type iat=first, iL=0; iat<last; iL++,iat++) {
      //particle index on this process
      size_type n2=0;
      Scalar_t elocal = 0.0,etemp; // local energy to be accumulated
      size_type ibond = myPtcl->GroupID[iat]*NumSpecies;

      Tensor_t astress;    

      PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
      while(nit != nit_end) {
        const PairData_t& cur(*nit);
        const SWParam<Scalar_t>& sw(*(SWSet[ibond+ cur.Group]));
        if(sw.inside(cur.sep)) {
          elocal += sw.evaluate(cur.Where,cur.sep,cur.displ,v2[n2],df);
          astress -= outerProduct(df, cur.displ);
          ++n2;
        }
        ++nit;
      } // while(nit != nit_end)

      for(size_type nj=0; nj<n2-1; nj++) {

        const SWPair<Scalar_t>& v3j=v2[nj];
        SingleParticlePos_t drj(v3j.dcos);
        Scalar_t rij = v3j.sep;
        
        for(size_type nk=nj+1; nk<n2; nk++) {

          const SWPair<Scalar_t>& v3k=v2[nk];
          SingleParticlePos_t drk(v3k.dcos);

          Scalar_t lcos = dot(drj, drk);
          Scalar_t x = (lcos - Theta0);
          Scalar_t xx = x*x;

          Scalar_t V3 =  v3j.g * v3k.g;
          Scalar_t dV3j =  xx * v3j.dg * v3k.g;
          Scalar_t dV3k =  xx * v3j.g  * v3k.dg;

          Scalar_t dVl = V3*2.0e0*x; 
          Scalar_t vlj = dVl*v3j.rinv;
          Scalar_t vlk = dVl*v3k.rinv;
        
          SingleParticlePos_t dfj(dV3j*drj + vlj*(drk-lcos*drj));
          SingleParticlePos_t dfk(dV3k*drk + vlk*(drj-lcos*drk));

          astress -= rij*outerProduct(dfj, drj);
          astress -= v3k.sep*outerProduct(dfk, drk);
        }//k
      }//j

      exx[iat]=astress(0,0);
      exy[iat]=astress(0,1);
      ezx[iat]=astress(0,2);
      eyy[iat]=astress(1,1);
      eyz[iat]=astress(1,2);
      ezz[iat]=astress(2,2);
   }
  }

  /*
  SWPotential::Scalar_t 
  SWPotential::evalForceNN(PtclPairList_t& nnlist, 
			   ParticlePos_t& f0, 
			   ParticleScalar_t& q, 
			   unsigned stressmode) const {

    int myid=nnlist.myID;
    int nfv=nnlist.Visitors;
    bool multiplethread = nnlist.MultiSet;

    PtclPairList_t::IndexList_t& fvind = nnlist.IDv;
    PtclPairList_t::VectorList_t& fv = nnlist.Fv;
    const ParticleIndex_t* o=NULL;
    if(multiplethread) o = myPtcl->getIndexAttrib(ParticleTags::openmp_tag);

    //the maximum number of neighbors per atom
    int maxnn = nnlist.getMaxNadj();
    vector<SWPair<Scalar_t> > v2(maxnn);
    vector<SingleParticlePos_t> fref(maxnn);

    Scalar_t  esumloc = 0.0;
    Tensor_t  stressloc;
    int nnloc = 0;
    SingleParticlePos_t df;

    for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
      int iat = nnlist.I[iL];
      int num_nn = nnlist.nadj(iL);
      int n2=0;
      Scalar_t elocal = 0.0,etemp; // local energy to be accumulated
      int ibond = myPtcl->GroupID[iat]*NumSpecies;
      for(int nni = 0; nni< num_nn; nni++, nnloc++) {
	Scalar_t r=nnlist.R[nnloc];
	const SWParam<Scalar_t>& sw= *(SWSet[ibond+ myPtcl->GroupID[nnlist.iadj(iL,nni)]]);
	if(sw.inside(r)) {
	  elocal += sw.evaluate(nnlist.iadj(iL,nni),r,nnlist.dR[nnloc],v2[n2],df);
	  f0[iat] += 2.0*df;
	  if(stressmode) stressloc -= r*outerProduct(df, v2[n2].dr);
	  fref[n2]=0.0;
	  n2++;
	}
      } // for jL < nn

      for(int nj=0; nj<n2-1; nj++) {

	const SWPair<Scalar_t>& v3j=v2[nj];

	SingleParticlePos_t drj = v3j.dr;
	Scalar_t rij = v3j.sep;
      
	for(int nk=nj+1; nk<n2; nk++) {

	  const SWPair<Scalar_t>& v3k=v2[nk];
	  SingleParticlePos_t drk = v3k.dr;

	  Scalar_t lcos = dot(drj, drk);
	  Scalar_t x = (lcos - Theta0);
	  Scalar_t xx = x*x;

	  Scalar_t V3 =  v3j.g * v3k.g;
	  Scalar_t dV3j =  xx * v3j.dg * v3k.g;
	  Scalar_t dV3k =  xx * v3j.g  * v3k.dg;

	  Scalar_t dVl = V3*2.0e0*x; 
	  Scalar_t vlj = dVl*v3j.one_over_r;
	  Scalar_t vlk = dVl*v3k.one_over_r;
	
	  SingleParticlePos_t dfj = dV3j*drj + vlj*(drk-lcos*drj);
	  SingleParticlePos_t dfk = dV3k*drk + vlk*(drj-lcos*drk);
	
	  elocal+= (V3*xx);
	
          fref[nj]-=dfj;
          fref[nk]-=dfk;
	  f0[iat] += (dfj+dfk);

	  if(stressmode) {
	    stressloc -= rij*outerProduct(dfj, drj);
	    stressloc -= v3k.sep*outerProduct(dfk, drk);
	  }
	}//k
      }//j

      if(multiplethread) {
	for(int nj=0; nj<n2; nj++) {
	  int jat=v2[nj].J;
	  if((*o)[jat]==myid) 
	    f0[jat]+= fref[nj];
	  else {
	    fv[nfv]=fref[nj]; fvind[nfv]=jat; nfv++;
	  }
	}
      } else {
	for(int nj=0; nj<n2; nj++) f0[v2[nj].J]+= fref[nj];
      }
      q[iat] = elocal;
      esumloc += elocal;
    }

    //Assignment to PtclPairList
    nnlist.Visitors = nfv;
    nnlist.Stress = stressloc;
    return esumloc;
  }
  */

}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: SWPotential.1.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
