//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//
// Tue Jun 20 15:58:36 CDT 2000 Local pressure calculation is added.
// Fri Dec 28 10:20:23 CST 2001 OpenMP parallelization
// Tue Nov 30 15:04:41 CST 2004 Multiple-species and put function to read the parameters
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Message/OpenMP.h"
#include "Potentials/SWPotential.h"
#include "Utilities/OhmmsInfo.h"
#include "ParticleBase/ParticleMessage.h"
#include "OhmmsData/ParameterSet.h"
#include "Potentials/SWParam.h"
#include "Potentials/StressAnalysis.h"
#include <set>

namespace OHMMS {

  SWPotential::SWPotential(): InFileName("none"){ 
    CollectVisitingForces = true;
    RecordPropertyList::registerRecord("sw_stress",new StressAnalysis<SWPotential>(*this));
  }

  SWPotential::SWPotential(int argc, char **argv):
    InFileName("none") 
  {
    CollectVisitingForces = true;
  }

  SWPotential::~SWPotential() {
    DEBUGMSG("Calling SWPotential::~SWPotential()");
  }

  void SWPotential::initialize() {
    myName = "SW"; // name is SW
  }
  

// Moved to SWPotential.1.cpp
//  SWPotential::Scalar_t 
//  SWPotential::evalForceNN(PtclPairList_t& nnlist, 
//			   ParticlePos_t& f0, 
//			   ParticleScalar_t& q, 
//			   unsigned stressmode) const {
//
//    int myid=nnlist.myID;
//    int nfv=nnlist.Visitors;
//    bool multiplethread = nnlist.MultiSet;
//
//    PtclPairList_t::IndexList_t& fvind = nnlist.IDv;
//    PtclPairList_t::VectorList_t& fv = nnlist.Fv;
//    const ParticleIndex_t* o=NULL;
//    if(multiplethread) o = myPtcl->getIndexAttrib(ParticleTags::openmp_tag);
//
//    //the maximum number of neighbors per atom
//    int maxnn = nnlist.getMaxNadj();
//    vector<SWPair<Scalar_t> > v2(maxnn);
//    vector<SingleParticlePos_t> fref(maxnn);
//
//    Scalar_t  esumloc = 0.0;
//    Tensor_t  stressloc;
//    int nnloc = 0;
//    SingleParticlePos_t df;
//
//    for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
//      int iat = nnlist.I[iL];
//      int num_nn = nnlist.nadj(iL);
//      int n2=0;
//      Scalar_t elocal = 0.0,etemp; // local energy to be accumulated
//      int ibond = myPtcl->GroupID[iat]*NumSpecies;
//      for(int nni = 0; nni< num_nn; nni++, nnloc++) {
//	Scalar_t r=nnlist.R[nnloc];
//	const SWParam<Scalar_t>& sw= *(SWSet[ibond+ myPtcl->GroupID[nnlist.iadj(iL,nni)]]);
//	if(sw.inside(r)) {
//	  elocal += sw.evaluate(nnlist.iadj(iL,nni),r,nnlist.dR[nnloc],v2[n2],df);
//	  f0[iat] += 2.0*df;
//	  if(stressmode) stressloc -= r*outerProduct(df, v2[n2].dr);
//	  fref[n2]=0.0;
//	  n2++;
//	}
//      } // for jL < nn
//
//      for(int nj=0; nj<n2-1; nj++) {
//
//	const SWPair<Scalar_t>& v3j=v2[nj];
//
//	SingleParticlePos_t drj = v3j.dr;
//	Scalar_t rij = v3j.sep;
//      
//	for(int nk=nj+1; nk<n2; nk++) {
//
//	  const SWPair<Scalar_t>& v3k=v2[nk];
//	  SingleParticlePos_t drk = v3k.dr;
//
//	  Scalar_t lcos = dot(drj, drk);
//	  Scalar_t x = (lcos - Theta0);
//	  Scalar_t xx = x*x;
//
//	  Scalar_t V3 =  v3j.g * v3k.g;
//	  Scalar_t dV3j =  xx * v3j.dg * v3k.g;
//	  Scalar_t dV3k =  xx * v3j.g  * v3k.dg;
//
//	  Scalar_t dVl = V3*2.0e0*x; 
//	  Scalar_t vlj = dVl*v3j.one_over_r;
//	  Scalar_t vlk = dVl*v3k.one_over_r;
//	
//	  SingleParticlePos_t dfj = dV3j*drj + vlj*(drk-lcos*drj);
//	  SingleParticlePos_t dfk = dV3k*drk + vlk*(drj-lcos*drk);
//	
//	  elocal+= (V3*xx);
//	
//          fref[nj]-=dfj;
//          fref[nk]-=dfk;
//	  f0[iat] += (dfj+dfk);
//
//	  if(stressmode) {
//	    stressloc -= rij*outerProduct(dfj, drj);
//	    stressloc -= v3k.sep*outerProduct(dfk, drk);
//	  }
//	}//k
//      }//j
//
//      if(multiplethread) {
//	for(int nj=0; nj<n2; nj++) {
//	  int jat=v2[nj].J;
//	  if((*o)[jat]==myid) 
//	    f0[jat]+= fref[nj];
//	  else {
//	    fv[nfv]=fref[nj]; fvind[nfv]=jat; nfv++;
//	  }
//	}
//      } else {
//	for(int nj=0; nj<n2; nj++) f0[v2[nj].J]+= fref[nj];
//      }
//      q[iat] = elocal;
//      esumloc += elocal;
//    }
//
//    //Assignment to PtclPairList
//    nnlist.Visitors = nfv;
//    nnlist.Stress = stressloc;
//    return esumloc;
//  }


  bool SWPotential::get(ostream& os) const { 
    return true;
  }
  bool SWPotential::put(istream&) {return true;}

  void SWPotential::reset() {
  }

  bool SWPotential::put(xmlNodePtr cur){
    //search order: src, file and filename
    const xmlChar* s=xmlGetProp(cur,(const xmlChar*)"src");
    if(s == NULL) { 
      s=xmlGetProp(cur,(const xmlChar*)"file");
    }
    const char* fname=0;
    if(s) {
      fname = (const char*)s;
    } else {//check if filename node exists
      xmlNodePtr tcur = cur->children;
      while(tcur != NULL) {
        string cname((const char*)tcur->name);
        if(cname == "filename") {
          fname = (const char*)xmlNodeListGetString(tcur->doc, tcur->xmlChildrenNode, 1);
        }
        tcur=tcur->next;
      }
    }

    if(fname) {
      LOGMSG("SW parameters are provided in " << fname)
      // build an XML tree from a the file;
      xmlDocPtr doc = xmlParseFile(fname);
      if (doc == NULL) return false;
    
      // Check the document is of the right kind
      xmlNodePtr tcur = xmlDocGetRootElement(doc);
      if (tcur == NULL) {
	fprintf(stderr,"empty document\n");
	xmlFreeDoc(doc);
	return false;
      }
      
      xmlNsPtr ns = NULL;
      string cname((const char*)(tcur->name));
      if(cname != "potential") {
	fprintf(stderr,"document of the wrong type, root node != potential");
	xmlFreeDoc(doc);
	return false;
      }
      const xmlChar* f=xmlGetProp(tcur,(const xmlChar*) "name");
      bool success=false;
      if(f) {
	cname = (const char*)f;
	if(cname == "SW") {
	  success= putSpecial(tcur);
	}
      }     
      xmlFreeDoc(doc);
      if(!success)
	ERRORMSG("SW potential initialization has failed. Check the root of " << fname)
      return success;
    } else {//embedded
      return putSpecial(cur);
    }
  }

  bool SWPotential::putSpecial(xmlNodePtr cur){

    if(myPtcl == 0) {
      ERRORMSG("Illegal operation. SWPotential::putSpecial is used before setting the particle")  
    }

    SpeciesSet& mySpecies = myPtcl->getSpeciesSet();

    size_type irmax = mySpecies.addAttrib("rmax");
    size_type imass = mySpecies.addAttrib("mass");

    Rmax = 3.77118;
    Theta0 = -1.0e0/3.0e0;  // only for tetraheral structures

    cur = cur->children;
    vector<xmlNodePtr> pii_cur, pij_cur;

    while(cur != NULL) {
      string cname((const char*)(cur->name));
      if(cname=="species") {
	xmlChar *ap=xmlGetProp(cur,(const xmlChar*)"name");
	xmlChar *mp=xmlGetProp(cur,(const xmlChar*)"mass");
	size_type newid=0;
	double qmass = 28.8;
	if(ap) {
	  newid =mySpecies.addSpecies((const char*)ap);
	}
	if(mp) {
	  qmass = atof((const char*)mp);
	}
	mySpecies(imass,newid)= qmass;
      } else if(cname == "pair") {
	xmlChar *ap=xmlGetProp(cur,(const xmlChar*)"speciesA");
	xmlChar *bp=xmlGetProp(cur,(const xmlChar*)"speciesB");
	size_type itype=0, jtype=0;
	if(ap) itype = mySpecies.addSpecies((const char*)ap);
	if(bp) jtype = mySpecies.addSpecies((const char*)bp);
	if(itype == jtype) {
	  pii_cur.push_back(cur);
	}
	else
	  pij_cur.push_back(cur);
      }
      cur = cur->next;
    }

    if(pii_cur.empty()) {
      size_type newid =mySpecies.addSpecies("Si");
      mySpecies(imass,newid)=28.8;
      mySpecies(irmax,newid) = Rmax*0.5;
      SWSet.push_back(new SWParam<Scalar_t>());
      NumSpecies=1;
    } else {
      NumSpecies = mySpecies.getTotalNum();
      for(size_type is=0; is<NumSpecies*NumSpecies; is++) SWSet.push_back(0);
      for(size_type is=0; is<pii_cur.size(); is++) {
	xmlChar *ap=xmlGetProp(pii_cur[is],(const xmlChar*)"speciesA");
	size_type itype=0;
	if(ap) {
	  itype = mySpecies.addSpecies((const char*)ap);
	}
	SWParam<Scalar_t> *vii = new SWParam<Scalar_t>(pii_cur[is]);
	SWSet[itype*NumSpecies+itype]=vii;

	//Determine the maxium cutoff radius
	Rmax = std::max(Rmax,vii->a); 
      }

      if(NumSpecies>1) {//use the symmetric form
	for(int is=0; is<NumSpecies; is++) {
	  for(int js=is+1; js<NumSpecies; js++) {
	    SWParam<Scalar_t> *vij 
	      = new SWParam<Scalar_t>(*SWSet[is*NumSpecies+is],*SWSet[js*NumSpecies+js]); 
	    SWSet[is*NumSpecies+js] = vij;
	    SWSet[js*NumSpecies+is] = vij;
	  }
	}
      }
    }
//     string sname("Si");
//     int stype = 0;
// #if defined(HAVE_LIBXML2)
//     ///process attributes: type or format
//     xmlAttrPtr att = cur->properties;
//     while(att != NULL) {
//       string aname((const char*)(att->name));
//       if(aname == "species") {
//         sname = (const char*)(att->children->content);
//       } else if(aname == "id") {
//          stype = atoi((const char*)(att->children->content));
//       }
//       att = att->next;
//     }
//     SpeciesSet* mySpecies = SpeciesCollection::getSpecies();
//     if(mySpecies->getTotalNum() > stype) {
//       mySpecies->Name[stype] = sname;
//     } 
// #endif
    return true;
  }
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: SWPotential.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
