//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_SHORTRANGEPOTENTIAL_H
#define OHMMS_SHORTRANGEPOTENTIAL_H

#include "OhmmsData/OhmmsElementBase.h"
#include "Particle/ParticleSet.h"

namespace OHMMS {

  /** Abstract class for an interaction with a finite cutoff
   */
  struct ShortRangePotential: public OhmmsElementBase {

    typedef ParticleSet                     Particle_t;
    typedef Particle_t::size_type           size_type;
    typedef Particle_t::Scalar_t            Scalar_t;
    typedef Particle_t::SingleParticlePos_t SingleParticlePos_t;
    typedef Particle_t::Tensor_t            Tensor_t;

    typedef Particle_t::ParticleIndex_t     ParticleIndex_t;
    typedef Particle_t::ParticleScalar_t    ParticleScalar_t;
    typedef Particle_t::ParticlePos_t       ParticlePos_t;
    typedef Particle_t::ParticleTensor_t    ParticleTensor_t;

    typedef Particle_t::PtclPairList_t      PtclPairList_t;

    ///The cut-off radius of an interaction
    Scalar_t Rmax;

    Scalar_t DistScale;

    ///Reference to a particle set
    ParticleSet *myPtcl;

    ///true, if the visiting forces need to be assembled
    bool CollectVisitingForces;

    ///Constructor
    ShortRangePotential();

    ///Desstructor
    virtual ~ShortRangePotential();

    ///Set a ParticleSet whose forces will be evaulated
    virtual void setParticleSet(Particle_t* ptclset) { myPtcl=ptclset;}

    ///reset the ParticleSet: no internal changes
    virtual void resetParticleSet(Particle_t* ptclset) = 0;

    /** evaluate partial forces of an interaction term
     *@param nnlist a PtclPairList&
     *@param f0 a ParticlePos_t& for on-thread forces 
     *@param q a ParticleScalar_t& for atomic energies
     *@return the sum of atomic energies on-thread
     *
     *This function is called by PotentialCollection::evalForce for each thread .
     *A ShortRangePotential implements the functional-dependent force evaluations.
     *For n-body interactions with n>2, it is generally more efficient to
     *evalaute the visiting forces according to Newton's 2nd law and add
     *the term after all the pieces are calculated.
     *PtclPairList has the data members, Fv and IDv, and Visitors to facilitate
     *the force evaluations. See PotentialCollention::evalForce.
     */
    virtual Scalar_t evalForceNN(PtclPairList_t& nnlist, 
                                 ParticlePos_t& f0, 
                                 unsigned stressmode) const =0;

    virtual Scalar_t evalEnergyNN(const PtclPairList_t& nnlist,Scalar_t rscale=1.0) const =0;

  };


}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 363 $   $Date: 2005-07-13 13:33:18 -0500 (Wed, 13 Jul 2005) $
 * $Id: ShortRangePotential.h 363 2005-07-13 18:33:18Z jnkim $ 
 ***************************************************************************/
