//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_STRESSANALYSIS_H
#define OHMMS_STRESSANALYSIS_H

#include "OhmmsData/RecordProperty.h"
#include "Particle/NNEngine.h"

namespace OHMMS {

  template<class MODEL>
  struct StressAnalysis: public RecordProperty {
    
    typedef std::size_t size_type;

    int OutputFormat;
    MODEL& ref_;
    
    StressAnalysis(MODEL& kp):ref_(kp){}
    
    void reset(const char* fileroot) {
      FileName = fileroot;
      FileName.append(".stress");
      typedef NNEngine::ParticleScalar_t ParticleScalar_t;
      //Register particle attributes
      ParticleScalar_t* exx =ref_.myPtcl->getScalarAttrib("exx");
      ParticleScalar_t* eyy =ref_.myPtcl->getScalarAttrib("eyy");
      ParticleScalar_t* ezz =ref_.myPtcl->getScalarAttrib("ezz");
      ParticleScalar_t* exy =ref_.myPtcl->getScalarAttrib("exy");
      ParticleScalar_t* eyz =ref_.myPtcl->getScalarAttrib("eyz");
      ParticleScalar_t* ezx =ref_.myPtcl->getScalarAttrib("ezx");
      ref_.myPtcl->addRestartAttrib("exx");
      ref_.myPtcl->addRestartAttrib("eyy");
      ref_.myPtcl->addRestartAttrib("ezz");
      ref_.myPtcl->addRestartAttrib("exy");
      ref_.myPtcl->addRestartAttrib("eyz");
      ref_.myPtcl->addRestartAttrib("ezx");
    }

    void report(int iter) {

      if(iter!=-1) return;

#pragma omp parallel 
      {
        omp_int_t ip = omp_get_thread_num();
	const NNEngine *nnListEngine = ref_.myPtcl->getNNEngine();
  	size_type first = nnListEngine->first_id(ip);
  	size_type last = nnListEngine->last_id(ip);
  	size_type nitem = last-first;
  	ref_.analize(nnListEngine->getPairAttrib(ip),first,last);
      }//end-of-omp
    }

    void finalize() {
    }
    
    bool put(xmlNodePtr cur) {
      return true;
    }
  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: StressAnalysis.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
