//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Richard Hennig and Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/*!\class AlloyEAMPotential
 * \author Richard Hennig
 * \author Jeongnim Kim
 * \brief Derived class from PotentialBase.  Implements AlloyEAM potential for single
 *  species.
 * \note  
 * Implemented by Richard Hennig 6/2001
 *
 * Based on MAlloyEAMPotential originally written by Thomas Lenosky, LLNL, 12/8/99
 *
 * Modified to confirm new interfaces of PotentialBase Jeongnim Kim 9/27/2001
 ************************************************************************/
#ifndef AlloyEAMPOTENTIAL_H
#define AlloyEAMPOTENTIAL_H

#include "Potentials/PotentialBase.h"
#include "OhmmsPETE/OhmmsMatrix.h"

class AlloyEAMParam;

class AlloyEAMPotential: public PotentialBase{

public:

  AlloyEAMPotential();
  AlloyEAMPotential(ThisParticle_t*);
  AlloyEAMPotential(int argc, char** argv);
  AlloyEAMPotential(vector<string>&);

  ~AlloyEAMPotential();

  void initialize();
  Scalar_t evalEnergy();
  Scalar_t evalForce(ParticlePos_t& f, ParticleScalar_t& e, Tensor_t&, 
                     unsigned stressmode = 0);
  Scalar_t e(const ParticlePos_t&, int i, int myid=0);// returns a local energy
  bool evalDynamicMatrix(DynMatrix_t&);

protected:

  AlloyEAMParam* EAMSet;

};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: AlloyEAMPotentialOMP.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
