//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Richard Hennig and Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/*! 
 * \author Richard Hennig
 * \author Jeongnim Kim
 * \note
 * Based on MEAM code written by Thomas Lenosky, LLNL, 12/8/99
 *
 * Modified for PoomaMD by Jeongnim Kim 12/8/99
 *
 * Modified for MgEAM by Richard Hennig 6/2001
 ************************************************************************/
#ifndef MgEAMPARAM_H
#define MgEAMPARAM_H

#include <vector>
using std::vector;

// Macros used by Lenosky

/*! \class SplineTypeMgEAM
 *  \brief Similar to SplineType in CubicSplineType.h.
 */
struct SplineTypeMgEAM {
  typedef double Scalar_t;
  vector<Scalar_t> X;
  vector<Scalar_t> Y;
  vector<Scalar_t> b, c, d;
  //Scalar_t Yp1;
  //Scalar_t Ypn;
  int Npt;
  int iset1,isetn,ider1,idern;
  SplineTypeMgEAM(){ }
  void readSpline(istream& is);
  void getSpline();
  Scalar_t splint(Scalar_t r);
  void splint(Scalar_t r, Scalar_t& val, Scalar_t& deriv); 
};


/*! \class EAMParam
 *  \brief Contain spline parameters for EAM potentials.
 *         Contains various splines for a EAM potential
 *  \note Using SplineType defined in CubicSplineType.h
 */
struct MgEAMParam {
  //Ranges of various splines in potential
  typedef double Scalar_t;
  Scalar_t clsphircut;    //Range of pair potential
  Scalar_t clsrhorcut;    //Range for rho(r)
  Scalar_t clsfrhocut;    //Range for f(rho)
  SplineTypeMgEAM phi, f, rho;

  MgEAMParam(const char *fname) {
    readMgEAMParam(fname);
    clsphircut = phi.X[phi.Npt];
    clsrhorcut = rho.X[rho.Npt];
    clsfrhocut = f.X[f.Npt];
    cout << "cut-offs: " << clsphircut << " " << clsrhorcut << " " << clsfrhocut << "\n";
  }

  void readMgEAMParam(const char *fname);

  Scalar_t rcutmax() { 
    return (clsphircut > clsrhorcut)? clsphircut : clsrhorcut;
  }

};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: MgEAMParam.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
