//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Propagators/ConstraintBase.h"
using namespace OHMMS;

ConstraintBase::ConstraintBase() {
}

ConstraintBase::~ConstraintBase() {

}

ConstraintBase::Scalar_t 
ConstraintBase::dof() {//degree's of freedom

  int nfix=0;
  for(int idim=0; idim<OHMMS_DIM; idim++) {
    if(mySys->Lattice.BoxBConds[idim]) nfix++;
  }

  if(mySys->hasAttrib("freeze")) {
    int nfree = 0;
    const ParticleIndex_t& fixed = *(mySys->getIndexAttrib("freeze"));
    for(int i=0; i<fixed.size(); i++)  if(!fixed[i]) nfree++;
    cout << "Number of free particles " << nfree << endl;
    return static_cast<Scalar_t>(OHMMS_DIM*nfree-nfix);
    //return  2.0e0/(3.0e0*static_cast<Scalar_t>(nfree));
  }
  else
    return static_cast<Scalar_t>(OHMMS_DIM*mySys->getTotalNum()-nfix);
    //return  2.0e0/(3.0e0*static_cast<Scalar_t>(mySys->getTotalNum()));

}

void ConstraintBase::set(Particle_t* ptclin) {
  mySys = ptclin;
}


void ConstraintBase::removeDrift(ParticlePos_t& p) {
  ParticlePos_t::Type_t center;
  for(int i=0; i<p.size(); i++) center+= p[i];
  Scalar_t dn = 1.0/static_cast<Scalar_t>(p.size());
  center *= dn;
  p -= center;
}

void ConstraintBase::apply(ParticlePos_t& p) { 
  if(mySys->hasAttrib("freeze")) {
  const ParticleIndex_t& fixed = *(mySys->getIndexAttrib("freeze"));
  for(int i=0; i<p.size(); i++)  if(fixed[i]) p[i] =0.0;
  }
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 298 $   $Date: 2005-02-15 16:05:07 -0600 (Tue, 15 Feb 2005) $
 * $Id: ConstraintBase.cpp 298 2005-02-15 22:05:07Z jnkim $ 
 ***************************************************************************/
