//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/***************************************************************************
 * class EPNHydroVerlet:
 *       Derived class from PropagatorBase to perform cell relaxations.
 *       Use a hydrostatic pressure and rescale the unit cell accordingly
 ***************************************************************************/
#ifndef OHHMS_EPNHYDROVERLETPROPAGATOR_H
#define OHHMS_EPNHYDROVERLETPROPAGATOR_H

#include "Propagators/PropagatorBase.h"

namespace OHMMS {
  /*!\class EPNHydroVerlet
   * \author Jeongnim Kim
   * \brief A propagator for TVN ensemble. Only uniform scaling is allowed.
   * \note If non-zero friction is given, this Propagator works as an
   * modified steepest-descent cell-relaxation algorithm.
   */
  class EPNHydroVerlet: public PropagatorBase { // using Verlet algorithm

  public:

    EPNHydroVerlet();
    ~EPNHydroVerlet() { }
 
    // implementing virutal functions
    void initialize(Particle_t* ptclin, Interaction_t* potin, bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(Interaction_t*);
    void applyBConds(bool wrapping);

  protected:

    ///
    int pressure_ind, pv_ind;
    ///reference temperature
    Scalar_t T_ref;
    Scalar_t P_ref, P_ref_kB;
    Scalar_t Dt;
    Scalar_t Friction;

    Scalar_t c0, c1, c2, oneover2dt;
    Scalar_t Vol0, Vol1, Vol2, VolF, VolMass;

  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 241 $   $Date: 2005-01-12 21:32:57 -0600 (Wed, 12 Jan 2005) $
 * $Id: EPNHydroVerlet.h 241 2005-01-13 03:32:57Z jnkim $ 
 ***************************************************************************/
