//////////////////////////////////////////////////////////////////
// (c) Copyright 2003- by Jeongnim Kim, Richard Hennig
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_EPNVERLETPROPAGATOR_H
#define OHMMS_EPNVERLETPROPAGATOR_H

#include "Propagators/PropagatorBase.h"
#include "OhmmsData/OhmmsElementBase.h"

namespace OHMMS {

/*!\class EPNPVERLET
 * \author Jeongnim Kim, Richard Hennig
 * \brief A propagator for PTN ensemble using whatever algorithm.
 */
  class EPNVerlet: public PropagatorBase { // using Verlet algorithm

  public:

    EPNVerlet();
    ~EPNVerlet() { }
 
    // implementing virutal functions of PropagatorBase
    void initialize(Particle_t* ptclin, PotentialBase* potin, bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(PotentialBase*);
    void applyBConds(bool wrapping);

  protected:
  
    void Get_Sigma_Tensor();
    void Get_Tot_stress(Tensor_t);
    void Get_Scaled_Force();

    ///
    int pressure_ind, pv_pot, pv_kin;
    ///reference temperature
    Scalar_t T_ref;
    Scalar_t P_ref;
    Scalar_t Dt;
    Scalar_t Friction;

    /// ParticlePos_t dS, S0, S1, S2;
    ParticlePos_t dS, S2;
    Scalar_t c0, c1, c2;
 
    Scalar_t thermostat, fac;
 
    Tensor_t G0, G1, dG, dh, h0, h1, h2, hinv;
    Tensor_t SigmaTensor, PiTensor, Tot_stress, stress, P_ext;
    ParticlePos_t Scaled_Force;
    Scalar_t d0, d1, d2, oneover2dt;
    Scalar_t VolMass, Damping;
  };

}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 241 $   $Date: 2005-01-12 21:32:57 -0600 (Wed, 12 Jan 2005) $
 * $Id: EPNVerlet.h 241 2005-01-13 03:32:57Z jnkim $ 
 ***************************************************************************/
