//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_LEAPFROGPROPAGATOR_H
#define OHMMS_LEAPFROGPROPAGATOR_H

#include "Propagators/PropagatorBase.h"
#include "OhmmsData/OhmmsElementBase.h"

namespace OHMMS {

  /*!\class LeapFrogPropagator
   * \author Jeongnim Kim
   * \brief A propagator for EVN ensemble using LeapFrog algorithm.
   */
  class LeapFrogPropagator: public PropagatorBase//, public OhmmsElementBase
  { // using Verlet algorithm

  public:

    LeapFrogPropagator();
    ~LeapFrogPropagator() { }
 
    // implementing virutal functions of PropagatorBase
    bool propagate(PotentialBase*);
    void adjust(bool trescale, bool wrapping);
    void applyBConds();

    /// implementing virtual functions for OhmmsElementBase
    bool get(ostream&) const;
    bool put(istream&);
    void reset();
    bool put(xmlNodePtr cur);

  protected:

    ///reference temperature
    bool ToApplyBConds;
    Scalar_t T_ref;
    Scalar_t Dt, Dtover2;
    ParticlePos_t Vp;
  };

}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 271 $   $Date: 2005-01-16 18:44:20 -0600 (Sun, 16 Jan 2005) $
 * $Id: EVNLeapFrog.h 271 2005-01-17 00:44:20Z jnkim $ 
 ***************************************************************************/
