//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_EVN_VELOCITYVERLETPROPAGATOR_H
#define OHMMS_EVN_VELOCITYVERLETPROPAGATOR_H

#include "Propagators/PropagatorBase.h"

namespace OHMMS {

  /*!\class EVNVelocityVerlet
   * \author Jeongnim Kim
   * \brief A propagator for EVN ensemble using Velocity-Verlet algorithm.
   */
  class EVNVelocityVerlet: public PropagatorBase { // using Verlet algorithm

  public:

    EVNVelocityVerlet();
    ~EVNVelocityVerlet() { }
 
    void initialize(Particle_t* ptclin, Interaction_t* potin, 
		    bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(Interaction_t*);
    void applyBConds(bool wrapping);

  protected:
    Scalar_t T_ref;
    Scalar_t Dt, Dtover2, DtDtover2;
    std::vector<Scalar_t> dtover2m;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 293 $   $Date: 2005-02-12 15:22:15 -0600 (Sat, 12 Feb 2005) $
 * $Id: EVNVelocityVerlet.h 293 2005-02-12 21:22:15Z jnkim $ 
 ***************************************************************************/
