//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_EVN_VERLETPROPAGATOR_H
#define OHMMS_EVN_VERLETPROPAGATOR_H

#include "Propagators/PropagatorBase.h"
#include "OhmmsData/OhmmsElementBase.h"

namespace OHMMS {

  /*!\class EVNVerlet
   * \author Jeongnim Kim
   * \brief A propagator for EVN ensemble using Verlet algorithm.
   *
   * \note If non-zero friction is used, using EVNVerlet results
   * in structural relaxation by steepest descent method. Because the
   * previous gradients as well as current gradients are used for update,
   * relaxation is very efficient. 
   */
  class EVNVerlet: public PropagatorBase { // using Verlet algorithm

  public:

    EVNVerlet();
    ~EVNVerlet() { }
 
    // implementing virutal functions of PropagatorBase
    void initialize(Particle_t* ptclin, Interaction_t* potin, bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(Interaction_t*);
    void applyBConds(bool wrapping);
    void copy(PropagatorBase* old);

  protected:

    size_type Rid;

    ///reference temperature
    Scalar_t T_ref;

    ///the time step
    Scalar_t Dt;

    ///the friction coefficient
    Scalar_t Friction;

    ///internal parameters for the EOM
    Scalar_t c0, c1, c2, oneover2dt;

  };

}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: EVNVerlet.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
