//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_FORCECONSTANT_H
#define OHMMS_FORCECONSTANT_H

#include "Propagators/PropagatorBase.h"
#include "OhmmsNumeric/Matrix.h"
#include <bitset>

namespace OHMMS {

  /*!\class ForceConstant
   * \author Jeongnim Kim
   * \brief A propagator to calculate phonon spectra for the fixed configuration.
   */
  class ForceConstant: public PropagatorBase { // using Verlet algorithm

  public:

    ForceConstant();
    ~ForceConstant();

    bool put(xmlNodePtr cur);
 
    void initialize(Particle_t* ptclin, Interaction_t* potin, bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(Interaction_t*);
    void applyBConds(bool wrapping);
    void copy(PropagatorBase* old) {}
    
  protected:
    
    void printForceConstant(const ParticlePos_t& F, int i, int j, bool plus);
    void printMode(const RealMatrix_t& FC, int i, int j, int writenode=0);
    void printFreq(const vector<Scalar_t>& Lambda, int writenode=0);
    
    enum {PhononFrequency = 0, PhononMode, FC};
    Scalar_t Displacement;
    bool GetMode, GetDOS;
    std::bitset<3> PrintOut;

    Particle_t    PrimaryCell;
    Particle_t::ParticleIndex_t LocPtcl;

    RealMatrix_t  FCMat;

    void getPhononDispersion();
    //int WriteForceConstant;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 376 $   $Date: 2005-08-28 15:32:27 -0500 (Sun, 28 Aug 2005) $
 * $Id: ForceConstant.h 376 2005-08-28 20:32:27Z jnkim $ 
 ***************************************************************************/
