//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002, 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Propagators/NoPropagator.h"
using namespace OHMMS;

NoPropagator::NoPropagator():GetForce(false), ForceUnitR(false), GetPressure(false){ 
  m_param.add(GetForce,"force","bool");
  m_param.add(GetPressure,"pressure","bool");
  m_param.add(ForceUnitR,"unit","bool");
  TDEnsemble.set(STATIC_BIT,true);
}

void 
NoPropagator::initialize(Particle_t* ptclin, Interaction_t* potin, bool makecopy) {

  Counter = -1;
  TDEnsemble.set(STATIC_BIT);

  initializeBase(ptclin, potin, makecopy, GetForce||GetPressure);

  if(!Initialized) { 
    //if(ForceUnitR && myPtcl->R.getUnit() == PosUnit::CartesianUnit) 
    if(GetPressure) {
      GetForce = true;
      exx_ind = E.add("exx-kB");
      eyy_ind = E.add("eyy-kB");
      ezz_ind = E.add("ezz-kB");
      exy_ind = E.add("exy-kB");
      eyz_ind = E.add("eyz-kB");
      ezx_ind = E.add("ezx-kB");
    }
    if(GetForce) myPtcl->addRestartAttrib(force_tag);
    Initialized = true; 
  }
}

void NoPropagator::setTemperature(bool rescale, Scalar_t tref) { } 

bool NoPropagator::propagate(Interaction_t* pot) {

  //myPtcl->convert2Unit(myPtcl->R);
  myPot->setCurrentPos(myPtcl->R);

  if(GetForce) {
    //add F and Eloc 
    ParticlePos_t& F = *myPtcl->getVectorAttrib(force_tag);
    E[POT_ENERGY] = pot->evalForce(F, GetPressure);

    if(GetPressure) {
      Tensor_t stress = pot->getStress();
      Scalar_t volinv =pressure_to_kbar/myPtcl->Lattice.Volume;
      E[exx_ind] = volinv*stress(0,0);
      E[eyy_ind] = volinv*stress(1,1);
      E[ezz_ind] = volinv*stress(2,2);
      E[exy_ind] = volinv*stress(0,1);
      E[eyz_ind] = volinv*stress(1,2);
      E[ezx_ind] = volinv*stress(0,2);
    }

  } else {
    E[POT_ENERGY] = pot->evalEnergy();
  }

  E[FREE_ENERGY] = E[POT_ENERGY]; //d_fe = d_pe;
  Counter++;
  return false; // do not propagate
}

void NoPropagator::applyBConds(bool wrapping) 
{ }
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 314 $   $Date: 2005-03-01 17:08:22 -0600 (Tue, 01 Mar 2005) $
 * $Id: NoPropagator.cpp 314 2005-03-01 23:08:22Z jnkim $ 
 ***************************************************************************/
