//////////////////////////////////////////////////////////////////
// (c) Copyright 2004- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/** @file TPNHydroLangevin.h
 * Implment TPN integrators based on Trotter factorization.
 */
#ifndef OHHMS_TPNHYDROLANEGVIN_H
#define OHHMS_TPNHYDROLANEGVIN_H

#include "Propagators/PropagatorBase.h"
#include "Propagators/TPLangevinBase.h"

namespace OHMMS {

  /** A propagator for TPN ensemble. 
   *
   * Isotropic Langevin TPN equations of motion based on
   *"Langevin dynamics in constant pressure extended systems",
   *D. Quigley and M. I. J Prbert, J. Chem. Phys. V120, 11432 (2004).
   *As prescribed by the paper, the mass of the volume is determined by
   * \f$W = DN k_{b}T/\omega_b^2\f$
   */
  class TPNHydroLangevin: private TPLangevinBase,
			  public PropagatorBase {

  public:

    TPNHydroLangevin();
    ~TPNHydroLangevin() { }
 
    //implementing virutal functions
    void initialize(Particle_t* ptclin, Interaction_t* potin, bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(Interaction_t*);
    void applyBConds(bool wrapping);

  private:

    ///The momentumm for the volume
    Scalar_t pVolume;

    ///Dimension*Volume
    T DV;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 237 $   $Date: 2005-01-12 21:15:58 -0600 (Wed, 12 Jan 2005) $
 * $Id: TPNHydroLangevin.h 237 2005-01-13 03:15:58Z jnkim $ 
 ***************************************************************************/
