//////////////////////////////////////////////////////////////////
// (c) Copyright 2004- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/** @file TPNHydroVerlet.h
 * A propagator using Nose-Hoover method to perform istropic TPN.
 */
#ifndef OHHMS_TPNHYDROVERLETPROPAGATOR_H
#define OHHMS_TPNHYDROVERLETPROPAGATOR_H

#include "Propagators/PropagatorBase.h"

namespace OHMMS {

  /**  A propagator for TVN ensemble.
   *
   * Velocity-verlet algorithm for isotropic constant pressure method
   * based on Nose-Hoover thermostat. 
   * Appendix D.3 of "Constant pressure molecualr dynamics algorthm",
   *G. J. Martyna, D. J. Tobias and M. L. Klein, J. Chem. Phys.
   * V101, 4177 (1994). 
   */
  class TPNHydroVerlet: public PropagatorBase { 

  public:

    TPNHydroVerlet();
    ~TPNHydroVerlet() { }
 
    // implementing virutal functions
    void initialize(Particle_t* ptclin, PotentialBase* potin, 
		    bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(PotentialBase*);
    void applyBConds(bool wrapping);

  protected:

    ///
    int pressure_ind, pv_ind;

    ///reference temperature
    Scalar_t T_ref, KE_ref;
    Scalar_t P_ref, P_ref_kB, P_v, dEdVol;
    Scalar_t Dt;
    Scalar_t dtover2,dtsqover2,cv_scale;
    Scalar_t DoverDOF;
    Scalar_t TwoPlusDoverDOF;
    Scalar_t OneOverDV;
    Scalar_t Vol, VolV, VolV1, VolF, VolMass;
    Scalar_t Chi, ChiV, ChiV1, ChiA, ChiMass;
    ParticlePos_t R1,F1;
    
    void updateVelocities(ParticlePos_t& V);
    void evalForceAndPressure(ParticlePos_t& F, ParticleScalar_t& Eloc);
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 238 $   $Date: 2005-01-12 21:22:22 -0600 (Wed, 12 Jan 2005) $
 * $Id: TPNHydroVerlet.h 238 2005-01-13 03:22:22Z jnkim $ 
 ***************************************************************************/
