//////////////////////////////////////////////////////////////////
// (c) Copyright 2004- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/** @file TPNLangevin.h
 * Implment TPN integrators based on Trotter factorization for general cell relaxations.
 */
#ifndef OHHMS_TPNLANEGVIN_H
#define OHHMS_TPNLANEGVIN_H

#include "Propagators/PropagatorBase.h"
#include "Propagators/TPLangevinBase.h"

namespace OHMMS {

  /** A propagator for TPN ensemble for general cell relxations. 
   *
   * General Langevin TPN equations of motion based on
   *"Langevin dynamics in constant pressure extended systems",
   *D. Quigley and M. I. J Prbert, J. Chem. Phys. V120, 11432 (2004).
   *As prescribed by the paper, the mass of the volume is determined by
   * \f$W = DN k_{b}T/\omega_b^2\f$
   */
  class TPNLangevin: private TPLangevinBase,
		     public PropagatorBase {

  public:

    TPNLangevin();
    ~TPNLangevin() { }
 
    //implementing virutal functions
    void initialize(Particle_t* ptclin, Interaction_t* potin, 
		    bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(Interaction_t*);
    void applyBConds(bool wrapping);

  private:

    Scalar_t VolumeInv;
    Tensor_t Stress_ind, Stress_ke, Stress_ext;
    Tensor_t Lattice0, pLattice0;
    Tensor_t Lattice_h, pLattice_h;
    Tensor_t CellMask;
    std::vector<Scalar_t> dtover2m;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 292 $   $Date: 2005-02-12 15:20:50 -0600 (Sat, 12 Feb 2005) $
 * $Id: TPNLangevin.h 292 2005-02-12 21:20:50Z jnkim $ 
 ***************************************************************************/
