//////////////////////////////////////////////////////////////////
// (c) Copyright 2003- by Jeongnim Kim, Benjamin Cho, Daniel Go, Alfonso Reina
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_NOSELEAPFROG_H
#define OHMMS_NOSELEAPFROG_H

#include "Propagators/PropagatorBase.h"

namespace OHMMS {

  /** A propagator for TVN ensemble using LeapFrog algorithm.
   *
   * Implements Nose-Hoover dynamics using Sympletic algorithm.
   */
  class TVNNoseHoover: public PropagatorBase//, public OhmmsElementBase
  { // using Leapfrog algorithm

  public:

    TVNNoseHoover();
    ~TVNNoseHoover() { }
 
    // implementing virutal functions of PropagatorBase
    void initialize(Particle_t* ptclin, Interaction_t* potin, bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(Interaction_t*);
    void applyBConds(bool wrapping);

  protected:

    ///reference temperature
    bool ToApplyBConds;
    Scalar_t T_ref;
    Scalar_t Dt, Dtover2;

    int nose_ke_ind;
    int nose_pe_ind;
    // Our variables
    Scalar_t DtoverQ;  // dt/Q
    Scalar_t KE_ref;   // temperature in eV
    Scalar_t Chi;      // Friction Coefficient
    Scalar_t Q;        // Inverse Mass  
    Scalar_t Theta;    // Ln(s) 
  };

}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 240 $   $Date: 2005-01-12 21:24:59 -0600 (Wed, 12 Jan 2005) $
 * $Id: TVNNoseHoover.h 240 2005-01-13 03:24:59Z jnkim $ 
 ***************************************************************************/
