//////////////////////////////////////////////////////////////////
//                        COPYRIGHT NOTICE
// (c) Copyright 1998-2002 by Jeongnim Kim
//   All rights reserved.
//////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////
//                    CONTACT INFORMATION
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// \author Jeongnim Kim
// \class BandByBandConjGradient
// \brief A templated class to perform band-by-band CG minimization
//        to calculate m eigen states.
// 
//
//   Polak-Ribiere factor (not used)
//      double x1 = dot(g,g)-dot(gprev,g);
//      double x2 = dot(gprev,gprev);
//      gamma = -x1/(x2+1e-40);
//
//      gamma = dot(g,g)/(dot(gprev,gprev)+1e-40); // <g|g>/<gp|gp>
//
//////////////////////////////////////////////////////////////////////
#ifndef OHMMS_CONJUGATEGRADIENT_LINEARSOLVER_H
#define OHMMS_CONJUGATEGRADIENT_LINEARSOLVER_H
#include "EigSolver/OhmmsBlasf.h"

template<class Ham_t, class Vec_t>
void 
ConjGradientLinerSolver(const Ham_t& ham, const Vec_t& b, Vec_t& psi) {
  typedef typename Vec_t::value_type T;
  int BasisDim = b.size();  
  Vec_t hpsi(BasisDim), g(BasisDim), cg(BasisDim), hcg(BasisDim), gprev(BasisDim);

  const T restol = 1e-12;
  T lambda, e0, e1, e2;
  T gamma, residue,resnew;

  int maxiter = 1000;
  ham.apply(psi.begin(), hpsi.begin());

  for(int ib=0; ib<BasisDim; ib++) g[ib] = b[ib] - hpsi[ib];

  for(int ib=0; ib<BasisDim; ib++)  cg[ib] = g[ib];

  residue = OhmmsBlas::dot(BasisDim,g.begin(),g.begin());

  int iter = 0;
  do {
    e1 = OhmmsBlas::dot(BasisDim,cg.begin(),g.begin());
    ham.apply(cg.begin(), hcg.begin());// new gradient      
    e2 = OhmmsBlas::dot(BasisDim,cg.begin(),hcg.begin());

    lambda = -0.5*e1/e2;
    for(int ib=0; ib<BasisDim; ib++) psi[ib] += lambda*cg[ib];

    for(int ib=0; ib<BasisDim; ib++)  hpsi[ib] += lambda*hcg[ib];

    for(int ib=0; ib<BasisDim; ib++) g[ib] = b[ib] - hpsi[ib];

    resnew = OhmmsBlas::dot(BasisDim,g.begin(),g.begin());
    gamma = resnew/residue;
    for(int ib=0; ib<BasisDim; ib++) cg[ib] = gamma*cg[ib] + g[ib];
    residue = resnew;

    if(residue < restol) break;
    iter++;
    cout << iter << " " << residue << endl;
  } while(iter < maxiter);

}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: ConjGradientLinearSolver.h 151 2004-11-02 23:51:14Z jnkim $
 ************************************************************************/
