//////////////////////////////////////////////////////////////////
//                        COPYRIGHT NOTICE
//   (c) Copyright 1998-2002 Jeongnim Kim.
//   All rights reserved.
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
#ifndef OHMMS_FERMIONWAVEFUNCTIONS_H
#define OHMMS_FERMIONWAVEFUNCTIONS_H
#include <vector>
#include <iostream>
using namespace std;


#include "OhmmsPETE/OhmmsMatrix.h"
struct 
FermionWaveFunctions: public Matrix<double,vector<double> > {

  typedef double T;
  typedef T*                   WaveFunctionRef_t;
  typedef const T*             WaveFunctionConstRef_t;
  typedef Matrix<T,vector<T> > BaseType_t;
  typedef FermionWaveFunctions This_t;

  FermionWaveFunctions();
  ~FermionWaveFunctions();
  FermionWaveFunctions(int n, int m);

  void init(int n, int m);

  inline WaveFunctionRef_t wfs(int iband) { return begin() + D2*iband;}
  inline WaveFunctionConstRef_t wfs(int iband) const { return begin() + D2*iband;}

  void randomize(int iband=-1);

  //! normalize a state
  void normalize(int iband);

  //! ortnogonalize a state with states whose index is smaller than iband
  //! if(iband == -1) full Gram-Schmidt orthonormalization is performed
  //! by recursive calling of orthonormalize
  void orthonormalize(int iband = -1);

  //! orthogonalize iband states with iother, iother is not affected
  void orthogonalize(int iband, int iother);

  //! orthogonalize a state with this, this is constant
  void orthogonalize(WaveFunctionRef_t, int minband, int maxband) const;

  //!  copy iband to an array
  void copy(int iband, T* ) const; 

  //! assign an array to iband
  void assign(int iband, const T* );

  void print(ostream&, int =0);
  vector<int> LocLoc, LocDim;
};
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: FermionWaveMatrix.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
