//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_CUBICSPLINEMODEL_H
#define OHMMS_CUBICSPLINEMODEL_H

#include <math.h>
#include "ScaleFunctors/CubicSplineFunctions.h"

template<class T>
struct CubicSplineOverR2 {
  CubicSpline<T> Sc;    // using cubic scaling

  CubicSplineOverR2() { }

  inline T operator()(T x0){ return Sc(x0)/x0/x0;} 
  inline T operator()(T x0, T& yrinv){ // note that yrinv = y/r
    T yp = Sc(x0,yrinv);               // derivative of a spline function
    T xinv = 1.0e0/x0; 
    T xinvsq = xinv*xinv;
    yrinv *= xinvsq;
    return yp*xinvsq - 2.0e0*yrinv;    // derivative
  } 

  inline T operator()(T x0, T& yrinv2, T& yprinv){   
    T zor2, dzor; // spline values
    T d2z = Sc(x0,zor2,dzor);
    T rinv2 = 1/x0/x0;
    yrinv2 = zor2*rinv2;
    yprinv = (dzor-2*zor2)*rinv2;
    return (d2z-4*dzor+6*zor2)*rinv2;
  } 

  inline void spline(){ Sc.spline();}
  inline void spline(const int n, T* x, T* y, const T yp1, const T ypn) {
    Sc.spline(n,x,y,yp1,ypn);
  }
};

#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: CubicSplineOverR2.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
