//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/***************************************************************************
 *
 ***************************************************************************/

#ifndef SLATERKOSTER_DD_H
#define SLATERKOSTER_DD_H

template<class T, class SF>
class SlaterKoster<T,2,2,SF>: public SlaterKosterBase<T> {
private:

  int I[5][5], J[5][5];
  bool NeedToSymm;
  T Eps;
  SF* V;

public:

  using SlaterKosterBase<T>::I0;
  using SlaterKosterBase<T>::J0;

  SlaterKoster(int i0, int j0, SF* , T eps=0, bool samespecies=true);
  void getOnsite(T* b);
  void getOffsite(T r, const T*, T* b);
  void getDeriv(T r, const T*, const T [][3], T* bx, T* by, T* bz);
  void getDeriv2(T r, const T*, const T [][3], const T [][3][3], 
       T* bx,int i, int j);
  void assign(T*);
  void reset(int nrow, int ncol);
};

// D-D block
template<class T, class SF>
SlaterKoster<T,2,2,SF>::SlaterKoster(int i0, int j0, SF* v, T eps,
				     bool samespecies): V(v), Eps(eps) {
  I0 = i0; J0 = j0;
  if(samespecies && I0 != J0)  NeedToSymm = true; 
  else                         NeedToSymm = false;
}

template<class T, class SF>
void SlaterKoster<T,2,2,SF>::reset(int nrow, int ncol) {
  for(int i=0; i<5; i++)
    for(int j=0; j<5; j++)
      I[i][j] = (j+J0)+(i+I0)*ncol;
}

template<class T, class SF>
void SlaterKoster<T,2,2,SF>::getOnsite(T* b) {
  b[I[0][0]] += Eps; b[I[1][1]] += Eps; b[I[2][2]] += Eps;
  b[I[3][3]] += Eps; b[I[4][4]] += Eps;
}


template<class T, class SF>
void 
SlaterKoster<T,2,2,SF>::assign(T* b) {
  for(int i=0; i < 5; i++) 
    for(int j=0; j < 5; j++)
      b[I[i][j]] = static_cast<T>(I[i][j]);
}

template<class T, class SF>
void 
SlaterKoster<T,2,2,SF>::getOffsite(T r, const T* p, T* b){
  //T vsig = Vsnorm*Vsig->f(r);
  //T vpi  = Vpnorm*Vpi->f(r);
  //T vdel = Vdnorm*Vdel->f(r);
  T vsig = V[0](r);
  T vpi  = V[1](r);
  T vdel = V[2](r);

  T v1 = 3.0e0*vsig - 4.0e0*vpi + vdel;
  T v2 = vpi - vdel;
  T v3 = vpi;
  T v4 = (vpi+ 2*vdel)*onethird;
  T v2ll = v2*p[0]*p[0], v2mm = v2*p[1]*p[1], v2nn = v2*p[2]*p[2];

  b[I[0][0]] += v1*p[SK_Dxy  ]*p[SK_Dxy  ] + v3-v2nn ;
  b[I[0][1]] += v1*p[SK_Dxy  ]*p[SK_Dyz  ] + v2*p[SK_Dzx];
  b[I[0][2]] += v1*p[SK_Dxy  ]*p[SK_Dzx  ] + v2*p[SK_Dyz];
  b[I[0][3]] += v1*p[SK_Dxy  ]*p[SK_Dx2y2]              ;
  b[I[0][4]] += v1*p[SK_Dxy  ]*p[SK_Dz2r2] - v2*p[SK_Dxy]*twooversqrt3;

  b[I[1][1]] += v1*p[SK_Dyz  ]*p[SK_Dyz  ] + v3-v2ll;
  b[I[1][2]] += v1*p[SK_Dyz  ]*p[SK_Dzx  ] + v2*p[SK_Dxy];
  b[I[1][3]] += v1*p[SK_Dyz  ]*p[SK_Dx2y2] - v2*p[SK_Dyz];
  b[I[1][4]] += v1*p[SK_Dyz  ]*p[SK_Dz2r2] + v2*p[SK_Dyz]*oneoversqrt3;

  b[I[2][2]] += v1*p[SK_Dzx  ]*p[SK_Dzx  ] + v3-v2mm    ;
  b[I[2][3]] += v1*p[SK_Dzx  ]*p[SK_Dx2y2] + v2*p[SK_Dzx];
  b[I[2][4]] += v1*p[SK_Dzx  ]*p[SK_Dz2r2] + v2*p[SK_Dzx]*oneoversqrt3;

  b[I[3][3]] += v1*p[SK_Dx2y2]*p[SK_Dx2y2] + v3-v2nn;
  b[I[3][4]] += v1*p[SK_Dx2y2]*p[SK_Dz2r2] - v2*p[SK_Dx2y2]*twooversqrt3;

  b[I[4][4]] += v1*p[SK_Dz2r2]*p[SK_Dz2r2] + v4+v2nn;

  b[I[1][0]] = b[I[0][1]];
  b[I[2][0]] = b[I[0][2]];
  b[I[2][1]] = b[I[1][2]];
  b[I[3][0]] = b[I[0][3]];
  b[I[3][1]] = b[I[1][3]];
  b[I[3][2]] = b[I[2][3]];
  b[I[4][0]] = b[I[0][4]];
  b[I[4][1]] = b[I[1][4]];
  b[I[4][2]] = b[I[2][4]];
  b[I[4][3]] = b[I[3][4]];

  //\todo NeedToSymm for the same species and different orbitals 
}



template<class T, class SF>
void 
SlaterKoster<T,2,2,SF>::getDeriv(T r, const T* p, const T pd[][3],
                                  T* bx, T* by, T* bz) {
  T vpi, vsig, vdel;
  //T dvsig = Vsnorm*Vsig->df(r,vsig);
  //T dvpi  = Vpnorm*Vpi->df(r,vpi); // denote that  f(r)/r is returned 
  //T dvdel = Vdnorm*Vdel->df(r,vdel);

  T dvsig = V[0](r,vsig);
  T dvpi  = V[1](r,vpi); // denote that  f(r)/r is returned 
  T dvdel = V[2](r,vdel);
  T v1 = 3.0e0*vsig - 4.0e0*vpi + vdel;
  T v2 = vpi - vdel;

  T dv1 = 3.0e0*dvsig - 4.0e0*dvpi + dvdel;
  T dv2 = dvpi - dvdel;
  T dv3 = dvpi;
  T dv4 = (dvpi + 2.0e0*dvdel)*onethird;

  T a[5][5];
  for(int i=0; i<5; i++) {
    for(int j=i; j<5; j++) {
      a[i][j] = dv1*p[i+SK_Dxy]*p[j+SK_Dxy];
    }
  }

  T dv2nn = dv2*p[SK_Pz]*p[SK_Pz];
  a[0][0]+= dv3 - dv2nn ;
  a[1][1]+= dv3 - dv2*p[SK_Px]*p[SK_Px];
  a[2][2]+= dv3 - dv2*p[SK_Py]*p[SK_Py];
  a[3][3]+= dv3 - dv2nn;
  a[4][4]+= dv4 + dv2nn;
  T twov2 = 2.0e0*v2;  

  // x-derivate
  int k=SK_Dxy;
  for(int i=0; i<5; i++) {
    int kp=i+SK_Dxy;
    for(int j=i; j<5; j++) {
      bx[I[i][j]] += a[i][j]*p[0] + v1*(pd[k][0]*p[kp]+p[k]*pd[kp][0]);
      kp++;
    }
    k++;
  }

  dv2nn = twov2*p[SK_Pz]*pd[SK_Pz][0];
  bx[I[0][0]]-=dv2nn; 
  bx[I[1][1]]-=twov2*p[SK_Px]*pd[SK_Px][0]; 
  bx[I[2][2]]-=twov2*p[SK_Py]*pd[SK_Py][0];
  bx[I[3][3]]-=dv2nn; 
  bx[I[4][4]]+=dv2nn;

  T lm_i=v2*pd[SK_Dxy][0]+dv2*p[SK_Dxy]*p[0];
  T mn_i=v2*pd[SK_Dyz][0]+dv2*p[SK_Dyz]*p[0];
  T nl_i=v2*pd[SK_Dzx][0]+dv2*p[SK_Dzx]*p[0];
  bx[I[0][1]]+= nl_i; bx[I[0][2]] += mn_i; bx[I[0][4]]-= twooversqrt3*lm_i;
  bx[I[1][2]]+= lm_i; bx[I[1][3]] -= mn_i; bx[I[1][4]]+= oneoversqrt3*mn_i;
  bx[I[2][3]]+= nl_i; bx[I[2][4]] += oneoversqrt3*nl_i;
  bx[I[3][4]]-= twooversqrt3*(v2*pd[SK_Dx2y2][0]+dv2*p[SK_Dx2y2]*p[0]);

  bx[I[1][0]] = bx[I[0][1]];
  bx[I[2][0]] = bx[I[0][2]];
  bx[I[2][1]] = bx[I[1][2]];
  bx[I[3][0]] = bx[I[0][3]];
  bx[I[3][1]] = bx[I[1][3]];
  bx[I[3][2]] = bx[I[2][3]];
  bx[I[4][0]] = bx[I[0][4]];
  bx[I[4][1]] = bx[I[1][4]];
  bx[I[4][2]] = bx[I[2][4]];
  bx[I[4][3]] = bx[I[3][4]];

  // y-derivate
  k=SK_Dxy;
  for(int i=0; i<5; i++) {
    int kp=i+SK_Dxy;
    for(int j=i; j<5; j++) {
      by[I[i][j]] += a[i][j]*p[1] + v1*(pd[k][1]*p[kp]+p[k]*pd[kp][1]);
      kp++;
    }
    k++;
  }

  dv2nn = twov2*p[SK_Pz]*pd[SK_Pz][1];
  by[I[0][0]]-=dv2nn; 
  by[I[1][1]]-=twov2*p[SK_Px]*pd[SK_Px][1]; 
  by[I[2][2]]-=twov2*p[SK_Py]*pd[SK_Py][1];
  by[I[3][3]]-=dv2nn; 
  by[I[4][4]]+=dv2nn;

  lm_i=v2*pd[SK_Dxy][1]+dv2*p[SK_Dxy]*p[1];
  mn_i=v2*pd[SK_Dyz][1]+dv2*p[SK_Dyz]*p[1];
  nl_i=v2*pd[SK_Dzx][1]+dv2*p[SK_Dzx]*p[1];
  by[I[0][1]]+= nl_i; by[I[0][2]] += mn_i; by[I[0][4]]-= twooversqrt3*lm_i;
  by[I[1][2]]+= lm_i; by[I[1][3]] -= mn_i; by[I[1][4]]+= oneoversqrt3*mn_i;
  by[I[2][3]]+= nl_i; by[I[2][4]] += oneoversqrt3*nl_i;
  by[I[3][4]]-= twooversqrt3*(v2*pd[SK_Dx2y2][1]+dv2*p[SK_Dx2y2]*p[1]);

  by[I[1][0]] = by[I[0][1]];
  by[I[2][0]] = by[I[0][2]];
  by[I[2][1]] = by[I[1][2]];
  by[I[3][0]] = by[I[0][3]];
  by[I[3][1]] = by[I[1][3]];
  by[I[3][2]] = by[I[2][3]];
  by[I[4][0]] = by[I[0][4]];
  by[I[4][1]] = by[I[1][4]];
  by[I[4][2]] = by[I[2][4]];
  by[I[4][3]] = by[I[3][4]];

  // z-derivate
  k=SK_Dxy;
  for(int i=0; i<5; i++) {
    int kp=i+SK_Dxy;
    for(int j=i; j<5; j++) {
      bz[I[i][j]] += a[i][j]*p[2] + v1*(pd[k][2]*p[kp]+p[k]*pd[kp][2]);
      kp++;
    }
    k++;
  }

  dv2nn = twov2*p[SK_Pz]*pd[SK_Pz][2];
  bz[I[0][0]]-=dv2nn; 
  bz[I[1][1]]-=twov2*p[SK_Px]*pd[SK_Px][2]; 
  bz[I[2][2]]-=twov2*p[SK_Py]*pd[SK_Py][2];
  bz[I[3][3]]-=dv2nn; 
  bz[I[4][4]]+=dv2nn;

  lm_i=v2*pd[SK_Dxy][2]+dv2*p[SK_Dxy]*p[2];
  mn_i=v2*pd[SK_Dyz][2]+dv2*p[SK_Dyz]*p[2];
  nl_i=v2*pd[SK_Dzx][2]+dv2*p[SK_Dzx]*p[2];
  bz[I[0][1]]+= nl_i; bz[I[0][2]] += mn_i; bz[I[0][4]]-= twooversqrt3*lm_i;
  bz[I[1][2]]+= lm_i; bz[I[1][3]] -= mn_i; bz[I[1][4]]+= oneoversqrt3*mn_i;
  bz[I[2][3]]+= nl_i; bz[I[2][4]] += oneoversqrt3*nl_i;
  bz[I[3][4]]-= twooversqrt3*(v2*pd[SK_Dx2y2][2]+dv2*p[SK_Dx2y2]*p[2]);

  bz[I[1][0]] = bz[I[0][1]];
  bz[I[2][0]] = bz[I[0][2]];
  bz[I[2][1]] = bz[I[1][2]];
  bz[I[3][0]] = bz[I[0][3]];
  bz[I[3][1]] = bz[I[1][3]];
  bz[I[3][2]] = bz[I[2][3]];
  bz[I[4][0]] = bz[I[0][4]];
  bz[I[4][1]] = bz[I[1][4]];
  bz[I[4][2]] = bz[I[2][4]];
  bz[I[4][3]] = bz[I[3][4]];

  //\todo NeedToSymm for the same species and different orbitals 
}

template<class T, class SF>
void 
SlaterKoster<T,2,2,SF>::getDeriv2(T r, const T* p, const T pd[][3], 
                                   const T pdd[][3][3], T* b, int d0, int d1) {
  T vpi, vsig, vdel, dvpi, dvsig, dvdel;
  //T d2vsig = Vsnorm*Vsig->d2f(r,vsig,dvsig);
  //T d2vpi  = Vpnorm*Vpi->d2f(r,vpi,dvpi);
  //T d2vdel = Vdnorm*Vdel->d2f(r,vdel,dvdel);

  T d2vsig = V[0](r,vsig,dvsig);
  T d2vpi  = V[1](r,vpi,dvpi);
  T d2vdel = V[2](r,vdel,dvdel);
  T v1 = 3.0e0*vsig - 4.0e0*vpi + vdel;
  T v2 = vpi - vdel;

  T dv1 = 3.0e0*dvsig - 4.0e0*dvpi + dvdel;
  T dv2 = dvpi - dvdel;
  T dv3 = dvpi;
  T dv4 = (dv3 + 2.0e0*dvdel)*onethird;

  T d2v1 = 3.0e0*d2vsig - 4.0e0*d2vpi + d2vdel;
  T d2v2 = d2vpi - d2vdel;
  T d2v3 = d2vpi;
  T d2v4 = (d2vpi + 2.0e0*d2vdel)*onethird;

  // PART (1)

  int k=SK_Dxy;
  for(int i=0; i<5; i++) {
    int kp=i+SK_Dxy;
    for(int j=i; j<5; j++) {
      b[I[i][j]] += d2v1*p[d0]*p[d1]*p[k]*p[kp]
                 +   dv1*(pd[d0][d1]*p[k]*p[kp]
                        + p[d0]*(pd[k][d1]*p[kp]+pd[kp][d1]*p[k])
                        + p[d1]*(pd[k][d0]*p[kp]+pd[kp][d0]*p[k]) )
                 +   v1*(pdd[k ][d0][d1]*p[kp]
                       + pdd[kp][d0][d1]*p[k ]
                       + pd[k ][d0]*pd[kp][d1]
                       + pd[kp][d0]*pd[k ][d1]) ;
      kp++;
    }
    k++;
  }

  // PART (2)
  T tmp1 = d2v3*p[d0]*p[d1] + dv3*pd[d0][d1];
  T tmp2 = d2v2*p[d0]*p[d1]*p[2]*p[2]
         + dv2*p[2]*(pd[d0][d1]*p[2]+2.0e0*(p[d0]*pd[2][d1]+p[d1]*pd[2][d0]))
         + 2.0e0*v2*(pd[2][d1]*pd[2][d0]+p[2]*pdd[2][d0][d1]);

  b[I[0][0]] += tmp1 - tmp2;
  b[I[1][1]] += tmp1 
              - d2v2*p[d0]*p[d1]*p[0]*p[0]
              - dv2*p[0]*(pd[d0][d1]*p[0]+2.0e0*(p[d0]*pd[0][d1]+p[d1]*pd[0][d0]))
              - 2.0e0*v2*(pd[0][d1]*pd[0][d0]+p[0]*pdd[0][d0][d1]);

  b[I[2][2]] += tmp1
              - d2v2*p[d0]*p[d1]*p[1]*p[1]
              - dv2*p[1]*(pd[d0][d1]*p[1]+2.0e0*(p[d0]*pd[1][d1]+p[d1]*pd[1][d0]))
              - 2.0e0*v2*(pd[1][d1]*pd[1][d0]+p[1]*pdd[1][d0][d1]);
  b[I[3][3]] += tmp1 - tmp2;
  b[I[4][4]] += d2v4*p[d0]*p[d1] + dv4*pd[d0][d1] + tmp2;


  // PART (3)
  T lm_ij=v2*pdd[SK_Dxy][d0][d1]
         +dv2*(pd[SK_Dxy][d1]*p[d0]+pd[SK_Dxy][d0]*p[d1]+p[SK_Dxy]*pd[d0][d1])
         +d2v2*p[SK_Dxy]*p[d0]*p[d1];

  T mn_ij=v2*pdd[SK_Dyz][d0][d1]
         +dv2*(pd[SK_Dyz][d1]*p[d0]+pd[SK_Dyz][d0]*p[d1]+p[SK_Dyz]*pd[d0][d1])
         +d2v2*p[SK_Dyz]*p[d0]*p[d1];

  T nl_ij=v2*pdd[SK_Dzx][d0][d1]
         +dv2*(pd[SK_Dzx][d1]*p[d0]+pd[SK_Dzx][d0]*p[d1]+p[SK_Dzx]*pd[d0][d1])
         +d2v2*p[SK_Dzx]*p[d0]*p[d1];

  T l2m2_ij= v2*pdd[SK_Dx2y2][d0][d1]
         +dv2*(pd[SK_Dx2y2][d1]*p[d0]+pd[SK_Dx2y2][d0]*p[d1]+p[SK_Dx2y2]*pd[d0][d1])
         +d2v2*p[SK_Dx2y2]*p[d0]*p[d1];

  b[I[0][1]] += nl_ij;  b[I[0][2]] += mn_ij;  b[I[0][4]] -= lm_ij*twooversqrt3;
  b[I[1][2]] += lm_ij;  b[I[1][3]] -= mn_ij;  b[I[1][4]] += mn_ij*oneoversqrt3;
  b[I[2][3]] += nl_ij;  b[I[2][4]] += nl_ij*oneoversqrt3;
  b[I[3][4]] -= l2m2_ij*twooversqrt3;

  b[I[1][0]] = b[I[0][1]];
  b[I[2][0]] = b[I[0][2]];
  b[I[2][1]] = b[I[1][2]];
  b[I[3][0]] = b[I[0][3]];
  b[I[3][1]] = b[I[1][3]];
  b[I[3][2]] = b[I[2][3]];
  b[I[4][0]] = b[I[0][4]];
  b[I[4][1]] = b[I[1][4]];
  b[I[4][2]] = b[I[2][4]];
  b[I[4][3]] = b[I[3][4]];

  //\todo NeedToSymm for the same species and different orbitals 

}
#endif


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 378 $   $Date: 2005-08-28 18:40:33 -0500 (Sun, 28 Aug 2005) $
 * $Id: SlaterKosterDD.h 378 2005-08-28 23:40:33Z jnkim $ 
 ***************************************************************************/
