//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/**@file FastSKTB.h
 * @brief Definition of FastSKTB
 */
#ifndef OHMMS_SLATERKOSTERTB_GAMMA_FAST_H
#define OHMMS_SLATERKOSTERTB_GAMMA_FAST_H

#include "SlaterKosterTB/SlaterKosterTB.h"
#include "SparseMatrix/SparseMatrix2MatrixCopy.h"
#include "EigSolver/LapackSymSolverPacked.h"
#include "Utilities/Clock.h"

namespace OHMMS {

  /** a SlaterKosterTB for gamma calculations using symmetric dense matrix
   *
   * Using eigen solvers for symmetric matrices
   */
  class FastSKTB: public SlaterKosterTB {

  public:

    FastSKTB();
    ~FastSKTB();

    Scalar_t el_ph_coupling(const vector<int>& psi_i, const PhononMode& phonon,
        		    vector<int>& psi_f, Scalar_t del);
    
  protected:
    
    void setEigenSolver();
    Scalar_t solveGES();
    Scalar_t evalForceTB(ParticlePos_t& f, ParticleScalar_t& e, 
		     Tensor_t&, unsigned stressmode);
    void tbDynamicMatrix(DynMatrix_t&);

    void assignBlocks(const PtclPairList_t& nnlist, PackedSymMatrix_t& h);
    void assignBlocks(const PtclPairList_t& nnlist, 
        PackedSymMatrix_t& h, PackedSymMatrix_t& s);

  private:
    typedef LapackPackedSymSolver<real_type,PackedSymMatrix_t,RealMatrix_t> EigSolver_t;
    EigSolver_t *thisSolver;
    RealMatrix_t A, B;
    Pooma::Clock timer;
    vector<Scalar_t> accCPU;
  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 378 $   $Date: 2005-08-28 18:40:33 -0500 (Sun, 28 Aug 2005) $
 * $Id: FastSKTB.h 378 2005-08-28 23:40:33Z jnkim $
 ***************************************************************************/
