//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Message/OpenMP.h"
#include "Particle/NNEngine.h"
#include "SlaterKosterTB/Models/GSPTB.h"
#include "Potentials/GenericTwoBodySRP.h"
#include "Potentials/EnvDepPotential.h"
#include "SlaterKoster/SlaterKosterMake.h"
#include "Utilities/SimpleParser.h"
using namespace OHMMS;

GSPTB::GSPTB() {}
GSPTB::~GSPTB() {
  // remove scaling functions
  for(int i=0; i<V.size(); i++) delete V[i];
}

ShortRangePotential*
GSPTB::init(SKList_t& SK_h, SKList_t& SK_s, 
	    Matrix<int>& bond, istream& fin){

  SpeciesSet& mySpecies(myPtcl->getSpeciesSet());

  int ib = -1, ia, ja, L1, L2, vloc;
  int l1, l2, lt1, lt2, m;
  Scalar_t eps;
  Scalar_t val, ro, rc, n, nc, r1, rcut;
  vector<string> slist, vlist;
  bool tofindend = true;
  while(tofindend) {
    int nw = getwords(slist, fin);
    if(nw == 0) continue;
    if(slist[0] == "bond") {
      ia = mySpecies.addSpecies(slist[1].c_str());
      ja = mySpecies.addSpecies(slist[2].c_str());

      // create a SK block for AB bond
      ib = SK_h.size();
      SK_h.push_back(new SlaterKosterComp<Scalar_t>);

      // assign unphysical L1 and L2
      L1 = -1; L2 = -1;
      bond(ia,ja) = ib;
    } else if(isdigit(slist[0][0])) { 
        l1 = atoi(slist[0].c_str());
        l2 = atoi(slist[1].c_str());
        m  = atoi(slist[2].c_str());
        lt1 = atoi(slist[3].c_str());
        lt2 = atoi(slist[4].c_str());

        if(m < 0) {
          eps = atof(slist[5].c_str());
	} else {
          eps = 0.0;
          val =  atof(slist[5].c_str());         
          ro =   atof(slist[6].c_str());         
          rc =   atof(slist[7].c_str());         
          n =    atof(slist[8].c_str());         
          nc =   atof(slist[9].c_str());         
          r1 =   atof(slist[10].c_str());         
          rcut = atof(slist[11].c_str());         
	}

        if(l1 != L1 || l2 != L2) {// new L1-L2 pair is added
          vloc = V.size();
          V.push_back(new vector<ScaleFunction_t>);
          SK_h[ib]->add(
			SKBlock<Scalar_t,ScaleFunction_t>
			::make(l1,l2,ia,ja,lt1,lt2,eps,V[vloc]));
        }

        L1 = l1; L2 = l2;
        if(m>=0) {
          SK_h[ib]->setRcut(rcut);
          V[vloc]->operator[](m).set(val,n,nc,ro,rc,r1,rcut);
	}
    } else if(slist[0] == "</TBPARAM>")
      {
	tofindend = false; 
      }
  } while(tofindend);


  ///////////////////////////////////
  // repulsive
  ///////////////////////////////////
  do {
   getwords(slist, fin);
  } while(!slist[0].find("rep"));

  tofindend = true;
  bool EnvDep = false;

  vector<Scalar_t> fc(5,0.0);

  typedef GenericTwoBodySRP<ScaleFunction_t> PairPotential_t;
  PairPotential_t *repPot=new PairPotential_t;
  repPot->setParticleSet(myPtcl); 
  Scalar_t rmax_rep(-10.0);
  do {
    int nw = getwords(slist, fin);
    if(nw == 0) continue;
    if(slist[0] == "bond") {
      getwords(vlist, fin);
      Scalar_t val = atof(vlist[0].c_str());
      Scalar_t ro  = atof(vlist[1].c_str());
      Scalar_t rc  = atof(vlist[2].c_str());
      Scalar_t n   = atof(vlist[3].c_str());
      Scalar_t nc  = atof(vlist[4].c_str());
      Scalar_t r1  = atof(vlist[5].c_str());
      Scalar_t rcut = atof(vlist[6].c_str());
      rmax_rep = std::max(rmax_rep,rcut);
      ScaleFunction_t* phi = new ScaleFunction_t;
      phi->set(val,n,nc,ro,rc,r1,rcut);
      //Only for the single species
      repPot->addPairPotential(0,0,phi);
    } else if(slist[0] == "env") {
      getwords(vlist, fin);
      fc[0] = atof(vlist[0].c_str()); 
      fc[1] = atof(vlist[1].c_str()); 
      fc[2] = atof(vlist[2].c_str()); 
      fc[3] = atof(vlist[3].c_str()); 
      fc[4] = atof(vlist[4].c_str()); 
      EnvDep = true;
    } else if(slist[0] == "</rep>") {
      tofindend = false; 
    }
  } while(tofindend);

  repPot->Rmax = rmax_rep;
  repPot->PairFactor=1.0e0;

  if(EnvDep) {
    EnvDepPotential<PairPotential_t> *ePot= new EnvDepPotential<PairPotential_t>(repPot);
    ePot->Rmax=rmax_rep;
    ePot->setFc(fc);
    return ePot;
  } else {
    return repPot;
  }
}
// real_type 
// GSPTB::e(const ParticlePos_t& R, int i, int myID) {
//   const PtclPairList_t& nnlist = myPtcl->getNNEngine()->getPairAttrib(myID);
//   real_type sum = 0.0e0;
//   for(int jL=0; jL< nnlist.nadj(i); ++jL) {
//     int j = nnlist.iadj(i,jL);  //atomic index of jL-th nn of i
//     int loc = nnlist.loc(i,jL); //location of R, dR of the pair
//     if(nnlist.R[loc] < Rmax) {
//       sum += Phi(nnlist.R[loc]);// 0.5 factory is not used here
//     }
//   }
//   return sum;
// }

//GSPTB::real_type 
//GSPTB::evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale) const {
//  real_type  esum = 0.0;
//  if(EnvDep) {
//    int nn = 0;      
//    for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
//      real_type rep = 0.0;
//      for(int nni=0; nni< nnlist.nadj(iL); nni++, nn++){
//	Scalar_t sep = nnlist.R[nn]*rscale;
//	if(sep < Rmax)  rep += Phi(sep);
//      }
//      esum += fc[0] + rep*(fc[1]+rep*(fc[2]+rep*(fc[3]+rep*fc[4])));
//    }
//  } else {
//    int nn = 0;      
//    for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
//      for(int nni=0; nni< nnlist.nadj(iL); nni++, nn++){
//	Scalar_t sep = nnlist.R[nn]*rscale;
//	if(sep < Rmax) esum += Phi(sep);
//      }
//    }
//  }
//  return esum;
//}
//
//GSPTB::Scalar_t 
//GSPTB::evalForceNN(PtclPairList_t& nnlist, 
//		   ParticlePos_t& f0, 
//		   ParticleScalar_t& q, 
//		   unsigned stressmode) const {
//  if(EnvDep) 
//    return evalForce_with_env(nnlist,f0,q,stressmode);
//  else
//    return evalForce_paironly(nnlist,f0,q,stressmode);
//}
//
///** @warning This is not correct with OpenMP.
// */
//GSPTB::Scalar_t 
//GSPTB::evalForce_with_env(PtclPairList_t& nnlist, 
//			  ParticlePos_t& f0, 
//			  ParticleScalar_t& q, 
//			  unsigned stressmode) const {
//  
//  int myid=nnlist.myID;
//  int nfv=nnlist.Visitors;
//  bool multiplethread = nnlist.MultiSet;
//
//  PtclPairList_t::IndexList_t& fvind = nnlist.IDv;
//  PtclPairList_t::VectorList_t& fv = nnlist.Fv;
//  const ParticleIndex_t* o=NULL;
//  if(multiplethread) o = myPtcl->getIndexAttrib(ParticleTags::openmp_tag);
//
//  vector<SingleParticlePos_t> fref(nnlist.getMaxNadj());
//  vector<int> J(nnlist.getMaxNadj());
//  vector<Scalar_t> dphi(nnlist.getLocalNum());
//  real_type esum = 0.0;
//  Tensor_t stressloc;
//  int nnloc = 0;      
//  for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
//    int iat = nnlist.I[iL];
//    real_type xxx=0.0e0;
//    for(int nni = 0; nni< nnlist.nadj(iL); nni++, nnloc++) {
//      xxx += Phi(nnlist.R[nnloc]);
//    }
//    real_type rep = fc[0] + xxx*(fc[1]+xxx*(fc[2]+xxx*(fc[3]+xxx*fc[4])));
//    dphi[iL] = (fc[1]+xxx*(2.e0*fc[2] +xxx*(3.e0*fc[3]+4.0e0*xxx*fc[4])))*0.5e0;
//    q[iat] += rep;
//    esum+= rep;
//  }
//
//
//  real_type drep,rep;
//#pragma omp barrier  
//  for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
//    int iat = nnlist.I[iL];
//    int nn_sub=0;
//    for(int nni = 0; nni< nnlist.nadj(iL); nni++, nnloc++) {
//      int jat = nnlist.iadj(iL,nni);  //atomic index of jL-th nn of i
//      if(nnlist.R[nnloc]<Rmax) {
//        //This is horrible.
//	drep = Phi(nnlist.R[nnloc],rep)*(dphi[iat]+dphi[jat]);
//	SingleParticlePos_t df = drep*nnlist.dR[nnloc];
//	f0[iat] += df;
//        fref[nn_sub] = -1.0*df;
//        J[nn_sub] = jat;
//	nn_sub++;
//	if(stressmode) {
//	  stressloc -=nnlist.R[nnloc]*outerProduct(df,nnlist.dR[nnloc]);    
//	}
//      }
//    }
//    if(multiplethread) {
//      for(int nj=0; nj<nn_sub; nj++) {
//	int jat=J[nj];
//	if((*o)[jat]==myid) 
//	  f0[jat]+= fref[nj];
//	else {
//	  fv[nfv]=fref[nj]; fvind[nfv]=jat; nfv++;
//	}
//      }
//    } else {
//      for(int nj=0; nj<nn_sub; nj++) f0[J[nj]]+= fref[nj];
//    }
//  }
// 
//  nnlist.Visitors = nfv;
//  nnlist.Stress=stressloc;
//
//  return esum;
//}
//
//GSPTB::Scalar_t 
//GSPTB::evalForce_paironly(PtclPairList_t& nnlist, 
//			  ParticlePos_t& f0, 
//			  ParticleScalar_t& q, 
//			  unsigned stressmode) const {
//
//  real_type rep, drep;
//  real_type esum = 0.0;
//  int nnloc = 0;      
//  Tensor_t stressloc;
//  for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
//    int iat = nnlist.I[iL];
//    int num_nn = nnlist.nadj(iL);
//    real_type elocal = 0.0; // local energy to be accumulated
//    for(int nni = 0; nni< num_nn; nni++, nnloc++) {
//      real_type r = nnlist.R[nnloc];
//      SingleParticlePos_t dr = nnlist.dR[nnloc];
//      
//      if(nnlist.R[nnloc] < Rmax) {
//	drep = 0.5e0*Phi(r,rep);
//	rep *= 0.5e0*r;
//	SingleParticlePos_t df = drep*dr;
//	f0[iat] += 2.0*df;
//	if(stressmode) stressloc -= r*outerProduct(df, dr);
//	elocal += rep;
//      }
//    }
//    q[iat] += elocal;
//    esum += elocal;
//  }
//
//  nnlist.Stress=stressloc;
//  return esum;
//}


//bool GSPTB::get(ostream& os) const { 
//  os << myName << endl;
//  return true;
//}
//bool GSPTB::put(istream&) {return true;}
//void GSPTB::reset() {
//
//}
//#if defined(HAVE_LIBXML2)
//bool GSPTB::put(xmlNodePtr cur){
////   string sname = (const char*)(xmlGetProp(cur, (const xmlChar *) "species"));
////   int stype = atoi((const char*)(xmlGetProp(cur, (const xmlChar *) "id")));
//
////   SpeciesBase* mySpecies = SpeciesCollection::getSpecies();
//
////   if(mySpecies->getTotalNum() > stype) {
////     mySpecies->Name[stype] = sname;
////   } else {
////     int newid =SpeciesCollection::addSpecies(sname.c_str());
////     char a[3];
////     sprintf(a,"%i",newid);
////     xmlSetProp(cur, (const xmlChar *) "id", (const xmlChar*)a);
////   }
//  return true;
//}
//#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 396 $   $Date: 2006-01-28 13:05:14 -0600 (Sat, 28 Jan 2006) $
 * $Id: GSPTB.cpp 396 2006-01-28 19:05:14Z jnkim $ 
 ***************************************************************************/
