//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

////////////////////////////////////////////////////////////////////////////
// p. 54 Table 2-3 Electronic Structure and properties of Solids
// Walter A. Harrison 
////////////////////////////////////////////////////////////////////////////
#include "SlaterKosterTB/Models/InvPowerTB.h"
#include "SlaterKoster/SlaterKosterMake.h"
#include "Utilities/SimpleParser.h"
#include <limits>
using namespace OHMMS;

InvPowerTB::InvPowerTB() { }
InvPowerTB::~InvPowerTB() { }

ShortRangePotential*
InvPowerTB::init(SKList_t& SK, SKList_t& SK_s, Matrix<int>& bond, 
		 istream& fin) {

  // make private references to Species
  //SpeciesBase* mySpecies = SpeciesCollection::getSpecies();
  //int irmax = SpeciesCollection::addAttrib("rmax");
  SpeciesSet& mySpecies(myPtcl->getSpeciesSet());
  int irmax = mySpecies.addAttrib("rmax");

  int ib = -1, ia, ja, L1, L2,vloc;
  int l1, l2, lt1, lt2, m;
  Scalar_t eps, r0, rcut=0.2, vnorm, np, rscale=1.0;
  bool useConstantScale=false;
  vector<string> slist;
  bool tofindend = true;
  do {
   int nw = getwords(slist,fin);
   if(nw == 0) continue;
   if(slist[0]== "scale") {
     rscale = atof(slist[1].c_str());
   } else if(slist[0] == "bond") {
      ia = mySpecies.addSpecies(slist[1].c_str());
      ja = mySpecies.addSpecies(slist[2].c_str());
      // create a SK block for AB bond
      ib = SK.size();

      // create a SK block for AB bond
      SK.push_back(new SlaterKosterComp<Scalar_t>);

      // assign unphysical L1 and L2
      L1 = -1; L2 = -1;
      bond(ia,ja) = ib;
   } else if(isdigit(slist[0][0])) { 
     l1 = atoi(slist[0].c_str());
     l2 = atoi(slist[1].c_str());
     m  = atoi(slist[2].c_str());
     lt1 = atoi(slist[3].c_str());
     lt2 = atoi(slist[4].c_str());

     if(m < 0) {
       eps = atof(slist[5].c_str());
     } else {
       eps = 0.0;
       vnorm = atof(slist[5].c_str());
       np = atof(slist[6].c_str());
       r0 = rscale*atof(slist[7].c_str());
       rcut = rscale*atof(slist[8].c_str());
       mySpecies(irmax,ia)=r0*0.6;
       mySpecies(irmax,ja)=r0*0.6;
     }

     if(l1 != L1 || l2 != L2) {// new L1-L2 pair is added
       if(fabs(np)< numeric_limits<Scalar_t>::epsilon()) {
         vloc = Vc.size();
         useConstantScale=true;
         Vc.push_back(new vector<ConstantType>);
         SK[ib]->add(SKBlock<Scalar_t,ConstantType>::make(l1,l2,ia,ja,lt1,lt2,eps,Vc[vloc]));
       } else {
         vloc = Vg.size();
         useConstantScale=false;
         Vg.push_back(new vector<GeneralPowerLawType>);
         SK[ib]->add(SKBlock<Scalar_t,GeneralPowerLawType>::make(l1,l2,ia,ja,lt1,lt2,eps,Vg[vloc]));
       }
     }
     
     L1 = l1; L2 = l2;
     if(m>=0) {
       if(useConstantScale)
         Vc[vloc]->operator[](m).set(vnorm,np,r0,rcut);
       else
         Vg[vloc]->operator[](m).set(vnorm,np,r0,rcut);
       SK[ib]->setRcut(rcut);
     }
   } else if(slist[0] == "</TBPARAM>") {
        tofindend = false;
   }
  } while(tofindend);
  return NULL;
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 352 $   $Date: 2005-06-16 09:12:55 -0500 (Thu, 16 Jun 2005) $
 * POOMA_VERSION_ID: $Id: InvPowerTB.cpp 352 2005-06-16 14:12:55Z jnkim $ 
 ***************************************************************************/
