//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Message/OpenMP.h"
#include "Particle/NNEngine.h"
#include "SlaterKosterTB/Models/SplineTB.h"
#include "SlaterKoster/SlaterKosterMake.h"
#include "Potentials/GenericTwoBodySRP.h"
#include "Utilities/SimpleParser.h"
#include "Utilities/OhmmsInfo.h"
using namespace OHMMS;

SplineTB::SplineTB() {}
SplineTB::~SplineTB() {
  // release the pointer to scaling functions
  for(int i=0; i<V.size(); i++) delete V[i];
  for(int i=0; i<Vs.size(); i++) delete Vs[i];
}


//////////////////////////////////////////////
// virtual function of TBModelBase
//////////////////////////////////////////////
ShortRangePotential* 
SplineTB::init(SKList_t& SK, SKList_t& SK_O, Matrix<int>& bond, 
	       istream& fin) {

  // make private references to Species
  SpeciesSet& mySpecies(myPtcl->getSpeciesSet());
  int ins     = mySpecies.addAttrib("ns");
  int irmax   = mySpecies.addAttrib("rmax");

  int Lmax = 0;
  int ib = -1, ia, ja, L1, L2, vloc, sloc;
  int l1, l2, lt1, lt2, m;
  Scalar_t eps;
  vector<Scalar_t> y,z;
  Scalar_t ri, rf, dr;
  int npts;
  
  vector<string> slist, vlist;
  bool tofindend = true;
  bool nonortho = true;
  do {
    int nw =getwords(slist, fin);
    if(nw == 0) continue;
    if(slist[0] == "type") {
      if(slist[1] == "orthogonal") nonortho = false;
    } else if(slist[0] == "bond") {
      ib = SK.size();
      ia = mySpecies.addSpecies(slist[1].c_str());
      ja = mySpecies.addSpecies(slist[2].c_str());

      // create a SK block for AB bond
      SK.push_back(new SlaterKosterComp<Scalar_t>);
      if(nonortho) 
	SK_O.push_back(new SlaterKosterComp<Scalar_t>);

      bond(ia,ja) = ib;

      // assign unphysical L1 and L2
      L1 = -1; L2 = -1;
    } else if(isdigit(slist[0][0])) { 
      l1 = atoi(slist[0].c_str());
      l2 = atoi(slist[1].c_str());
      m  = atoi(slist[2].c_str());
      lt1 = atoi(slist[3].c_str());
      lt2 = atoi(slist[4].c_str());

      if(m < 0) {
	eps = atof(slist[5].c_str());
      } else {
	eps = 0.0;
	npts = atoi(slist[5].c_str());
	ri = atof(slist[6].c_str());
	rf = atof(slist[7].c_str());
      }

      if(l1 != L1 || l2 != L2) {// new L1-L2 pair is added
	vloc = V.size();
	V.push_back(new vector<ScaleFunction_t>);
	SK[ib]->add(SKBlock<Scalar_t,ScaleFunction_t>::make(l1,l2,ia,ja,lt1,lt2,eps,V[vloc]));
	if(nonortho) {
	  sloc = Vs.size();
	  Vs.push_back(new vector<ScaleFunction_t>);
          SK_O[ib]->add(SKBlock<Scalar_t,ScaleFunction_t>::make(l1,l2,ia,ja,lt1,lt2,1.0,Vs[sloc]));
	}
      }

      L1 = l1; L2 = l2;
      if(m>=0) {
	y.resize(npts);
	z.resize(npts);
	for(int ipts=0; ipts<npts; ipts++)  {
	  getwords(vlist,fin);
	  //if(ipts == 0) ri = atof(vlist[0].c_str());
	  //else if(ipts == npts-1) rf = atof(vlist[0].c_str());
	  y[ipts] = atof(vlist[0].c_str());
	  z[ipts] = atof(vlist[1].c_str());
	}
	SK[ib]->setRcut(rf);
	//V[vloc]->operator[](m).spline(npts-1,ri, rf, y.begin());
	V[vloc]->operator[](m).spline(npts-1,ri, rf, &(y[0]));

	if(nonortho) {
	  SK_O[ib]->setRcut(rf);
          //Vs[sloc]->operator[](m).spline(npts-1,ri, rf, z.begin());
          Vs[sloc]->operator[](m).spline(npts-1,ri, rf, &(z[0]));
	}
      }
    } else if(slist[0] == "</TBPARAM>") {
      tofindend = false; 
    }
  } while(tofindend);

  ///////////////////////////////////
  // look for repulsive potential
  ///////////////////////////////////
  tofindend = true;
  do {
   int nw = getXwords(slist, fin);
   if(nw == 0) continue;
   if(slist[0] == "rep") {
     tofindend = false;
     if(slist[2] == "none") {
       LOGMSG("\tThis TB model does not have classical terms")
       return NULL; // this does not use repulsive potential
     }
   }
  } while(!fin.eof() && tofindend);

  tofindend = true;

  GenericTwoBodySRP<ScaleFunction_t> *repPot= new GenericTwoBodySRP<ScaleFunction_t>;
  repPot->setParticleSet(myPtcl);
  Scalar_t rmax_rep(-10.0e0);

  do {
   int nw = getwords(slist, fin);
   if(nw == 0) continue;
   if(slist[0] == "bond") {
      npts = atoi(slist[3].c_str());
      ri = atof(slist[4].c_str());
      rf = atof(slist[5].c_str());

      for(int ipts=0; ipts<npts; ipts++)  {
	getwords(vlist,fin);
	y[ipts] = atof(vlist[0].c_str());
      }

      ScaleFunction_t* phi = new ScaleFunction_t;
      phi->spline(npts-1,ri,rf,&(y[0]));
      repPot->addPairPotential(mySpecies.addSpecies(slist[1].c_str()),
          mySpecies.addSpecies(slist[2].c_str()),phi);
      rmax_rep=std::max(rmax_rep,rf);
      //Phi.push_back(ScaleFunction_t());
      //Phi[ib].spline(npts-1, ri, rf, &(y[0]));
      //Rmax = rf;
   } else if(slist[0] == "</rep>") {
     tofindend = false;
   }
  } while(tofindend);

  if(rmax_rep<0) {
    //No repulsive potential is used. 
    delete repPot;
    return 0;
  } else {
    repPot->Rmax=rmax_rep;
    return repPot;
  }
}


//struct TempPot: public ShortRangePotential {
//
//  typedef CubicSplineDiff<Scalar_t> ScaleFunction_t;
//
//  /// implementing virtual functions for OhmmsElementBase
//  bool get(ostream&) const;
//  bool put(istream&);
//  bool put(xmlNodePtr cur);
//  void reset();
//
//  void setParticleSet(Particle_t* ptclin) { myPtcl = ptclin;}
//  Scalar_t evalForceNN(PtclPairList_t& nnlist, ParticlePos_t& f0, 
//      		 unsigned stressmode) const;
//
//  Scalar_t evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale=1.0) const;
//
//  vector<ScaleFunction_t*> Phi; // pair potential
//
//};
//
//// Scalar_t 
//// SplineTB::e(const ParticlePos_t& R, int i, int myID) {
////   if(HasPairPotential) {
////     /////////////////////////////////////
////     // tired of testing: only single species works for now
////     /////////////////////////////////////
////     const PtclPairList_t& nnlist = myPtcl->nnListEngine->getPairAttrib(myID);
////     Scalar_t sum = 0.0e0;
////     for(int jL=0; jL< nnlist.nadj(i); ++jL) {
////       int j = nnlist.iadj(i,jL);  //atomic index of jL-th nn of i
////       int loc = nnlist.loc(i,jL); //location of R, dR of the pair
////       if(nnlist.R[loc] < Rmax) {
//// 	sum += Phi[0](nnlist.R[loc]);// 0.5 factory is not used here
////       }
////     }
////     return sum;
////   } else {
////     return 0.0;
////   }
//// }
//
//TempPote::real_type
//TempPote::evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale) const {
//
//  ///@todo multi-species 
//  if(!HasPairPotential) return 0.0;
//  real_type esum = 0.0;
//  int nnloc=0;
//  for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
//    int num_nn = nnlist.nadj(iL);
//    for(int nni = 0; nni< num_nn; nni++, nnloc++) {
//      Scalar_t sep = nnlist.R[nnloc]*rscale;
//      if(sep < Rmax) {
//	esum += 0.5*Phi[0](sep);
//      }
//    }
//  }
//  return esum;
//}
//
//TempPote::Scalar_t 
//TempPote::evalForceNN(PtclPairList_t& nnlist, 
//		      ParticlePos_t& f0, 
//		      ParticleScalar_t& q, 
//		      unsigned stressmode) const {
//
//  if(!HasPairPotential) return 0.0;
//  real_type rep, sumloc = 0.0e0,psum = 0.0e0;
//  real_type dist, drep;
//  int nnloc = 0;
//  Tensor_t stressloc;
//  for(int iL=0; iL<nnlist.getLocalNum(); iL++) {
//    int iat = nnlist.I[iL];
//    int itype = myPtcl->GroupID[iat];    
//    for(int nni=0; nni< nnlist.nadj(iL); nni++, nnloc++){
//      int jtype = myPtcl->GroupID[nnlist.iadj(iL,nni)];    
//      if(nnlist.R[nnloc] < Rmax) {
//	dist = nnlist.R[nnloc];
//	drep = 0.5e0*Phi[BondMap(itype,jtype)](dist,rep);
//	rep *= 0.5e0*dist;
//	SingleParticlePos_t df = drep*nnlist.dR[nnloc];
//	f0[iat] += 2*df;
//	if(stressmode) {
//	  stressloc -=dist*outerProduct(df,nnlist.dR[nnloc]);    
//	}
//	q[iat] += rep;
//	sumloc += rep;
//      }
//    }
//  }
//
//  nnlist.Stress=stressloc;
//  return sumloc;
//}
//
////bool SplineTB::evalDynamicMatrix(DynMatrix_t& ) {
////    if(!nnListEngine)
////      nnListEngine = NNEngine::getNNEngine(*myPtcl,Rmax);
////    const PtclPairList_t* nnlist = nnListEngine->getPairAttrib();
////    T rep, sum = 0.0e0,psum = 0.0e0;
////    for(int i=0; i< myPtcl->getLocalNum(); i++) {
////      for(int jL=0; jL<nnlist->nadj(i); ++jL) {
////        int j = nnlist->iadj(i,jL);  //atomic index of jL-th nn of i
////        int loc = nnlist->loc(i,jL); //location of R, dR of the pair
////        if(nn->R[loc] < Rcut) {
////          T rep, drep;
////          T d2rep = 0.5e0*Phi.d2f(nn->R[loc],rep,drep);
////          drep *= 0.5e0;
////          SingleParticlePos_t dr = nn->dR[loc];
////          d2rep -= drep;
////          DynMatrix_t& tmp = dynmat.offdiag(numpairs);
////          tmp(0,0) =  d2rep*dr[0]*dr[0] + drep;
////          tmp(0,1) =  d2rep*dr[0]*dr[1];
////          tmp(0,2) =  d2rep*dr[0]*dr[2];
////          tmp(1,1) =  d2rep*dr[1]*dr[1] + drep;
////          tmp(1,2) =  d2rep*dr[1]*dr[2];
////          tmp(2,2) =  d2rep*dr[2]*dr[2] + drep;
////          tmp(1,0) = tmp(0,1);
////          tmp(2,0) = tmp(0,2);
////          tmp(2,1) = tmp(1,2);
////          sum += 0.5*rep*nn->R[jl]*nn->R[jl];
////          numpairs++;
////        }
////      }
////    }
////    return sum;
////  return false;
////}
//
//bool SplineTB::get(ostream& os) const { 
//  os << myName << endl;
//  return true;
//}
//bool SplineTB::put(istream&) {return true;}
//void SplineTB::reset() {
//
//}
//#if defined(HAVE_LIBXML2)
//bool SplineTB::put(xmlNodePtr cur){
//  return true;
//}
//#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 330 $   $Date: 2005-05-26 17:44:25 -0500 (Thu, 26 May 2005) $
 * $Id: SplineTB.cpp 330 2005-05-26 22:44:25Z jnkim $ 
 ***************************************************************************/
