//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
///////////////////////////////////////////////////////////////////////
// TBModelBase is an abstract class from which a model TB is derived.
//  pure virtial functions should be implemneted by a derived model.
//  returns the maximum angular momentum.
//  virtual int initialize(SKArray_t&)  =0;
//  virtual int initialize(SKArray_t&, SKArray_t&)  =0;
/////////////////////////////////////////////////////////////////////

#include "OhmmsUnits.h"
#include "Particle/NNEngine.h"
#include "SlaterKosterTB/Models/ZincBlendSurface.h"
#include "Utilities/SimpleParser.h"
#include "OhmmsPETE/TinyMatrix.h"
using namespace OHMMS;

void 
ZincBlendSurface::OnsiteMixtureType::setShift(int l, int loc, int x, int y, 
                                              Scalar_t delx, Scalar_t dely) {
  int n=2*l+1;
  SpeciesX = x;
  SpeciesY = y;
  for(int k=loc; k<loc+n; k++) {DeltaX[k]=delx; DeltaY[k]=dely;}
}

void 
ZincBlendSurface::OnsiteMixtureType::add(Scalar_t* onsite, Scalar_t mix) {
  Scalar_t mix_c = 1.0-mix;
  int nb=DeltaX.size();
  for(int ib=0; ib<nb; ib++)  {
    onsite[ib*nb+ib]+= (DeltaX[ib]*mix_c+DeltaY[ib]*mix);
  } 
}

void 
ZincBlendSurface::OnsiteMixtureType::print(ostream& os) {
  os << "SpecieX = " << SpeciesX << " " << SpeciesY << endl;
  os << "Diagonal mixture " << endl;
  for(int i=0; i<DeltaX.size(); i++) os << DeltaX[i] << " " << DeltaY[i] << endl;
  os << endl;
}
bool ZincBlendSurface::put(istream& fin) {

  // make private references to Species
  SpeciesSet& mySpecies(myPtcl->getSpeciesSet());
  int ins = mySpecies.addAttrib("ns");
  int nbasis = static_cast<int>(mySpecies(ins,0));

  surfShift.resize(mySpecies.getTotalNum());
  ssShift.resize(mySpecies.getTotalNum());
  diagMix.resize(mySpecies.getTotalNum());
  for(int i=0; i<mySpecies.getTotalNum(); i++) {
    ssShift[i] = 0.0;
    surfShift[i] = 100.0;
    diagMix[i]=0;
  }

  vector<string> slist;
  bool tofindend=true;
  do{
    int nw = getwords(slist,fin);
    if(nw == 0) continue;
    if(slist[0] == "</onsite>") tofindend = false;
    else {
      if(slist[0] == "mix") {
	int ia = mySpecies.addSpecies(slist[1].c_str());
	if(diagMix[ia]==0) diagMix[ia] = new OnsiteMixtureType(nbasis);
        int s=atoi(slist[2].c_str());
        int loc=atoi(slist[3].c_str());
	int speciesx = mySpecies.addSpecies(slist[4].c_str());
	int speciesy = mySpecies.addSpecies(slist[6].c_str());
        Scalar_t delx = atof(slist[5].c_str());
        Scalar_t dely = atof(slist[7].c_str());
        diagMix[ia]->setShift(s,loc,speciesx,speciesy,delx,dely);
      } else {
	int ia = mySpecies.addSpecies(slist[0].c_str());
	ssShift[ia] = atof(slist[1].c_str());
	surfShift[ia] = atof(slist[2].c_str());
      }
    }
  } while(tofindend);

//  for(int i=0; i<mySpecies->getTotalNum(); i++) {
//    if(diagMix[i]) {
//      diagMix[i]->print(cout);
//    }
//  }
  i00 = 0;        i01 = 1;     i02 = 2;     i03 = 3;
  i10 = nbasis;   i11 = i10+1; i12 = i11+1; i13 = i12+1;
  i20 = 2*nbasis; i21 = i20+1; i22 = i21+1; i23 = i22+1;
  i30 = 3*nbasis; i31 = i30+1; i32 = i31+1; i33 = i32+1;
  if(nbasis == 5) i44 = 4*nbasis+nbasis-1;
  else            i44 = 0;
  return true;
}

void ZincBlendSurface::initialize() {}
bool ZincBlendSurface::get(ostream&) const {
  return true;
}
bool ZincBlendSurface::put(xmlNodePtr cur) {
  return true;
}
                                                                                
void ZincBlendSurface::reset() {}
                                                                                

//#define FIXEDTBMETHOD
#if defined(FIXEDTBMETHOD)
void 
ZincBlendSurface::add(SparseMatrix_t& H, int myID) {

  TinyMatrix<int,2,2> Bond;
  Bond(0,0) = 0;
  Bond(0,1) = 1;
  Bond(1,0) = 2;
  Bond(1,1) = 3;

  DiagBlocks.resize(32);
  OffBlocks.resize(8);

  typedef Particle_t::SingleParticlePos_t SingleParticlePos_t;
  const double oneoversqrt3 = 0.5773502691896257645091487805019574556475;
  vector<SingleParticlePos_t> bond(4);
  bond[0] = SingleParticlePos_t( oneoversqrt3,  oneoversqrt3,  oneoversqrt3);
  bond[1] = SingleParticlePos_t(-oneoversqrt3, -oneoversqrt3,  oneoversqrt3);
  bond[2] = SingleParticlePos_t( oneoversqrt3, -oneoversqrt3, -oneoversqrt3);
  bond[3] = SingleParticlePos_t(-oneoversqrt3,  oneoversqrt3, -oneoversqrt3);

  TinyMatrix<double,TB_BLOCK,TB_BLOCK> d;
  (*SK_h)[Bond(0,0)]->getOnsite(d.data());
  cout << d << endl;
  int BondMap[2][2][2][2][2];
  Scalar_t v_dang[4]; 
  int idiag=0;
  for(int i=0; i<2; i++) {
    if(i) v_dang[0] = surfShift[1]*0.25e0;
    else  v_dang[0] = 0.0;
    for(int j=0; j<2;j++) {
      if(j) v_dang[1] = surfShift[1]*0.25e0;
      else  v_dang[1] = 0.0;
      for(int k=0; k<2; k++) {
	if(k) v_dang[2] = surfShift[1]*0.25e0;
	else  v_dang[2] = 0.0;
	for(int l=0; l<2; l++) {
	  if(l) v_dang[3] = surfShift[1]*0.25e0;
	  else  v_dang[3] = 0.0;

	  DiagBlocks[idiag] = d;
	  BondMap[0][i][j][k][l] = idiag;
	  if(idiag) {
	    DiagBlocks[idiag](4,4) += ssShift[1];

	    Scalar_t vsum = v_dang[0] + v_dang[1] + v_dang[2] + v_dang[3];
	    DiagBlocks[idiag](0,0) += vsum; // (0,0) 0+0*5
	    DiagBlocks[idiag](1,1) += vsum; // (1,1) 1+1*5
	    DiagBlocks[idiag](2,2) += vsum; // (2,2) 2+2*5
	    DiagBlocks[idiag](3,3) += vsum; // (3,3) 3+3*5
	    
	    vsum = v_dang[0] - v_dang[1] + v_dang[2] - v_dang[3];
	    DiagBlocks[idiag](0,1) += vsum; // (0,1) 1+0*5
	    DiagBlocks[idiag](1,0) += vsum; // (1,0) 0+1*5
	    DiagBlocks[idiag](2,3) += vsum; // (2,3) 3+2*5
	    DiagBlocks[idiag](3,2) += vsum; // (3,2) 2+3*5
	    
	    vsum = v_dang[0] - v_dang[1] - v_dang[2] + v_dang[3];
	    DiagBlocks[idiag](0,2) += vsum; // (0,2) 2+0*5
	    DiagBlocks[idiag](1,3) += vsum; // (1,3) 3+1*5
	    DiagBlocks[idiag](2,0) += vsum; // (2,0) 0+0*5
	    DiagBlocks[idiag](3,1) += vsum; // (3,1) 1+3*5
	    
	    vsum = v_dang[0] + v_dang[1] - v_dang[2] - v_dang[3];
	    DiagBlocks[idiag](0,3) += vsum; //(0,3) 3+0*5
	    DiagBlocks[idiag](1,2) += vsum; //(1,2) 2+1*5
	    DiagBlocks[idiag](2,1) += vsum; //(2,1) 1+2*5
	    DiagBlocks[idiag](3,0) += vsum; //(3,0) 0+3*5
	  }
	  idiag++;
	}
      }
    }
  }

  (*SK_h)[Bond(1,1)]->getOnsite(d.data());
  for(int i=0; i<2; i++) {
    if(i) v_dang[0] = surfShift[0]*0.25e0;
    else  v_dang[0] = 0.0;
    for(int j=0; j<2;j++) {
      if(j) v_dang[1] = surfShift[0]*0.25e0;
      else  v_dang[1] = 0.0;
      for(int k=0; k<2; k++) {
	if(k) v_dang[2] = surfShift[0]*0.25e0;
	else  v_dang[2] = 0.0;
	for(int l=0; l<2; l++) {
	  if(l) v_dang[3] = surfShift[0]*0.25e0;
	  else  v_dang[3] = 0.0;
	  int nbond=i+j+k+l;
	  DiagBlocks[idiag] = d;
	  BondMap[1][i][j][k][l] = idiag;
	  if(idiag>16) {
	    DiagBlocks[idiag](4,4) += ssShift[0];

	    Scalar_t vsum = v_dang[0] + v_dang[1] + v_dang[2] + v_dang[3];
	    DiagBlocks[idiag](0,0) += vsum; // (0,0) 0+0*5
	    DiagBlocks[idiag](1,1) += vsum; // (1,1) 1+1*5
	    DiagBlocks[idiag](2,2) += vsum; // (2,2) 2+2*5
	    DiagBlocks[idiag](3,3) += vsum; // (3,3) 3+3*5
	    
	    vsum = v_dang[0] - v_dang[1] + v_dang[2] - v_dang[3];
	    DiagBlocks[idiag](0,1) -= vsum; // (0,1) 1+0*5
	    DiagBlocks[idiag](1,0) -= vsum; // (1,0) 0+1*5
	    DiagBlocks[idiag](2,3) += vsum; // (2,3) 3+2*5
	    DiagBlocks[idiag](3,2) += vsum; // (3,2) 2+3*5

	    vsum = v_dang[0] - v_dang[1] - v_dang[2] + v_dang[3];
	    DiagBlocks[idiag](0,2) -= vsum; // (0,2) 2+0*5
	    DiagBlocks[idiag](1,3) += vsum; // (1,3) 3+1*5
	    DiagBlocks[idiag](2,0) -= vsum; // (2,0) 0+0*5
	    DiagBlocks[idiag](3,1) += vsum; // (3,1) 1+3*5

	    vsum = v_dang[0] + v_dang[1] - v_dang[2] - v_dang[3];
	    DiagBlocks[idiag](0,3) -= vsum; //(0,3) 3+0*5
	    DiagBlocks[idiag](1,2) += vsum; //(1,2) 2+1*5
	    DiagBlocks[idiag](2,1) += vsum; //(2,1) 1+2*5
	    DiagBlocks[idiag](3,0) -= vsum; //(3,0) 0+3*5
	  }
	  idiag++;
	}
      }
    }
  }

  Scalar_t pvec[8];
  for(int ib=0; ib<4; ib++) {
    pvec[0] = bond[ib][0];
    pvec[1] = bond[ib][1];
    pvec[2] = bond[ib][2];
    (*SK_h)[Bond(0,1)]->getOffsite(1.0,pvec,OffBlocks[ib].data());
  }
  for(int ib=0; ib<4; ib++) {
    pvec[0] = -bond[ib][0];
    pvec[1] = -bond[ib][1];
    pvec[2] = -bond[ib][2];
    (*SK_h)[Bond(1,0)]->getOffsite(1.0,pvec,OffBlocks[ib+4].data());
  }


  idiag=0;

  ofstream fout("uniqueblocks.dat");
  fout.setf(ios::left,ios::adjustfield);
  fout.setf(ios::scientific, ios::floatfield);
  fout << "Diag 32 " << TB_BLOCK << " " << TB_BLOCK << endl;
  for(int itype=0; itype<2; itype++) {
    for(int i=0; i<2; i++) {
      for(int j=0; j<2;j++) {
	for(int k=0; k<2; k++) {
	  for(int l=0; l<2; l++) {
	    fout << idiag << " " << itype << " " << i << " " << j << " " << k << " "<< l << endl
		 << DiagBlocks[idiag];
	    idiag++;
	  }
	}
      }
    }
  }
  fout << "Ofdiag 8 " <<  TB_BLOCK << " " << TB_BLOCK << endl;
  bond[0] = SingleParticlePos_t( oneoversqrt3,  oneoversqrt3,  oneoversqrt3);
  bond[1] = SingleParticlePos_t(-oneoversqrt3, -oneoversqrt3,  oneoversqrt3);
  bond[2] = SingleParticlePos_t( oneoversqrt3, -oneoversqrt3, -oneoversqrt3);
  bond[3] = SingleParticlePos_t(-oneoversqrt3,  oneoversqrt3, -oneoversqrt3);

  fout << "0  1  1  1 " << endl << OffBlocks[0];
  fout << "0 -1 -1  1 " << endl << OffBlocks[1];
  fout << "0  1 -1 -1 " << endl << OffBlocks[2];
  fout << "0 -1  1 -1 " << endl << OffBlocks[3];
  fout << "1 -1 -1 -1 " << endl << OffBlocks[4];
  fout << "1  1  1 -1 " << endl << OffBlocks[5];
  fout << "1 -1  1  1 " << endl << OffBlocks[6];
  fout << "1  1 -1  1 " << endl << OffBlocks[7];
}

///Temporary routine to write large sparse matrices without allocation
// void 
// ZincBlendSurface::add(SparseMatrix_t& H, int myID) {

//   typedef Particle_t::SingleParticlePos_t SingleParticlePos_t;
//   const double oneoversqrt3 = 0.5773502691896257645091487805019574556475;
//   vector<SingleParticlePos_t> bond(4);
//   bond[0] = SingleParticlePos_t( oneoversqrt3,  oneoversqrt3,  oneoversqrt3);
//   bond[1] = SingleParticlePos_t(-oneoversqrt3, -oneoversqrt3,  oneoversqrt3);
//   bond[2] = SingleParticlePos_t( oneoversqrt3, -oneoversqrt3, -oneoversqrt3);
//   bond[3] = SingleParticlePos_t(-oneoversqrt3,  oneoversqrt3, -oneoversqrt3);
//   Scalar_t v_dang[4]; 

//   const PtclPairList_t& nnlist = myPtcl->nnListEngine->getPairAttrib(myID);

//   int nn = 0;
 
//   TinyMatrix<double,5,5> hon;
//   TinyMatrix<int,2,2> Bond;
//   Bond(0,0) = 0;
//   Bond(0,1) = 1;
//   Bond(1,0) = 2;
//   Bond(1,1) = 3;

//   ofstream fout("smat.out");
//   fout.setf(ios::left,ios::adjustfield);
//   fout.setf(ios::scientific, ios::floatfield);
//   fout << "#block size " << 5    << " " << 5  << endl;
//   fout << "#number of diagonal blocks " << nnlist.getLocalNum() << endl;
//   fout << "#number of off-diagonal blocks " << nnlist.getTotNadj() << endl;

//   for(int iL=0; iL<nnlist.getLocalNum(); iL++) {

//     int iG =nnlist.I[iL];
//     int itype = myPtcl->GroupID[iG];
//     int ij = Bond(myPtcl->GroupID[iG],myPtcl->GroupID[iG]);

//     hon = 0.0;
//     (*SK_h)[ij]->getOnsite(hon.data());

//     ///Scalar_t* hon = H.diag(myID,iL);
//     if(nnlist.nadj(iL) < 4) {      

//       hon(4,4) += ssShift[itype];
//       Scalar_t cosv = 0.0e0;
//       for(int ib=0; ib<4; ib++) v_dang[ib] = surfShift[itype]*0.25e0;

//       int loc = nn;
//       for(int nl=0; nl < nnlist.nadj(iL);  nl++, loc++) {
// 	int ib=0;
// 	cosv = dot(bond[ib],nnlist.dR[loc]);
// 	while(fabs(cosv) < 0.9 && ib < 3) {
// 	  cosv = dot(bond[++ib],nnlist.dR[loc]);
// 	} 
// 	v_dang[ib] = 0.0e0;
//       } // nl = neibors

//       if(cosv>0) { // --- bond
// 	Scalar_t vsum = v_dang[0] + v_dang[1] + v_dang[2] + v_dang[3];
// 	hon(0,0) += vsum; // (0,0) 0+0*5
// 	hon(1,1) += vsum; // (1,1) 1+1*5
// 	hon(2,2) += vsum; // (2,2) 2+2*5
// 	hon(3,3) += vsum; // (3,3) 3+3*5

// 	vsum = v_dang[0] - v_dang[1] + v_dang[2] - v_dang[3];
// 	hon(0,1) += vsum; // (0,1) 1+0*5
// 	hon(1,0) += vsum; // (1,0) 0+1*5
// 	hon(2,3) += vsum; // (2,3) 3+2*5
// 	hon(3,2) += vsum; // (3,2) 2+3*5

// 	vsum = v_dang[0] - v_dang[1] - v_dang[2] + v_dang[3];
// 	hon(0,2) += vsum; // (0,2) 2+0*5
// 	hon(1,3) += vsum; // (1,3) 3+1*5
// 	hon(2,0) += vsum; // (2,0) 0+0*5
// 	hon(3,1) += vsum; // (3,1) 1+3*5

// 	vsum = v_dang[0] + v_dang[1] - v_dang[2] - v_dang[3];
// 	hon(0,3) += vsum; //(0,3) 3+0*5
// 	hon(1,2) += vsum; //(1,2) 2+1*5
// 	hon(2,1) += vsum; //(2,1) 1+2*5
// 	hon(3,0) += vsum; //(3,0) 0+3*5
//       } else { // --- bond
// 	Scalar_t vsum = v_dang[0] + v_dang[1] + v_dang[2] + v_dang[3];
// 	hon(0,0) += vsum; // (0,0) 0+0*5
// 	hon(1,1) += vsum; // (1,1) 1+1*5
// 	hon(2,2) += vsum; // (2,2) 2+2*5
// 	hon(3,3) += vsum; // (3,3) 3+3*5

// 	vsum = v_dang[0] - v_dang[1] + v_dang[2] - v_dang[3];
// 	hon(0,1) -= vsum; // (0,1) 1+0*5
// 	hon(1,0) -= vsum; // (1,0) 0+1*5
// 	hon(2,3) += vsum; // (2,3) 3+2*5
// 	hon(3,2) += vsum; // (3,2) 2+3*5

// 	vsum = v_dang[0] - v_dang[1] - v_dang[2] + v_dang[3];
// 	hon(0,2) -= vsum; // (0,2) 2+0*5
// 	hon(1,3) += vsum; // (1,3) 3+1*5
// 	hon(2,0) -= vsum; // (2,0) 0+0*5
// 	hon(3,1) += vsum; // (3,1) 1+3*5

// 	vsum = v_dang[0] + v_dang[1] - v_dang[2] - v_dang[3];
// 	hon(0,3) -= vsum; //(0,3) 3+0*5
// 	hon(1,2) += vsum; //(1,2) 2+1*5
// 	hon(2,1) += vsum; //(2,1) 1+2*5
// 	hon(3,0) -= vsum; //(3,0) 0+3*5
//       }
//     } // dangling bond terminated

//     fout << "Atom = " << iG << " " << iG << " " << iG*5 << " " << iG*5 << endl;
//     for(int ib=0; ib<5; ib++) {
//       for(int jb=0;jb<5; jb++)
// 	fout << setw(18) << setprecision(8) << hon(ib,jb);
//       fout << endl;
//     }
//     nn += nnlist.nadj(iL); // increase the nn
//   }

//   Scalar_t pvec[8];
//   nn = 0;  
//   for(int iL=0; iL<nnlist.getLocalNum(); iL++) {

//     int iG =nnlist.I[iL];
//     int itype = myPtcl->GroupID[iG];

//     for(int nni=0; nni< nnlist.nadj(iL); nni++, nn++){
//       int jG = myPtcl->ID[nnlist.iadj(iL, nni)];
//       int ij = Bond(myPtcl->GroupID[iG],myPtcl->GroupID[jG]);
//       Scalar_t sep = nnlist.R[nn];

//       if( sep > (*SK_h)[ij]->rcut() ) continue;
//       hon = 0.0;	  
//       //pre-calculate values related to cosine vectors
//       getPD(nnlist.dR[nn], pvec, (*SK_h)[ij]->maxL());
//       (*SK_h)[ij]->getOffsite(sep, pvec, hon.data());
//       fout<< "Pair = " << iG << " " << jG << " " << iG*5 << " " << jG*5 << endl;
//       for(int ib=0; ib<5; ib++) {
// 	for(int jb=0;jb<5; jb++)
// 	  fout << setw(18) << setprecision(8) << hon(ib,jb);
// 	fout << endl;
//       }
//     }
//   }

// }
#else
void 
ZincBlendSurface::diagonal(SparseMatrix_t& H, const PtclPairList_t& nnlist) {

  int myID=nnlist.myID;
  const double oneoversqrt3 = 0.5773502691896257645091487805019574556475;
  vector<SingleParticlePos_t> bond(4);
  bond[0] = SingleParticlePos_t( oneoversqrt3,  oneoversqrt3,  oneoversqrt3);
  bond[1] = SingleParticlePos_t(-oneoversqrt3, -oneoversqrt3,  oneoversqrt3);
  bond[2] = SingleParticlePos_t( oneoversqrt3, -oneoversqrt3, -oneoversqrt3);
  bond[3] = SingleParticlePos_t(-oneoversqrt3,  oneoversqrt3, -oneoversqrt3);
  Scalar_t v_dang[4]; 

  typedef PtclPairList_t::PairData_t    PairData_t;
  const SparseMatrix_t::Index_t* hindex = H.getIndex();
  const ParticleIndex_t& grID(myPtcl->GroupID);

  for(int iL=0; iL<nnlist.getLocalNum(); iL++) {

    int iat(nnlist.I[iL]);
    int itype(grID[iat]);
    int num_nn(nnlist.nadj(iL));
    Scalar_t* restrict hon = H.diag(myID,iL);
    if(num_nn < 4) {      
      hon[i44] += ssShift[itype];
      Scalar_t cosv = 0.0e0;
      for(int ib=0; ib<4; ib++) v_dang[ib] = surfShift[itype]*0.25e0;
      PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
      while(nit != nit_end) {
        const PairData_t& cur(*nit);
	int ib=0;
	cosv = dot(bond[ib],cur.displ);
	while(fabs(cosv) < 0.9 && ib < 3) {
	  cosv = dot(bond[++ib],cur.displ);
	} 
	v_dang[ib] = 0.0e0;
        ++nit;
      } // nl = neibors

      if(cosv>0) { // --- bond
	Scalar_t vsum = v_dang[0] + v_dang[1] + v_dang[2] + v_dang[3];
	hon[i00] += vsum; // (0,0) 0+0*5
	hon[i11] += vsum; // (1,1) 1+1*5
	hon[i22] += vsum; // (2,2) 2+2*5
	hon[i33] += vsum; // (3,3) 3+3*5

	vsum = v_dang[0] - v_dang[1] + v_dang[2] - v_dang[3];
	hon[i01] += vsum; // (0,1) 1+0*5
	hon[i10] += vsum; // (1,0) 0+1*5
	hon[i23] += vsum; // (2,3) 3+2*5
	hon[i32] += vsum; // (3,2) 2+3*5

	vsum = v_dang[0] - v_dang[1] - v_dang[2] + v_dang[3];
	hon[i02] += vsum; // (0,2) 2+0*5
	hon[i13] += vsum; // (1,3) 3+1*5
	hon[i20] += vsum; // (2,0) 0+0*5
	hon[i31] += vsum; // (3,1) 1+3*5

	vsum = v_dang[0] + v_dang[1] - v_dang[2] - v_dang[3];
	hon[i03] += vsum; //(0,3) 3+0*5
	hon[i12] += vsum; //(1,2) 2+1*5
	hon[i21] += vsum; //(2,1) 1+2*5
	hon[i30] += vsum; //(3,0) 0+3*5
      } else { // --- bond
	Scalar_t vsum = v_dang[0] + v_dang[1] + v_dang[2] + v_dang[3];
	hon[i00] += vsum; // (0,0) 0+0*5
	hon[i11] += vsum; // (1,1) 1+1*5
	hon[i22] += vsum; // (2,2) 2+2*5
	hon[i33] += vsum; // (3,3) 3+3*5

	vsum = v_dang[0] - v_dang[1] + v_dang[2] - v_dang[3];
	hon[i01] -= vsum; // (0,1) 1+0*5
	hon[i10] -= vsum; // (1,0) 0+1*5
	hon[i23] += vsum; // (2,3) 3+2*5
	hon[i32] += vsum; // (3,2) 2+3*5

	vsum = v_dang[0] - v_dang[1] - v_dang[2] + v_dang[3];
	hon[i02] -= vsum; // (0,2) 2+0*5
	hon[i13] += vsum; // (1,3) 3+1*5
	hon[i20] -= vsum; // (2,0) 0+0*5
	hon[i31] += vsum; // (3,1) 1+3*5

	vsum = v_dang[0] + v_dang[1] - v_dang[2] - v_dang[3];
	hon[i03] -= vsum; //(0,3) 3+0*5
	hon[i12] += vsum; //(1,2) 2+1*5
	hon[i21] += vsum; //(2,1) 1+2*5
	hon[i30] -= vsum; //(3,0) 0+3*5
      }
    } else {
      if(diagMix[itype]) {
	int ny(0);
	int y(diagMix[itype]->SpeciesY);
        PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
        while(nit != nit_end) {
	  if((*nit).Group == y) ny++;
          ++nit;
        }
	Scalar_t mix_ratio = static_cast<Scalar_t>(ny)/static_cast<Scalar_t>(num_nn);
	diagMix[itype]->add(hon,mix_ratio);
      }
    }// dangling bond terminated
  }

  //   cout << iat << endl;
  //   for(int ib=0; ib<BS; ib++) {
  //     for(int jb=0; jb<BS; jb++)
  //        cout << hon(ib,jb) <<" ";
  //     cout << endl;
  //   }
  //   cout << endl;
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: ZincBlendSurface.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
