//////////////////////////////////////////////////////////////////
// (c) Copyright 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "SlaterKosterTB/SKParameters.h"
#include "SlaterKosterTB/TBModelBase.h"

namespace OHMMS {

  SKParameters::SKParameters():curSK(0),curOnsite(0){ }

  SKParameters::~SKParameters() {
    for(int i=0; i<SK_h.size(); i++) delete SK_h[i];
    for(int i=0; i<SK_s.size(); i++) delete SK_s[i];
    for(int i=0; i<OtherPot.size(); i++) delete OtherPot[i];
    if(curSK) delete curSK;
  }

  void SKParameters::setParticleSet(Particle_t* ptclin) {
    if(myPtcl == ptclin) return;
    myPtcl = ptclin;
    for(int i=0; i<OtherPot.size(); i++) OtherPot[i]->setParticleSet(myPtcl);
  }

  void SKParameters::resetParticleSet(Particle_t* ptclin) {
    myPtcl = ptclin;
    for(int i=0; i<OtherPot.size(); i++) 
      OtherPot[i]->resetParticleSet(myPtcl);
  }

  SKParameters::Scalar_t 
  SKParameters::evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale) const {
    Scalar_t esum=0.0;
    for(int i=0; i<OtherPot.size(); i++) 
      esum += OtherPot[i]->evalEnergyNN(nnlist,rscale);
    return esum;
  }

  SKParameters::Scalar_t 
  SKParameters::evalForceNN(PtclPairList_t& nnlist, ParticlePos_t& f0, 
      unsigned stressmode) const {
    Scalar_t esum=0.0;
    for(int i=0; i<OtherPot.size(); i++) 
      esum += OtherPot[i]->evalForceNN(nnlist,f0,stressmode);
    return esum;
  }

  void SKParameters::setOnsiteSK(SparseMatrix_t* rhoM) {
    if(curOnsite) {
      curOnsite->Rho = rhoM;
    }
  }

  void SKParameters::setOnsite(SparseMatrix_t& hmat, const PtclPairList_t& nnlist) {
    if(curOnsite) curOnsite->diagonal(hmat,nnlist);
  }

}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 364 $   $Date: 2005-07-13 13:33:56 -0500 (Wed, 13 Jul 2005) $
 * $Id: SKParameters.cpp 364 2005-07-13 18:33:56Z jnkim $ 
 ***************************************************************************/
