//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/**@file SlaterKosterTBGamma.h
 * @brief Definition of SlaterKosterTBGamma
 */
#ifndef OHMMS_SLATERKOSTERTB_GAMMA_H
#define OHMMS_SLATERKOSTERTB_GAMMA_H

#include "SlaterKosterTB/SlaterKosterTB.h"

namespace OHMMS {

  class TBSCFBase;
  class SymEigenSolver;

  /** a SlaterKosterTB for gamma calculations
   *
   * Using eigen solvers for symmetric matrices
   */
  class SlaterKosterTBGamma: public SlaterKosterTB {

  public:

    SlaterKosterTBGamma();
    ~SlaterKosterTBGamma();

    Scalar_t el_ph_coupling(const vector<int>& psi_i, const PhononMode& phonon,
        		    vector<int>& psi_f, Scalar_t del);
    
  protected:
    
    void setEigenSolver();
    Scalar_t solveGES();
    Scalar_t evalForceTB(ParticlePos_t& f, ParticleScalar_t& e, 
		     Tensor_t&, unsigned stressmode);
    void     tbDynamicMatrix(DynMatrix_t&);

  private:

    TBSCFBase* scfEngine;
    SymEigenSolver* thisSolver;
  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 330 $   $Date: 2005-05-26 17:44:25 -0500 (Thu, 26 May 2005) $
 * $Id: SlaterKosterTBGamma.h 330 2005-05-26 22:44:25Z jnkim $
 ***************************************************************************/
