//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include <sstream>
using namespace std;
#include "SlaterKosterTB/TBPotentialFactory.h"
#include "SlaterKosterTB/SlaterKosterTBGamma.h"
#include "SlaterKosterTB/SlaterKosterTBKPT.h"
#include "SlaterKosterTB/FastSKTB.h"
#include "Utilities/OhmmsInfo.h"

namespace OHMMS {

  PotentialBase* createTBPotential(int argc, char **argv) {
    int i=0; 
    bool gamma = true;


    while(i<argc) {
      if(!strcmp(argv[i],"--kpoint")) {
	gamma = false;
      } else if(!strcmp(argv[i],"--so")) {
	gamma = false;
      } 
      i++;
    }

    if(gamma) 
      return new SlaterKosterTBGamma;
    else // calling a hermitian solver
      return new SlaterKosterTBKPT;
  }


  PotentialBase*  createTBPotential(xmlNodePtr tbPtr, xmlNodePtr layoutPtr) {

    bool gamma = true;
    string eng("default");

#if defined(HAVE_LIBXML2)
    const xmlChar* a=xmlGetProp(tbPtr,(const xmlChar*)"engine");
    if(a) {
      eng=(const char*)a;
    }
    xmlNodePtr cur = tbPtr->children;
    while (cur != NULL) {
      string cname((const char*)(cur->name));
      if(cname == "kpoint") {
	gamma = false;
      } else if(cname == "particlelayout") {
        layoutPtr = cur; //overwrite layoutPtr with "//potential/particlelayout"
      }
      cur = cur->next;
    }
#endif
  
    SlaterKosterTB* tbpot=0;  
    if(eng == "fast") {
      LOGMSG("Creating FastSKTB")
      tbpot = new FastSKTB;
    } else {
      if(gamma) {
        LOGMSG("Creating SlaterKosterTBGamma")
        tbpot = new SlaterKosterTBGamma;
      } else {// calling a hermitian solver
        LOGMSG("Creating SlaterKosterTBKPT")
        tbpot = new SlaterKosterTBKPT;
      }
    }
   
    tbpot->setLayout(layoutPtr);
    tbpot->put(tbPtr);
    return tbpot;
  }

}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: TBPotentialFactory.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
