//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
///////////////////////////////////////////////////////////////////////
// TBModelBase is an abstract class from which a model TB is derived.
//  pure virtial functions should be implemneted by a derived model.
//  returns the maximum angular momentum.
//  virtual int initialize(SKArray_t&)  =0;
//  virtual int initialize(SKArray_t&, SKArray_t&)  =0;
/////////////////////////////////////////////////////////////////////
#ifndef OHMMS_SLATERKOSTERTB_MODELBASE_FACTORY_H
#define OHMMS_SLATERKOSTERTB_MODELBASE_FACTORY_H

#include "SlaterKosterTB/TBModelBase.h"
namespace OHMMS {

  //forward declaration of SKParameters
  //class SKParameters;
  template <class T> class SKParameters;

  struct TBModelFactory {

    static bool init(const string& fname,
		     SKParameters<Scalar_t>& skp,
		     TBOnsiteBase* OnSite,
		     vector<ShortRangePotential*>& pot_s);
	 
    /*!\fn PotentialBase* int
     *\param mname the name of a model TB
     *\param SK_h SlaterKoster composite blocks for TB Hamiltonian matrix
     *\param SK_s SlaterKoster composite blocks for Overlap matrix
     *\param Bond mapping between SK_h[Bond(i,j)] for species i and j
     *\param fin input stream
     *\return A repulsive potential if a model TB requires one
     */
    static ShortRangePotential*
    init(const string& mname, SKArray_t& SK_h,  SKArray_t& SK_s, 
	 Matrix<int>& Bond, istream& fin);

    static vector<TBModelBase*> TB;
  };

  struct TBOnsiteFactory {

    /*\fn TBOnsiteBase* int
     *\param oname the name of a model for onsite terms
     *\param fin input stream
     *\param A onsite model with additional diagonal terms
     */
    static TBOnsiteBase* init(const string& oname, istream& fin);
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: TBModelFactory.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
