//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2004- by Lars Jonsson and Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include <iostream>
#include <fstream>
#include <iomanip>
#include <unistd.h>
#include <sys/types.h>
#ifdef __SGI_IRIX
#include <sys/sysmp.h>
#endif
#include "SparseMatrix/NumaInfo.h"

void NumaInfo::print_host_info() { 
  std::cout << "The host is " << hname << " with " 
	    << Ncpus << " cpus on " << Nnodes << " nodes\n";
}  

void NumaInfo::print_thread_info() {
  std::cout << "The present job has " << Nthreads 
	    << " threads running on processors (module, slot):\n";
  for(int j=0; j<(Nthreads/4); j++) {
    std::cout << "  " << std::setw(3) << j*4 << " - " << std::setw(3) << j*4+3 << ": " 
	      << std::setw(3) << table[0][j*4] 
	      << " (" << std::setw(2) << table[1][table[0][j*4]] 
	      << "," << std::setw(1) << table[2][table[0][j*4]] << ")";
    for(int i=j*4+1; i<j*4+4; i++) 
      std::cout << ", " << std::setw(3) << table[0][i] 
		<< " (" << std::setw(2) << table[1][table[0][i]] 
		<< "," << std::setw(1) << table[2][table[0][i]] << ")";
    std::cout << std::endl;
  }
  
  if(Nthreads%4) {
    std::cout << "  " << std::setw(3) << (Nthreads/4)*4 
	      << " - " << std::setw(3) << Nthreads-1 << ": " 
	      << std::setw(3) << table[0][(Nthreads/4)*4] 
	      << " (" << std::setw(2) << table[1][table[0][(Nthreads/4)*4]] 
	      << "," << std::setw(1) << table[2][table[0][(Nthreads/4)*4]] << ")";
    for(int i=(Nthreads/4)*4+1; i<Nthreads; i++) 
      std::cout << ", " << std::setw(3) << table[0][i] 
		<< " (" << std::setw(2) << table[1][table[0][i]] 
		<< "," << std::setw(1) << table[2][table[0][i]] << ")";
    std::cout << std::endl;
  }
}

NumaInfo::NumaInfo() {
#ifdef __SGI_IRIX
    gethostname (hname,10);
    std::ifstream fnode("nodenum");
    std::ifstream fcpu("cpunum");
    int node[3][1024];
    int ic=0, in=0, idmy;
    fcpu >> idmy;
    while(!fcpu.eof()) {
      table[0][ic] = idmy; fcpu >> table[1][ic] >> table[2][ic]; ic++; fcpu >> idmy;
    }
    fnode >> idmy;
    while(!fnode.eof()) {
      node[0][in]=idmy; fnode >> node[1][in] >> node[2][in]; in++; fnode >> idmy;
    }
    for(int i=0; i<ic; i++) 
      for(int j=0; j<in; j++) 
	if((table[1][i]==node[1][j])&&(table[2][i]==node[2][j])) 
	  table[3][i]=node[0][j];
    Ncpus=ic; Nnodes=in;
    Nthreads=omp_get_max_threads();
#pragma parallel 
    { 
      table[0][omp_get_thread_num()]=sysmp(MP_GETMUSTRUN);
    }
#endif
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 185 $   $Date: 2004-11-18 20:40:37 -0600 (Thu, 18 Nov 2004) $
 * $Id: NumaInfo.cpp 185 2004-11-19 02:40:37Z jnkim $ 
 ***************************************************************************/
