//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim, Seunwon Lee and Lars Jonsson
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_SPARSEMATRIX_OMP_H
#define OHMMS_SPARSEMATRIX_OMP_H

#include <complex>
#include "SparseMatrix/SparseIndex.h"
#include "OhmmsPETE/OhmmsVector.h"
#include "OhmmsData/OhmmsElementBase.h"
//class WorkSpace;

class SparseMatrixOMP: public OhmmsElementBase {

public:

  typedef OHMMS_PRECISION    T;
  typedef T*                 pointer;
  typedef const T*           const_pointer;

  typedef std::complex<T>    dcomplex_t;
  typedef OHMMS::SparseIndex Index_t;

  typedef Vector<T>          Container_t;

  SparseMatrixOMP();
  SparseMatrixOMP(Index_t* ind);
  ~SparseMatrixOMP(); 

  void setIndex(Index_t* sInd);

  const Index_t* getIndex() const { return myIndex;}
  Index_t* getIndex() { return myIndex;}

  //return true if the internal storage is empty
  inline bool empty() const { return NumThreads == 0;}
  ///create
  void create();
  void resize(omp_int_t pid);
  void clear(int assignmode = 0, omp_int_t myID=0);
  void clearDiagonal(omp_int_t myID=0);

  //SparseMatrixOMP& operator +=(T);
  void add2Diag(T mu, omp_int_t myid=0);
  void axpy(const SparseMatrixOMP& a, T mu, omp_int_t myid=0);

  void apply(const T* restrict win, T* restrict wout, bool mpitarget=false) const;
  void apply(const dcomplex_t* restrict win, 
	     dcomplex_t* restrict  wout) const;
  void apply(T** restrict win, int i, 
	     T** restrict wout, int j, 
	     int maxnb, int ifactor) const;

  inline int getBlockSize() const { return Dmax;}

  inline pointer diag(omp_int_t myid, int i) { 
    return d_diag[myid]->data()+i*Dmax;
  }
  inline pointer offdiag(omp_int_t myid, int i) { 
    return d_offdiag[myid]->data()+i*Dmax;
  }
  inline const_pointer diag(omp_int_t myid, int i) const { 
    return d_diag[myid]->data()+i*Dmax;
  }
  inline const_pointer offdiag(omp_int_t myid, int i) const { 
    return d_offdiag[myid]->data()+i*Dmax;
  }

  // returns the size of column on myid
  inline int numcol(omp_int_t myid) const {
    return  myIndex->col_loc(myIndex->last_diag(myid))
           -myIndex->col_loc(myIndex->first_diag(myid));
  }
  
  //implement virtual functions
  bool get(std::ostream& ) const;
  bool put(std::istream& ) { return true;}
  bool put(xmlNodePtr cur);
  void reset();

private:

  int Num; 
  int WS_size;
  int Dmax;
  omp_int_t NumThreads;

  Index_t* myIndex;
  vector<int> MaxNumPairs;
  vector<Container_t*> d_diag, d_offdiag;

  //T***  d_diag;
  //T***  d_offdiag;
  //WorkSpace::Block_t* myWorkSpace;
};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: SparseMatrixOMP.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

