//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Lars Jonsson, Seungwon Lee and Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include <iostream>
#include "Message/OpenMP.h"
#include "SparseMatrix/WorkSpace.h"
//#include "SparseMatrix/NumaInfo.h"

//initialization of the static data
int WorkSpace::NPROC = 1;
std::vector<WorkSpace::Block_t*> WorkSpace::Blocks;
//NumaInfo* WorkSpace::NI = 0;

#ifdef __SGI_IRIX
extern "C" {
   __uint64_t va2pa_lj(void *);
}
#define VADR2NODE(A) (((int) (va2pa_lj(A) >> 32)))
#endif

template<class T>
void WorkSpaceBlock<T>::create(int nproc, int ndata) {
  SIZE=ndata;
  ws_begin.resize(nproc,0);
  ws_next.resize(nproc,0);
#pragma omp parallel
  {
    int pid=omp_get_thread_num();
    ws_begin[pid] = new double[SIZE+4096];
    for(int i=0;i<SIZE+4096;i++) (ws_begin[pid])[i]=1.e-9*i;
    ws_begin[pid]+=4096;
    ws_next[pid]=ws_begin[pid];
  }
}

WorkSpace::Block_t* 
WorkSpace::create(int ndata) {

  //if(!NI) NI = new NumaInfo();
  NPROC=omp_get_max_threads();

  Block_t* newblock = 0;

  //check if there is unused block 
  int iblock=0;
  while(newblock == 0 && iblock<Blocks.size()) {
    if(Blocks[iblock]->capacity()>=ndata && Blocks[iblock]->used(0) == 0) {
      newblock = Blocks[iblock];
      std::cout << "Using an existing unused block " << iblock 
		<< " size of "<<Blocks[iblock]->capacity() << std::endl;
    }
    iblock++;
  }

  if(newblock == 0) {
    newblock = new Block_t(NPROC,ndata);
    std::cout << "Creating a new block " << ndata << std::endl;
    Blocks.push_back(newblock);
  }

  return newblock;
//  #ifdef __SGI_IRIX
//    for(i=0;i<NPROC;i++) {
//      iproc=NI->get_proc(i);
//      cout << "Thread " << i << ": proc = " << iproc << ", module = " << NI->get_module(iproc) 
//           << ", slot = " << NI->get_slot(iproc) << ", node = " << NI->get_node(iproc) 
//           << ", ws begins on pa " << VADR2NODE(ws_begin[i])
//           << ", ws ends on pa " << VADR2NODE(ws_begin[i]+SIZE-1) << endl;
//      if((NI->get_node(iproc))!=(VADR2NODE(ws_begin[i]))) cout << "Warning: thread "
//         << i << " has bad workspace start" << endl;
//      if((NI->get_node(iproc))!=(VADR2NODE(ws_begin[i]+SIZE-1))) cout << "Warning: thread "
//         << i << " has bad workspace end" << endl;
//    }
//    cout.flush();
//  #endif
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 201 $   $Date: 2004-12-22 10:34:56 -0600 (Wed, 22 Dec 2004) $
 * $Id: WorkSpace.cpp 201 2004-12-22 16:34:56Z jnkim $ 
 ***************************************************************************/

