//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Lars Jonsson, Seungwon Lee and Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_WORKSPACE_H
#define OHMMS_WORKSPACE_H

#include <vector>
#include <map>

/**class to encapsulate memory blocks that can are distributed over openmp threads
 */
template<class T>
struct WorkSpaceBlock {
  
  int SIZE;
  std::vector<T*>  ws_begin;
  std::vector<T*>  ws_next;
  
  WorkSpaceBlock():SIZE(0){}
  WorkSpaceBlock(int nproc, int ndata){create(nproc,ndata);}
  ~WorkSpaceBlock() { }

  inline void setValue(int pid, T x) {
    std::fill(ws_begin[pid],ws_next[pid],x);
  }

  inline int used(int pid) { return ws_next[pid]-ws_begin[pid];}
  inline int remain(int pid) { return SIZE-used(pid);}
  inline double* New(int pid, int num){
    double* ptemp;
    if(remain(pid)>=num) 
      {ptemp=ws_next[pid]; ws_next[pid]+=num; return ptemp;}
    else
      return 0;
  }

  inline void resize(int pid, int ndata) {
    //first, release the data
    if(ndata>SIZE) {
      if(ws_begin[pid]) delete [] ws_begin[pid];
      SIZE = ndata;
      ws_begin[pid] = new double[SIZE+4096];
      for(int i=0;i<SIZE+4096;i++) (ws_begin[pid])[i]=1.e-9*i;
      ws_begin[pid]+=4096;
    }
    ws_next[pid]=ws_begin[pid];
  }

  void create(int nproc, int ndata);
  inline int capacity() { return SIZE;}
  inline T* begin(int pid) {return ws_begin[pid];}
  inline T* next(int pid) {return ws_next[pid];}
  inline void release(int pid) {ws_next[pid]=ws_begin[pid];}
};

//class NumaInfo;

struct WorkSpace {

  typedef WorkSpaceBlock<double> Block_t;
  static int NPROC;
  //static NumaInfo* NI;
  static std::vector<Block_t*> Blocks;
  static Block_t* create(int num);
};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 255 $   $Date: 2005-01-14 11:41:16 -0600 (Fri, 14 Jan 2005) $
 * $Id: WorkSpace.h 255 2005-01-14 17:41:16Z jnkim $ 
 ***************************************************************************/

