//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Message/OpenMP.h"
#include "SparseMatrix/SparseIndexNN.h"
using namespace OHMMS;

SparseIndexNN::SparseIndexNN():  NumThreads(0) { 

  DEBUGMSG("Calling SparseIndexNN::SparseIndexNN() with " ) 

}

void SparseIndexNN::create(int n) {
  NumThreads = n;
  myPairs.resize(n,3);
  myAtoms.resize(n,3);
}

SparseIndexNN::~SparseIndexNN() {  
  DEBUGMSG("Calling SparseIndexNN::~SparseIndexNN() with " ) 
} 

void SparseIndexNN::update(Particle_t& newptcl) {
  
  ///for the time being, always calculate this
  getPairlist(newptcl,-1);

  /// update the pair list for array
  int nn = 0;
  for(int myid=0; myid < NumThreads; myid++) {
    myPairs(myid,2) = PairAttrib[myid].getTotNadj();
    myPairs(myid,0) = nn;
    nn +=  PairAttrib[myid].getTotNadj();
    myPairs(myid,1) = nn;
  }

}

bool SparseIndexNN::get(ostream& os) const{

  os << "Maximum Row-block size = " << MaxRowBlock << endl;
  os << "Maximum Col-block size = " << MaxColBlock << endl;

  for(int myID=0; myID<NumThreads; myID++)  {
    os << "The range of atoms for Node " << myID << " ";
    os << first_diag(myID) << " " << last_diag(myID) << endl;
    int iG = first_diag(myID);
    for(int iL=0; iL<num_diag(myID); iL++, iG++) {
      os << iG << " ";
      for(int nni=0; nni<nadj(iL,myID); nni++) {
	os << iadj(iL, nni, myID) << " ";
      }
      os << endl;
    }
  }
  return true;
}

void SparseIndexNN::reset() {
}
bool SparseIndexNN::put(istream&){
  return true;
}

bool SparseIndexNN::put(xmlNodePtr cur){
  return true;
}
void SparseIndexNN::resetSpecial(ParticleIndex_t& id){

  if(Reordered) {
    RowBlock.resize(TotalNum+1);
    ColBlock.resize(TotalNum+1);
    int nr = 0;
    int nc = 0;
    for(int i=0; i<TotalNum; i++) {
      RowBlock[i] = nr;
      ColBlock[i] = nc;
      nr += RowUnit[id[i]];
      nc += ColUnit[id[i]];
    }
    RowBlock[TotalNum] = nr;
    ColBlock[TotalNum] = nc;
    TotalRows = nr;
    TotalCols = nc;
    Reorodered = false;
  }
}


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * OHMMS_VERSION: $Id: SparseIndexNN.cpp 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
