//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_SPARSEVECTORINDEXOMP3_GRIDBASE_H
#define OHMMS_SPARSEVECTORINDEXOMP3_GRIDBASE_H

#include "Particle/NNEngine.h"
#include "OhmmsNumeric/Matrix.h"

/*! \class SparseIndex
 * A specialized sparse index class for TB Hamiltonian matrices
 * inherited from NNEngine for neighbor list
 */
namespace OHMMS {

  struct SparseIndexNN: public NNEngine {

    int NumThreads;
    int MaxRowBlock, MaxColBlock, MaxBlockSize;
    int TotalRows, Totalcols;

    vector<int> RowUnit, ColUnit;
    vector<int> RowBlock;
    vector<int> ColBlock;  
    IndexMatrix_t myPairs, myAtoms;
    
    SparseIndexNN();
    virtual ~SparseIndexNN();
    
    void create(int n);

    inline int rows() const { return TotalRows;}
    inline int cols() const { return TotalCols;}
    
    inline int numatoms(int myid=0) const {
      return PtclDist[myid+1]-PtclDist[myid];
    }
    inline int numpairs(int myid=0) const {
      return PairAttrib[myid].getTotNadj();
    }

    inline int maxpairs(int myid=0) const {
      return PairAttrib[myid].getMaxNadj();
    }
    
    inline int nadj(int iL, int myid) const { 
      return PairAttrib[myid].nadj(iL);
    }

    inline int iadj(int iL, int nni, int myid) const { 
      return PairAttrib[myid].iadj(iL, nni); 
    }

    inline int row_loc(int i) const { return RowBlock[i];}
    inline int col_loc(int i) const { return ColBlock[i];}
    inline int row_block(int ig) const { return RowBlock[ig+1]-RowBlock[ig];}
    inline int col_block(int ig) const { return ColBlock[ig+1]-ColBlock[ig];}
    
    inline int first_diag(int myid =0) const { return PtclDist[myid];}
    inline int last_diag(int myid =0) const { return PtclDist[myid+1];}
    inline int num_diag(int myid =0) const { return PtclDist[myid+1]-PtclDist[myid];}
    
    inline int first_offdiag(int myid=0) const { return myPairs(myid,0);}
    inline int last_offdiag(int myid=0) const {return myPairs(myid,1); }
    inline int num_offdiag(int myid=0) const { return myPairs(myid,2);}

    
    template<class NS> void 
    set_blocks(const NS& rowblock,  const NS& colblock) {
      RowUnit.resize(rowblock);
      ColUnit.resize(colblock);
      int rowmax=0;
      for(int i=0; i<rowblock.size(); i++) {
	RowUnit[i] = static_cast<int>(rowblock[i]);
	rowmax = max(rowmax,RowUnit[i]);
      }
      int colmax=0;
      for(int i=0; i<colblock.size(); i++) {
	ColUnit[i] = static_cast<int>(colblock[i]);
	colmax = max(colmax,ColUnit[i]);
      }
      MaxRowBlock = rowmax; MaxColBlock = colmax; MaxBlockSize = rowmax*colmax;
    } 

#ifdef ENABLE_OPENMP
    inline int where(int ig) const { //return NodeID[ig];}
      int res=0;
      while(ig >= last_diag(res)) {res++;}
      return res;
    }
#else
    inline int where(int ig) const { return 0;}
#endif

    bool get(ostream& os) const;
    bool put(istream&);

    bool put(xmlNodePtr cur);

    void resetSpecial(ParticleIndex_t&);
  };
}

#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * OHMMS_VERSION: $Id: SparseIndexNN.h 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
