//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_SPARSEVECTORINDEXOMP_H
#define OHMMS_SPARSEVECTORINDEXOMP_H

#ifndef OHMMS_PTCLPAIRLISTBASE_H
#include "ParticleBase/PtclPairListBase.h"
#endif
#include "Particle/OhmmsParticle.h"
using namespace OHMMS;
#include "Utilities/OhmmsObject.h"

/*! \class VectorIndex
 * A specialized sparse index class for TB Hamiltonian matrices
 * Has a reference to PairList and creates extra indices.
 */
struct SparseIndexOMP: public OhmmsObject {

  typedef PtclPairListBase<double,3> ThisPairList_t;
  typedef ParticleAttrib<int>        IonID_t;
  typedef vector<double>             NS_t;

  bool OwnIndices;
  int NumThreads, TotalNum;
  int MaxRowBlock, MaxColBlock, MaxBlockSize;
  int TotalRows, TotalCols; 

  ThisPairList_t *myList;
  int *NodeID;
  int *IonID;
  int **myAtoms;
  int **myPairs;
  int **J;
  int **M;

  vector<int> RowBlock;
  vector<int> ColBlock;  

  SparseIndexOMP(const char* = "none");
  virtual ~SparseIndexOMP(); // myList has to be provided externally

#ifdef ENABLE_OPENMP
  void init(ThisPairList_t* ptclpairs, const NS_t& ns, IonID_t& ionid, 
            IonID_t& nodeid);
  void init(ThisPairList_t* ptclpairs, const NS_t& ns, 
            const NS_t& nb, IonID_t& ionid, IonID_t& nodeid);
#else
  void init(ThisPairList_t* ptclpairs, const NS_t& ns, IonID_t& ionid); 
  void init(ThisPairList_t* ptclpairs, const NS_t& ns, 
            const NS_t& nb, IonID_t& ionid);
#endif

  inline int nrows() const { return TotalRows;}
  inline int ncols() const { return TotalCols;}

  inline int getTotNum() const { return TotalNum;}
  inline int getTotNadj() const { return myPairs[NumThreads-1][1];}

  inline int numatoms(int myID=0) const { return myAtoms[myID][2];}
  inline int numpairs(int myID=0) const { return myPairs[myID][2];}
  inline int maxpairs(int myID=0) const { return myPairs[myID][3];}

  inline int nadj(int iL, int myID = 0) const { 
    return M[myID][iL+1] - M[myID][iL];
  }

  inline int iadj(int iL, int nni, int myID = 0) const { 
    return J[myID][ M[myID][iL]+nni ];
  }

  ////////////////////////////////////////////////////////
  // WARNING: Assuming each row and col is identicial
  //////////////////////////////////////////////////////
  inline int row_loc(int i) const { return RowBlock[0]*i;}
  inline int col_loc(int i) const { return ColBlock[0]*i;}

  inline int row_block(int ig) const { return RowBlock[ IonID[ig] ];}
  inline int col_block(int ig) const { return ColBlock[ IonID[ig] ];}
  inline int kind(int ig) const { return IonID[ig];}

  // not needed
  //!< The row-index for the nn-th pair
  //inline int irow(int myID, int iL) const { return iL + myAtoms[myID][0];}
  //!< The column-index for the nn-th pair
  //inline int icol(int nn) const { return myList->J[nn];}
  ////inline int icol(int myID, int nn) const { return J[myID][nn];}

  inline int first_diag(int myid =0) const { return myAtoms[myid][0];}
  inline int last_diag(int myid =0) const { return myAtoms[myid][1];}
  inline int num_diag(int myid =0) const { return myAtoms[myid][2];}
  inline int first_offdiag(int myid = 0) const { return myPairs[myid][0];}
  inline int last_offdiag(int myid = 0) const { return  myPairs[myid][1];}
  inline int num_offdiag(int myid = 0) const { return  myPairs[myid][2];}

  inline int where(int ig) const { return NodeID[ig];}

  inline int begin_offdiag(int iL, int myid = 0) const { return M[myid][iL]; }

  virtual void update();

  void set_offsets(const NS_t& rowblock, const NS_t& colblock);
  //void make_mp_map(int myid);
  void create();
  bool write_data(ostream&);
  bool read_data(istream&);
};

#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * OHMMS_VERSION: $Id: SparseIndexOMP.h 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
