//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

// -*- C++ -*-
#ifndef OHMMS_SPARSEMATRIXSERIAL_H
#define OHMMS_SPARSEMATRIXSERIAL_H

#include <complex>
using namespace std;
#ifndef OHMMS_PTCLPAIRLISTBASE_H
#include "ParticleBase/PtclPairListBase.h"
#endif
#ifndef OHMMS_PETE_MATRIX_H
#include "OhmmsPETE/OhmmsMatrix.h"
#endif
//#include "SparseMatrix/SparseVectorIndex.h"
#include "SparseMatrix/SparseIndex.h"

class SparseMatrixSerial {
public:
  typedef double             T;
  typedef complex<T>         dcomplex_t;
  typedef Matrix<T>          FullMatrix_t;
  typedef Matrix<dcomplex_t> FullCMatrix_t;
  typedef SparseIndexOMP     Index_t;
public:

  SparseMatrixSerial();
  SparseMatrixSerial(Index_t* ind);
  ~SparseMatrixSerial(); 

  inline const Index_t* index() const { return myIndex;}

  void init(Index_t*);
  void reset(int assignmode = 0);

  SparseMatrixSerial& operator +=(T);

  bool precondition(T* g, const T* wfs) const { return false;}

  void apply(const FullMatrix_t& win, FullMatrix_t& wout);
  void apply(const T* win, T* wout) const;
  void apply(const dcomplex_t* win, dcomplex_t* wout) const;

  void copy(Matrix<T>& mout, int packing=0) const;
  void copy(Matrix<dcomplex_t>& mout, const complex<T>*, int packing = 0) const;
  void copy(Matrix<dcomplex_t>& mout, int packin =0) const;

  void product(const FullMatrix_t&, const FullMatrix_t&, const vector<T>& w);
  void product(const T* a, const T*);
  void product(SparseMatrixSerial&, const FullCMatrix_t&, const FullCMatrix_t&, 
               const vector<T>& w);

  inline T* diag(int, int i) { return d_diag[i];}
  inline T* offdiag(int, int i) { return d_offdiag[i];}
  inline const T* diag(int, int i) const { return d_diag[i];}
  inline const T* offdiag(int, int i) const { return d_offdiag[i];}

  inline int first_diag(int ) const { return 0;}
  inline int last_diag(int ) const { return Num;}
  inline int first_offdiag(int ) const { return 0;}
  inline int last_offdiag(int ) const { return NumPairs;}

  void print(ostream& os);

private:

  int Num; 
  int NumPairs;
  int WS_size;
  int Dmax;

  Index_t* myIndex;
  T**  d_diag;
  T**  d_offdiag;

};

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseMatrixSerial.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

