//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

template<class T, class I>
SparseMatrix<T,I>::SparseMatrix()
:myIndex(NULL), d_diag(NULL), d_offdiag(NULL), WS_size(-1), Num(0) 
{
}

template<class T, class Index_t>
SparseMatrix<T,Index_t>::SparseMatrix(Index_t* ind)
:d_diag(NULL), d_offdiag(NULL), WS_size(-1), Num(0)
{
  init(ind);
}

template<class T, class I>
SparseMatrix<T,I>::~SparseMatrix() { 

  if(d_diag) { 
    //for(int i=0; i<Num; i++) delete [] d_diag[i];
    //Total memory is allocated to d_diag[0], d_diag[i] for i>0 only points
    //to the addresses
    delete [] d_diag[0];
    delete d_diag;
  }
  if(d_offdiag) { 
    //for(int i=0; i<NumPairs; i++) delete [] d_offdiag[i];
    //Total memory is allocated to d_diag[0], d_diag[i] for i>0 only points
    //to the addresses
    delete [] d_offdiag[0];
    delete d_offdiag;
  }
}

template<class T, class Index_t>
void
SparseMatrix<T,Index_t>::init(Index_t* newindex) {

  if(newindex) { myIndex = newindex; }

  myIndex->update();

  if(Num<myIndex->nrows()) { // first time, allocate pointer holders

    Dmax = myIndex->maxBasis;
    NumPairs = myIndex->numpairs();
    Num = myIndex->nrows();

    int nshift = Dmax*Dmax;

    if(!d_diag) { //
      d_diag = new T*[Num];
      d_diag[0] = new T[Dmax*Dmax*Num];
      for(int i=1; i<Num; i++) d_diag[i] = d_diag[i-1]+nshift;
      //for(int i=0; i<Num; i++) d_diag[i] = new T[Dmax*Dmax];
    }

    if(!d_offdiag) { //
      WS_size = (NumPairs + NumPairs/10);// add 10%
      d_offdiag = new T*[WS_size];
      d_offdiag[0] = new T[Dmax*Dmax*WS_size];
      for(int i=1; i<WS_size; i++) d_offdiag[i] = d_offdiag[i-1]+ nshift;
      //for(int i=0; i<WS_size; i++) d_offdiag[i] = new T[Dmax*Dmax];
    }
  }

}
//////////////////////////////////////////////////////////
// will check if f77 is better.
//////////////////////////////////////////////////////////
template<class T>
inline void assign(T* d, int n, T val) {
  for(int i=0; i<n; i++) d[i] = val;
}
//////////////////////////////////////////////////////////

template<class T, class I>
void
SparseMatrix<T,I>::reset(bool changediag) {

  NumPairs = myIndex->numpairs();

  if(WS_size < NumPairs) {// checking the size

    delete [] d_offdiag[0];
    delete d_offdiag;
    
    WS_size = (NumPairs + NumPairs/10);// add 10%
    d_offdiag = new T*[WS_size];
    d_offdiag[0] = new T[Dmax*Dmax*WS_size];
    int nshift = Dmax*Dmax;
    for(int i=1; i<WS_size; i++) d_offdiag[i] = d_offdiag[i-1]+ nshift;
    //for(int i=0; i<WS_size; i++) d_offdiag[i] = new T[Dmax*Dmax];
  }

  if(changediag) assign(d_diag[0], Dmax*Dmax*Num, 0.0);    
  assign(d_offdiag[0],Dmax*Dmax*NumPairs, 0.0);
}

template<class T, class I>
template<class T1> 
SparseMatrix<T,I>& SparseMatrix<T,I>::operator +=(T1 lambda) 
{
  register T xxx = static_cast<T>(lambda);
  for(int i=0;i<Num;i++){
    int nb = myIndex->Basis(i);
    for(int ib=0;ib<nb;ib++){
      d_diag[i][ib*nb+ib] += xxx;
    }
  }
}

template<class T>
void multadd(const T* a, const T* x, T* y, int n, int m) {
  int ii=0;
  for(int i=0; i<n; i++) 
    for(int j=0; j<m; j++)
      y[i] += a[ii++]*x[j];
}

template<class T, class I>
template<class T1>
void SparseMatrix<T,I>::apply(const T1* win, T1* wout) const {

  int* offset = myIndex->Offset;
  int* basis = myIndex->Basis;

  for(int i=0; i<offset[Num]; i++) wout[i] = 0.0;

  for(int i=0; i < Num; i++){
    multadd(d_diag[i], win+offset[i], wout+offset[i], basis[i], basis[i]);
  }

  for(int nn=0; nn < NumPairs; nn++) {
    int i = myIndex->icol(nn);
    int j = myIndex->irow(nn);
    multadd(d_offdiag[nn], win+offset[j], wout+offset[i], basis[j], basis[i]);
  }
}

template<class T, class I>
template<class Mat>
void SparseMatrix<T,I>::copy(Mat& mout) const {

  int* offset = myIndex->Offset;
  int* basis = myIndex->Basis;

  for(int i=0; i < Num; i++){
    int d1 = basis[i];
    mout.copy(TinyMatrixRef<T>(d_diag[i],d1,d1), offset[i], offset[i]);
  }

  for(int nn=0; nn < NumPairs; nn++) {
    int i=myIndex->icol(nn);
    int j=myIndex->irow(nn);
    if(j>=i)
      mout.add(TinyMatrixRef<T>(d_offdiag[nn],basis[i], basis[j]),
               offset[i], offset[j]);
  }
}

template<class T, class I>
template<class Mat>
void
SparseMatrix<T,I>::copy(Mat& h, const complex<T>* phi) const {

  int *basis = myIndex->Basis;
  int *offset = myIndex->Offset;

  for(int i=0; i<Num; i++) {
    int i0=offset[i];
    int k=0;
    for(int ib=0; ib<basis[i]; ib++) {
      int j0=offset[i];
      for(int jb=0; jb<basis[i]; jb++) h(i0,j0++) += d_diag[i][k++];
      i0++;
    }
  //h.copy(TinyMatrixRef<T>(d_diag[i],basis(i),basis(i)), offset(i), offset(i));
  }

  for(int nn=0; nn < myIndex->numpairs(); nn++) {
    int i=myIndex->irow(nn); // row index in a full matrix
    int j=myIndex->icol(nn); // col index in a full matrix
    if(j>=i) {
      int i0=offset[i];
      int k=0;
      for(int ib=0; ib<basis[i]; ib++) {
	int j0=offset[j];
	for(int jb=0; jb<basis[j]; jb++) 
          h(i0,j0++) += phi[nn]*d_offdiag[nn][k++];
	i0++;
      }
    }
  }
}

template<class T, class Index_t>
template<class Mat_t>
void 
SparseMatrix<T,Index_t>::product(const Mat_t& a, const Mat_t& b,
                                 const vector<T>& w){
  int ncol = a.ncol();
  int nrow = a.nrow();
  int* offset = myIndex->Offset;
  int* basis = myIndex->Basis;

  // diagonal blocks
  for(int i=0; i<Num; i++) {
    int i0= offset[i];
    int kk = 0;
    for(int ib=0; ib<basis[i]; ib++) {
      int j0 = offset[i];
      for(int jb=0; jb<basis[i]; jb++) {
        T tmp = 0.0e0;
        for(int m=0; m<nrow; m++) {
          tmp += w[m]*a(m,i0)*b(m,j0);
        }
        d_diag[i][kk++] += tmp;
        j0++;
      }
      i0++;
    }
  }

  // off-diagonal blocks
  for(int nn=0; nn<myIndex->numpairs(); nn++) {
    int i=myIndex->irow(nn); // row index in a full matrix
    int j=myIndex->icol(nn); // col index in a full matrix
    if(i == j) continue;
    if(myIndex->Loc[nn] == nn) { // first-case only
      int i0 = offset[i];
      int kk = 0;
      for(int ib=0; ib<basis[i]; ib++) {
        int j0 = offset[j];
        for(int jb=0; jb<basis[j]; jb++) {
          T tmp = 0.0e0;
          for(int m=0; m<nrow; m++) {
            tmp+= w[m]*a(m,i0)*b(m,j0);
	  }
          d_offdiag[nn][kk++] += tmp;
          j0++;
	}
        i0++;
      }
    }
  }
}
template<class T>
inline T mult_real(const complex<T>& a, const complex<T>& b) {
  return a.real()*b.real() + a.imag()*b.imag();
}

template<class T>
inline T mult_imag(const complex<T>& a, const complex<T>& b) {
  return a.real()*b.imag() - a.imag()*b.real();
}

template<class T, class Index_t>
template<class Mat_t>
void 
SparseMatrix<T,Index_t>::product_real(const Mat_t& a, const Mat_t& b,
                                      const vector<T>& w){
  int ncol = a.ncol();
  int nrow = a.nrow();
  int* offset = myIndex->Offset;
  int* basis = myIndex->Basis;

  // diagonal blocks
  for(int i=0; i<Num; i++) {
    int i0= offset[i];
    int kk = 0;
    for(int ib=0; ib<basis[i]; ib++) {
      int j0 = offset[i];
      for(int jb=0; jb<basis[i]; jb++) {
        T tmp = 0.0e0;
        for(int m=0; m<nrow; m++) {
          tmp += w[m]*mult_real(a(m,i0),b(m,j0));
        }
        d_diag[i][kk++] += tmp;
        j0++;
      }
      i0++;
    }
  }

  // off-diagonal blocks
  for(int nn=0; nn<myIndex->numpairs(); nn++) {
    if(myIndex->Loc[nn] == nn) { // first-case only
      int i=myIndex->irow(nn); // row index in a full matrix
      int j=myIndex->icol(nn); // col index in a full matrix
      int i0 = offset[i];
      int kk = 0;
      for(int ib=0; ib<basis[i]; ib++) {
        int j0 = offset[j];
        for(int jb=0; jb<basis[j]; jb++) {
          T tmp = 0.0e0;
          for(int m=0; m<nrow; m++) {
            tmp+= w[m]*mult_real(a(m,i0),b(m,j0));
	  }
          d_offdiag[nn][kk++] += tmp;
          j0++;
	}
        i0++;
      }
    }
  }
}
template<class T, class Index_t>
template<class Mat_t>
void 
SparseMatrix<T,Index_t>::product_imag(const Mat_t& a, const Mat_t& b,
                                      const vector<T>& w){
  int ncol = a.ncol();
  int nrow = a.nrow();
  int* offset = myIndex->Offset;
  int* basis = myIndex->Basis;

  // diagonal blocks
  for(int i=0; i<Num; i++) {
    int i0= offset[i];
    int kk = 0;
    for(int ib=0; ib<basis[i]; ib++) {
      int j0 = offset[i];
      for(int jb=0; jb<basis[i]; jb++) {
        T tmp = 0.0e0;
        for(int m=0; m<nrow; m++) {
          tmp += w[m]*mult_imag(a(m,i0),b(m,j0));
        }
        d_diag[i][kk++] += tmp;
        j0++;
      }
      i0++;
    }
  }

  // off-diagonal blocks
  for(int nn=0; nn<myIndex->numpairs(); nn++) {
    if(myIndex->Loc[nn] == nn) { // first-case only
      int i=myIndex->irow(nn); // row index in a full matrix
      int j=myIndex->icol(nn); // col index in a full matrix
      int i0 = offset[i];
      int kk = 0;
      for(int ib=0; ib<basis[i]; ib++) {
        int j0 = offset[j];
        for(int jb=0; jb<basis[j]; jb++) {
          T tmp = 0.0e0;
          for(int m=0; m<nrow; m++) {
            tmp+= w[m]*mult_imag(a(m,i0),b(m,j0));
	  }
          d_offdiag[nn][kk++] += tmp;
          j0++;
	}
        i0++;
      }
    }
  }
}

template<class T, class Index_t>
void 
SparseMatrix<T,Index_t>::print(ostream& os) {

  os << "diagonal blocks\n";
  for(int i=0; i<Num; i++) {
    os << "Atom = " << i << endl;
    int kk=0;
    for(int ib=0; ib<myIndex->Basis[i]; ib++) {
      for(int jb=0;jb<myIndex->Basis[i]; jb++)
        os << setw(12) << setprecision(3) << d_diag[i][kk++];
      os << endl;
    }
  }

  os << "off-diagonal blocks\n";
  // off-diagonal blocks
  for(int nn=0; nn<myIndex->numpairs(); nn++) {
    int i=myIndex->irow(nn); // row index in a full matrix
    int j=myIndex->icol(nn); // col index in a full matrix
    int kk = 0;
    os << "Pair = " << i << " " << j << endl;
    for(int ib=0; ib<myIndex->Basis[i]; ib++) {
      for(int jb=0;jb<myIndex->Basis[j]; jb++)
        os << setw(12) << setprecision(3) << d_offdiag[nn][kk++];
      os << endl;
    }
  }

}

//  template<class T, class I>
//  template<class SK_t, class Bond_t, class IonID_t>
//  void 
//  SparseMatrix<T,I>::make(SK_t& sk, const Bond_t& bond, const IonID_t& ionid)
//  {

//    const typename I::ThisPairList_t* nnlist = myIndex->myList;

//    T pvec[8];
//    for(int i=0; i< myIndex->nrows(); i++) {
//      sk[bond(ionid[i],ionid[i])]->getOnsite(d_diag[i]);
//    }
 
//    for(int nn=0; nn<myIndex->numpairs(); nn++) {

//      int i=myIndex->irow(nn); // row index in a full matrix
//      int j=myIndex->icol(nn); // col index in a full matrix
//      int ij = bond(ionid[i],ionid[j]);

//      if( nnlist->R[nn] < sk[ij]->rcut() ) {
//        getPD(nnlist->dR[nn], pvec, sk[ij]->maxL());
//        sk[ij]->getOffsite(nnlist->R[nn], pvec, d_offdiag[nn]);
//      }
//    }
//  }

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseMatrixSerialTemplated.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

