//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

// -*- C++ -*-
#ifndef OHMMS_SPARSEMATRIXSERIAL_H
#define OHMMS_SPARSEMATRIXSERIAL_H

#include <complex>
using namespace std;
#include "OhmmsPETE/TinyMatrixRef.h"

template<class T, class Index_t>
class SparseMatrix {

protected:

  int Num; 
  int NumPairs;
  int WS_size;

  Index_t* myIndex;
  T**  d_diag;
  T**  d_offdiag;
  int Dmax;

public:

  SparseMatrix();
  SparseMatrix(Index_t* ind);
  ~SparseMatrix(); 

  inline const Index_t* index() const { return myIndex;}

  void init(Index_t*);

  void reset(bool changediag = false);

  template<class T1>
  SparseMatrix<T,Index_t>& operator +=(T1);

  template<class T1>
  void apply(const T1* win, T1* wout) const;

  template<class Mat>
  void copy(Mat& mout) const;

  template<class Mat>
  void copy(Mat& mout, const complex<T>* ) const;


  template<class Mat>
  void product(const Mat&, const Mat&, const vector<T>& w);

  template<class Mat>
  void product_real(const Mat&, const Mat&, const vector<T>& w);

  template<class Mat>
  void product_imag(const Mat&, const Mat&, const vector<T>& w);

  inline T* diag(int i) { return d_diag[i];}
  inline T* offdiag(int i) { return d_offdiag[i];}
  inline const T* diag(int i) const { return d_diag[i];}
  inline const T* offdiag(int i) const { return d_offdiag[i];}

  inline T* offdiag(int i, int j) { // expensive operation
    if(i == j) return d_diag[i];
    else       return d_offdiag[myIndex->loc(i,j)];
  }
  
  //template<class SK_t, class Bond_t, class IonID_t>
  //void make(SK_t&, const Bond_t&, const IonID_t&);

  void print(ostream& os);
};

#include "SparseMatrix/SparseMatrixSerial.cpp"

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseMatrixSerialTemplated.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

