//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

// -*- C++ -*-
#ifndef OHMMS_SPARSEMATRIXSERIAL_SPINORBIT_H
#define OHMMS_SPARSEMATRIXSERIAL_SPINORBIT_H

#include <complex>
using namespace std;
#ifndef OHMMS_PTCLPAIRLISTBASE_H
#include "ParticleBase/PtclPairListBase.h"
#endif
#ifndef OHMMS_PETE_MATRIX_H
#include "OhmmsPETE/OhmmsMatrix.h"
#endif
//#include "SparseMatrix/SparseVectorIndex.h"
#include "SparseMatrix/SparseIndexOMP.h"
#include "SlaterKoster/SpinOrbitBase.h"

class SparseMatrixSerial {
public:
  typedef double     T;
  typedef complex<T> dcomplex_t;
  typedef SparseIndexOMP Index_t;
//#ifdef OHMMS_DIM
//  typedef VectorIndex<PtclPairListBase<T,OHMMS_DIM> > Index_t;
//#else
//  typedef VectorIndex<PtclPairListBase<T,3> > Index_t;
//#endif
  typedef vector<SpinOrbitComp<T>* >    SOArray_t;
  typedef Matrix<complex<T> >           FullMatrix_t;

protected:

  int Num; 
  int NumPairs;
  int WS_size;

  Index_t* myIndex;
  T**  d_diag;
  T**  d_offdiag;
  int Dmax;
  SOArray_t SO;
  vector<Matrix<dcomplex_t>* > SOB; // spin-orbit block
  
public:

  SparseMatrixSerial(Index_t*);
  SparseMatrixSerial();
  ~SparseMatrixSerial(); 

  inline const Index_t* index() const { return myIndex;}

  void init(Index_t*);

  void reset(int assignmode = 0);

  SparseMatrixSerial& operator +=(T);

  void apply(const dcomplex_t* win, dcomplex_t* wout) const;

  void copy(FullMatrix_t& mout, int packing=0) const;

  void copy(FullMatrix_t& mout, const dcomplex_t*, int pacing=0) const;

  template<class Mat>
  void product(SparseMatrixSerial&, 
               const Mat&, const Mat&, const vector<T>& w) {}

  inline T* diag(int, int i) { return d_diag[i];}
  inline T* offdiag(int, int i) { return d_offdiag[i];}
  inline const T* diag(int, int i) const { return d_diag[i];}
  inline const T* offdiag(int, int i) const { return d_offdiag[i];}

  inline int first_diag(int ) const { return 0;}
  inline int last_diag(int ) const { return Num;}
  inline int first_offdiag(int ) const { return 0;}
  inline int last_offdiag(int ) const { return NumPairs;}

//    inline T* diag(int i) { return d_diag[i];}
//    inline T* offdiag(int i) { return d_offdiag[i];}
//    inline const T* diag(int i) const { return d_diag[i];}
//    inline const T* offdiag(int i) const { return d_offdiag[i];}

//    inline T* offdiag(int i, int j) { // expensive operation
//      if(i == j) return d_diag[i];
//      else       return d_offdiag[myIndex->loc(i,j)];
//    }
  
  void print(ostream& os);
private:
  void read_so_parameters();
};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseMatrixSpinOrbit.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

