//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim and Nichols A. Romero
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// Implemented by Jeongnim Kim, May 2002
// Modifided by Jeongnim Kim and Nichols A. Romero, May/30 2002
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_GAUSSIANDOS_H
#define OHMMS_GAUSSIANDOS_H
#include <vector>

template<class T>
struct GaussianDOS {

  T lower, upper, diff;
  std::vector<T> DOS;

  explicit GaussianDOS(T emin, T emax, int nbin) {
    init(emin,emax,nbin);
  }

  void init(T emin, T emax, int nbin) {
    lower = emin; upper = emax;
    diff = (upper-lower)/static_cast<T>(nbin);
    DOS.resize(nbin+1,T());
  }

  inline void accumulate(T a, T sig, T w=1.0) {
    T siginv = 0.5/sig/sig;
    T x = lower; 
    int i=0;
    T wnorm = w/(sqrt(2*3.1415926535)*sig);
    while(x<upper) {
      DOS[i] += wnorm*exp(-(x-a)*(x-a)*siginv);
      i++; x+=diff;
    }        
  }

  void print(ostream& os) {
    T x = lower; 
    for(int i=0; i<DOS.size(); i++) {
      os << x << " " << DOS[i] << endl; x+=diff;
    }
  }
};

#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: GaussianDOS.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
