//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Utilities/OhmmsSpecies.h"
using namespace std;

SpeciesBase::SpeciesBase() {
  TotalNum = 0;  
  Name.reserve(10); // expect less than 10 species
}

SpeciesBase::~SpeciesBase() {
  AttribList_t::iterator dit = d_attrib.begin();      
  for(; dit != d_attrib.end(); ++dit) delete (*dit);
}

// int SpeciesBase::addAttrib() {
//   int n = d_attrib.size();
//   d_attrib.push_back(new SpeciesAttrib_t(TotalNum));
//   return n;
// }

/** add a new attribute to the list
 *@param aname the name of an attribute
 */
int SpeciesBase::addAttrib(const char* aname) {
  map<string,int>::iterator it = attrib_ind.find(aname);
  if(it == attrib_ind.end()) {
    int a=d_attrib.size();
    attrib_ind[aname] = a;
    d_attrib.push_back(new SpeciesAttrib_t(TotalNum));
    return a;
  } else {
    return (*it).second;
  }
}

bool SpeciesBase::get(ostream& os) const {

  os << "Total Number of Species Attributes = " << d_attrib.size() << endl;
  os << "Total Number of Species  = " << TotalNum << endl;

  for(int isp=0; isp<TotalNum; isp++)  {
    os  << Name[isp] << " " ;
    for(int id=0; id< d_attrib.size(); id++) {
      os << d_attrib[id]->operator[](isp) << " ";
    }
    os << endl;
  }
  return true;
}

bool SpeciesBase::put(istream& is) {
  return true;
}

void SpeciesBase::reset() {

}

bool SpeciesBase::put(xmlNodePtr cur) {
  cur = cur->children;
  while(cur != NULL) {
    string cname((const char*)(cur->name));
    if(cname == "group") {
      int species_id=-1;
      xmlAttrPtr att = cur->properties;
      while(att != NULL) {
	string aname((const char*)(att->name));
	string vname((const char*)(att->children->content));
	if(aname == "name") {
	  species_id = getSpeciesID(vname.c_str());
	} 
	att = att->next;
      }
      if(species_id<0) break;//cannot have a negative species_id
      xmlNodePtr tcur = cur->children;
      while(tcur != NULL) {	//check parameters
	string tcname((const char*)tcur->name);
	if(tcname == "parameter") {
	  att = tcur->properties;
	  int attrib_id = -1;
	  double val;
	  while(att != NULL) {
	    string aname((const char*)(att->name));
	    if(aname == "name") {
	      attrib_id = addAttrib((const char*)(att->children->content));
	    }
	    att = att->next;
	  }
	  if(attrib_id>=0) {//cannot have a negative attrib_id
	    putContent(val,tcur);
	    d_attrib[attrib_id]->operator[](species_id)=val;
	  }
	}
	tcur = tcur->next;
      }
    } 
    cur = cur->next;
  }
  return true;
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: OhmmsSpecies.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

